@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -23,10 +23,9 @@
 */
 
 // Standard K-3D interface implementations for embedding
-#include <k3dsdk/application_implementation.h>
+#include <k3dsdk/application_detail.h>
 #include <k3dsdk/plugin_factory_collection.h>
 #include <k3dsdk/render_farm.h>
-#include <k3dsdk/shader_collection.h>
 
 // Standard K-3D interfaces and helpers
 #include <k3dsdk/algebra.h>
@@ -36,6 +35,7 @@
 #include <k3dsdk/create_plugins.h>
 #include <k3dsdk/fstream.h>
 #include <k3dsdk/gl_info.h>
+#include <k3dsdk/gzstream.h>
 #include <k3dsdk/i18n.h>
 #include <k3dsdk/ideletable.h>
 #include <k3dsdk/idocument.h>
@@ -47,9 +47,13 @@
 #include <k3dsdk/nodes.h>
 #include <k3dsdk/options_policy.h>
 #include <k3dsdk/property.h>
+#include <k3dsdk/render_farm_detail.h>
 #include <k3dsdk/register_application.h>
 #include <k3dsdk/scripting.h>
+#include <k3dsdk/shader_cache_detail.h>
+#include <k3dsdk/share_detail.h>
 #include <k3dsdk/system.h>
+#include <k3dsdk/user_interface_detail.h>
 #include <k3dsdk/utility.h>
 #include <k3dsdk/version.h>
 #include <k3dsdk/xml.h>
@@ -111,7 +115,6 @@ k3d::iuser_interface_plugin* g_user_interface = 0;
 boost::filesystem::path g_options_path;
 std::string g_plugin_paths;
 boost::filesystem::path g_shader_cache_path;
-std::string g_shader_source_paths;
 boost::filesystem::path g_share_path;
 boost::filesystem::path g_user_interface_path;
 boost::filesystem::path g_null_user_interface_path;
@@ -168,7 +171,6 @@ void set_default_options(bool& Quit, bool& Error)
 	g_options_path = data_path / "options.k3d";
 	g_plugin_paths = (module_path / "plugins").native_file_string();
 	g_shader_cache_path = data_path / "shadercache";
-	g_shader_source_paths = (module_path / "share" / "shaders").native_file_string();
 	g_share_path = module_path / "share";
 	g_user_interface_path = module_path / "libk3dngui.dll";
 	g_null_user_interface_path = module_path / "libk3dnui.dll";
@@ -303,17 +305,6 @@ string_array parse_command_line_2(const string_array Options, bool& Quit, bool&
 
 			g_plugin_paths = (*option);
 		}
-		else if((*option) == "--shaders")
-		{
-			++option;
-			if(option == Options.end())
-			{
-				handle_error("You must supply a delimited set of directories with the --shaders option!", Quit, Error);
-				return string_array();
-			}
-
-			g_shader_source_paths = (*option);
-		}
 		else if((*option) == "--shadercache")
 		{
 			++option;
@@ -381,12 +372,15 @@ void check_dependencies(bool& Quit, bool& Error)
 	k3d::log() << info << "plugin path(s): " << g_plugin_paths << std::endl;
 #endif // !K3D_STATIC_BUILD
 	k3d::log() << info << "shader cache path: " << g_shader_cache_path.native_file_string() << std::endl;
-	k3d::log() << info << "shader source path(s): " << g_shader_source_paths << std::endl;
 	k3d::log() << info << "share path: " << g_share_path.native_file_string() << std::endl;
 	k3d::log() << info << "user interface: " << g_user_interface_path.native_file_string() << std::endl;
 	k3d::log() << info << "home directory: " << k3d::system::get_home_directory().native_file_string() << std::endl;
 	k3d::log() << info << "temp directory: " << k3d::system::get_temp_directory().native_file_string() << std::endl;
 
+#ifdef K3D_HAVE_NLS
+	k3d::log() << info << "locale directory: " << K3D_LOCALE << std::endl;
+#endif // K3D_HAVE_NLS
+
 	// The options file must be specified and must exist ...
 	if(g_options_path.empty())
 	{
@@ -433,18 +427,6 @@ void check_dependencies(bool& Quit, bool& Error)
 		return;
 	}
 
-	// Every shader source path specified must exist ...
-	k3d::system::paths_t shader_source_paths;
-	k3d::system::decompose_path_list(g_shader_source_paths, shader_source_paths);
-	for(k3d::system::paths_t::const_iterator shader_source_path = shader_source_paths.begin(); shader_source_path != shader_source_paths.end(); ++shader_source_path)
-	{
-		if(!boost::filesystem::exists(*shader_source_path))
-		{
-			handle_error("Shader source path [" + shader_source_path->native_file_string() + "] does not exist.  Shutting down.", Quit, Error);
-			return;
-		}
-	}
-
 	// The share path must exist ...
 	if(!boost::filesystem::exists(g_share_path))
 	{
@@ -458,9 +440,6 @@ void check_dependencies(bool& Quit, bool& Error)
 		handle_error("Could not connect to an OpenGL server.  Shutting down.", Quit, Error);
 		return;
 	}
-
-	// Display details about the OpenGL implementation ...
-	k3d::gl::implementation(k3d::log());
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -482,7 +461,7 @@ void create_user_interface(k3d::plugin_factory_collection& Plugins, bool& Quit,
 		handle_error("UI plugin module [" + module_name + "] does not exist", Quit, Error);
 		return;
 	}
-	Plugins.load_module(g_user_interface_path, k3d::plugin_factory_collection::IGNORE_PROXIES, k3d::plugin_factory_collection::DONT_GENERATE_PROXIES);
+	Plugins.load_module(g_user_interface_path, k3d::plugin_factory_collection::IGNORE_PROXIES);
 
 #endif // !K3D_STATIC_BUILD
 
@@ -551,7 +530,7 @@ void load_modules(k3d::plugin_factory_collection& Plugins, bool& Quit, bool& Err
 
 #else // K3D_STATIC_BUILD
 
-	Plugins.load_modules(g_plugin_paths, true, k3d::plugin_factory_collection::IGNORE_PROXIES, k3d::plugin_factory_collection::DONT_GENERATE_PROXIES);
+	Plugins.load_modules(g_plugin_paths, true, k3d::plugin_factory_collection::LOAD_PROXIES);
 
 #endif // !K3D_STATIC_BUILD
 }
@@ -626,16 +605,17 @@ string_array parse_command_line_3(const string_array Options, bool& Quit, bool&
 
 			if(scriptname == "-")
 			{
+				k3d::log() << info << "Running script [STDIN]" << std::endl;
 				scriptname = "STDIN";
 				k3d::iscript_engine::context_t context;
-				k3d::execute_script(std::cin, scriptname, context, recognized, executed);
+				k3d::script::execute(k3d::script::code(std::cin), scriptname, context, recognized, executed);
 			}
 			else
 			{
 				k3d::log() << info << "Running script [" << scriptname << "]" << std::endl;
-				k3d::filesystem::ifstream file(boost::filesystem::path(scriptname, boost::filesystem::native));
+				k3d::filesystem::igzstream file(boost::filesystem::path(scriptname, boost::filesystem::native));
 				k3d::iscript_engine::context_t context;
-				k3d::execute_script(file, scriptname, context, recognized, executed);
+				k3d::script::execute(k3d::script::code(file), scriptname, context, recognized, executed);
 			}
 
 			if(!recognized)
@@ -688,11 +668,15 @@ int main(int argc, char* argv[])
 {
 	const std::string program_name = boost::filesystem::path(argv[0], boost::filesystem::native).leaf();
 
+#ifdef K3D_HAVE_NLS
+
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, K3D_LOCALE);
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
 	textdomain(PACKAGE);
 
+#endif
+
 	logbufs_t logbufs;
 
 	try
@@ -723,6 +707,12 @@ int main(int argc, char* argv[])
 		if(quit)
 			return error ? 1 : 0;
 
+		// Set the shader cache path ...
+		k3d::set_shader_cache_path(g_shader_cache_path);
+
+		// Set the share path ...
+		k3d::set_share_path(g_share_path);
+
 		// Load user options ...
 		k3d::options::file_storage user_options(g_options_path);
 		k3d::options::set_storage(user_options);
@@ -740,7 +730,7 @@ int main(int argc, char* argv[])
 		// Give the UI a chance to handle command-line options ...
 		if(g_user_interface)
 		{
-			options = g_user_interface->parse_command_line(options, g_share_path, quit, error);
+			options = g_user_interface->parse_command_line(options, quit, error);
 			if(quit)
 				return error ? 1 : 0;
 		}
@@ -752,22 +742,15 @@ int main(int argc, char* argv[])
 		if(quit)
 			return error ? 1 : 0;
 
-		// Load shaders ...
-		k3d::shader_collection shaders;
-		shaders.message_signal().connect(sigc::ptr_fun(startup_message_handler));
-		shaders.load_shaders(g_shader_source_paths);
-
 		// Setup a render farm ...
-		k3d::render_farm render_farm(g_options_path);
+		k3d::render_farm_implementation render_farm(g_options_path);
+		k3d::set_render_farm(render_farm);
+
+		// Register our user interface ...
+		k3d::set_user_interface(*dynamic_cast<k3d::iuser_interface*>(g_user_interface));
 
 		// Create the main application object ...
-		k3d::application_implementation application(
-			plugins,
-			shaders,
-			render_farm,
-			*dynamic_cast<k3d::iuser_interface*>(g_user_interface),
-			g_shader_cache_path,
-			g_share_path);
+		k3d::application_implementation application(plugins);
 
 		// We want to be notified when the user requests a shutdown ...
 		application.exit_signal().connect(sigc::ptr_fun(exit_request_handler));
@@ -810,4 +793,3 @@ int main(int argc, char* argv[])
 	return 0;
 }
 
-
