@@ -114,11 +114,6 @@ public:
 	void load(xml::element& Element, const ipersistent::load_context& Context)
 	{
 		std::string value = Element.text;
-
-		/** \deprecated Handle legacy values that were stored as attributes */
-		if(value.empty())
-			value = xml::attribute_text(Element, "value");
-
 		property_policy_t::set_value(from_string(value, property_policy_t::internal_value()));
 	}
 
@@ -196,11 +191,6 @@ public:
 	void load(xml::element& Element, const ipersistent::load_context& Context)
 	{
 		std::string value = Element.text;
-
-		/** \deprecated Handle legacy values that were stored as attributes */
-		if(value.empty())
-			value = xml::attribute_text(Element, "value");
-
 		property_policy_t::set_value(dynamic_cast<value_t>(Context.lookup.lookup_object(from_string(value, static_cast<ipersistent_lookup::id_type>(0)))));
 	}
 
@@ -403,13 +393,13 @@ public:
 		return m_deleted_signal;
 	}
 
-	bool property_set_value(const boost::any Value)
+	bool property_set_value(const boost::any Value, iunknown* const Hint)
 	{
 		const value_t* const new_value = boost::any_cast<value_t>(&Value);
 		if(!new_value)
 			return false;
 
-		set_value(*new_value);
+		name_policy_t::set_value(*new_value, Hint);
 		return true;
 	}
 
@@ -506,20 +496,20 @@ public:
 		return m_deleted_signal;
 	}
 
-	bool property_set_value(const boost::any Value)
+	bool property_set_value(const boost::any Value, iunknown* const Hint)
 	{
 		const std::string* const new_value = boost::any_cast<std::string>(&Value);
 		if(!new_value)
 			return false;
 
 		try
-			{
-				set_value(boost::lexical_cast<value_t>(*new_value));
-				return true;
-			}
+		{
+			set_value(boost::lexical_cast<value_t>(*new_value), Hint);
+			return true;
+		}
 		catch(...)
-			{
-			}
+		{
+		}
 
 		return false;
 	}
@@ -610,13 +600,12 @@ public:
 		return m_deleted_signal;
 	}
 
-	bool property_set_value(const boost::any Value)
+	bool property_set_value(const boost::any Value, iunknown* const Hint)
 	{
 		const value_t* const new_value = boost::any_cast<value_t>(&Value);
 		if(!new_value)
 			return false;
-
-		set_value(*new_value);
+		name_policy_t::set_value(*new_value, Hint);
 		return true;
 	}
 
@@ -625,6 +614,11 @@ public:
 		return m_mode;
 	}
 
+	const std::string property_path_type()
+	{
+		return m_type;
+	}
+
 	const ipath_property::reference_t property_path_reference()
 	{
 		return m_reference;
@@ -645,11 +639,16 @@ public:
 		return m_reference_changed_signal;
 	}
 
-	const std::string property_path_type()
+	const ipath_property::pattern_filters_t pattern_filters()
 	{
-		return m_type;
+		return m_pattern_filters;
 	}
 
+	void add_pattern_filter(const ipath_property::pattern_filter& PatternFilter)
+	{
+		m_pattern_filters.push_back(PatternFilter);
+	}
+	
 protected:
 	template<typename init_t>
 	path_property(const init_t& Init) :
@@ -659,8 +658,8 @@ protected:
 		m_label(Init.label()),
 		m_description(Init.description()),
 		m_mode(Init.path_mode()),
-		m_reference(RELATIVE_REFERENCE),
-		m_type(Init.path_type())
+		m_type(Init.path_type()),
+		m_reference(RELATIVE_REFERENCE)
 	{
 		Init.property_collection().register_property(*this);
 	}
@@ -677,9 +676,10 @@ private:
 	const char* const m_description;
 	deleted_signal_t m_deleted_signal;
 	const ipath_property::mode_t m_mode;
+	const std::string m_type;
 	ipath_property::reference_t m_reference;
 	ipath_property::path_reference_changed_signal_t m_reference_changed_signal;
-	const std::string m_type;
+	ipath_property::pattern_filters_t m_pattern_filters;
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -751,7 +751,7 @@ public:
 		return m_deleted_signal;
 	}
 
-	bool property_set_value(const boost::any Value)
+	bool property_set_value(const boost::any Value, iunknown* const Hint)
 	{
 		const std::string* const new_value = boost::any_cast<std::string>(&Value);
 		if(!new_value)
@@ -759,7 +759,7 @@ public:
 
 		try
 		{
-			set_value(boost::lexical_cast<value_t>(*new_value));
+			set_value(boost::lexical_cast<value_t>(*new_value), Hint);
 			return true;
 		}
 		catch(...)
@@ -863,7 +863,7 @@ public:
 		return m_deleted_signal;
 	}
 
-	bool property_set_value(const boost::any Value)
+	bool property_set_value(const boost::any Value, iunknown* const Hint)
 	{
 		const std::string* const new_value = boost::any_cast<std::string>(&Value);
 		if(!new_value)
@@ -871,7 +871,7 @@ public:
 
 		try
 		{
-			set_value(boost::lexical_cast<value_t>(*new_value));
+			set_value(boost::lexical_cast<value_t>(*new_value), Hint);
 			return true;
 		}
 		catch(...)
@@ -881,11 +881,21 @@ public:
 		return false;
 	}
 
-	enumeration_values_t property_values()
+	enumeration_values_t enumeration_values()
 	{
 		return m_values;
 	}
 
+	sigc::connection connect_enumeration_values_changed(const sigc::slot0<void>& Slot)
+	{
+		return m_values_changed_signal.connect(Slot);
+	}
+
+	void notify_enumeration_values_changed()
+	{
+		m_values_changed_signal.emit();
+	}
+
 protected:
 	template<typename init_t>
 	enumeration_property(const init_t& Init) :
@@ -910,6 +920,7 @@ private:
 	const char* const m_label;
 	const char* const m_description;
 	const ienumeration_property::enumeration_values_t& m_values;
+	sigc::signal0<void> m_values_changed_signal;
 	deleted_signal_t m_deleted_signal;
 };
 
@@ -974,13 +985,13 @@ public:
 		return m_deleted_signal;
 	}
 
-	bool property_set_value(const boost::any Value)
+	bool property_set_value(const boost::any Value, iunknown* const Hint)
 	{
 		const value_t* const new_value = boost::any_cast<value_t>(&Value);
 		if(!new_value)
 			return false;
 
-		set_value(*new_value);
+		set_value(*new_value, Hint);
 		return true;
 	}
 
@@ -1077,13 +1088,13 @@ public:
 		return m_deleted_signal;
 	}
 
-	bool property_set_value(const boost::any Value)
+	bool property_set_value(const boost::any Value, iunknown* const Hint)
 	{
 		inode* const * new_value = boost::any_cast<inode*>(&Value);
 		if(!new_value)
 			return false;
 
-		set_value(dynamic_cast<value_t>(*new_value));
+		name_policy_t::set_value(dynamic_cast<value_t>(*new_value), Hint);
 		return true;
 	}
 
@@ -1188,13 +1199,13 @@ public:
 		return m_deleted_signal;
 	}
 
-	bool property_set_value(const boost::any Value)
+	bool property_set_value(const boost::any Value, iunknown* const Hint)
 	{
 		const value_t* const new_value = boost::any_cast<value_t>(&Value);
 		if(!new_value)
 			return false;
 
-		set_value(*new_value);
+		name_policy_t::set_value(*new_value, Hint);
 		return true;
 	}
 
@@ -1297,10 +1308,10 @@ class no_constraint :
 	public undo_policy_t
 {
 public:
-	void set_value(const value_t& Value)
+	void set_value(const value_t& Value, iunknown* const Hint = 0)
 	{
 		if(Value != undo_policy_t::internal_value())
-			undo_policy_t::set_value(Value);
+			undo_policy_t::set_value(Value, Hint);
 	}
 
 protected:
@@ -1431,12 +1442,12 @@ class with_constraint :
 	public undo_policy_t
 {
 public:
-	void set_value(value_t Value)
+	void set_value(value_t Value, iunknown* const Hint = 0)
 	{
 		m_constraint->constrain(Value);
 
 		if(Value != undo_policy_t::internal_value())
-			undo_policy_t::set_value(Value);
+			undo_policy_t::set_value(Value, Hint);
 	}
 
 protected:
@@ -1469,9 +1480,9 @@ protected:
 	}
 
 	/// This little bit of magic makes it possible for base classes (such as node_storage) to update their own values while observing the correct undo policy
-	void internal_set_value(const value_t& Value)
+	void internal_set_value(const value_t& Value, iunknown* const Hint)
 	{
-		set_value(Value);
+		set_value(Value, Hint);
 	}
 };
 
@@ -1509,12 +1520,12 @@ protected:
 	typedef with_undo<value_t, storage_policy_t> this_t;
 
 	/// This little bit of magic makes it possible for base classes (such as node_storage) to update their own values while observing the correct undo policy
-	void internal_set_value(const value_t& Value)
+	void internal_set_value(const value_t& Value, iunknown* const Hint)
 	{
-		set_value(Value);
+		set_value(Value, Hint);
 	}
 
-	void set_value(const value_t& Value)
+	void set_value(const value_t& Value, iunknown* const Hint = 0)
 	{
 		if(ready_to_record())
 			{
@@ -1523,7 +1534,7 @@ protected:
 				storage_policy_t::start_recording(m_state_recorder);
 			}
 
-		storage_policy_t::set_value(Value);
+		storage_policy_t::set_value(Value, Hint);
 	}
 
 private:
@@ -1581,10 +1592,10 @@ protected:
 	}
 
 	/// Sets a new value for the data
-	void set_value(const value_t& Value)
+	void set_value(const value_t& Value, iunknown* const Hint = 0)
 	{
 		m_value = Value;
-		signal_policy_t::set_value(Value);
+		signal_policy_t::set_value(Hint);
 	}
 
 	/// Optionally called to store the new state of the data after one-or-more modifications
@@ -1680,7 +1691,7 @@ protected:
 
 	void on_node_deleted()
 	{
-		internal_set_value(0);
+		internal_set_value(0, 0);
 	}
 
 	/// Optionally called to store the original state of the data prior to modification
@@ -1691,7 +1702,7 @@ protected:
 	}
 
 	/// Sets a new value for the data
-	void set_value(value_t Value)
+	void set_value(value_t Value, iunknown* const Hint)
 	{
 		if(m_node)
 		{
@@ -1708,7 +1719,7 @@ protected:
 				m_node_changed_connection = node_change_signal->node_changed_signal().connect(signal_policy_t::changed_signal().make_slot());
 		}
 
-		signal_policy_t::set_value(Value);
+		signal_policy_t::set_value(Hint);
 	}
 
 	/// Optionally called to store the new state of the data after one-or-more modifications
@@ -1720,7 +1731,7 @@ protected:
 
 private:
 	/// This little bit of magic allows us to "call" the derived undo policy when our state is modified internally, e.g. if the node is deleted
-	virtual void internal_set_value(const value_t& Value)
+	virtual void internal_set_value(const value_t& Value, iunknown* const Hint)
 	{
 	}
 
@@ -1760,55 +1771,54 @@ class demand_storage :
 	public signal_policy_t
 {
 public:
-	/// Returns true iff the underlying data hasn't been created, yet
-	bool empty()
+	// This policy only works for data stored by-pointer
+	BOOST_STATIC_ASSERT((boost::is_pointer<value_t>::value));
+
+	/// Returns a slot that will invoke the reset() method
+	sigc::slot1<void, iunknown*> make_reset_slot()
 	{
-		return !m_value;
+		return sigc::mem_fun(*this, &demand_storage<value_t, signal_policy_t>::internal_reset);
+	}
+
+	/// Resets the underlying data so it will be created-again next read
+	void reset(value_t NewValue = 0, iunknown* const Hint = 0)
+	{
+		m_cache.reset(NewValue);
+		signal_policy_t::set_value(Hint);
 	}
 
 	/// Accesses the underlying data, creating it if it doesn't already exist
 	value_t internal_value()
 	{
-		if(!m_value)
-			m_value = m_need_data_signal.emit();
+		if(!m_cache.get())
+		{
+			// Note: we create the value and update its state in two steps
+			// because m_data_slot() may cause this method to be executed in a loop
+			m_cache.reset(new typename boost::remove_pointer<value_t>::type());
+			m_data_slot(*m_cache);
+		}
 
-		return m_value;
-	}
-
-	/// Resets the underlying data so it will be created-again next read
-	void reset(value_t NewValue = 0)
-	{
-		delete m_value;
-		m_value = NewValue;
-
-		signal_policy_t::set_value(NewValue);
-	}
-
-	/// Defines a signal that will be emitted to obtain data on-demand
-	typedef sigc::signal0<value_t> need_data_signal_t;
-
-	/// Signal emitted to obtain data on-demand
-	need_data_signal_t& need_data_signal()
-	{
-		return m_need_data_signal;
+		return m_cache.get();
 	}
 
 protected:
 	template<typename init_t>
 	demand_storage(const init_t& Init) :
 		signal_policy_t(Init),
-		m_value(0)
+		m_data_slot(Init.slot())
 	{
 	}
 
-	~demand_storage()
-	{
-		delete m_value;
-	}
-
 private:
-	value_t m_value;
-	need_data_signal_t m_need_data_signal;
+	/// Resets the underlying data so it will be created-again next read
+	void internal_reset(iunknown* const Hint = 0)
+	{
+		m_cache.reset();
+		signal_policy_t::set_value(Hint);
+	}
+
+	sigc::slot1<void, typename boost::remove_pointer<value_t>::type&> m_data_slot;
+	std::auto_ptr<typename boost::remove_pointer<value_t>::type> m_cache;
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -1820,14 +1830,35 @@ class computed_storage :
 	public signal_policy_t
 {
 public:
+	// This policy only works for data stored by-value
+	BOOST_STATIC_ASSERT((!boost::is_pointer<value_t>::value));
+
+	/// Returns a slot that will invoke the reset() method
+	sigc::slot1<void, iunknown*> make_reset_slot()
+	{
+		return sigc::mem_fun(*this, &computed_storage<value_t, signal_policy_t>::reset);
+	}
+
+	/// Resets the underlying data so it will be recalculated the next time it's read
+	void reset(iunknown* const Hint = 0)
+	{
+		m_cache.reset();
+		signal_policy_t::set_value(0);
+	}
+
 	/// Accesses the underlying data, creating it if it doesn't already exist
 	value_t internal_value()
 	{
-		return m_data_slot();
-	}
+		if(!m_cache.get())
+		{
+			// Note: we create the cache and set its value in two steps
+			// because m_data_slot() may cause this method to be executed in a loop
+			m_cache.reset(new value_t());
+			*m_cache = m_data_slot();
+		}
 
-	/// Defines a slot that will be called to obtain data on-demand
-	typedef sigc::slot0<value_t> data_slot_t;
+		return *m_cache;
+	}
 
 protected:
 	template<typename init_t>
@@ -1838,7 +1869,8 @@ protected:
 	}
 
 private:
-	data_slot_t m_data_slot;
+	sigc::slot0<value_t> m_data_slot;
+	std::auto_ptr<value_t> m_cache;
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -1858,7 +1890,7 @@ protected:
 	{
 	}
 
-	void set_value(const value_t& Value)
+	void set_value(iunknown* const Hint)
 	{
 	}
 
@@ -1875,8 +1907,8 @@ template<typename value_t>
 class change_signal
 {
 public:
-	/// Defines a signal emitted when the underlying data changes
-	typedef sigc::signal0<void> changed_signal_t;
+	/// Defines a signal emitted when the underlying data changes.  The signal includes an optional "hint" that describes the nature of the change.
+	typedef sigc::signal1<void, iunknown*> changed_signal_t;
 
 	/// Returns a reference to the signal that is emitted whenever the underlying data changes
 	changed_signal_t& changed_signal()
@@ -1894,22 +1926,21 @@ protected:
 	{
 	}
 
-	void set_value(const value_t& Value)
+	void set_value(iunknown* const Hint)
 	{
-		m_changed_signal.emit();
+		m_changed_signal.emit(Hint);
 	}
 
 	void finish_recording(istate_recorder& StateRecorder)
 	{
-		StateRecorder.current_change_set()->undo_signal().connect(m_changed_signal.make_slot());
-		StateRecorder.current_change_set()->redo_signal().connect(m_changed_signal.make_slot());
+		StateRecorder.current_change_set()->undo_signal().connect(sigc::bind(m_changed_signal.make_slot(), static_cast<iunknown*>(0)));
+		StateRecorder.current_change_set()->redo_signal().connect(sigc::bind(m_changed_signal.make_slot(), static_cast<iunknown*>(0)));
 	}
 
 private:
 	changed_signal_t m_changed_signal;
 };
 
-
 /// Convenience macro for declaring k3d::data::container nodes
 #define k3d_data(value_type, name_policy, signal_policy, undo_policy, storage_policy, constraint_policy, property_policy, serialization_policy) \
 	k3d::data::container<value_type, serialization_policy<value_type, property_policy<value_type, name_policy<constraint_policy<value_type, undo_policy<value_type, storage_policy<value_type, signal_policy<value_type> > > > > > > >
@@ -2423,10 +2454,10 @@ void undoable_delete(object_t* const Object, idocument& Document)
 	// If we aren't recording undos, delete it the old-fashioned way ...
 	state_change_set* const changeset = Document.state_recorder().current_change_set();
 	if(!changeset)
-		{
-			delete Object;
-			return;
-		}
+	{
+		delete Object;
+		return;
+	}
 
 	// Create an instance container that only deletes the object if it hasn't been undone
 	typedef detail::instance_container<object_t> container_t;
@@ -2443,4 +2474,3 @@ using namespace k3d::data;
 
 #endif // !K3DSDK_DATA_H
 
-
