@@ -18,7 +18,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 /** \file
-		\brief Provides a concrete implementation of k3d::idocument, which encapsulates an open K-3D document
+		\brief Provides a concrete implementation of idocument, which encapsulates an open K-3D document
 		\author Tim Shead (tshead@k-3d.com)
 		\author Dan Erikson (derikson@montana.com)
 */
@@ -40,6 +40,7 @@
 #include "inode_collection.h"
 #include "iplugin_factory.h"
 #include "iselectable.h"
+#include "node_name_map.h"
 #include "persistent.h"
 #include "plugins.h"
 #include "property_collection.h"
@@ -61,15 +62,18 @@ using namespace k3d::xml;
 #include <iterator>
 #include <memory>
 
-namespace
+namespace k3d
 {
 
+namespace detail
+{
+	
 /////////////////////////////////////////////////////////////////////////////
 // state_recorder_implementation
 
-/// Provides a concrete implementation of k3d::istate_recorder
+/// Provides a concrete implementation of istate_recorder
 class state_recorder_implementation :
-	public k3d::istate_recorder
+	public istate_recorder
 {
 public:
 	state_recorder_implementation() :
@@ -86,7 +90,7 @@ public:
 			delete_node(*node);
 
 		if(m_current_recording.get())
-			k3d::log() << warning << "Still recording undo/redo data at shutdown, this indicates an undo/redo leak" << std::endl;
+			log() << warning << "Still recording undo/redo data at shutdown, this indicates an undo/redo leak" << std::endl;
 	}
 
 	void delete_node(node* const Node)
@@ -99,27 +103,27 @@ public:
 		delete Node;
 	}
 
-	void start_recording(std::auto_ptr<k3d::state_change_set> ChangeSet)
+	void start_recording(std::auto_ptr<state_change_set> ChangeSet)
 	{
 		return_if_fail(ChangeSet.get());
 
 		if(m_current_recording.get())
 		{
-			k3d::log() << warning << "Starting undo/redo recording while recording is already in progress, this indicates an undo/redo leak" << std::endl;
-			std::auto_ptr<k3d::state_change_set> changeset = stop_recording();
+			log() << warning << "Starting undo/redo recording while recording is already in progress, this indicates an undo/redo leak" << std::endl;
+			std::auto_ptr<state_change_set> changeset = stop_recording();
 			commit_change_set(changeset, "Unfinished changeset");
 		}
-		
+
 		return_if_fail(!m_current_recording.get());
 		m_current_recording = ChangeSet;
 	}
 
-	k3d::state_change_set* current_change_set()
+	state_change_set* current_change_set()
 	{
 		return m_current_recording.get();
 	}
 
-	std::auto_ptr<k3d::state_change_set> stop_recording()
+	std::auto_ptr<state_change_set> stop_recording()
 	{
 		// Sanity checks ...
 		return_val_if_fail(m_current_recording.get(), m_current_recording);
@@ -128,13 +132,13 @@ public:
 		m_current_recording->recording_done_signal().emit();
 
 		// Now that recording is complete, get rid of leftover connections ...
-		k3d::state_change_set::recording_done_signal_t::slot_list_type slots = m_current_recording->recording_done_signal().slots();
+		state_change_set::recording_done_signal_t::slot_list_type slots = m_current_recording->recording_done_signal().slots();
 		slots.erase(slots.begin(), slots.end());
 
 		return m_current_recording;
 	}
 
-	void commit_change_set(std::auto_ptr<k3d::state_change_set> ChangeSet, const std::string& Label)
+	void commit_change_set(std::auto_ptr<state_change_set> ChangeSet, const std::string& Label)
 	{
 		// Sanity checks ...
 		return_if_fail(ChangeSet.get());
@@ -186,7 +190,7 @@ public:
 		m_current_node_changed_signal.emit();
 	}
 
-	k3d::istate_recorder::node_added_signal_t& node_added_signal()
+	istate_recorder::node_added_signal_t& node_added_signal()
 	{
 		return m_node_added_signal;
 	}
@@ -203,7 +207,7 @@ public:
 
 private:
 	/// Stores the current change set
-	std::auto_ptr<k3d::state_change_set> m_current_recording;
+	std::auto_ptr<state_change_set> m_current_recording;
 	/// Stores the root node(s) (if any)
 	nodes_t m_root_nodes;
 	/// Stores a reference to the current node (if any)
@@ -222,10 +226,10 @@ private:
 // node_collection_implementation
 
 class node_collection_implementation :
-	public k3d::inode_collection
+	public inode_collection
 {
 public:
-	node_collection_implementation(k3d::istate_recorder& StateRecorder) :
+	node_collection_implementation(istate_recorder& StateRecorder) :
 		m_state_recorder(StateRecorder)
 	{
 	}
@@ -238,9 +242,9 @@ public:
 	{
 		// Ensure no NULLs creep in ...
 		nodes_t nodes(Nodes);
-		nodes.erase(std::remove(nodes.begin(), nodes.end(), static_cast<k3d::inode*>(0)), nodes.end());
+		nodes.erase(std::remove(nodes.begin(), nodes.end(), static_cast<inode*>(0)), nodes.end());
 		if(nodes.size() != Nodes.size())
-			k3d::log() << warning << "NULL node cannot be inserted into node collection and will be ignored" << std::endl;
+			log() << warning << "NULL node cannot be inserted into node collection and will be ignored" << std::endl;
 
 		// We want to emit a signal whenever an node's name changes ...
 		for(nodes_t::const_iterator node = nodes.begin(); node != nodes.end(); ++node)
@@ -267,9 +271,9 @@ public:
 	{
 		// Ensure no NULLs creep in ...
 		nodes_t nodes(Nodes);
-		nodes.erase(std::remove(nodes.begin(), nodes.end(), static_cast<k3d::inode*>(0)), nodes.end());
+		nodes.erase(std::remove(nodes.begin(), nodes.end(), static_cast<inode*>(0)), nodes.end());
 		if(nodes.size() != Nodes.size())
-			k3d::log() << warning << "NULL node will be ignored" << std::endl;
+			log() << warning << "NULL node will be ignored" << std::endl;
 
 		// If we're recording undo/redo data, record the new state ...
 		if(m_state_recorder.current_change_set())
@@ -306,22 +310,22 @@ public:
 	void on_close_document()
 	{
 		// Give nodes a chance to shut down ...
-		for(k3d::inode_collection::nodes_t::iterator node = m_nodes.begin(); node != m_nodes.end(); ++node)
+		for(inode_collection::nodes_t::iterator node = m_nodes.begin(); node != m_nodes.end(); ++node)
 		{
 			(*node)->deleted_signal().emit();
 		}
 
 		// Zap nodes ...
-		for(k3d::inode_collection::nodes_t::iterator node = m_nodes.begin(); node != m_nodes.end(); ++node)
-			delete dynamic_cast<k3d::ideletable*>(*node);
+		for(inode_collection::nodes_t::iterator node = m_nodes.begin(); node != m_nodes.end(); ++node)
+			delete dynamic_cast<ideletable*>(*node);
 	}
 
 private:
 	class add_nodes_container :
-		public k3d::istate_container
+		public istate_container
 	{
 	public:
-		add_nodes_container(k3d::inode_collection& Collection, const k3d::inode_collection::nodes_t& Nodes) :
+		add_nodes_container(inode_collection& Collection, const inode_collection::nodes_t& Nodes) :
 			m_collection(Collection),
 			m_nodes(Nodes)
 		{
@@ -337,15 +341,15 @@ private:
 		}
 
 	private:
-		k3d::inode_collection& m_collection;
-		const k3d::inode_collection::nodes_t m_nodes;
+		inode_collection& m_collection;
+		const inode_collection::nodes_t m_nodes;
 	};
 
 	class remove_nodes_container :
-		public k3d::istate_container
+		public istate_container
 	{
 	public:
-		remove_nodes_container(k3d::inode_collection& Collection, const k3d::inode_collection::nodes_t& Nodes) :
+		remove_nodes_container(inode_collection& Collection, const inode_collection::nodes_t& Nodes) :
 			m_collection(Collection),
 			m_nodes(Nodes)
 		{
@@ -361,14 +365,14 @@ private:
 		}
 
 	private:
-		k3d::inode_collection& m_collection;
-		const k3d::inode_collection::nodes_t m_nodes;
+		inode_collection& m_collection;
+		const inode_collection::nodes_t m_nodes;
 	};
 
 	/// Provides storage for undo/redo information
-	k3d::istate_recorder& m_state_recorder;
+	istate_recorder& m_state_recorder;
 	/// Provides undo-able storage for a collection of nodes
-	k3d::inode_collection::nodes_t m_nodes;
+	inode_collection::nodes_t m_nodes;
 	/// Signal for notifying observers when nodes are added to the collection
 	add_nodes_signal_t m_add_nodes_signal;
 	/// Signal for notifying observers when nodes are removed from the collection
@@ -381,11 +385,11 @@ private:
 // dag_implementation
 
 class dag_implementation :
-	public k3d::idag,
+	public idag,
 	public sigc::trackable
 {
 public:
-	dag_implementation(k3d::istate_recorder& StateRecorder) :
+	dag_implementation(istate_recorder& StateRecorder) :
 		m_state_recorder(StateRecorder)
 	{
 	}
@@ -397,8 +401,8 @@ public:
 	void set_dependencies(dependencies_t& Dependencies)
 	{
 		// Don't let any NULLs creep in ...
-		if(Dependencies.erase(static_cast<k3d::iproperty*>(0)))
-			k3d::log() << warning << "Cannot assign a dependency to a NULL property" << std::endl;
+		if(Dependencies.erase(static_cast<iproperty*>(0)))
+			log() << warning << "Cannot assign a dependency to a NULL property" << std::endl;
 
 		// If we're recording undo/redo data, record the new state ...
 		if(m_state_recorder.current_change_set())
@@ -418,7 +422,7 @@ public:
 			{
 				m_change_connections[dependency->first] =
 					dependency->second->property_changed_signal().connect(
-						k3d::signal::make_loop_safe_slot(
+						signal::make_loop_safe_slot(
 							dependency->first->property_changed_signal()));
 			}
 		}
@@ -432,10 +436,10 @@ public:
 
 		// Synthesize change notifications for every property whose parent was set ...
 		for(dependencies_t::iterator dependency = Dependencies.begin(); dependency != Dependencies.end(); ++dependency)
-			dependency->first->property_changed_signal().emit();
+			dependency->first->property_changed_signal().emit(0);
 	}
 
-	k3d::iproperty* dependency(k3d::iproperty& Target)
+	iproperty* dependency(iproperty& Target)
 	{
 		return get_dependency(&Target)->second;
 	}
@@ -460,7 +464,7 @@ public:
 			connection->second.disconnect();
 	}
 
-	void on_property_deleted(k3d::iproperty* Property)
+	void on_property_deleted(iproperty* Property)
 	{
 		dependencies_t::iterator dependency = m_dependencies.find(Property);
 		return_if_fail(dependency != m_dependencies.end());
@@ -482,7 +486,7 @@ public:
 		for(dependencies_t::iterator dependency = m_dependencies.begin(); dependency != m_dependencies.end(); ++dependency)
 		{
 			if(dependency->second == Property)
-				new_dependencies.insert(std::make_pair(dependency->first, static_cast<k3d::iproperty*>(0)));
+				new_dependencies.insert(std::make_pair(dependency->first, static_cast<iproperty*>(0)));
 		}
 
 		if(new_dependencies.size())
@@ -490,14 +494,14 @@ public:
 	}
 
 private:
-	dependencies_t::iterator get_dependency(k3d::iproperty* Property)
+	dependencies_t::iterator get_dependency(iproperty* Property)
 	{
 		assert(Property);
 
 		dependencies_t::iterator result = m_dependencies.find(Property);
 		if(result == m_dependencies.end())
 		{
-			result = m_dependencies.insert(std::make_pair(Property, static_cast<k3d::iproperty*>(0))).first;
+			result = m_dependencies.insert(std::make_pair(Property, static_cast<iproperty*>(0))).first;
 			m_delete_connections[Property] = Property->property_deleted_signal().connect(sigc::bind(sigc::mem_fun(*this, &dag_implementation::on_property_deleted), Property));
 		}
 
@@ -505,10 +509,10 @@ private:
 	}
 
 	class set_dependencies_container :
-		public k3d::istate_container
+		public istate_container
 	{
 	public:
-		set_dependencies_container(k3d::idag& Dag, const k3d::idag::dependencies_t& Dependencies) :
+		set_dependencies_container(idag& Dag, const idag::dependencies_t& Dependencies) :
 			m_dag(Dag),
 			m_dependencies(Dependencies)
 		{
@@ -524,15 +528,15 @@ private:
 		}
 
 	private:
-		k3d::idag& m_dag;
-		k3d::idag::dependencies_t m_dependencies;
+		idag& m_dag;
+		idag::dependencies_t m_dependencies;
 	};
 
 	class delete_property_container :
-		public k3d::istate_container
+		public istate_container
 	{
 	public:
-		delete_property_container(dag_implementation& Dag, k3d::iproperty* const Property) :
+		delete_property_container(dag_implementation& Dag, iproperty* const Property) :
 			m_dag(Dag),
 			m_property(Property)
 		{
@@ -549,16 +553,16 @@ private:
 
 	private:
 		dag_implementation& m_dag;
-		k3d::iproperty* const m_property;
+		iproperty* const m_property;
 	};
 
-	k3d::istate_recorder& m_state_recorder;
+	istate_recorder& m_state_recorder;
 
 	/// Stores inter-property dependencies
 	dependencies_t m_dependencies;
 
 	/// Defines storage for per-property signal connections
-	typedef std::map<k3d::iproperty*, sigc::connection> connections_t;
+	typedef std::map<iproperty*, sigc::connection> connections_t;
 	/// Stores connections between property change signals (so a change to a source property is automatically passed-along to its dependent properties)
 	connections_t m_change_connections;
 	/// Stores connections to property delete signals (so we can clean-up when a property goes away)
@@ -572,14 +576,14 @@ private:
 
 /// Encapsulates an open K-3D document
 class public_document_implementation :
-	public k3d::idocument,
-	public k3d::command_node::implementation,
-	public k3d::property_collection,
+	public idocument,
+	public command_node::implementation,
+	public property_collection,
 	public sigc::trackable
 {
 public:
-	public_document_implementation(k3d::istate_recorder& StateRecorder, k3d::inode_collection& Nodes, k3d::idag& Dag) :
-		k3d::command_node::implementation("document", 0),
+	public_document_implementation(istate_recorder& StateRecorder, inode_collection& Nodes, idag& Dag) :
+		command_node::implementation("document", 0),
 		property_collection(),
 		m_state_recorder(StateRecorder),
 		m_nodes(Nodes),
@@ -587,8 +591,8 @@ public:
 		m_path(init_owner(*this) + init_name("path") + init_label(_("Document Path")) + init_description(_("Document Path")) + init_value(boost::filesystem::path())),
 		m_title(init_owner(*this) + init_name("title") + init_label(_("Document Title")) + init_description(_("Document Title")) + init_value<std::string>(""))
 	{
-		// We want to be asked before closing the application
-		k3d::application().safe_to_close_signal().connect(sigc::mem_fun(*this, &public_document_implementation::safe_to_close_application));
+ 		// Automatically add nodes to the unique node name collection
+ 		m_nodes.add_nodes_signal().connect(sigc::mem_fun(m_unique_node_names, &node_name_map::add_nodes));
 	}
 
 	~public_document_implementation()
@@ -596,74 +600,69 @@ public:
 		// Note: our close signal gets called by our owner - yes, it's a hack
 	}
 
-	k3d::idocument& document()
+	idocument& document()
 	{
 		return *this;
 	}
 
-	bool safe_to_close_application()
-	{
-		return m_safe_to_close_signal.empty() || m_safe_to_close_signal.emit();
-	}
-
-	k3d::inode_collection& nodes()
+	inode_collection& nodes()
 	{
 		return m_nodes;
 	}
 
-	k3d::idag& dag()
+	idag& dag()
 	{
 		return m_dag;
 	}
 
-	k3d::istate_recorder& state_recorder()
+	istate_recorder& state_recorder()
 	{
 		return m_state_recorder;
 	}
 
-	k3d::iproperty& path()
+ 	inode_name_map& unique_node_names()
+ 	{
+ 		return m_unique_node_names;
+ 	}
+
+	iproperty& path()
 	{
 		return m_path;
 	}
 
-	k3d::iproperty& title()
+	iproperty& title()
 	{
 		return m_title;
 	}
 
-	void enable_plugin_serialization(const k3d::uuid& ClassID, k3d::ipersistent& Handler)
+	void enable_plugin_serialization(const uuid& ClassID, ipersistent& Handler)
 	{
-		return_if_fail(ClassID != k3d::uuid::null());
+		return_if_fail(ClassID != uuid::null());
 		m_plugin_serialization_handlers.insert(std::make_pair(ClassID, &Handler));
 	}
 
-	const k3d::idocument::plugin_serialization_handlers_t& plugin_serialization_handlers()
+	const idocument::plugin_serialization_handlers_t& plugin_serialization_handlers()
 	{
 		return m_plugin_serialization_handlers;
 	}
-	
-	k3d::idocument::safe_to_close_signal_t& safe_to_close_signal()
-	{
-		return m_safe_to_close_signal;
-	}
 
-	k3d::idocument::close_signal_t& close_signal()
+	idocument::close_signal_t& close_signal()
 	{
 		return m_close_signal;
 	}
 
 private:
-	/// Used to find-out whether it's safe to close the document
-	safe_to_close_signal_t m_safe_to_close_signal;
 	/// Notifies observers that the document is being closed
 	close_signal_t m_close_signal;
 
 	/// Records changes made by the user for Undo / Redo purposes
-	k3d::istate_recorder& m_state_recorder;
+	istate_recorder& m_state_recorder;
 	/// Stores document nodes ...
-	k3d::inode_collection& m_nodes;
+	inode_collection& m_nodes;
 	/// Stores a reference to an implementation of idag
-	k3d::idag& m_dag;
+	idag& m_dag;
+ 	/// Stores unique node names
+ 	node_name_map m_unique_node_names;
 
 	/// Stores the full document filepath
 	k3d_data(boost::filesystem::path, immutable_name, change_signal, no_undo, local_storage, no_constraint, writable_property, no_serialization) m_path;
@@ -717,34 +716,30 @@ documents_t& documents()
 	return g_documents;
 }
 
-} // namespace
-
-namespace k3d
-{
+} // namespace detail
 
 idocument* create_document()
 {
-	std::auto_ptr<document_implementation> document(new document_implementation());
-	documents().push_back(document.get());
+	std::auto_ptr<detail::document_implementation> document(new detail::document_implementation());
+	detail::documents().push_back(document.get());
 	return document.release()->m_document;
 }
 
 void close_document(idocument& Document)
 {
-	for(documents_t::iterator document = documents().begin(); document != documents().end(); ++document)
+	for(detail::documents_t::iterator document = detail::documents().begin(); document != detail::documents().end(); ++document)
 	{
 		if((*document)->m_document == &Document)
 		{
 			delete *document;
-			documents().erase(document);
+			detail::documents().erase(document);
 
 			return;
 		}
 	}
 
-	k3d::log() << error << "close_document(): could not find document to destroy" << std::endl;
+	log() << error << "close_document(): could not find document to destroy" << std::endl;
 }
 
 } // namespace k3d
 
-
