@@ -50,7 +50,7 @@ public:
 	/// Returns the set of all properties provided by this object
 	virtual const properties_t& properties() = 0;
 	/// Defines a signal emitted whenever the collection changes - note: this signal is emitted when the collection changes (properties are added or deleted), not when individual property values are changed
-	typedef sigc::signal0<void> properties_changed_signal_t;
+	typedef sigc::signal1<void, iunknown*> properties_changed_signal_t;
 	virtual properties_changed_signal_t& properties_changed_signal() = 0;
 
 protected:
