@@ -44,23 +44,6 @@ class iuser_interface :
 	public virtual iunknown
 {
 public:
-	/// Enumerates mouse actions that can be displayed diagrammatically for tutorial purposes
-	typedef enum
-	{
-		MOUSE_DONE,
-		MB_NONE,
-		LMB_DRAG,
-		LMB_CLICK,
-		LMB_DOUBLE_CLICK,
-		MMB_DRAG,
-		MMB_CLICK,
-		MMB_DOUBLE_CLICK,
-		RMB_DRAG,
-		RMB_CLICK,
-		RMB_DOUBLE_CLICK,
-		LMBRMB_DRAG,
-	} mouse_action_t;
-
 	/// Specifies batch-mode (no user intervention, good for executing scripts)
 	virtual bool batch_mode() = 0;
 	/// Displays a URL in the user's preferred web browser
@@ -84,8 +67,6 @@ public:
 		\return true iff the user wants to continue the tutorial, false to quit
 	*/
 	virtual bool tutorial_message(const std::string& Message) = 0;
-	/// Displays an iconic mouse (for tutorial purposes) in a dialog box
-	virtual void tutorial_mouse_message(const std::string& Message, const mouse_action_t Action, const key_modifiers Modifiers) = 0;
 
 	/**
 		\brief Prompts the user for a filepath, checking for old choices, and storing the current choice for reuse
