@@ -119,6 +119,11 @@ std::ostream& backtrace(std::ostream& Stream)
 	return Stream;
 }
 
+void print_debug(const std::string& Message)
+{
+	log() << debug << Message << std::endl;
+}
+
 /////////////////////////////////////////////////////////////////////////////
 // tag_buf
 
