@@ -2,7 +2,7 @@
 #define K3DSDK_MESH_H
 
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -57,11 +57,11 @@ class point :
 	public memory::cache<point>
 {
 public:
-	point(const vector3& Position);
+	point(const point3& Position);
 	point(const double X, const double Y, const double Z);
 
 	/// Stores the position of the point in 3D space
-	vector3 position;
+	point3 position;
 	/// Stores vertex data for the point
 	parameters_t vertex_data;
 	/// Stores tag (SDS) data for the point
@@ -180,9 +180,9 @@ public:
 };
 
 /// Calculates the normal for an edge loop (returns a zero-length normal for degenerate cases)
-vector3 normal(const split_edge* const Loop);
+normal3 normal(const split_edge* const Loop);
 /// Calculates the normal for a face (returns a zero-length normal for degenerate cases)
-vector3 normal(const face& Face);
+normal3 normal(const face& Face);
 
 /////////////////////////////////////////////////////////////////////////////
 // polyhedron
@@ -751,11 +751,72 @@ void for_each_face(mesh& Mesh, T Functor)
 {
 	for(mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
 	{
+		Functor(**polyhedron);
 		for(polyhedron::faces_t::const_iterator face = (*polyhedron)->faces.begin(); face != (*polyhedron)->faces.end(); ++face)
 			Functor(**face);
 	}
 }
 
+/// Function template in the spirit of std::for_each that applies a functor to every linear curve in a mesh
+template<typename T>
+void for_each_linear_curve(mesh& Mesh, T Functor)
+{
+	for(mesh::linear_curve_groups_t::iterator group = Mesh.linear_curve_groups.begin(); group != Mesh.linear_curve_groups.end(); ++group)
+	{
+		Functor(**group);
+		for(linear_curve_group::curves_t::iterator curve = (*group)->curves.begin(); curve != (*group)->curves.end(); ++curve)
+			Functor(**curve);
+	}
+}
+
+/// Function template in the spirit of std::for_each that applies a functor to every cubic curve in a mesh
+template<typename T>
+void for_each_cubic_curve(mesh& Mesh, T Functor)
+{
+	for(mesh::cubic_curve_groups_t::iterator group = Mesh.cubic_curve_groups.begin(); group != Mesh.cubic_curve_groups.end(); ++group)
+	{
+		Functor(**group);
+		for(cubic_curve_group::curves_t::iterator curve = (*group)->curves.begin(); curve != (*group)->curves.end(); ++curve)
+			Functor(**curve);
+	}
+}
+
+/// Function template in the spirit of std::for_each that applies a functor to every nucurve in a mesh
+template<typename T>
+void for_each_nucurve(mesh& Mesh, T Functor)
+{
+	for(mesh::nucurve_groups_t::iterator group = Mesh.nucurve_groups.begin(); group != Mesh.nucurve_groups.end(); ++group)
+	{
+		Functor(**group);
+		for(nucurve_group::curves_t::iterator curve = (*group)->curves.begin(); curve != (*group)->curves.end(); ++curve)
+			Functor(**curve);
+	}
+}
+
+/// Function template in the spirit of std::for_each that applies a functor to every bilinear patch in a mesh
+template<typename T>
+void for_each_bilinear_patch(mesh& Mesh, T Functor)
+{
+	for(mesh::bilinear_patches_t::iterator patch = Mesh.bilinear_patches.begin(); patch != Mesh.bilinear_patches.end(); ++patch)
+		Functor(**patch);
+}
+
+/// Function template in the spirit of std::for_each that applies a functor to every bicubic patch in a mesh
+template<typename T>
+void for_each_bicubic_patch(mesh& Mesh, T Functor)
+{
+	for(mesh::bicubic_patches_t::iterator patch = Mesh.bicubic_patches.begin(); patch != Mesh.bicubic_patches.end(); ++patch)
+		Functor(**patch);
+}
+
+/// Function template in the spirit of std::for_each that applies a functor to every bilinear patch in a mesh
+template<typename T>
+void for_each_nupatch(mesh& Mesh, T Functor)
+{
+	for(mesh::nupatches_t::iterator patch = Mesh.nupatches.begin(); patch != Mesh.nupatches.end(); ++patch)
+		Functor(**patch);
+}
+
 /// Function template in the spirit of std::for_each that applies a functor to every part of a mesh
 template<typename T>
 T for_each_component(mesh& Mesh, T Functor)
