@@ -50,7 +50,7 @@ namespace detail
 
 //////////////////////////////////////////////////////////////////////////////
 // os_load_module
-	
+
 #if defined K3D_PLATFORM_WIN32
 
 	#include <windows.h>
@@ -60,7 +60,10 @@ namespace detail
 		if(file_extension(FilePath) != "dll")
 			return;
 
+		const UINT old_error_mode = SetErrorMode(SetErrorMode(SEM_FAILCRITICALERRORS)); // Disable error dialogs when loading DLLs
 		HINSTANCE module = LoadLibrary(FilePath.native_file_string().c_str());
+		SetErrorMode(old_error_mode);
+
 		if(!module)
 		{
 			log() << error << "Module [" << FilePath.native_file_string() << "] could not be loaded: error " << GetLastError() << std::endl;
@@ -322,7 +325,7 @@ private:
 
 /// Stores a mapping of plugin class id to plugin factory, so we can lookup recently-loaded factories quickly
 std::map<uuid, iplugin_factory*> proxied_factories;
-/// Stores a mapping of plugin class id to plugin module path, so we load modules that were proxied
+/// Stores a mapping of plugin class id to plugin module path, so we can load modules that were proxied
 std::map<uuid, boost::filesystem::path> proxied_modules;
 
 iplugin_factory* load_proxied_factory(const uuid& ClassID)
@@ -340,7 +343,7 @@ iplugin_factory* load_proxied_factory(const uuid& ClassID)
 		return 0;
 
 	// It's a K-3D module, all-right - give it a chance to register its plugins
-	log() << info << "Loading plugin module" << proxied_modules[ClassID].leaf() << std::endl;
+	log() << info << "Loading plugin module " << proxied_modules[ClassID].native_file_string() << std::endl;
 
 	message_signal_t message_signal;
 	iplugin_factory_collection::factories_t factories;
@@ -349,7 +352,7 @@ iplugin_factory* load_proxied_factory(const uuid& ClassID)
 	register_plugins(registry);
 	for(iplugin_factory_collection::factories_t::iterator factory = factories.begin(); factory != factories.end(); ++factory)
 		proxied_factories[(*factory)->class_id()] = (*factory);
-	
+
 	return proxied_factories[ClassID];
 }
 
@@ -362,7 +365,7 @@ class application_plugin_factory_proxy :
 	public iapplication_plugin_factory
 {
 public:
-	application_plugin_factory_proxy(const uuid& Class, const std::string& Name, const std::string& ShortDescription, const iplugin_factory::categories_t& Categories, const iplugin_factory::quality_t Quality, const types_t& Interfaces) :
+	application_plugin_factory_proxy(const uuid& Class, const std::string& Name, const std::string& ShortDescription, const iplugin_factory::categories_t& Categories, const iplugin_factory::quality_t Quality, const iplugin_factory::interfaces_t& Interfaces) :
 		m_factory(0),
 		m_application_factory(0),
 		m_class_id(Class),
@@ -378,56 +381,61 @@ public:
 	{
 		if(m_factory)
 			return m_factory->class_id();
-			
+
 		return m_class_id;
 	}
-	
+
 	const std::string name()
 	{
 		if(m_factory)
 			return m_factory->name();
-			
+
 		return m_name;
 	}
-	
+
 	const std::string short_description()
 	{
 		if(m_factory)
 			return m_factory->short_description();
-			
+
 		return m_short_description;
 	}
-	
+
 	const categories_t& categories()
 	{
 		if(m_factory)
 			return m_factory->categories();
-			
+
 		return m_categories;
 	}
-	
+
 	quality_t quality()
 	{
 		if(m_factory)
 			return m_factory->quality();
-			
+
 		return m_quality;
 	}
-	
+
 	bool implements(const std::type_info& InterfaceType)
 	{
 		if(m_factory)
 			return m_factory->implements(InterfaceType);
-	
-		for(types_t::const_iterator interface = m_interfaces.begin(); interface != m_interfaces.end(); ++interface)
+
+		for(iplugin_factory::interfaces_t::const_iterator iface = m_interfaces.begin(); iface != m_interfaces.end(); ++iface)
 		{
-			if((**interface) == InterfaceType)
+			if((**iface) == InterfaceType)
 				return true;
 		}
-		
+
 		return false;
 	}
 
+	const interfaces_t interfaces()
+	{
+		return m_interfaces;
+	}
+
 	iunknown* create_plugin()
 	{
 		if(!m_factory)
@@ -435,7 +443,7 @@ public:
 			m_factory = load_proxied_factory(m_class_id);
 			m_application_factory = dynamic_cast<iapplication_plugin_factory*>(m_factory);
 		}
-		
+
 		return_val_if_fail(m_application_factory, 0);
 		return m_application_factory->create_plugin();
 	}
@@ -443,13 +451,13 @@ public:
 private:
 	iplugin_factory* m_factory;
 	iapplication_plugin_factory* m_application_factory;
-	
+
 	const uuid m_class_id;
 	const std::string m_name;
 	const std::string m_short_description;
 	const iplugin_factory::categories_t m_categories;
 	const iplugin_factory::quality_t m_quality;
-	const types_t m_interfaces;
+	const iplugin_factory::interfaces_t m_interfaces;
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -461,7 +469,7 @@ class document_plugin_factory_proxy :
 	public idocument_plugin_factory
 {
 public:
-	document_plugin_factory_proxy(const uuid& Class, const std::string& Name, const std::string& ShortDescription, const iplugin_factory::categories_t& Categories, const iplugin_factory::quality_t Quality, const types_t& Interfaces) :
+	document_plugin_factory_proxy(const uuid& Class, const std::string& Name, const std::string& ShortDescription, const iplugin_factory::categories_t& Categories, const iplugin_factory::quality_t Quality, const iplugin_factory::interfaces_t& Interfaces) :
 		m_factory(0),
 		m_document_factory(0),
 		m_class_id(Class),
@@ -477,56 +485,61 @@ public:
 	{
 		if(m_factory)
 			return m_factory->class_id();
-			
+
 		return m_class_id;
 	}
-	
+
 	const std::string name()
 	{
 		if(m_factory)
 			return m_factory->name();
-			
+
 		return m_name;
 	}
-	
+
 	const std::string short_description()
 	{
 		if(m_factory)
 			return m_factory->short_description();
-			
+
 		return m_short_description;
 	}
-	
+
 	const categories_t& categories()
 	{
 		if(m_factory)
 			return m_factory->categories();
-			
+
 		return m_categories;
 	}
-	
+
 	quality_t quality()
 	{
 		if(m_factory)
 			return m_factory->quality();
-			
+
 		return m_quality;
 	}
-	
+
 	bool implements(const std::type_info& InterfaceType)
 	{
 		if(m_factory)
 			return m_factory->implements(InterfaceType);
-			
-		for(types_t::const_iterator interface = m_interfaces.begin(); interface != m_interfaces.end(); ++interface)
+
+		for(iplugin_factory::interfaces_t::const_iterator iface = m_interfaces.begin(); iface != m_interfaces.end(); ++iface)
 		{
-			if((**interface) == InterfaceType)
+			if((**iface) == InterfaceType)
 				return true;
 		}
-		
+
 		return false;
 	}
 
+	const interfaces_t interfaces()
+	{
+		return m_interfaces;
+	}
+
 	inode* create_plugin(idocument& Document)
 	{
 		if(!m_factory)
@@ -534,7 +547,7 @@ public:
 			m_factory = load_proxied_factory(m_class_id);
 			m_document_factory = dynamic_cast<idocument_plugin_factory*>(m_factory);
 		}
-		
+
 		return_val_if_fail(m_document_factory, 0);
 		return m_document_factory->create_plugin(Document);
 	}
@@ -542,13 +555,13 @@ public:
 private:
 	iplugin_factory* m_factory;
 	idocument_plugin_factory* m_document_factory;
-	
+
 	const uuid m_class_id;
 	const std::string m_name;
 	const std::string m_short_description;
 	const iplugin_factory::categories_t m_categories;
 	const iplugin_factory::quality_t m_quality;
-	const types_t m_interfaces;
+	const iplugin_factory::interfaces_t m_interfaces;
 };
 
 } // namespace detail
@@ -560,21 +573,23 @@ struct plugin_factory_collection::implementation
 {
 	bool proxy_module(const boost::filesystem::path& Path, const boost::filesystem::path& ProxyPath)
 	{
-		m_message_signal.emit(string_cast(boost::format(_("Proxying plugin module %1%")) % Path.leaf()));
-		
+		m_message_signal.emit(string_cast(boost::format(_("Proxying plugin module %1%")) % Path.native_file_string()));
+
 		try
 		{
 			filesystem::ifstream proxy_stream(ProxyPath);
 			xml::element xml_document;
 			proxy_stream >> xml_document;
-			
+
 			if(xml_document.name != "k3dml")
 				throw std::runtime_error("Not a k3dml document");
-				
+
 			xml::element* const xml_module = xml::find_element(xml_document, "module");
 			if(!xml_module)
 				throw std::runtime_error("Missing <module> tag");
-				
+
+			/** \todo Handle proxied module classes the same way we do it for un-proxied modules */
+/*
 			const uuid module_class = xml::attribute_value<uuid>(*xml_module, "class", uuid::null());
 			if(m_modules.count(module_class))
 			{
@@ -582,11 +597,12 @@ struct plugin_factory_collection::implementation
 				return false;
 			}
 			m_modules.insert(module_class);
-				
+*/
+
 			xml::element* const xml_plugins = xml::find_element(*xml_module, "plugins");
 			if(!xml_plugins)
 				throw std::runtime_error("Missing <plugins> tag");
-				
+
 			for(xml::element::elements_t::iterator xml_plugin = xml_plugins->children.begin(); xml_plugin != xml_plugins->children.end(); ++xml_plugin)
 			{
 				if(xml_plugin->name != "plugin")
@@ -607,12 +623,12 @@ struct plugin_factory_collection::implementation
 					{
 						if(xml_category->name != "category")
 							continue;
-							
+
 						plugin_categories.push_back(xml_category->text);
 					}
 				}
 
-				types_t plugin_interfaces;
+				iplugin_factory::interfaces_t plugin_interfaces;
 				if(xml::element* const xml_interfaces = xml::find_element(*xml_plugin, "interfaces"))
 				{
 					for(xml::element::elements_t::iterator xml_interface = xml_interfaces->children.begin(); xml_interface != xml_interfaces->children.end(); ++xml_interface)
@@ -650,11 +666,11 @@ struct plugin_factory_collection::implementation
 			log() << error << "Error proxying plugin module " << ProxyPath.native_file_string() << std::endl;
 			return false;
 		}
-		
+
 		return false;
 	}
 
-	void load_module(const boost::filesystem::path& Path, const load_proxy_t LoadProxies, const generate_proxy_t GenerateProxies)
+	void load_module(const boost::filesystem::path& Path, const load_proxy_t LoadProxies)
 	{
 		// If the module can be proxied for fast startup, do that instead ...
 		boost::filesystem::path proxy_path = boost::filesystem::path(Path.native_file_string() + ".proxy", boost::filesystem::native);
@@ -663,7 +679,7 @@ struct plugin_factory_collection::implementation
 			if(boost::filesystem::exists(proxy_path) && proxy_module(Path, proxy_path))
 				return;
 		}
-			
+
 		// OK, just load the module ...
 		register_module_entry_point register_module = 0;
 		register_plugins_entry_point register_plugins = 0;
@@ -677,69 +693,17 @@ struct plugin_factory_collection::implementation
 		register_module(module_id);
 		if(m_modules.count(module_id))
 		{
-			log() << info << "Skipping duplicate module " << Path.leaf() << std::endl;
+			log() << info << "Skipping duplicate module " << Path.native_file_string() << std::endl;
 			return;
 		}
 		m_modules.insert(module_id);
 
 		// It's a K-3D module, all-right - give it a chance to register its plugins
-		m_message_signal.emit(string_cast(boost::format(_("Loading plugin module %1%")) % Path.leaf()));
+		m_message_signal.emit(string_cast(boost::format(_("Loading plugin module %1%")) % Path.native_file_string()));
 
 		iplugin_factory_collection::factories_t new_factories;
 		detail::plugin_registry registry(m_message_signal, m_factories, new_factories);
 		register_plugins(registry);
-		
-		// Optionally generate proxies for the module
-		if(GenerateProxies == GENERATE_PROXIES)
-		{
-			xml::element xml_proxy("k3dml",
-				xml::attribute("package", K3D_PACKAGE),
-				xml::attribute("version", K3D_VERSION),
-				xml::attribute("host", K3D_HOST));
-				
-			xml::element& xml_module = xml_proxy.append(
-				xml::element("module",
-					xml::attribute("name", Path.leaf()),
-					xml::attribute("class", module_id)));
-					
-			xml::element& xml_plugins = xml_module.append(xml::element("plugins"));
-			
-			for(iplugin_factory_collection::factories_t::iterator factory = new_factories.begin(); factory != new_factories.end(); ++factory)
-			{
-				xml::element& xml_plugin = xml_plugins.append(
-					xml::element("plugin",
-						xml::attribute("name", (*factory)->name()),
-						xml::attribute("class", (*factory)->class_id()),
-						xml::attribute("quality", (*factory)->quality())));
-			
-				if(dynamic_cast<iapplication_plugin_factory*>(*factory))
-					xml_plugin.append(xml::attribute("type", "application"));
-				else if(dynamic_cast<idocument_plugin_factory*>(*factory))
-					xml_plugin.append(xml::attribute("type", "document"));
-				else
-				{
-					log() << error << "Unknown factory type, skipping proxy generation" << std::endl;
-					return;
-				}
-						
-				xml_plugin.append(xml::element("short_description", (*factory)->short_description()));
-				
-				xml::element& xml_categories = xml_plugin.append(xml::element("categories"));
-				for(iplugin_factory::categories_t::const_iterator category = (*factory)->categories().begin(); category != (*factory)->categories().end(); ++category)
-					xml_categories.append(xml::element("category", *category));
-					
-				xml::element& xml_interfaces = xml_plugin.append(xml::element("interfaces"));
-				const types_t types = registered_types();
-				for(types_t::const_iterator type = types.begin(); type != types.end(); ++type)
-				{
-					if((*factory)->implements(**type))
-						xml_interfaces.append(xml::element("interface", type_string(**type)));
-				}
-			}
-			
-			filesystem::ofstream proxy_file(proxy_path);
-			proxy_file << xml::declaration() << xml_proxy;
-		}
 	}
 
 	/// Stores a signal that will be emitted to display loading progress
@@ -783,12 +747,12 @@ void plugin_factory_collection::bind_module(const std::string& ModuleName, regis
 	RegisterPlugins(registry);
 }
 
-void plugin_factory_collection::load_module(const boost::filesystem::path& Path, const load_proxy_t LoadProxies, const generate_proxy_t GenerateProxies)
+void plugin_factory_collection::load_module(const boost::filesystem::path& Path, const load_proxy_t LoadProxies)
 {
-	m_implementation->load_module(Path, LoadProxies, GenerateProxies);
+	m_implementation->load_module(Path, LoadProxies);
 }
 
-void plugin_factory_collection::load_modules(const boost::filesystem::path& Path, const bool Recursive, const load_proxy_t LoadProxies, const generate_proxy_t GenerateProxies)
+void plugin_factory_collection::load_modules(const boost::filesystem::path& Path, const bool Recursive, const load_proxy_t LoadProxies)
 {
 	m_implementation->m_message_signal.emit(
 		string_cast(boost::format(_("Searching for plugins in %1%")) % Path.native_file_string()));
@@ -798,22 +762,22 @@ void plugin_factory_collection::load_modules(const boost::filesystem::path& Path
 		if(boost::filesystem::is_directory(*path))
 		{
 			if(Recursive)
-				load_modules(*path, Recursive, LoadProxies, GenerateProxies);
+				load_modules(*path, Recursive, LoadProxies);
 		}
 		else
 		{
-			load_module(*path, LoadProxies, GenerateProxies);
+			load_module(*path, LoadProxies);
 		}
 	}
 }
 
-void plugin_factory_collection::load_modules(const std::string& Paths, const bool Recursive, const load_proxy_t LoadProxies, const generate_proxy_t GenerateProxies)
+void plugin_factory_collection::load_modules(const std::string& Paths, const bool Recursive, const load_proxy_t LoadProxies)
 {
 	system::paths_t paths;
 	system::decompose_path_list(Paths, paths);
 
 	for(system::paths_t::const_iterator path = paths.begin(); path != paths.end(); ++path)
-		load_modules(*path, Recursive, LoadProxies, GenerateProxies);
+		load_modules(*path, Recursive, LoadProxies);
 }
 
 const iplugin_factory_collection::factories_t& plugin_factory_collection::factories()
