@@ -37,9 +37,6 @@ namespace k3d
 /// Recompiles a shader from source iff it's out-of-date
 bool compile_shader(const boost::filesystem::path& SourceFile, const std::string& RenderEngineType, const std::string& RenderEngine);
 
-/// Returns a shader by name (could return NULL)
-std::auto_ptr<sl::shader> lookup_shader(const std::string& ShaderName);
-
 } // namespace k3d
 
 #endif // !K3DSDK_SHADERS_H
