@@ -182,7 +182,14 @@ bool is_argument_type(const token_t& InputToken)
 {
 	static const char* types[] =
 	{
-		"float", "color", "point", "vector", "normal", "matrix", "string"
+		"float",
+		"string",
+		"point",
+		"vector",
+		"normal",
+		"hpoint",
+		"matrix",
+		"color"
 	};
 
 	for(unsigned long i = 0; i < sizeof(types) / sizeof(const char*); i++)
@@ -207,24 +214,24 @@ bool is_string(const token_t& InputToken)
 /////////////////////////////////////////////////////////////////////////////
 // parse_arguments
 
-class vector3
+class point3
 {
 public:
 	/// Stores the vector values
 	double n[3];
 
-	vector3()
+	point3()
 	{
 		n[0] = n[1] = n[2] = 0.0;
 	}
 
-	friend std::ostream& operator<<(std::ostream& Stream, const vector3& RHS)
+	friend std::ostream& operator<<(std::ostream& Stream, const point3& RHS)
 	{
 		Stream << RHS.n[0] << " " << RHS.n[1] << " " << RHS.n[2];
 		return Stream;
 	}
 
-	friend std::istream& operator>>(std::istream& Stream, vector3& RHS)
+	friend std::istream& operator>>(std::istream& Stream, point3& RHS)
 	{
 		Stream >> RHS.n[0];
 		RHS.n[1] = RHS.n[2] = RHS.n[0];
@@ -271,6 +278,9 @@ k3d::sl::shader::arguments_t parse_arguments(token_collection_t& Arguments, cons
 									case k3d::sl::argument::FLOAT:
 										extended_type = k3d::sl::argument::EX_FLOAT;
 										break;
+									case k3d::sl::argument::STRING:
+										extended_type = k3d::sl::argument::EX_STRING;
+										break;
 									case k3d::sl::argument::POINT:
 										extended_type = k3d::sl::argument::EX_POINT;
 										break;
@@ -280,17 +290,15 @@ k3d::sl::shader::arguments_t parse_arguments(token_collection_t& Arguments, cons
 									case k3d::sl::argument::NORMAL:
 										extended_type = k3d::sl::argument::EX_NORMAL;
 										break;
-									case k3d::sl::argument::COLOR:
-										extended_type = k3d::sl::argument::EX_COLOR;
+									case k3d::sl::argument::HPOINT:
+										extended_type = k3d::sl::argument::EX_HPOINT;
 										break;
 									case k3d::sl::argument::MATRIX:
 										extended_type = k3d::sl::argument::EX_MATRIX;
 										break;
-									case k3d::sl::argument::STRING:
-										extended_type = k3d::sl::argument::EX_STRING;
+									case k3d::sl::argument::COLOR:
+										extended_type = k3d::sl::argument::EX_COLOR;
 										break;
-									default:
-										assert(0);	// Who invented a new type without telling me?
 								}
 
 							++token;
@@ -366,7 +374,7 @@ k3d::sl::shader::arguments_t parse_arguments(token_collection_t& Arguments, cons
 					if((type == k3d::sl::argument::COLOR) || (type == k3d::sl::argument::POINT) || (type == k3d::sl::argument::VECTOR) || (type == k3d::sl::argument::NORMAL))
 						{
 							std::istringstream ibuffer(default_value);
-							vector3 temp;
+							point3 temp;
 							ibuffer >> temp;
 
 							std::ostringstream obuffer;
@@ -390,7 +398,7 @@ k3d::sl::shader::arguments_t parse_arguments(token_collection_t& Arguments, cons
 					// These can't be extracted from SL code
 					const std::string label;
 					const std::string description;
-					
+
 					// Push the new argument into our results ...
 					results.push_back(k3d::sl::argument(name, label, description, storage_class, type, extended_type, array_size, space, output, default_value));
 
@@ -536,9 +544,6 @@ std::ostream& operator<<(std::ostream& Stream, const argument::storage_class_t&
 			case argument::VARYING:
 				Stream << "varying";
 				break;
-			default:
-				assert(0);
-				break;
 		}
 
 	return Stream;
@@ -568,6 +573,9 @@ std::ostream& operator<<(std::ostream& Stream, const argument::type_t& Value)
 			case argument::FLOAT:
 				Stream << "float";
 				break;
+			case argument::STRING:
+				Stream << "string";
+				break;
 			case argument::POINT:
 				Stream << "point";
 				break;
@@ -577,17 +585,14 @@ std::ostream& operator<<(std::ostream& Stream, const argument::type_t& Value)
 			case argument::NORMAL:
 				Stream << "normal";
 				break;
-			case argument::COLOR:
-				Stream << "color";
+			case argument::HPOINT:
+				Stream << "hpoint";
 				break;
 			case argument::MATRIX:
 				Stream << "matrix";
 				break;
-			case argument::STRING:
-				Stream << "string";
-				break;
-			default:
-				assert(0);
+			case argument::COLOR:
+				Stream << "color";
 				break;
 		}
 
@@ -601,18 +606,20 @@ std::istream& operator>>(std::istream& Stream, argument::type_t& Value)
 
 	if(text == "float")
 		Value = argument::FLOAT;
+	else if(text == "string")
+		Value = argument::STRING;
 	else if(text == "point")
 		Value = argument::POINT;
 	else if(text == "vector")
 		Value = argument::VECTOR;
 	else if(text == "normal")
 		Value = argument::NORMAL;
-	else if(text == "color")
-		Value = argument::COLOR;
+	else if(text == "hpoint")
+		Value = argument::HPOINT;
 	else if(text == "matrix")
 		Value = argument::MATRIX;
-	else if(text == "string")
-		Value = argument::STRING;
+	else if(text == "color")
+		Value = argument::COLOR;
 	else
 		{
 			log() << error << "Unknown Shading Language Type: [" << text << "]" << std::endl;
@@ -652,21 +659,6 @@ std::ostream& operator<<(std::ostream& Stream, const argument::extended_type_t&
 			case argument::EX_PRESSURE:
 				Stream << "pressure";
 				break;
-			case argument::EX_POINT:
-				Stream << "point";
-				break;
-			case argument::EX_VECTOR:
-				Stream << "vector";
-				break;
-			case argument::EX_NORMAL:
-				Stream << "normal";
-				break;
-			case argument::EX_COLOR:
-				Stream << "color";
-				break;
-			case argument::EX_MATRIX:
-				Stream << "matrix";
-				break;
 			case argument::EX_STRING:
 				Stream << "string";
 				break;
@@ -676,8 +668,23 @@ std::ostream& operator<<(std::ostream& Stream, const argument::extended_type_t&
 			case argument::EX_SPACE:
 				Stream << "space";
 				break;
-			default:
-				assert(0);
+			case argument::EX_POINT:
+				Stream << "point";
+				break;
+			case argument::EX_VECTOR:
+				Stream << "vector";
+				break;
+			case argument::EX_NORMAL:
+				Stream << "normal";
+				break;
+			case argument::EX_HPOINT:
+				Stream << "hpoint";
+				break;
+			case argument::EX_MATRIX:
+				Stream << "matrix";
+				break;
+			case argument::EX_COLOR:
+				Stream << "color";
 				break;
 		}
 
@@ -707,22 +714,24 @@ std::istream& operator>>(std::istream& Stream, argument::extended_type_t& Value)
 		Value = argument::EX_FORCE;
 	else if(text == "pressure")
 		Value = argument::EX_PRESSURE;
-	else if(text == "point")
-		Value = argument::EX_POINT;
-	else if(text == "vector")
-		Value = argument::EX_VECTOR;
-	else if(text == "normal")
-		Value = argument::EX_NORMAL;
-	else if(text == "color")
-		Value = argument::EX_COLOR;
-	else if(text == "matrix")
-		Value = argument::EX_MATRIX;
 	else if(text == "string")
 		Value = argument::EX_STRING;
 	else if(text == "texture")
 		Value = argument::EX_TEXTURE;
 	else if(text == "space")
 		Value = argument::EX_SPACE;
+	else if(text == "point")
+		Value = argument::EX_POINT;
+	else if(text == "vector")
+		Value = argument::EX_VECTOR;
+	else if(text == "normal")
+		Value = argument::EX_NORMAL;
+	else if(text == "hpoint")
+		Value = argument::EX_HPOINT;
+	else if(text == "matrix")
+		Value = argument::EX_MATRIX;
+	else if(text == "color")
+		Value = argument::EX_COLOR;
 	else
 		{
 			log() << error << "Unknown Extended Shading Language Type: [" << text << "]" << std::endl;
@@ -753,9 +762,6 @@ std::ostream& operator<<(std::ostream& Stream, const shader::type_t& Value)
 			case shader::IMAGER:
 				Stream << "imager";
 				break;
-			default:
-				assert(0);
-				break;
 		}
 
 	return Stream;
@@ -910,7 +916,3 @@ void generate_metafile(std::ostream& Stream, const shaders_t Shaders)
 } // namespace sl
 
 } // namespace k3d
-
-
-
-
