@@ -92,7 +92,7 @@ public:
 	}
 
 	/// Scale element
-	void scale(const k3d::vector2 scaling)
+	void scale(const k3d::point2 scaling)
 	{
 		double scaling_x = scaling[0];
 		double scaling_y = scaling[1];
@@ -286,7 +286,7 @@ public:
 	}
 
 	/// Set element position
-	void position(const k3d::vector2 position)
+	void position(const k3d::point2 position)
 	{
 		m_position_x = static_cast<int>(std::floor(position[0])); // integral translation
 		m_position_y = static_cast<int>(std::floor(position[1])); // integral translation
@@ -382,7 +382,7 @@ public:
 				}
 				
 				// Color transform
-				k3d::vector3 rgb_vector(I_red, I_green, I_blue);
+				k3d::point3 rgb_vector(I_red, I_green, I_blue);
 				rgb_vector = rgb_vector * color_matrix;
 				
 				// Matte multiply (always) ...
