@@ -45,8 +45,8 @@ class bitmap_instance :
 public:
 	bitmap_instance(k3d::idocument& Document) :
 		base_t(Document),
-		m_position(init_owner(*this) + init_name("position") + init_label(_("Position")) + init_description(_("Position")) + init_value(k3d::vector3(0.0, 0.0, 0.0))),
-		m_scale(init_owner(*this) + init_name("scale") + init_label(_("Scale")) + init_description(_("Scale")) + init_value(k3d::vector3(1.0, 1.0, 1.0))),
+		m_position(init_owner(*this) + init_name("position") + init_label(_("Position")) + init_description(_("Position")) + init_value(k3d::point3(0.0, 0.0, 0.0))),
+		m_scale(init_owner(*this) + init_name("scale") + init_label(_("Scale")) + init_description(_("Scale")) + init_value(k3d::point3(1.0, 1.0, 1.0))),
 		m_angle(init_owner(*this) + init_name("angle") + init_label(_("Angle")) + init_description(_("Angle")) + init_value(k3d::radians(0.0)) + init_precision(2) + init_step_increment(k3d::radians(1.0)) + init_units(typeid(k3d::measurement::angle))),
 		m_premultiplied(init_owner(*this) + init_name("premultiplied") + init_label(_("Premultiplied")) + init_description(_("Image is pre-multiplied")) + init_value(true)),
 		m_color_matrix(init_owner(*this) + init_name("color_matrix") + init_label(_("Color Matrix")) + init_description(_("Color matrix")) + init_value(k3d::identity3D()))
@@ -57,37 +57,36 @@ public:
 		m_premultiplied.changed_signal().connect(sigc::mem_fun(*this, &bitmap_instance::on_reset_bitmap));
 		m_color_matrix.changed_signal().connect(sigc::mem_fun(*this, &bitmap_instance::on_reset_bitmap));
 		m_input.changed_signal().connect(sigc::mem_fun(*this, &bitmap_instance::on_reset_bitmap));
-		m_output.need_data_signal().connect(sigc::mem_fun(*this, &bitmap_instance::on_create_bitmap));
 	}
 
-	void on_reset_bitmap()
+	void on_reset_bitmap(k3d::iunknown* const Hint)
 	{
-		m_output.reset();
+		m_output.reset(0, Hint);
 	}
 
-	k3d::bitmap* on_create_bitmap()
+	void on_create_bitmap(k3d::bitmap& Bitmap)
 	{
 		// Get the input bitmap ...
 		k3d::bitmap* const input = m_input.value();
 		if(!input)
-			return 0;
+			return;
 
 		// Cache properties ...
 		k3d::matrix4 color_matrix = m_color_matrix.value();
 		bool premultiplied = m_premultiplied.value();
 		double angle = m_angle.value();
-		k3d::vector3 scale = m_scale.value();
-		k3d::vector3 position = m_position.value();
+		k3d::point3 scale = m_scale.value();
+		k3d::point3 position = m_position.value();
 
 		// Transform ...
 		bitmap_element* element = new bitmap_element(input);
 		element->color(color_matrix, premultiplied);
-		element->scale(k3d::vector2(scale[0], scale[1]));
+		element->scale(k3d::point2(scale[0], scale[1]));
 		element->rotate(angle);
-		element->position(k3d::vector2(position[0], position[1]));
+		element->position(k3d::point2(position[0], position[1]));
 		element->true_translation();
 
-		return element->get_bitmap();
+		Bitmap = *element->get_bitmap();
 	}
 
 	k3d::iplugin_factory& factory()
@@ -111,8 +110,8 @@ public:
 	}
 
 private:
-	k3d_data(k3d::vector3, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_position;
-	k3d_data(k3d::vector3, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_scale;
+	k3d_data(k3d::point3, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_position;
+	k3d_data(k3d::point3, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_scale;
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, measurement_property, with_serialization) m_angle;
 	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_premultiplied;
 	k3d_data(k3d::matrix4, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_color_matrix;
