@@ -45,11 +45,11 @@ public:
 		m_value.changed_signal().connect(sigc::mem_fun(*this, &bitmap_matte_colordiff::on_value_change));
 	}
 
-	void on_value_change()
+	void on_value_change(k3d::iunknown* const Hint)
 	{
 		m_value_cache = m_value.value();
 
-		on_reset_bitmap();
+		on_reset_bitmap(Hint);
 	}
 
         void on_filter(const k3d::pixel& I, k3d::pixel& O)
