@@ -46,12 +46,11 @@ public:
 		base(Document)
 	{
 		m_input.changed_signal().connect(sigc::mem_fun(*this, &bitmap_matte_invert::on_reset_bitmap));
-		m_output.need_data_signal().connect(sigc::mem_fun(*this, &bitmap_matte_invert::on_create_bitmap));
 	}
-	
-	void on_reset_bitmap()
+
+	void on_reset_bitmap(k3d::iunknown* const Hint)
 	{
-		m_output.reset();
+		m_output.reset(0, Hint);
 	}
 
 	struct invert_alpha
@@ -62,18 +61,16 @@ public:
 		}
 	};
 
-	k3d::bitmap* on_create_bitmap()
+	void on_create_bitmap(k3d::bitmap& Bitmap)
 	{
 		// If we don't have any input bitmap, we're done ...
 		const k3d::bitmap* const input = m_input.value();
 		if(!input)
-			return 0;
+			return;
 
 		// Otherwise, we make a copy of the input bitmap and modify the copy ...
-		k3d::bitmap* const output = new k3d::bitmap(*input);
-		std::for_each(output->begin(), output->end(), invert_alpha());
-
-		return output;
+		Bitmap = *input;
+		std::for_each(Bitmap.begin(), Bitmap.end(), invert_alpha());
 	}
 
 	k3d::iplugin_factory& factory()
