@@ -47,20 +47,19 @@ public:
 		m_width.changed_signal().connect(sigc::mem_fun(*this, &bitmap_size::on_reset_bitmap));
 		m_height.changed_signal().connect(sigc::mem_fun(*this, &bitmap_size::on_reset_bitmap));
 		m_input.changed_signal().connect(sigc::mem_fun(*this, &bitmap_size::on_reset_bitmap));
-		m_output.need_data_signal().connect(sigc::mem_fun(*this, &bitmap_size::on_create_bitmap));
 	}
 
-	void on_reset_bitmap()
+	void on_reset_bitmap(k3d::iunknown* const Hint)
 	{
-		m_output.reset();
+		m_output.reset(0, Hint);
 	}
 
-	k3d::bitmap* on_create_bitmap()
+	void on_create_bitmap(k3d::bitmap& Bitmap)
 	{
 		// If we don't have any input bitmap, we're done ...
 		const k3d::bitmap* const input = m_input.value();
 		if(!input)
-			return 0;
+			return;
 
 		// Cache properties ...
 		const k3d::pixel_size_t width = m_width.value();
@@ -70,28 +69,26 @@ public:
 		const long width_diff = width - input->width();
 		const long height_diff = height - input->height();
 
-		k3d::bitmap* output = new k3d::bitmap(*input);
+		Bitmap = *input;
 		k3d::pixel_size_t left, right, top, bottom;
 
 		left = right = width_diff > 0 ? (width_diff/2) : 0;
 		top = bottom = height_diff > 0 ? (height_diff/2) : 0;
 		if (left != 0 || right != 0 || top != 0 || bottom != 0)
 		{
-			k3d::bitmap* const output_padded = new k3d::bitmap(left + output->width() + right, top + output->height() + bottom);
-			bitmap_padding(*output, *output_padded, left, right, top, bottom);
-			output = output_padded;
+			k3d::bitmap* const output_padded = new k3d::bitmap(left + Bitmap.width() + right, top + Bitmap.height() + bottom);
+			bitmap_padding(Bitmap, *output_padded, left, right, top, bottom);
+			Bitmap = *output_padded;
 		}
 
 		left = right = width_diff < 0 ? std::abs(width_diff/2) : 0;
 		top = bottom = height_diff < 0 ? std::abs(height_diff/2) : 0;
 		if (left != 0 || right != 0 || top != 0 || bottom != 0)
 		{
-			k3d::bitmap* const output_cropped = new k3d::bitmap(output->width() - left - right, output->height() - top - bottom);
-			bitmap_croping(*output, *output_cropped, left, right, top, bottom);
-			output = output_cropped;
+			k3d::bitmap* const output_cropped = new k3d::bitmap(Bitmap.width() - left - right, Bitmap.height() - top - bottom);
+			bitmap_croping(Bitmap, *output_cropped, left, right, top, bottom);
+			Bitmap = *output_cropped;
 		}
-
-		return output;
 	}
 
 	k3d::iplugin_factory& factory()
