@@ -51,13 +51,13 @@ public:
 		m_alpha_threshold.changed_signal().connect(sigc::mem_fun(*this, &bitmap_threshold::on_value_change));
 	}
 
-	void on_value_change()
+	void on_value_change(k3d::iunknown* const Hint)
 	{
 		m_red_cache   = m_red_threshold.value();
 		m_green_cache = m_green_threshold.value();
 		m_blue_cache  = m_blue_threshold.value();
 		m_alpha_cache = m_alpha_threshold.value();
-		on_reset_bitmap();
+		on_reset_bitmap(Hint);
 	}
 
         void on_filter(const k3d::pixel& I, k3d::pixel& O)
