@@ -52,36 +52,28 @@ public:
 		m_size_z(init_owner(*this) + init_name("size_z") + init_label(_("Z Size")) + init_description(_("Size on Z axis")) + init_value(1.0) + init_precision(2) + init_step_increment(0.1) + init_constraint(constraint::minimum(0.1)) + init_units(typeid(k3d::measurement::distance))),
 		m_color(init_owner(*this) + init_name("color") + init_label(_("Color")) + init_description(_("Ellipsoid color")) + init_value(k3d::color(1, 1, 1)))
 	{
-		m_x.changed_signal().connect(sigc::mem_fun(*this, &blobby_ellipsoid_implementation::on_reset_geometry));
-		m_y.changed_signal().connect(sigc::mem_fun(*this, &blobby_ellipsoid_implementation::on_reset_geometry));
-		m_z.changed_signal().connect(sigc::mem_fun(*this, &blobby_ellipsoid_implementation::on_reset_geometry));
-
-		m_size_x.changed_signal().connect(sigc::mem_fun(*this, &blobby_ellipsoid_implementation::on_reset_geometry));
-		m_size_y.changed_signal().connect(sigc::mem_fun(*this, &blobby_ellipsoid_implementation::on_reset_geometry));
-		m_size_z.changed_signal().connect(sigc::mem_fun(*this, &blobby_ellipsoid_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &blobby_ellipsoid_implementation::on_create_geometry));
+		m_x.changed_signal().connect(make_reset_mesh_slot());
+		m_y.changed_signal().connect(make_reset_mesh_slot());
+		m_z.changed_signal().connect(make_reset_mesh_slot());
+		m_size_x.changed_signal().connect(make_reset_mesh_slot());
+		m_size_y.changed_signal().connect(make_reset_mesh_slot());
+		m_size_z.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		std::auto_ptr<k3d::mesh> mesh(new k3d::mesh());
-
 		k3d::point* position = new k3d::point(m_x.value(), m_y.value(), m_z.value());
-		mesh->points.push_back(position);
+		Mesh.points.push_back(position);
 
-		k3d::matrix4 transformation = k3d::scaling3D(k3d::vector3(m_size_x.value(), m_size_y.value(), m_size_z.value()));
+		k3d::matrix4 transformation = k3d::scaling3D(k3d::point3(m_size_x.value(), m_size_y.value(), m_size_z.value()));
 		k3d::blobby::ellipsoid* ellipsoid = new k3d::blobby::ellipsoid(position, transformation);
 		ellipsoid->vertex_data["Cs"] = m_color.value();
 
-		mesh->blobbies.push_back(new k3d::blobby(ellipsoid));
+		Mesh.blobbies.push_back(new k3d::blobby(ellipsoid));
+	}
 
-		return mesh.release();
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -131,39 +123,31 @@ public:
 		m_z2(init_owner(*this) + init_name("z2") + init_label(_("Z2 Position")) + init_description(_("Second segment end Z position")) + init_value(0.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_color(init_owner(*this) + init_name("color") + init_label(_("Color")) + init_description(_("Segment color")) + init_value(k3d::color(1, 1, 1)))
 	{
-		m_x1.changed_signal().connect(sigc::mem_fun(*this, &blobby_segment_implementation::on_reset_geometry));
-		m_y1.changed_signal().connect(sigc::mem_fun(*this, &blobby_segment_implementation::on_reset_geometry));
-		m_z1.changed_signal().connect(sigc::mem_fun(*this, &blobby_segment_implementation::on_reset_geometry));
-		m_x2.changed_signal().connect(sigc::mem_fun(*this, &blobby_segment_implementation::on_reset_geometry));
-		m_y2.changed_signal().connect(sigc::mem_fun(*this, &blobby_segment_implementation::on_reset_geometry));
-		m_z2.changed_signal().connect(sigc::mem_fun(*this, &blobby_segment_implementation::on_reset_geometry));
-
-		m_radius.changed_signal().connect(sigc::mem_fun(*this, &blobby_segment_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &blobby_segment_implementation::on_create_geometry));
+		m_x1.changed_signal().connect(make_reset_mesh_slot());
+		m_y1.changed_signal().connect(make_reset_mesh_slot());
+		m_z1.changed_signal().connect(make_reset_mesh_slot());
+		m_x2.changed_signal().connect(make_reset_mesh_slot());
+		m_y2.changed_signal().connect(make_reset_mesh_slot());
+		m_z2.changed_signal().connect(make_reset_mesh_slot());
+		m_radius.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		std::auto_ptr<k3d::mesh> mesh(new k3d::mesh());
-
 		k3d::point* start = new k3d::point(m_x1.value(), m_y1.value(), m_z1.value());
 		k3d::point* end = new k3d::point(m_x2.value(), m_y2.value(), m_z2.value());
-		mesh->points.push_back(start);
-		mesh->points.push_back(end);
+		Mesh.points.push_back(start);
+		Mesh.points.push_back(end);
 
 		k3d::matrix4 id = k3d::identity3D();
 		k3d::blobby::segment* segment = new k3d::blobby::segment(start, end, m_radius.value(), id);
 		segment->vertex_data["Cs"] = m_color.value();
 
-		mesh->blobbies.push_back(new k3d::blobby(segment));
+		Mesh.blobbies.push_back(new k3d::blobby(segment));
+	}
 
-		return mesh.release();
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -208,39 +192,33 @@ public:
 		m_input_mesh1(init_owner(*this) + init_name("input_mesh1") + init_label(_("Input mesh 1")) + init_description(_("First addition operand (should be a blobby)")) + init_value<k3d::mesh*>(0)),
 		m_input_mesh2(init_owner(*this) + init_name("input_mesh2") + init_label(_("Input mesh 2")) + init_description(_("Second addition operand (should be a blobby)")) + init_value<k3d::mesh*>(0))
 	{
-		m_input_mesh1.changed_signal().connect(sigc::mem_fun(*this, &blobby_add_operator_implementation::on_reset_geometry));
-		m_input_mesh2.changed_signal().connect(sigc::mem_fun(*this, &blobby_add_operator_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &blobby_add_operator_implementation::on_create_geometry));
+		m_input_mesh1.changed_signal().connect(make_reset_mesh_slot());
+		m_input_mesh2.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
-	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
 		// Get the input geometry ...
 		const k3d::mesh* const input_mesh1 = m_input_mesh1.value();
 		const k3d::mesh* const input_mesh2 = m_input_mesh2.value();
 
 		// Create output geometry ...
-		k3d::mesh* const output = new k3d::mesh();
 		if(input_mesh1)
-			k3d::deep_copy(*input_mesh1, *output);
+			k3d::deep_copy(*input_mesh1, Mesh);
 
 		if(input_mesh2)
-			k3d::deep_copy(*input_mesh2, *output);
+			k3d::deep_copy(*input_mesh2, Mesh);
 
 		k3d::blobby::add* addition = new k3d::blobby::add();
-		for(k3d::mesh::blobbies_t::const_iterator blob = output->blobbies.begin(); blob != output->blobbies.end(); blob++)
+		for(k3d::mesh::blobbies_t::const_iterator blob = Mesh.blobbies.begin(); blob != Mesh.blobbies.end(); blob++)
 			addition->add_operand((*blob)->root);
 
-		output->blobbies.clear();
-		output->blobbies.push_back(new k3d::blobby(addition));
+		Mesh.blobbies.clear();
+		Mesh.blobbies.push_back(new k3d::blobby(addition));
+	}
 
-		return output;
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -279,39 +257,33 @@ public:
 		m_input_mesh1(init_owner(*this) + init_name("input_mesh1") + init_label(_("Input mesh 1")) + init_description(_("First multiplication operand (should be a blobby)")) + init_value<k3d::mesh*>(0)),
 		m_input_mesh2(init_owner(*this) + init_name("input_mesh2") + init_label(_("Input mesh 2")) + init_description(_("Second multiplication operand (should be a blobby)")) + init_value<k3d::mesh*>(0))
 	{
-		m_input_mesh1.changed_signal().connect(sigc::mem_fun(*this, &blobby_mult_operator_implementation::on_reset_geometry));
-		m_input_mesh2.changed_signal().connect(sigc::mem_fun(*this, &blobby_mult_operator_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &blobby_mult_operator_implementation::on_create_geometry));
+		m_input_mesh1.changed_signal().connect(make_reset_mesh_slot());
+		m_input_mesh2.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
-	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
 		// Get the input geometry ...
 		const k3d::mesh* const input_mesh1 = m_input_mesh1.value();
 		const k3d::mesh* const input_mesh2 = m_input_mesh2.value();
 
 		// Create output geometry ...
-		k3d::mesh* const output = new k3d::mesh();
 		if(input_mesh1)
-			k3d::deep_copy(*input_mesh1, *output);
+			k3d::deep_copy(*input_mesh1, Mesh);
 
 		if(input_mesh2)
-			k3d::deep_copy(*input_mesh2, *output);
+			k3d::deep_copy(*input_mesh2, Mesh);
 
 		k3d::blobby::multiply* multiplication = new k3d::blobby::multiply();
-		for(k3d::mesh::blobbies_t::const_iterator blob = output->blobbies.begin(); blob != output->blobbies.end(); blob++)
+		for(k3d::mesh::blobbies_t::const_iterator blob = Mesh.blobbies.begin(); blob != Mesh.blobbies.end(); blob++)
 			multiplication->add_operand((*blob)->root);
 
-		output->blobbies.clear();
-		output->blobbies.push_back(new k3d::blobby(multiplication));
+		Mesh.blobbies.clear();
+		Mesh.blobbies.push_back(new k3d::blobby(multiplication));
+	}
 
-		return output;
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -350,39 +322,33 @@ public:
 		m_input_mesh1(init_owner(*this) + init_name("input_mesh1") + init_label(_("Input mesh 1")) + init_description(_("First maximum operand (should be a blobby)")) + init_value<k3d::mesh*>(0)),
 		m_input_mesh2(init_owner(*this) + init_name("input_mesh2") + init_label(_("Input mesh 2")) + init_description(_("Second maximum operand (should be a blobby)")) + init_value<k3d::mesh*>(0))
 	{
-		m_input_mesh1.changed_signal().connect(sigc::mem_fun(*this, &blobby_max_operator_implementation::on_reset_geometry));
-		m_input_mesh2.changed_signal().connect(sigc::mem_fun(*this, &blobby_max_operator_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &blobby_max_operator_implementation::on_create_geometry));
+		m_input_mesh1.changed_signal().connect(make_reset_mesh_slot());
+		m_input_mesh2.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
-	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
 		// Get the input geometry ...
 		const k3d::mesh* const input_mesh1 = m_input_mesh1.value();
 		const k3d::mesh* const input_mesh2 = m_input_mesh2.value();
 
 		// Create output geometry ...
-		k3d::mesh* const output = new k3d::mesh();
 		if(input_mesh1)
-			k3d::deep_copy(*input_mesh1, *output);
+			k3d::deep_copy(*input_mesh1, Mesh);
 
 		if(input_mesh2)
-			k3d::deep_copy(*input_mesh2, *output);
+			k3d::deep_copy(*input_mesh2, Mesh);
 
 		k3d::blobby::max* maximum = new k3d::blobby::max();
-		for(k3d::mesh::blobbies_t::const_iterator blob = output->blobbies.begin(); blob != output->blobbies.end(); blob++)
+		for(k3d::mesh::blobbies_t::const_iterator blob = Mesh.blobbies.begin(); blob != Mesh.blobbies.end(); blob++)
 			maximum->add_operand((*blob)->root);
 
-		output->blobbies.clear();
-		output->blobbies.push_back(new k3d::blobby(maximum));
+		Mesh.blobbies.clear();
+		Mesh.blobbies.push_back(new k3d::blobby(maximum));
+	}
 
-		return output;
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -421,39 +387,33 @@ public:
 		m_input_mesh1(init_owner(*this) + init_name("input_mesh1") + init_label(_("Input mesh 1")) + init_description(_("First minimum operand (should be a blobby)")) + init_value<k3d::mesh*>(0)),
 		m_input_mesh2(init_owner(*this) + init_name("input_mesh2") + init_label(_("Input mesh 2")) + init_description(_("Second minimum operand (should be a blobby)")) + init_value<k3d::mesh*>(0))
 	{
-		m_input_mesh1.changed_signal().connect(sigc::mem_fun(*this, &blobby_min_operator_implementation::on_reset_geometry));
-		m_input_mesh2.changed_signal().connect(sigc::mem_fun(*this, &blobby_min_operator_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &blobby_min_operator_implementation::on_create_geometry));
+		m_input_mesh1.changed_signal().connect(make_reset_mesh_slot());
+		m_input_mesh2.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
-	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
 		// Get the input geometry ...
 		const k3d::mesh* const input_mesh1 = m_input_mesh1.value();
 		const k3d::mesh* const input_mesh2 = m_input_mesh2.value();
 
 		// Create output geometry ...
-		k3d::mesh* const output = new k3d::mesh();
 		if(input_mesh1)
-			k3d::deep_copy(*input_mesh1, *output);
+			k3d::deep_copy(*input_mesh1, Mesh);
 
 		if(input_mesh2)
-			k3d::deep_copy(*input_mesh2, *output);
+			k3d::deep_copy(*input_mesh2, Mesh);
 
 		k3d::blobby::min* minimum = new k3d::blobby::min();
-		for(k3d::mesh::blobbies_t::const_iterator blob = output->blobbies.begin(); blob != output->blobbies.end(); blob++)
+		for(k3d::mesh::blobbies_t::const_iterator blob = Mesh.blobbies.begin(); blob != Mesh.blobbies.end(); blob++)
 			minimum->add_operand((*blob)->root);
 
-		output->blobbies.clear();
-		output->blobbies.push_back(new k3d::blobby(minimum));
+		Mesh.blobbies.clear();
+		Mesh.blobbies.push_back(new k3d::blobby(minimum));
+	}
 
-		return output;
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -493,43 +453,37 @@ public:
 		m_input_mesh2(init_owner(*this) + init_name("input_mesh2") + init_label(_("Input mesh 2")) + init_description(_("Blobby minuend")) + init_value<k3d::mesh*>(0)),
 		m_type(init_owner(*this) + init_name("subtraction_type") + init_label(_("Subtraction type")) + init_description(_("Swap subtrahend and minuend")) + init_enumeration(subtract_values()) + init_value(SUBTRACTION))
 	{
-		m_input_mesh1.changed_signal().connect(sigc::mem_fun(*this, &blobby_sub_operator_implementation::on_reset_geometry));
-		m_input_mesh2.changed_signal().connect(sigc::mem_fun(*this, &blobby_sub_operator_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &blobby_sub_operator_implementation::on_create_geometry));
+		m_input_mesh1.changed_signal().connect(make_reset_mesh_slot());
+		m_input_mesh2.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
-	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
 		// Get the input geometry ...
 		const k3d::mesh* const input_mesh1 = m_input_mesh1.value();
 		const k3d::mesh* const input_mesh2 = m_input_mesh2.value();
 
 		// Create output geometry ...
-		k3d::mesh* const output = new k3d::mesh();
 		if(input_mesh1)
-			k3d::deep_copy(*input_mesh1, *output);
+			k3d::deep_copy(*input_mesh1, Mesh);
 
 		if(input_mesh2)
-			k3d::deep_copy(*input_mesh2, *output);
+			k3d::deep_copy(*input_mesh2, Mesh);
 
-		if(output->blobbies.size() >= 2)
+		if(Mesh.blobbies.size() >= 2)
 			{
-				k3d::mesh::blobbies_t::const_iterator blobby = output->blobbies.begin();
+				k3d::mesh::blobbies_t::const_iterator blobby = Mesh.blobbies.begin();
 				k3d::blobby::opcode* opcode1 = (*blobby++)->root;
 				k3d::blobby::opcode* opcode2 = (*blobby++)->root;
 				k3d::blobby::subtract* subtraction = new k3d::blobby::subtract(opcode1, opcode2);
 
-				output->blobbies.clear();
-				output->blobbies.push_back(new k3d::blobby(subtraction));
+				Mesh.blobbies.clear();
+				Mesh.blobbies.push_back(new k3d::blobby(subtraction));
 			}
+	}
 
-		return output;
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -556,7 +510,7 @@ private:
 		REVERSE_SUBTRACTION
 	} subtract_t;
 
-	friend const k3d::ienumeration_property::enumeration_values_t& subtract_values()
+	static const k3d::ienumeration_property::enumeration_values_t& subtract_values()
 	{
 		static k3d::ienumeration_property::enumeration_values_t values;
 		if(values.empty())
@@ -618,43 +572,37 @@ public:
 		m_input_mesh2(init_owner(*this) + init_name("input_mesh2") + init_label(_("Input mesh 2")) + init_description(_("Blobby divisor")) + init_value<k3d::mesh*>(0)),
 		m_type(init_owner(*this) + init_name("division_type") + init_label(_("Division type")) + init_description(_("Swap dividend and divisor")) + init_enumeration(divide_values()) + init_value(DIVISION))
 	{
-		m_input_mesh1.changed_signal().connect(sigc::mem_fun(*this, &blobby_div_operator_implementation::on_reset_geometry));
-		m_input_mesh2.changed_signal().connect(sigc::mem_fun(*this, &blobby_div_operator_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &blobby_div_operator_implementation::on_create_geometry));
+		m_input_mesh1.changed_signal().connect(make_reset_mesh_slot());
+		m_input_mesh2.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
-	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
 		// Get the input geometry ...
 		const k3d::mesh* const input_mesh1 = m_input_mesh1.value();
 		const k3d::mesh* const input_mesh2 = m_input_mesh2.value();
 
 		// Create output geometry ...
-		k3d::mesh* const output = new k3d::mesh();
 		if(input_mesh1)
-			k3d::deep_copy(*input_mesh1, *output);
+			k3d::deep_copy(*input_mesh1, Mesh);
 
 		if(input_mesh2)
-			k3d::deep_copy(*input_mesh2, *output);
+			k3d::deep_copy(*input_mesh2, Mesh);
 
-		if(output->blobbies.size() >= 2)
+		if(Mesh.blobbies.size() >= 2)
 			{
-				k3d::mesh::blobbies_t::const_iterator blobby = output->blobbies.begin();
+				k3d::mesh::blobbies_t::const_iterator blobby = Mesh.blobbies.begin();
 				k3d::blobby::opcode* opcode1 = (*blobby++)->root;
 				k3d::blobby::opcode* opcode2 = (*blobby++)->root;
 				k3d::blobby::divide* division = new k3d::blobby::divide(opcode1, opcode2);
 
-				output->blobbies.clear();
-				output->blobbies.push_back(new k3d::blobby(division));
+				Mesh.blobbies.clear();
+				Mesh.blobbies.push_back(new k3d::blobby(division));
 			}
+	}
 
-		return output;
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -681,7 +629,7 @@ private:
 		REVERSE_DIVISION
 	} divide_t;
 
-	friend const k3d::ienumeration_property::enumeration_values_t& divide_values()
+	static const k3d::ienumeration_property::enumeration_values_t& divide_values()
 	{
 		static k3d::ienumeration_property::enumeration_values_t values;
 		if(values.empty())
