@@ -50,79 +50,69 @@ public:
 		base(Document),
 		m_voxels(init_owner(*this) + init_name("voxels") + init_label(_("Voxels")) + init_description(_("Voxel subdivisions along blobby's bounding-box width, height and depth ")) + init_value(30) + init_precision(0) + init_step_increment(1) + init_constraint(constraint::minimum(1L)) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &polygonize_blobbies_implementation::on_reset_geometry));
-		m_material.changed_signal().connect(sigc::mem_fun(*this, &polygonize_blobbies_implementation::on_reset_geometry));
-		m_voxels.changed_signal().connect(sigc::mem_fun(*this, &polygonize_blobbies_implementation::on_reset_geometry));
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &polygonize_blobbies_implementation::on_create_geometry));
+		m_material.changed_signal().connect(make_reset_mesh_slot());
+		m_voxels.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	/** \todo Improve the implementation so we don't have to do this */
+	k3d::iunknown* on_rewrite_hint(iunknown* const Hint)
 	{
-		m_output_mesh.reset();
+		// Force updates to re-allocate our mesh, for simplicity
+		return 0;
 	}
 
-	k3d::mesh* on_create_geometry()
-	{
-		// If we don't have any input mesh, we're done ...
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
-
-		// Otherwise, we convert input mesh polyhedra to blobbies ...
-		k3d::mesh* const output = new k3d::mesh();
-		update_geometry(*input, *output);
-
-		return output;
-	}
-
-	void update_geometry(const k3d::mesh& Input, k3d::mesh& Output)
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
 		const unsigned long voxels = m_voxels.value();
 		k3d::imaterial* const material = m_material.value();
 
 		// Polygonize each blobby ...
-		for(k3d::mesh::blobbies_t::const_iterator blobby = Input.blobbies.begin(); blobby != Input.blobbies.end(); blobby++)
+		for(k3d::mesh::blobbies_t::const_iterator blobby = InputMesh.blobbies.begin(); blobby != InputMesh.blobbies.end(); blobby++)
+		{
+			Mesh.polyhedra.push_back(new k3d::polyhedron());
+			k3d::polyhedron& polyhedron = *Mesh.polyhedra.back();
+
+			// Polygonize blobby
+			typedef std::vector<k3d::point3> vertices_t;
+			vertices_t blobby_vertices;
+			vertices_t blobby_normals;
+			typedef std::vector< std::vector<unsigned long> > polygons_t;
+			polygons_t blobby_polygons;
+			polygonize_blobby(*blobby, voxels, blobby_vertices, blobby_normals, blobby_polygons);
+
+			// Add geometry to the mesh ...
+			vertices_t::const_iterator vertex = blobby_vertices.begin();
+			vertices_t::const_iterator normal = blobby_normals.begin();
+			vertices_t::const_iterator vertices_end = blobby_vertices.end();
+			for(; vertex != vertices_end; vertex++, normal++)
 			{
-				Output.polyhedra.push_back(new k3d::polyhedron());
-				k3d::polyhedron& polyhedron = *Output.polyhedra.back();
-
-				// Polygonize blobby
-				typedef std::vector<k3d::vector3> vertices_t;
-				vertices_t blobby_vertices;
-				vertices_t blobby_normals;
-				typedef std::vector< std::vector<unsigned long> > polygons_t;
-				polygons_t blobby_polygons;
-				polygonize_blobby(*blobby, voxels, blobby_vertices, blobby_normals, blobby_polygons);
-
-				// Add geometry to the mesh ...
-				vertices_t::const_iterator vertex = blobby_vertices.begin();
-				vertices_t::const_iterator normal = blobby_normals.begin();
-				vertices_t::const_iterator vertices_end = blobby_vertices.end();
-				for(; vertex != vertices_end; vertex++, normal++)
-					{
-						k3d::point* point = new k3d::point(*vertex);
-						point->vertex_data["N"] = k3d::ri::normal(-(*normal));
-						Output.points.push_back(point);
-					}
-
-				for(unsigned long i = 0; i != blobby_polygons.size(); ++i)
-					{
-						if(!blobby_polygons[i].size())
-							continue;
-
-						std::vector<k3d::split_edge*> edges;
-						for(unsigned long j = 0; j != blobby_polygons[i].size(); ++j)
-							edges.push_back(new k3d::split_edge(Output.points[blobby_polygons[i][j]]));
-						for(unsigned long j = 0; j != blobby_polygons[i].size(); ++j)
-							edges[j]->face_clockwise = edges[(j+1) % blobby_polygons[i].size()];
-
-						polyhedron.faces.push_back(new k3d::face(edges.front(), material));
-					}
-
-				// Set companions and check solidness
-				k3d::set_companions(polyhedron);
-				assert_warning(is_solid(polyhedron));
+				k3d::point* point = new k3d::point(*vertex);
+				point->vertex_data["N"] = k3d::ri::normal(k3d::to_normal(-(*normal)));
+				Mesh.points.push_back(point);
 			}
+
+			for(unsigned long i = 0; i != blobby_polygons.size(); ++i)
+			{
+				if(!blobby_polygons[i].size())
+					continue;
+
+				std::vector<k3d::split_edge*> edges;
+				for(unsigned long j = 0; j != blobby_polygons[i].size(); ++j)
+					edges.push_back(new k3d::split_edge(Mesh.points[blobby_polygons[i][j]]));
+				for(unsigned long j = 0; j != blobby_polygons[i].size(); ++j)
+					edges[j]->face_clockwise = edges[(j+1) % blobby_polygons[i].size()];
+
+				polyhedron.faces.push_back(new k3d::face(edges.front(), material));
+			}
+
+			// Set companions and check solidness
+			k3d::set_companions(polyhedron);
+			assert_warning(is_solid(polyhedron));
+		}
+	}
+
+	void on_update_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -159,4 +149,3 @@ k3d::iplugin_factory& polygonize_blobbies_factory()
 
 } // namespace libk3dblobbies
 
-
