@@ -27,11 +27,12 @@
 #include <k3dsdk/classes.h>
 #include <k3dsdk/gl.h>
 #include <k3dsdk/i18n.h>
-#include <k3dsdk/iapplication.h>
 #include <k3dsdk/measurement.h>
 #include <k3dsdk/module.h>
 #include <k3dsdk/node.h>
 #include <k3dsdk/persistent.h>
+#include <k3dsdk/snappable.h>
+#include <k3dsdk/snap_target.h>
 #include <k3dsdk/transform.h>
 #include <k3dsdk/transformable.h>
 #include <k3dsdk/vectors.h>
@@ -43,9 +44,9 @@ namespace libk3dcore
 // axes
 
 class axes :
-	public k3d::bounded<k3d::gl::drawable<k3d::transformable<k3d::persistent<k3d::node> > > >
+	public k3d::snappable<k3d::bounded<k3d::gl::drawable<k3d::transformable<k3d::persistent<k3d::node> > > > >
 {
-	typedef k3d::bounded<k3d::gl::drawable<k3d::transformable<k3d::persistent<k3d::node> > > > base;
+	typedef k3d::snappable<k3d::bounded<k3d::gl::drawable<k3d::transformable<k3d::persistent<k3d::node> > > > > base;
 
 public:
 	axes(k3d::idocument& Document) :
@@ -57,14 +58,32 @@ public:
 		m_GridSize(init_owner(*this) + init_name("gridsize") + init_label(_("Grid Size")) + init_description(_("The size of each grid square")) + init_value(2.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_GridCount(init_owner(*this) + init_name("gridcount") + init_label(_("Grid Count")) + init_description(_("Number of squares along each grid")) + init_value(5) + init_precision(0) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar)) + init_constraint(constraint::minimum(1L)))
 	{
-		m_Axes.changed_signal().connect(sigc::mem_fun(*this, &axes::async_redraw_all));
-		m_XYPlane.changed_signal().connect(sigc::mem_fun(*this, &axes::async_redraw_all));
-		m_YZPlane.changed_signal().connect(sigc::mem_fun(*this, &axes::async_redraw_all));
-		m_XZPlane.changed_signal().connect(sigc::mem_fun(*this, &axes::async_redraw_all));
-		m_GridSize.changed_signal().connect(sigc::mem_fun(*this, &axes::async_redraw_all));
-		m_GridCount.changed_signal().connect(sigc::mem_fun(*this, &axes::async_redraw_all));
+		m_Axes.changed_signal().connect(make_async_redraw_slot());
+		m_XYPlane.changed_signal().connect(make_async_redraw_slot());
+		m_YZPlane.changed_signal().connect(make_async_redraw_slot());
+		m_XZPlane.changed_signal().connect(make_async_redraw_slot());
+		m_GridSize.changed_signal().connect(make_async_redraw_slot());
+		m_GridCount.changed_signal().connect(make_async_redraw_slot());
+		m_input_matrix.changed_signal().connect(make_async_redraw_slot());
 
-		m_input_matrix.changed_signal().connect(sigc::mem_fun(*this, &axes::async_redraw_all));
+		add_snap_target(new k3d::snap_target(_("Grid"), sigc::mem_fun(*this, &axes::grid_target_position), sigc::mem_fun(*this, &axes::grid_target_orientation)));
+	}
+
+	bool grid_target_position(const k3d::point3& Position, k3d::point3& TargetPosition)
+	{
+		const double grid_size = m_GridSize.value();
+
+		TargetPosition = k3d::point3(
+			k3d::round(Position[0] / grid_size) * grid_size,
+			k3d::round(Position[1] / grid_size) * grid_size,
+			k3d::round(Position[2] / grid_size) * grid_size);
+
+		return true;
+	}
+
+	bool grid_target_orientation(const k3d::point3& Position, k3d::vector3& Look, k3d::vector3& Up)
+	{
+		return false;
 	}
 
 	const k3d::bounding_box3 extents()
@@ -99,104 +118,104 @@ public:
 
 		// Draw axes and labels
 		if(m_Axes.value())
-			{
-				// Draw X axis
-				k3d::gl::color3d(x_color);
-				glBegin(GL_LINE_LOOP);
-				glVertex3d(-size, 0.0, 0.0);
-				glVertex3d(size, 0.0, 0.0);
-				glEnd();
+		{
+			// Draw X axis
+			k3d::gl::color3d(x_color);
+			glBegin(GL_LINE_LOOP);
+			glVertex3d(-size, 0.0, 0.0);
+			glVertex3d(size, 0.0, 0.0);
+			glEnd();
 
-				// Draw Y axis
-				k3d::gl::color3d(y_color);
-				glBegin(GL_LINE_LOOP);
-				glVertex3d(0.0, -size, 0.0);
-				glVertex3d(0.0, size, 0.0);
-				glEnd();
+			// Draw Y axis
+			k3d::gl::color3d(y_color);
+			glBegin(GL_LINE_LOOP);
+			glVertex3d(0.0, -size, 0.0);
+			glVertex3d(0.0, size, 0.0);
+			glEnd();
 
-				// Draw Z axis
-				k3d::gl::color3d(z_color);
-				glBegin(GL_LINE_LOOP);
-				glVertex3d(0.0, 0.0, -size);
-				glVertex3d(0.0, 0.0, size);
-				glEnd();
+			// Draw Z axis
+			k3d::gl::color3d(z_color);
+			glBegin(GL_LINE_LOOP);
+			glVertex3d(0.0, 0.0, -size);
+			glVertex3d(0.0, 0.0, size);
+			glEnd();
 
-				// Draw axis labels ...
-				k3d::gl::color3d(grid_color);
+			// Draw axis labels ...
+			k3d::gl::color3d(grid_color);
 
-				double labelposition = size * 1.1;
+			double labelposition = size * 1.1;
 
-				glListBase(State.gl_ascii_font_list_base);
+			glListBase(State.gl_ascii_font_list_base);
 
-				glRasterPos3d(labelposition, 0, 0);
-				glCallLists(2, GL_UNSIGNED_BYTE, "+X");
-				glRasterPos3d(0, labelposition, 0);
-				glCallLists(2, GL_UNSIGNED_BYTE, "+Y");
-				glRasterPos3d(0, 0, labelposition);
-				glCallLists(2, GL_UNSIGNED_BYTE, "+Z");
+			glRasterPos3d(labelposition, 0, 0);
+			glCallLists(2, GL_UNSIGNED_BYTE, "+X");
+			glRasterPos3d(0, labelposition, 0);
+			glCallLists(2, GL_UNSIGNED_BYTE, "+Y");
+			glRasterPos3d(0, 0, labelposition);
+			glCallLists(2, GL_UNSIGNED_BYTE, "+Z");
 
-				glRasterPos3d(-labelposition, 0, 0);
-				glCallLists(2, GL_UNSIGNED_BYTE, "-X");
-				glRasterPos3d(0, -labelposition, 0);
-				glCallLists(2, GL_UNSIGNED_BYTE, "-Y");
-				glRasterPos3d(0, 0, -labelposition);
-				glCallLists(2, GL_UNSIGNED_BYTE, "-Z");
-			}
+			glRasterPos3d(-labelposition, 0, 0);
+			glCallLists(2, GL_UNSIGNED_BYTE, "-X");
+			glRasterPos3d(0, -labelposition, 0);
+			glCallLists(2, GL_UNSIGNED_BYTE, "-Y");
+			glRasterPos3d(0, 0, -labelposition);
+			glCallLists(2, GL_UNSIGNED_BYTE, "-Z");
+		}
 
 		// Setup grid color
 		k3d::gl::color3d(grid_color);
 
 		// Draw XY plane
 		if(m_XYPlane.value())
+		{
+			glBegin(GL_LINES);
+			for(long i = -grid_count; i <= grid_count; ++i)
 			{
-				glBegin(GL_LINES);
-				for(long i = -grid_count; i <= grid_count; ++i)
-					{
-						glVertex3d(i * grid_size, -size, 0.0);
-						glVertex3d(i * grid_size, size, 0.0);
-						glVertex3d(-size, i * grid_size, 0.0);
-						glVertex3d(size, i * grid_size, 0.0);
-					}
-				glEnd();
+				glVertex3d(i * grid_size, -size, 0.0);
+				glVertex3d(i * grid_size, size, 0.0);
+				glVertex3d(-size, i * grid_size, 0.0);
+				glVertex3d(size, i * grid_size, 0.0);
 			}
+			glEnd();
+		}
 
 		// Draw YZ plane
 		if(m_YZPlane.value())
+		{
+			glBegin(GL_LINES);
+			for(long i = -grid_count; i <= grid_count; ++i)
 			{
-				glBegin(GL_LINES);
-				for(long i = -grid_count; i <= grid_count; ++i)
-					{
-						glVertex3d(0.0, i * grid_size, -size);
-						glVertex3d(0.0, i * grid_size, size);
-						glVertex3d(0.0, -size, i * grid_size);
-						glVertex3d(0.0, size, i * grid_size);
-					}
-				glEnd();
+				glVertex3d(0.0, i * grid_size, -size);
+				glVertex3d(0.0, i * grid_size, size);
+				glVertex3d(0.0, -size, i * grid_size);
+				glVertex3d(0.0, size, i * grid_size);
 			}
+			glEnd();
+		}
 
 		// Draw XZ plane
 		if(m_XZPlane.value())
+		{
+			glBegin(GL_LINES);
+			for(long i = -grid_count; i <= grid_count; ++i)
 			{
-				glBegin(GL_LINES);
-				for(long i = -grid_count; i <= grid_count; ++i)
-					{
-						glVertex3d(i * grid_size, 0.0, -size);
-						glVertex3d(i * grid_size, 0.0, size);
-						glVertex3d(-size, 0.0, i * grid_size);
-						glVertex3d(size, 0.0, i * grid_size);
-					}
-				glEnd();
+				glVertex3d(i * grid_size, 0.0, -size);
+				glVertex3d(i * grid_size, 0.0, size);
+				glVertex3d(-size, 0.0, i * grid_size);
+				glVertex3d(size, 0.0, i * grid_size);
 			}
+			glEnd();
+		}
 	}
 
 	void on_gl_select(const k3d::gl::render_state& State, const k3d::gl::select_state& SelectState)
 	{
 	}
 
-	void snap(const k3d::vector3& InputCoordinates, k3d::vector3& SnapCoordinates, std::string& SnapDescription)
+	void snap(const k3d::point3& InputCoordinates, k3d::point3& SnapCoordinates, std::string& SnapDescription)
 	{
 		// Convert coordinates to our frame ...
-		k3d::vector3 input_coordinates = k3d::inverse(k3d::node_to_world_matrix(*this)) * InputCoordinates;
+		k3d::point3 input_coordinates = k3d::inverse(k3d::node_to_world_matrix(*this)) * InputCoordinates;
 
 		// Snap coordinates ...
 		input_coordinates[0] = k3d::round(input_coordinates[0] / m_GridSize.value()) * m_GridSize.value();
