@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -35,6 +35,7 @@
 #include <k3dsdk/persistent.h>
 #include <k3dsdk/property_group_collection.h>
 #include <k3dsdk/selection.h>
+#include <k3dsdk/transform.h>
 #include <k3dsdk/transformable.h>
 
 #ifdef	WIN32
@@ -64,6 +65,8 @@ public:
 	camera(k3d::idocument& Document) :
 		base(Document),
 		m_navigation_target(init_owner(*this) + init_name("navigation_target") + init_label(_("Navigation Target")) + init_description(_("Navigation Target")) + init_value<k3d::itransform_source*>(this)),
+		m_world_target(init_owner(*this) + init_name("world_target") + init_label(_("World target")) + init_description(_("World target")) + init_value(k3d::point3(0, -5, 0))),
+		m_target_distance(init_owner(*this) + init_name("target_distance") + init_label(_("Target distance")) + init_description(_("Target distance from camera position")) + init_value(0) + init_precision(2) + init_units(typeid(k3d::measurement::distance)) + init_slot(sigc::mem_fun(*this, &camera::get_target_distance))),
 		m_aspect_ratio(init_owner(*this) + init_name("aspect_ratio") + init_label(_("Aspect Ratio")) + init_description(_("Choose a predefined aspect ratio")) + init_enumeration(k3d::aspect_ratio_values()) + init_value(std::string(""))),
 		m_show_projection(init_owner(*this) + init_name("show_projection") + init_label(_("Show Projection")) + init_description(_("Show viewing projection")) + init_value(false)),
 		m_orthographic(init_owner(*this) + init_name("orthographic") + init_label(_("Orthographic")) + init_description(_("Orthographic projection")) + init_value(false)),
@@ -74,9 +77,12 @@ public:
 		m_near(init_owner(*this) + init_name("near") + init_label(_("Near")) + init_description(_("Near Plane Distance")) + init_value(1.0) + init_constraint(constraint::minimum(0.0)) + init_precision(1) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_far(init_owner(*this) + init_name("far") + init_label(_("Far")) + init_description(_("Far Plane Distance")) + init_value(1000.0) + init_constraint(constraint::minimum(0.0)) + init_precision(1) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_crop_window_left(init_owner(*this) + init_name("crop_window_left") + init_label(_("Crop Window Left")) + init_description(_("Crop window left")) + init_value(0.0) + init_constraint(constraint::minimum(0.0, constraint::maximum(1.0))) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::scalar))),
-		m_crop_window_right(init_owner(*this) + init_name("crop_window_right") + init_label(_("CropWwindow Right")) + init_description(_("Crop Window Right")) + init_value(1.0) + init_constraint(constraint::minimum(0.0, constraint::maximum(1.0))) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::scalar))),
+		m_crop_window_right(init_owner(*this) + init_name("crop_window_right") + init_label(_("Crop Window Right")) + init_description(_("Crop Window Right")) + init_value(1.0) + init_constraint(constraint::minimum(0.0, constraint::maximum(1.0))) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::scalar))),
 		m_crop_window_top(init_owner(*this) + init_name("crop_window_top") + init_label(_("Crop Window Top")) + init_description(_("Crop Window Top")) + init_value(0.0) + init_constraint(constraint::minimum(0.0, constraint::maximum(1.0))) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::scalar))),
 		m_crop_window_bottom(init_owner(*this) + init_name("crop_window_bottom") + init_label(_("Crop Window Bottom")) + init_description(_("Crop window bottom")) + init_value(1.0) + init_constraint(constraint::minimum(0.0, constraint::maximum(1.0))) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::scalar))),
+		m_show_reference_plane(init_owner(*this) + init_name("show_reference_plane") + init_label(_("Show Reference Plane")) + init_description(_("Show Reference Plane")) + init_value(false)),
+		m_reference_plane(init_owner(*this) + init_name("reference_plane") + init_label(_("Reference Plane")) + init_description(_("Reference Plane Distance")) + init_value(0.0) + init_constraint(constraint::minimum(0.0)) + init_precision(1) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
+		m_reference_plane_color(init_owner(*this) + init_name("reference_plane_color") + init_label(_("Reference Plane Color")) + init_description(_("Reference Plane Color")) + init_value(k3d::color(0, 0, 0.7))),
 		m_perspective_projection(m_left, m_right, m_top, m_bottom, m_near, m_far),
 		m_orthographic_projection(m_left, m_right, m_top, m_bottom, m_near, m_far)
 	{
@@ -90,6 +96,9 @@ public:
 		projection_group.properties.push_back(&static_cast<k3d::iproperty&>(m_bottom));
 		projection_group.properties.push_back(&static_cast<k3d::iproperty&>(m_near));
 		projection_group.properties.push_back(&static_cast<k3d::iproperty&>(m_far));
+		projection_group.properties.push_back(&static_cast<k3d::iproperty&>(m_show_reference_plane));
+		projection_group.properties.push_back(&static_cast<k3d::iproperty&>(m_reference_plane));
+		projection_group.properties.push_back(&static_cast<k3d::iproperty&>(m_reference_plane_color));
 
 		k3d::iproperty_group_collection::group output_group("Output");
 		output_group.properties.push_back(&static_cast<k3d::iproperty&>(m_crop_window_left));
@@ -102,29 +111,33 @@ public:
 
 		m_aspect_ratio.changed_signal().connect(sigc::mem_fun(*this, &camera::on_aspect_ratio_changed));
 
-		m_show_projection.changed_signal().connect(sigc::mem_fun(*this, &camera::async_redraw_all));
-		m_orthographic.changed_signal().connect(sigc::mem_fun(*this, &camera::async_redraw_all));
-		m_left.changed_signal().connect(sigc::mem_fun(*this, &camera::async_redraw_all));
-		m_right.changed_signal().connect(sigc::mem_fun(*this, &camera::async_redraw_all));
-		m_top.changed_signal().connect(sigc::mem_fun(*this, &camera::async_redraw_all));
-		m_bottom.changed_signal().connect(sigc::mem_fun(*this, &camera::async_redraw_all));
-		m_near.changed_signal().connect(sigc::mem_fun(*this, &camera::async_redraw_all));
-		m_far.changed_signal().connect(sigc::mem_fun(*this, &camera::async_redraw_all));
+		m_show_projection.changed_signal().connect(make_async_redraw_slot());
+		m_orthographic.changed_signal().connect(make_async_redraw_slot());
+		m_left.changed_signal().connect(make_async_redraw_slot());
+		m_right.changed_signal().connect(make_async_redraw_slot());
+		m_top.changed_signal().connect(make_async_redraw_slot());
+		m_bottom.changed_signal().connect(make_async_redraw_slot());
+		m_near.changed_signal().connect(make_async_redraw_slot());
+		m_far.changed_signal().connect(make_async_redraw_slot());
+		m_show_reference_plane.changed_signal().connect(make_async_redraw_slot());
+		m_reference_plane.changed_signal().connect(make_async_redraw_slot());
+		m_reference_plane_color.changed_signal().connect(make_async_redraw_slot());
+		m_crop_window_left.changed_signal().connect(make_async_redraw_slot());
+		m_crop_window_right.changed_signal().connect(make_async_redraw_slot());
+		m_crop_window_top.changed_signal().connect(make_async_redraw_slot());
+		m_crop_window_bottom.changed_signal().connect(make_async_redraw_slot());
+		m_input_matrix.changed_signal().connect(make_async_redraw_slot());
 
-		m_crop_window_left.changed_signal().connect(sigc::mem_fun(*this, &camera::async_redraw_all));
-		m_crop_window_right.changed_signal().connect(sigc::mem_fun(*this, &camera::async_redraw_all));
-		m_crop_window_top.changed_signal().connect(sigc::mem_fun(*this, &camera::async_redraw_all));
-		m_crop_window_bottom.changed_signal().connect(sigc::mem_fun(*this, &camera::async_redraw_all));
-
-		m_input_matrix.changed_signal().connect(sigc::mem_fun(*this, &camera::async_redraw_all));
+		m_input_matrix.changed_signal().connect(sigc::mem_fun(*this, &camera::on_position_changed));
+		m_world_target.changed_signal().connect(sigc::mem_fun(*this, &camera::on_position_changed));
 	}
 
-	void set_visible_selection(const bool Visible)
+	void set_selection_weight(const double Weight)
 	{
-		if(Visible != get_visible_selection())
+		if(Weight != get_selection_weight())
 			k3d::gl::redraw_all(document(), k3d::gl::irender_engine::ASYNCHRONOUS);
 
-		base::set_visible_selection(Visible);
+		base::set_selection_weight(Weight);
 	}
 
 	k3d::itransform_source& transformation()
@@ -156,6 +169,11 @@ public:
 		return *target;
 	}
 
+	k3d::iproperty& world_target()
+	{
+		return m_world_target;
+	}
+
 	k3d::iproperty& crop_left()
 	{
 		return m_crop_window_left;
@@ -176,43 +194,55 @@ public:
 		return m_crop_window_bottom;
 	}
 
-	void on_aspect_ratio_changed()
+	void on_aspect_ratio_changed(k3d::iunknown*)
 	{
 		const std::string new_ratio = m_aspect_ratio.value();
 
 		const k3d::aspect_ratios_t& ratios = k3d::aspect_ratios();
 		for(k3d::aspect_ratios_t::const_iterator ratio = ratios.begin(); ratio != ratios.end(); ++ratio)
-			{
-				if(ratio->name != new_ratio)
-					continue;
+		{
+			if(ratio->name != new_ratio)
+				continue;
 
-				m_left.set_value(-0.5 * ratio->value);
-				m_right.set_value(0.5 * ratio->value);
-				m_top.set_value(0.5);
-				m_bottom.set_value(-0.5);
+			m_left.set_value(-0.5 * ratio->value);
+			m_right.set_value(0.5 * ratio->value);
+			m_top.set_value(0.5);
+			m_bottom.set_value(-0.5);
 
-				return;
-			}
+			return;
+		}
 
 		assert_not_reached();
 	}
 
+	void on_position_changed(k3d::iunknown* const Hint)
+	{
+		m_target_distance.reset(Hint);
+	}
+
+	double get_target_distance()
+	{
+		const k3d::point3 position = k3d::node_to_world_matrix(navigation_target()) * k3d::point3(0, 0, 0);
+		return k3d::distance(position, m_world_target.value());
+	}
+
 	void on_gl_draw(const k3d::gl::render_state& State)
 	{
-		const bool show_projection = m_show_projection.value();
-
 		glDisable(GL_LIGHTING);
 		glDisable(GL_TEXTURE_1D);
 		glDisable(GL_TEXTURE_2D);
 
-		k3d::gl::color3d(get_visible_selection() ? k3d::color(1, 1, 1) : k3d::color(0, 0, 0));
+		k3d::gl::color3d(get_selection_weight() ? k3d::color(1, 1, 1) : k3d::color(0, 0, 0));
 		glLineWidth(1.0f);
 		glDisable(GL_LINE_STIPPLE);
 
 		draw();
 
-		if(show_projection)
+		if(m_show_projection.value())
 			draw_projection();
+
+		if(m_show_reference_plane.value())
+			draw_reference_plane();
 	}
 
 	void on_gl_select(const k3d::gl::render_state& State, const k3d::gl::select_state& SelectState)
@@ -232,73 +262,143 @@ public:
 		const double near = m_near.value();
 		const double far = m_far.value();
 
+		const double reference_plane = m_reference_plane.value();
+		const k3d::color reference_plane_color = m_reference_plane_color.value();
+
 		if(orthographic)
-			{
-				glBegin(GL_LINE_LOOP);
-					glVertex3d(left, top, near);
-					glVertex3d(right, top, near);
-					glVertex3d(right, bottom, near);
-					glVertex3d(left, bottom, near);
-				glEnd();
+		{
+			// Draw the near plane ...
+			glBegin(GL_LINE_LOOP);
+				glVertex3d(left, top, near);
+				glVertex3d(right, top, near);
+				glVertex3d(right, bottom, near);
+				glVertex3d(left, bottom, near);
+			glEnd();
 
-				glBegin(GL_LINE_LOOP);
-					glVertex3d(left, top, far);
-					glVertex3d(right, top, far);
-					glVertex3d(right, bottom, far);
-					glVertex3d(left, bottom, far);
-				glEnd();
+			// Draw the reference plane ...
+			glBegin(GL_LINE_LOOP);
+				glVertex3d(left, top, reference_plane);
+				glVertex3d(right, top, reference_plane);
+				glVertex3d(right, bottom, reference_plane);
+				glVertex3d(left, bottom, reference_plane);
+			glEnd();
 
-				glBegin(GL_LINES);
-					glVertex3d(left, top, near);
-					glVertex3d(left, top, far);
+			// Draw the far plane ...
+			glBegin(GL_LINE_LOOP);
+				glVertex3d(left, top, far);
+				glVertex3d(right, top, far);
+				glVertex3d(right, bottom, far);
+				glVertex3d(left, bottom, far);
+			glEnd();
 
-					glVertex3d(right, top, near);
-					glVertex3d(right, top, far);
+			// Draw the frustum ...
+			glBegin(GL_LINES);
+				glVertex3d(left, top, near);
+				glVertex3d(left, top, far);
 
-					glVertex3d(right, bottom, near);
-					glVertex3d(right, bottom, far);
+				glVertex3d(right, top, near);
+				glVertex3d(right, top, far);
 
-					glVertex3d(left, bottom, near);
-					glVertex3d(left, bottom, far);
-				glEnd();
-			}
+				glVertex3d(right, bottom, near);
+				glVertex3d(right, bottom, far);
+
+				glVertex3d(left, bottom, near);
+				glVertex3d(left, bottom, far);
+			glEnd();
+		}
 		else
-			{
-				const double p = far / near;
+		{
+			const double p = far / near;
+			const double left2 = left * p;
+			const double right2 = right * p;
+			const double top2 = top * p;
+			const double bottom2 = bottom * p;
 
-				const double left2 = left * p;
-				const double right2 = right * p;
-				const double top2 = top * p;
-				const double bottom2 = bottom * p;
+			const double q = reference_plane / near;
+			const double left3 = left * q;
+			const double right3 = right * q;
+			const double top3 = top * q;
+			const double bottom3 = bottom * q;
 
-				glBegin(GL_LINE_LOOP);
-					glVertex3d(left, top, near);
-					glVertex3d(right, top, near);
-					glVertex3d(right, bottom, near);
-					glVertex3d(left, bottom, near);
-				glEnd();
+			// Draw the near plane ...
+			glBegin(GL_LINE_LOOP);
+				glVertex3d(left, top, near);
+				glVertex3d(right, top, near);
+				glVertex3d(right, bottom, near);
+				glVertex3d(left, bottom, near);
+			glEnd();
 
-				glBegin(GL_LINE_LOOP);
-					glVertex3d(left2, top2, far);
-					glVertex3d(right2, top2, far);
-					glVertex3d(right2, bottom2, far);
-					glVertex3d(left2, bottom2, far);
-				glEnd();
+			// Draw the reference plane ...
+			glBegin(GL_LINE_LOOP);
+				glVertex3d(left3, top3, reference_plane);
+				glVertex3d(right3, top3, reference_plane);
+				glVertex3d(right3, bottom3, reference_plane);
+				glVertex3d(left3, bottom3, reference_plane);
+			glEnd();
 
-				glBegin(GL_LINES);
-					glVertex3d(left, top, near);
-					glVertex3d(left2, top2, far);
+			// Draw the far plane ...
+			glBegin(GL_LINE_LOOP);
+				glVertex3d(left2, top2, far);
+				glVertex3d(right2, top2, far);
+				glVertex3d(right2, bottom2, far);
+				glVertex3d(left2, bottom2, far);
+			glEnd();
 
-					glVertex3d(right, top, near);
-					glVertex3d(right2, top2, far);
+			// Draw the frustum ...
+			glBegin(GL_LINES);
+				glVertex3d(left, top, near);
+				glVertex3d(left2, top2, far);
 
-					glVertex3d(right, bottom, near);
-					glVertex3d(right2, bottom2, far);
+				glVertex3d(right, top, near);
+				glVertex3d(right2, top2, far);
 
-					glVertex3d(left, bottom, near);
-					glVertex3d(left2, bottom2, far);
-				glEnd();
-			}
+				glVertex3d(right, bottom, near);
+				glVertex3d(right2, bottom2, far);
+
+				glVertex3d(left, bottom, near);
+				glVertex3d(left2, bottom2, far);
+			glEnd();
+		}
+	}
+
+	void draw_reference_plane()
+	{
+		const bool orthographic = m_orthographic.value();
+		const double left = m_left.value();
+		const double right = m_right.value();
+		const double top = m_top.value();
+		const double bottom = m_bottom.value();
+		const double near = m_near.value();
+
+		const double reference_plane = m_reference_plane.value();
+		const k3d::color reference_plane_color = m_reference_plane_color.value();
+
+		if(orthographic)
+		{
+			k3d::gl::color3d(reference_plane_color);
+			glBegin(GL_LINE_LOOP);
+				glVertex3d(left, top, reference_plane);
+				glVertex3d(right, top, reference_plane);
+				glVertex3d(right, bottom, reference_plane);
+				glVertex3d(left, bottom, reference_plane);
+			glEnd();
+		}
+		else
+		{
+			const double q = reference_plane / near;
+			const double left3 = left * q;
+			const double right3 = right * q;
+			const double top3 = top * q;
+			const double bottom3 = bottom * q;
+
+			k3d::gl::color3d(reference_plane_color);
+			glBegin(GL_LINE_LOOP);
+				glVertex3d(left3, top3, reference_plane);
+				glVertex3d(right3, top3, reference_plane);
+				glVertex3d(right3, bottom3, reference_plane);
+				glVertex3d(left3, bottom3, reference_plane);
+			glEnd();
+		}
 	}
 
 	void draw()
@@ -314,6 +414,8 @@ public:
 		const double filmradius = 0.2;
 		const double filmwidth = 0.125 * 0.5;
 
+		glTranslated(0, 0, -0.25);
+
 		// Draw the camera body ...
 		glBegin(GL_LINE_LOOP);
 		glVertex3d(-bodywidth, bodyheight, bodylength);
@@ -536,6 +638,8 @@ private:
 	};
 
 	k3d_data(k3d::itransform_source*, immutable_name, change_signal, with_undo, node_storage, no_constraint, node_property, node_serialization) m_navigation_target;
+	k3d_data(k3d::point3, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_world_target;
+	k3d_data(double, immutable_name, change_signal, no_undo, computed_storage, no_constraint, read_only_property, no_serialization) m_target_distance;
 	k3d_data(std::string, immutable_name, change_signal, with_undo, local_storage, no_constraint, enumeration_property, with_serialization) m_aspect_ratio;
 	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_show_projection;
 	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_orthographic;
@@ -549,6 +653,9 @@ private:
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, with_constraint, measurement_property, with_serialization) m_crop_window_right;
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, with_constraint, measurement_property, with_serialization) m_crop_window_top;
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, with_constraint, measurement_property, with_serialization) m_crop_window_bottom;
+	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_show_reference_plane;
+	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, with_constraint, measurement_property, with_serialization) m_reference_plane;
+	k3d_data(k3d::color, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_reference_plane_color;
 
 	perspective_projection m_perspective_projection;
 	orthographic_projection m_orthographic_projection;
@@ -561,4 +668,3 @@ k3d::iplugin_factory& camera_factory()
 
 } // namespace libk3dcore
 
-
