@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -21,14 +21,14 @@
 		\author Tim Shead (tshead@k-3d.com)
 */
 
-#include <k3dsdk/i18n.h>
 #include <k3dsdk/algebra.h>
+#include <k3dsdk/i18n.h>
 #include <k3dsdk/itransform_sink.h>
 #include <k3dsdk/itransform_source.h>
 #include <k3dsdk/measurement.h>
+#include <k3dsdk/module.h>
 #include <k3dsdk/node.h>
 #include <k3dsdk/persistent.h>
-#include <k3dsdk/module.h>
 
 namespace libk3dcore
 {
@@ -50,8 +50,8 @@ public:
 		m_target(init_owner(*this) + init_name("target_matrix") + init_label(_("Target matrix")) + init_description(_("Target matrix")) + init_value(k3d::identity3D())),
 		m_output(init_owner(*this) + init_name("output_matrix") + init_label(_("Output matrix")) + init_description(_("Read only")) + init_slot(sigc::mem_fun(*this, &look_at::output_value)))
 	{
-		m_input.changed_signal().connect(m_output.changed_signal().make_slot());
-		m_target.changed_signal().connect(m_output.changed_signal().make_slot());
+		m_input.changed_signal().connect(m_output.make_reset_slot());
+		m_target.changed_signal().connect(m_output.make_reset_slot());
 	}
 
 	k3d::iproperty& transform_source_output()
@@ -69,9 +69,9 @@ public:
 		const k3d::matrix4 input_matrix = m_input.value();
 		const k3d::matrix4 target_matrix = m_target.value();
 
-		const k3d::vector3 from = input_matrix * k3d::vector3(0, 0, 0);
-		const k3d::vector3 to = target_matrix * k3d::vector3(0, 0, 0);
-		const k3d::vector3 spherical = (to - from).Spherical();
+		const k3d::point3 from = input_matrix * k3d::point3(0, 0, 0);
+		const k3d::point3 to = target_matrix * k3d::point3(0, 0, 0);
+		const k3d::vector3 spherical = k3d::spherical(k3d::to_vector(to - from));
 
 		return input_matrix * rotation3D(k3d::quaternion(k3d::euler_angles(0, -spherical[2], spherical[1], k3d::euler_angles::ZXYstatic)));
 	}
@@ -111,4 +111,3 @@ k3d::iplugin_factory& look_at_factory()
 
 } // namespace libk3dcore
 
-
