@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -21,9 +21,9 @@
 		\author Tim Shead (tshead@k-3d.com)
 */
 
-#include <k3dsdk/i18n.h>
 #include <k3dsdk/algebra.h>
 #include <k3dsdk/classes.h>
+#include <k3dsdk/i18n.h>
 #include <k3dsdk/itransform_sink.h>
 #include <k3dsdk/itransform_source.h>
 #include <k3dsdk/measurement.h>
@@ -54,11 +54,11 @@ public:
 		m_z(init_owner(*this) + init_name("z") + init_label(_("Z")) + init_description(_("Z offset")) + init_value(0.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_output(init_owner(*this) + init_name("output_matrix") + init_label(_("Output matrix")) + init_description(_("Read only")) + init_slot(sigc::mem_fun(*this, &position::output_value)))
 	{
-		m_input.changed_signal().connect(m_output.changed_signal().make_slot());
-		m_space.changed_signal().connect(m_output.changed_signal().make_slot());
-		m_x.changed_signal().connect(m_output.changed_signal().make_slot());
-		m_y.changed_signal().connect(m_output.changed_signal().make_slot());
-		m_z.changed_signal().connect(m_output.changed_signal().make_slot());
+		m_input.changed_signal().connect(m_output.make_reset_slot());
+		m_space.changed_signal().connect(m_output.make_reset_slot());
+		m_x.changed_signal().connect(m_output.make_reset_slot());
+		m_y.changed_signal().connect(m_output.make_reset_slot());
+		m_z.changed_signal().connect(m_output.make_reset_slot());
 	}
 
 	k3d::iproperty& transform_source_output()
@@ -73,7 +73,7 @@ public:
 
 	k3d::matrix4 output_value()
 	{
-		return m_input.value() * m_space.value() * translation3D(k3d::normal3(m_x.value(), m_y.value(), m_z.value())) * k3d::inverse(m_space.value());
+		return m_input.value() * m_space.value() * translation3D(k3d::vector3(m_x.value(), m_y.value(), m_z.value())) * k3d::inverse(m_space.value());
 	}
 
 	k3d::iplugin_factory& factory()
@@ -114,4 +114,3 @@ k3d::iplugin_factory& position_factory()
 
 } // namespace libk3dcore
 
-
