@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -24,8 +24,8 @@
 		\author Romain Behar (romainbehar@yahoo.com)
 */
 
-#include <k3dsdk/i18n.h>
 #include <k3dsdk/axis.h>
+#include <k3dsdk/i18n.h>
 #include <k3dsdk/measurement.h>
 #include <k3dsdk/mesh_modifier.h>
 #include <k3dsdk/mesh_selection_sink.h>
@@ -42,7 +42,7 @@ namespace detail
 class bulge_points
 {
 public:
-	bulge_points(const k3d::vector3& Origin, const k3d::vector3& Min, const k3d::vector3& Max, const k3d::axis Axis, const bool Radial, const bool displace_x, const bool displace_y, const bool displace_z, const double bulge_factor) :
+	bulge_points(const k3d::point3& Origin, const k3d::point3& Min, const k3d::point3& Max, const k3d::axis Axis, const bool Radial, const bool displace_x, const bool displace_y, const bool displace_z, const double bulge_factor) :
 		m_origin(Origin),
 		m_min(Min),
 		m_max(Max),
@@ -56,88 +56,88 @@ public:
 	{
 	}
 
-	const k3d::vector3 bulge(k3d::vector3& Coords)
+	const k3d::point3 bulge(k3d::point3& Coords)
 	{
-		k3d::vector3 coords = Coords + m_origin;
+		k3d::point3 coords = Coords + m_origin;
 
 		int index = 0;
 		switch(m_axis)
-			{
-				case k3d::X:
-					index = 0;
-				break;
-				case k3d::Y:
-					index = 1;
-				break;
-				case k3d::Z:
-					index = 2;
-				break;
-				default:
-					assert_not_reached();
+		{
+			case k3d::X:
+				index = 0;
+			break;
+			case k3d::Y:
+				index = 1;
+			break;
+			case k3d::Z:
+				index = 2;
+			break;
+			default:
+				assert_not_reached();
 
-			}
+		}
 
 		double delta = 0;
 		double length = m_size[index];
 		if(length != 0)
-			{
-				double min = m_min[index];
-				double max = m_max[index];
-				double value = coords[index];
+		{
+			double min = m_min[index];
+			double max = m_max[index];
+			double value = coords[index];
 
-				double down = value - min;
-				double up = max - value;
+			double down = value - min;
+			double up = max - value;
 
-				delta = 2 * down * up / (length*length);
-			}
+			delta = 2 * down * up / (length*length);
+		}
 
 		double bulge_amount = delta;
 		if(m_radial)
-			{
-				double distance = coords.Length();
-				double scale;
-				if(0 == distance)
-					scale = 1.0;
-				else
-					scale = (distance + m_bulge_factor * bulge_amount) / distance;
+		{
+			double distance = k3d::to_vector(coords).length();
+			double scale;
+			if(0 == distance)
+				scale = 1.0;
+			else
+				scale = (distance + m_bulge_factor * bulge_amount) / distance;
 
-				if(m_displace_x && (k3d::X != m_axis))
-					coords[0] *= scale;
+			if(m_displace_x && (k3d::X != m_axis))
+				coords[0] *= scale;
 
-				if(m_displace_y && (k3d::Y != m_axis))
-					coords[1] *= scale;
+			if(m_displace_y && (k3d::Y != m_axis))
+				coords[1] *= scale;
 
-				if(m_displace_z && (k3d::Z != m_axis))
-					coords[2] *= scale;
-			}
+			if(m_displace_z && (k3d::Z != m_axis))
+				coords[2] *= scale;
+		}
 		else
-			{
-				double offset = m_bulge_factor * bulge_amount;
+		{
+			double offset = m_bulge_factor * bulge_amount;
 
-				if(m_displace_x && (k3d::X != m_axis))
-					coords[0] += offset;
+			if(m_displace_x && (k3d::X != m_axis))
+				coords[0] += offset;
 
-				if(m_displace_y && (k3d::Y != m_axis))
-					coords[1] += offset;
+			if(m_displace_y && (k3d::Y != m_axis))
+				coords[1] += offset;
 
-				if(m_displace_z && (k3d::Z != m_axis))
-					coords[2] += offset;
-			}
+			if(m_displace_z && (k3d::Z != m_axis))
+				coords[2] += offset;
+		}
 
 		return coords - m_origin;
 	}
 
 private:
-	const k3d::vector3 m_origin;
-	const k3d::vector3 m_min;
-	const k3d::vector3 m_max;
+	const k3d::point3 m_origin;
+	const k3d::point3 m_min;
+	const k3d::point3 m_max;
 	const k3d::axis m_axis;
 	const bool m_radial;
 	const bool m_displace_x;
 	const bool m_displace_y;
 	const bool m_displace_z;
 	const double m_bulge_factor;
-	const k3d::vector3 m_size;
+	const k3d::point3 m_size;
 };
 
 } // namespace detail
@@ -160,63 +160,27 @@ public:
 		m_axis(init_owner(*this) + init_name("axis") + init_label(_("Axis")) + init_description(_("Axis to bulge along")) + init_value(k3d::Z) + init_enumeration(k3d::axis_values())),
 		m_type(init_owner(*this) + init_name("type") + init_label(_("Type")) + init_description(_("Bulge type")) + init_value(RADIAL) + init_enumeration(type_values()))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &bulge_points::on_reset_geometry));
-		m_mesh_selection.changed_signal().connect(sigc::mem_fun(*this, &bulge_points::on_reset_geometry));
+		m_mesh_selection.changed_signal().connect(make_reset_mesh_slot());
 
-		m_bulge_factor.changed_signal().connect(sigc::mem_fun(*this, &bulge_points::on_reshape_geometry));
-		m_displace_x.changed_signal().connect(sigc::mem_fun(*this, &bulge_points::on_reshape_geometry));
-		m_displace_y.changed_signal().connect(sigc::mem_fun(*this, &bulge_points::on_reshape_geometry));
-		m_displace_z.changed_signal().connect(sigc::mem_fun(*this, &bulge_points::on_reshape_geometry));
-		m_axis.changed_signal().connect(sigc::mem_fun(*this, &bulge_points::on_reshape_geometry));
-		m_type.changed_signal().connect(sigc::mem_fun(*this, &bulge_points::on_reshape_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &bulge_points::on_create_geometry));
+		m_bulge_factor.changed_signal().connect(make_update_mesh_slot());
+		m_displace_x.changed_signal().connect(make_update_mesh_slot());
+		m_displace_y.changed_signal().connect(make_update_mesh_slot());
+		m_displace_z.changed_signal().connect(make_update_mesh_slot());
+		m_axis.changed_signal().connect(make_update_mesh_slot());
+		m_type.changed_signal().connect(make_update_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		m_output_mesh.reset();
+		k3d::deep_copy(InputMesh, Mesh);
+		k3d::replace_selection(m_mesh_selection.value(), Mesh);
 	}
 
-	void on_reshape_geometry()
+	void on_update_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		if(m_output_mesh.empty())
-			return;
+		return_if_fail(InputMesh.points.size() == Mesh.points.size());
 
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return;
-
-		k3d::mesh* const output = const_cast<k3d::mesh*>(m_output_mesh.internal_value());
-		if(!output)
-			return;
-
-		reshape_geometry(*input, *output);
-
-		m_output_mesh.changed_signal().emit();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		// If we don't have any input mesh, we're done ...
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
-
-		// Otherwise, we make a copy of the input mesh and modify the copy ...
-		k3d::mesh* const output = new k3d::mesh();
-		k3d::deep_copy(*input, *output);
-		k3d::replace_selection(m_mesh_selection.value(), *output);
-		reshape_geometry(*input, *output);
-
-		return output;
-	}
-
-	void reshape_geometry(const k3d::mesh& Source, k3d::mesh& Target)
-	{
-		return_if_fail(Source.points.size() == Target.points.size());
-
-		const k3d::bounding_box3 bounds = k3d::bounds(Source);
+		const k3d::bounding_box3 bounds = k3d::bounds(InputMesh);
 
 		const double bulge_factor = m_bulge_factor.value();
 		const double displace_x = m_displace_x.value();
@@ -225,9 +189,9 @@ public:
 		const k3d::axis axis = m_axis.value();
 
 		detail::bulge_points bulger(
-			k3d::vector3(0, 0, 0),
-			k3d::vector3(bounds.nx, bounds.ny, bounds.nz),
-			k3d::vector3(bounds.px, bounds.py, bounds.pz),
+			k3d::point3(0, 0, 0),
+			k3d::point3(bounds.nx, bounds.ny, bounds.nz),
+			k3d::point3(bounds.px, bounds.py, bounds.pz),
 			axis,
 			m_type.value() == RADIAL,
 			displace_x,
@@ -236,8 +200,8 @@ public:
 			bulge_factor
 			);
 
-		for(unsigned long i = 0; i != Target.points.size(); ++i)
-			Target.points[i]->position = k3d::mix(Source.points[i]->position, bulger.bulge(Source.points[i]->position), Target.points[i]->selection_weight);
+		for(unsigned long i = 0; i != Mesh.points.size(); ++i)
+			Mesh.points[i]->position = k3d::mix(InputMesh.points[i]->position, bulger.bulge(InputMesh.points[i]->position), Mesh.points[i]->selection_weight);
 	}
 
 	k3d::iplugin_factory& factory()
@@ -326,4 +290,3 @@ k3d::iplugin_factory& bulge_points_factory()
 
 } // namespace libk3ddeformation
 
-
