@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -22,16 +22,16 @@
 		\author Romain Behar (romainbehar@yahoo.com)
 */
 
-#include <k3dsdk/i18n.h>
 #include <k3dsdk/axis.h>
 #include <k3dsdk/bounding_box.h>
+#include <k3dsdk/i18n.h>
 #include <k3dsdk/imaterial.h>
-#include <k3dsdk/node.h>
-#include <k3dsdk/persistent.h>
 #include <k3dsdk/measurement.h>
 #include <k3dsdk/mesh_modifier.h>
 #include <k3dsdk/mesh_selection_sink.h>
 #include <k3dsdk/module.h>
+#include <k3dsdk/node.h>
+#include <k3dsdk/persistent.h>
 
 #include <iterator>
 
@@ -49,61 +49,26 @@ class cylindrical_wave_points :
 public:
 	cylindrical_wave_points(k3d::idocument& Document) :
 		base(Document),
-		m_along(init_owner(*this) + init_name("along") + init_label(_("Along")) + init_description(_("Wave points along given axis")) + init_value(k3d::X) + init_enumeration(k3d::axis_values())),
+		m_along(init_owner(*this) + init_name("along") + init_label(_("Along")) + init_description(_("Wave points along given axis")) + init_value(k3d::Z) + init_enumeration(k3d::axis_values())),
 		m_amplitude(init_owner(*this) + init_name("amplitude") + init_label(_("Amplitude")) + init_description(_("Wave amplitude")) + init_value(5.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_wavelength(init_owner(*this) + init_name("wavelength") + init_label(_("Wavelength")) + init_description(_("Wavelength")) + init_value(10.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_phase(init_owner(*this) + init_name("phase") + init_label(_("Phase")) + init_description(_("Wave phase")) + init_value(0.0) + init_precision(2) + init_step_increment(k3d::radians(1.0)) + init_units(typeid(k3d::measurement::angle)))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &cylindrical_wave_points::on_reset_geometry));
-		m_mesh_selection.changed_signal().connect(sigc::mem_fun(*this, &cylindrical_wave_points::on_reset_geometry));
+		m_mesh_selection.changed_signal().connect(make_reset_mesh_slot());
 
-		m_along.changed_signal().connect(sigc::mem_fun(*this, &cylindrical_wave_points::on_reshape_geometry));
-		m_amplitude.changed_signal().connect(sigc::mem_fun(*this, &cylindrical_wave_points::on_reshape_geometry));
-		m_wavelength.changed_signal().connect(sigc::mem_fun(*this, &cylindrical_wave_points::on_reshape_geometry));
-		m_phase.changed_signal().connect(sigc::mem_fun(*this, &cylindrical_wave_points::on_reshape_geometry));
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &cylindrical_wave_points::on_create_geometry));
+		m_along.changed_signal().connect(make_update_mesh_slot());
+		m_amplitude.changed_signal().connect(make_update_mesh_slot());
+		m_wavelength.changed_signal().connect(make_update_mesh_slot());
+		m_phase.changed_signal().connect(make_update_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		m_output_mesh.reset();
+		k3d::deep_copy(InputMesh, Mesh);
+		k3d::replace_selection(m_mesh_selection.value(), Mesh);
 	}
 
-	void on_reshape_geometry()
-	{
-		if(m_output_mesh.empty())
-			return;
-
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return;
-
-		k3d::mesh* const output = const_cast<k3d::mesh*>(m_output_mesh.internal_value());
-		if(!output)
-			return;
-
-		reshape_geometry(*input, *output);
-
-		m_output_mesh.changed_signal().emit();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		// If we don't have any input mesh, we're done ...
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
-
-		// Otherwise, we make a copy of the input mesh and modify the copy ...
-		k3d::mesh* const output = new k3d::mesh();
-		k3d::deep_copy(*input, *output);
-		k3d::replace_selection(m_mesh_selection.value(), *output);
-		reshape_geometry(*input, *output);
-
-		return output;
-	}
-
-	void reshape_geometry(const k3d::mesh& Source, k3d::mesh& Target)
+	void on_update_mesh(const k3d::mesh& Source, k3d::mesh& Target)
 	{
 		// Sanity checks ...
 		assert(Source.points.size() == Target.points.size());
@@ -118,18 +83,18 @@ public:
 			return;
 
 		for(unsigned long i = 0; i != Target.points.size(); ++i)
-			{
-				k3d::vector3 position = Source.points[i]->position;
+		{
+			k3d::point3 position = Source.points[i]->position;
 
-				const double wave_position = phase + (k3d::pi_times_2() * position[along] / wavelength);
-				const double offset = amplitude * sin(wave_position);
+			const double wave_position = phase + (k3d::pi_times_2() * position[along] / wavelength);
+			const double offset = amplitude * sin(wave_position);
 
-				const k3d::vector3 direction((k3d::X != along) * position[0], (k3d::Y != along) * position[1], (k3d::Z != along) * position[2]);
-				if(0 == direction.Length2())
-					continue;
+			const k3d::vector3 direction((k3d::X != along) * position[0], (k3d::Y != along) * position[1], (k3d::Z != along) * position[2]);
+			if(0 == direction.length2())
+				continue;
 
-				Target.points[i]->position = k3d::mix(Source.points[i]->position, position + offset * k3d::normalize(direction), Target.points[i]->selection_weight);
-			}
+			Target.points[i]->position = k3d::mix(Source.points[i]->position, position + offset * k3d::normalize(direction), Target.points[i]->selection_weight);
+		}
 	}
 
 	k3d::iplugin_factory& factory()
@@ -169,4 +134,3 @@ k3d::iplugin_factory& cylindrical_wave_points_factory()
 
 } // namespace libk3ddeformation
 
-
