@@ -75,7 +75,7 @@ public:
 		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &deformation_bone::on_reset_geometry));
 		m_input_matrix.changed_signal().connect(sigc::mem_fun(*this, &deformation_bone::on_reset_geometry));
 
-		m_length.changed_signal().connect(sigc::mem_fun(*this, &deformation_bone::async_redraw_all));
+		m_length.changed_signal().connect(make_async_redraw_slot());
 
 		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &deformation_bone::on_create_geometry));
 	}
@@ -131,7 +131,7 @@ public:
 
 	void on_gl_draw(const k3d::gl::render_state& State)
 	{
-		glColor3dv(get_visible_selection() ? k3d::vector3(1, 1, 1) : k3d::vector3(0, 0, 0));
+		glColor3dv(get_visible_selection() ? k3d::point3(1, 1, 1) : k3d::point3(0, 0, 0));
 		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
 		glDisable(GL_TEXTURE_1D);
 		glDisable(GL_TEXTURE_2D);
