@@ -49,64 +49,28 @@ class linear_wave_points :
 public:
 	linear_wave_points(k3d::idocument& Document) :
 		base(Document),
-		m_axis(init_owner(*this) + init_name("axis") + init_label(_("Axis")) + init_description(_("Axis")) + init_value(k3d::Y) + init_enumeration(k3d::axis_values())),
+		m_axis(init_owner(*this) + init_name("axis") + init_label(_("Axis")) + init_description(_("Axis")) + init_value(k3d::Z) + init_enumeration(k3d::axis_values())),
 		m_along(init_owner(*this) + init_name("along") + init_label(_("Along")) + init_description(_("Along")) + init_value(k3d::X) + init_enumeration(k3d::axis_values())),
-		m_amplitude(init_owner(*this) + init_name("amplitude") + init_label(_("Amplitude")) + init_description(_("Amplitude")) + init_value(5.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
+		m_amplitude(init_owner(*this) + init_name("amplitude") + init_label(_("Amplitude")) + init_description(_("Amplitude")) + init_value(1.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_wavelength(init_owner(*this) + init_name("wavelength") + init_label(_("Wavelength")) + init_description(_("Wavelength")) + init_value(10.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_phase(init_owner(*this) + init_name("phase") + init_label(_("Phase")) + init_description(_("Phase")) + init_value(0.0) + init_precision(2) + init_step_increment(k3d::radians(1.0)) + init_units(typeid(k3d::measurement::angle)))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &linear_wave_points::on_reset_geometry));
-		m_mesh_selection.changed_signal().connect(sigc::mem_fun(*this, &linear_wave_points::on_reset_geometry));
+		m_mesh_selection.changed_signal().connect(make_reset_mesh_slot());
 
-		m_axis.changed_signal().connect(sigc::mem_fun(*this, &linear_wave_points::on_reshape_geometry));
-		m_along.changed_signal().connect(sigc::mem_fun(*this, &linear_wave_points::on_reshape_geometry));
-		m_amplitude.changed_signal().connect(sigc::mem_fun(*this, &linear_wave_points::on_reshape_geometry));
-		m_wavelength.changed_signal().connect(sigc::mem_fun(*this, &linear_wave_points::on_reshape_geometry));
-		m_phase.changed_signal().connect(sigc::mem_fun(*this, &linear_wave_points::on_reshape_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &linear_wave_points::on_create_geometry));
+		m_axis.changed_signal().connect(make_update_mesh_slot());
+		m_along.changed_signal().connect(make_update_mesh_slot());
+		m_amplitude.changed_signal().connect(make_update_mesh_slot());
+		m_wavelength.changed_signal().connect(make_update_mesh_slot());
+		m_phase.changed_signal().connect(make_update_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		m_output_mesh.reset();
+		k3d::deep_copy(InputMesh, Mesh);
+		k3d::replace_selection(m_mesh_selection.value(), Mesh);
 	}
 
-	void on_reshape_geometry()
-	{
-		if(m_output_mesh.empty())
-			return;
-
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return;
-
-		k3d::mesh* const output = const_cast<k3d::mesh*>(m_output_mesh.internal_value());
-		if(!output)
-			return;
-
-		reshape_geometry(*input, *output);
-
-		m_output_mesh.changed_signal().emit();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		// If we don't have any input mesh, we're done ...
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
-
-		// Otherwise, we make a copy of the input mesh and modify the copy ...
-		k3d::mesh* const output = new k3d::mesh();
-		k3d::deep_copy(*input, *output);
-		k3d::replace_selection(m_mesh_selection.value(), *output);
-		reshape_geometry(*input, *output);
-
-		return output;
-	}
-
-	void reshape_geometry(const k3d::mesh& Source, k3d::mesh& Target)
+	void on_update_mesh(const k3d::mesh& Source, k3d::mesh& Target)
 	{
 		// Sanity checks ...
 		assert(Source.points.size() == Target.points.size());
@@ -117,7 +81,7 @@ public:
 		const double wavelength = m_wavelength.value();
 		const double phase = m_phase.value();
 
-		const k3d::vector3 offset_filter(k3d::X == axis, k3d::Y == axis, k3d::Z == axis);
+		const k3d::point3 offset_filter(k3d::X == axis, k3d::Y == axis, k3d::Z == axis);
 
 		// Filter-out infinite frequencies ...
 		if(0 == wavelength)
@@ -125,7 +89,7 @@ public:
 
 		for(unsigned long i = 0; i != Target.points.size(); ++i)
 			{
-				k3d::vector3 position = Source.points[i]->position;
+				k3d::point3 position = Source.points[i]->position;
 
 				const double wave_position = phase + (k3d::pi_times_2() * position[along] / wavelength);
 				const double offset = amplitude * sin(wave_position);
@@ -172,4 +136,3 @@ k3d::iplugin_factory& linear_wave_points_factory()
 
 } // namespace libk3ddeformation
 
-
