@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -23,14 +23,13 @@
 */
 
 #include <k3dsdk/i18n.h>
-#include <k3dsdk/axis.h>
 #include <k3dsdk/imaterial.h>
-#include <k3dsdk/node.h>
-#include <k3dsdk/persistent.h>
 #include <k3dsdk/measurement.h>
 #include <k3dsdk/mesh_modifier.h>
 #include <k3dsdk/mesh_selection_sink.h>
 #include <k3dsdk/module.h>
+#include <k3dsdk/node.h>
+#include <k3dsdk/persistent.h>
 
 #include <iterator>
 
@@ -52,65 +51,24 @@ public:
 		m_y(init_owner(*this) + init_name("y") + init_label(_("Y")) + init_description(_("Y scale")) + init_value(1.0) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::scalar))),
 		m_z(init_owner(*this) + init_name("z") + init_label(_("Z")) + init_description(_("Z scale")) + init_value(1.0) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &scale_points::on_reset_geometry));
-		m_mesh_selection.changed_signal().connect(sigc::mem_fun(*this, &scale_points::on_reset_geometry));
-
-		m_x.changed_signal().connect(sigc::mem_fun(*this, &scale_points::on_reshape_geometry));
-		m_y.changed_signal().connect(sigc::mem_fun(*this, &scale_points::on_reshape_geometry));
-		m_z.changed_signal().connect(sigc::mem_fun(*this, &scale_points::on_reshape_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &scale_points::on_create_geometry));
+		m_mesh_selection.changed_signal().connect(make_update_mesh_slot());
+		m_x.changed_signal().connect(make_update_mesh_slot());
+		m_y.changed_signal().connect(make_update_mesh_slot());
+		m_z.changed_signal().connect(make_update_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	void on_create_mesh(const k3d::mesh& Input, k3d::mesh& Output)
 	{
-		m_output_mesh.reset();
+		k3d::deep_copy(Input, Output);
 	}
 
-	void on_reshape_geometry()
+	void on_update_mesh(const k3d::mesh& Input, k3d::mesh& Output)
 	{
-		if(m_output_mesh.empty())
-			return;
+		k3d::replace_selection(m_mesh_selection.value(), Output);
 
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return;
-
-		k3d::mesh* const output = const_cast<k3d::mesh*>(m_output_mesh.value());
-		if(!output)
-			return;
-
-		reshape_geometry(*input, *output);
-
-		m_output_mesh.changed_signal().emit();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		// If we don't have any input mesh, we're done ...
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
-
-		// Otherwise, we make a copy of the input mesh and modify the copy ...
-		k3d::mesh* const output = new k3d::mesh();
-		k3d::deep_copy(*input, *output);
-		k3d::replace_selection(m_mesh_selection.value(), *output);
-		reshape_geometry(*input, *output);
-
-		return output;
-	}
-
-	void reshape_geometry(const k3d::mesh& Source, k3d::mesh& Target)
-	{
-		// Sanity checks ...
-		assert(Source.points.size() == Target.points.size());
-
-		// Scale points ...
-		const k3d::matrix4 matrix = k3d::scaling3D(k3d::vector3(m_x.value(), m_y.value(), m_z.value()));
-
-		for(unsigned long i = 0; i != Target.points.size(); ++i)
-			Target.points[i]->position = k3d::mix(Source.points[i]->position, matrix * Source.points[i]->position, Target.points[i]->selection_weight);
+		const k3d::matrix4 matrix = k3d::scaling3D(k3d::point3(m_x.value(), m_y.value(), m_z.value()));
+		for(unsigned long i = 0; i != Output.points.size(); ++i)
+			Output.points[i]->position = k3d::mix(Input.points[i]->position, matrix * Input.points[i]->position, Output.points[i]->selection_weight);
 	}
 
 	k3d::iplugin_factory& factory()
@@ -149,4 +107,3 @@ k3d::iplugin_factory& scale_points_factory()
 
 } // namespace libk3ddeformation
 
-
