@@ -49,60 +49,24 @@ class shear_points :
 public:
 	shear_points(k3d::idocument& Document) :
 		base(Document),
-		m_direction(init_owner(*this) + init_name("direction") + init_label(_("Direction")) + init_description(_("Shear direction")) + init_value(k3d::X) + init_enumeration(k3d::axis_values())),
-		m_axis(init_owner(*this) + init_name("axis") + init_label(_("Axis")) + init_description(_("Shear axis")) + init_value(k3d::Y) + init_enumeration(k3d::axis_values())),
+		m_direction(init_owner(*this) + init_name("direction") + init_label(_("Direction")) + init_description(_("Shear direction")) + init_value(k3d::Z) + init_enumeration(k3d::axis_values())),
+		m_axis(init_owner(*this) + init_name("axis") + init_label(_("Axis")) + init_description(_("Shear axis")) + init_value(k3d::X) + init_enumeration(k3d::axis_values())),
 		m_shear_factor(init_owner(*this) + init_name("shear_factor") + init_label(_("Shear factor")) + init_description(_("Shear factor")) + init_value(0.0) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &shear_points::on_reset_geometry));
-		m_mesh_selection.changed_signal().connect(sigc::mem_fun(*this, &shear_points::on_reset_geometry));
+		m_mesh_selection.changed_signal().connect(make_reset_mesh_slot());
 
-		m_direction.changed_signal().connect(sigc::mem_fun(*this, &shear_points::on_reshape_geometry));
-		m_axis.changed_signal().connect(sigc::mem_fun(*this, &shear_points::on_reshape_geometry));
-		m_shear_factor.changed_signal().connect(sigc::mem_fun(*this, &shear_points::on_reshape_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &shear_points::on_create_geometry));
+		m_direction.changed_signal().connect(make_update_mesh_slot());
+		m_axis.changed_signal().connect(make_update_mesh_slot());
+		m_shear_factor.changed_signal().connect(make_update_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		m_output_mesh.reset();
+		k3d::deep_copy(InputMesh, Mesh);
+		k3d::replace_selection(m_mesh_selection.value(), Mesh);
 	}
 
-	void on_reshape_geometry()
-	{
-		if(m_output_mesh.empty())
-			return;
-
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return;
-
-		k3d::mesh* const output = const_cast<k3d::mesh*>(m_output_mesh.internal_value());
-		if(!output)
-			return;
-
-		reshape_geometry(*input, *output);
-
-		m_output_mesh.changed_signal().emit();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		// If we don't have any input mesh, we're done ...
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
-
-		// Otherwise, we make a copy of the input mesh and modify the copy ...
-		k3d::mesh* const output = new k3d::mesh();
-		k3d::deep_copy(*input, *output);
-		k3d::replace_selection(m_mesh_selection.value(), *output);
-		reshape_geometry(*input, *output);
-
-		return output;
-	}
-
-	void reshape_geometry(const k3d::mesh& Source, k3d::mesh& Target)
+	void on_update_mesh(const k3d::mesh& Source, k3d::mesh& Target)
 	{
 		// Sanity checks ...
 		assert(Source.points.size() == Target.points.size());
