@@ -91,57 +91,22 @@ public:
 		base(Document),
 		m_smoothing(init_owner(*this) + init_name("smoothing") + init_label(_("Smoothing")) + init_description(_("Smoothing factor")) + init_value(1.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &smooth_points::on_reset_geometry));
-		m_mesh_selection.changed_signal().connect(sigc::mem_fun(*this, &smooth_points::on_reset_geometry));
+		m_mesh_selection.changed_signal().connect(make_reset_mesh_slot());
 
-		m_smoothing.changed_signal().connect(sigc::mem_fun(*this, &smooth_points::on_reshape_geometry));
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &smooth_points::on_create_geometry));
+		m_smoothing.changed_signal().connect(make_update_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		m_output_mesh.reset();
+		k3d::deep_copy(InputMesh, Mesh);
+		k3d::replace_selection(m_mesh_selection.value(), Mesh);
 	}
 
-	void on_reshape_geometry()
-	{
-		if(m_output_mesh.empty())
-			return;
-
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return;
-
-		k3d::mesh* const output = const_cast<k3d::mesh*>(m_output_mesh.internal_value());
-		if(!output)
-			return;
-
-		reshape_geometry(*input, *output);
-
-		m_output_mesh.changed_signal().emit();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		// If we don't have any input mesh, we're done ...
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
-
-		// Otherwise, we make a copy of the input mesh and modify the copy ...
-		k3d::mesh* const output = new k3d::mesh();
-		k3d::deep_copy(*input, *output);
-		k3d::replace_selection(m_mesh_selection.value(), *output);
-		reshape_geometry(*input, *output);
-
-		return output;
-	}
-
-	void reshape_geometry(const k3d::mesh& Source, k3d::mesh& Target)
+	void on_update_mesh(const k3d::mesh& Source, k3d::mesh& Target)
 	{
 		// Sanity checks ...
-		assert(Source.polyhedra.size() == Target.polyhedra.size());
-		assert(Source.points.size() == Target.points.size());
+		return_if_fail(Source.polyhedra.size() == Target.polyhedra.size());
+		return_if_fail(Source.points.size() == Target.points.size());
 
 		const double smoothing = m_smoothing.value();
 
@@ -164,7 +129,7 @@ public:
 				if(edges.empty())
 					continue;
 
-				k3d::vector3 position(0, 0, 0);
+				k3d::point3 position(0, 0, 0);
 				for(detail::edges_t::const_iterator edge = edges.begin(); edge != edges.end(); ++edge)
 					position += (*edge)->companion->vertex->position;
 
