@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -22,15 +22,15 @@
 		\author Romain Behar (romainbehar@yahoo.com)
 */
 
-#include <k3dsdk/i18n.h>
 #include <k3dsdk/bounding_box.h>
+#include <k3dsdk/i18n.h>
 #include <k3dsdk/imaterial.h>
-#include <k3dsdk/node.h>
-#include <k3dsdk/persistent.h>
 #include <k3dsdk/measurement.h>
 #include <k3dsdk/mesh_modifier.h>
 #include <k3dsdk/mesh_selection_sink.h>
 #include <k3dsdk/module.h>
+#include <k3dsdk/node.h>
+#include <k3dsdk/persistent.h>
 
 #include <iterator>
 
@@ -48,55 +48,20 @@ class sphereize_points :
 public:
 	sphereize_points(k3d::idocument& Document) :
 		base(Document),
-		m_sphere_factor(init_owner(*this) + init_name("sphere_factor") + init_label(_("Sphere factor")) + init_description(_("Sphere factor")) + init_value(0.0) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::scalar)))
+		m_sphere_factor(init_owner(*this) + init_name("sphere_factor") + init_label(_("Sphere factor")) + init_description(_("Sphere factor")) + init_value(1.0) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &sphereize_points::on_reset_geometry));
-		m_mesh_selection.changed_signal().connect(sigc::mem_fun(*this, &sphereize_points::on_reset_geometry));
+		m_mesh_selection.changed_signal().connect(make_reset_mesh_slot());
 
-		m_sphere_factor.changed_signal().connect(sigc::mem_fun(*this, &sphereize_points::on_reshape_geometry));
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &sphereize_points::on_create_geometry));
+		m_sphere_factor.changed_signal().connect(make_update_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		m_output_mesh.reset();
+		k3d::deep_copy(InputMesh, Mesh);
+		k3d::replace_selection(m_mesh_selection.value(), Mesh);
 	}
 
-	void on_reshape_geometry()
-	{
-		if(m_output_mesh.empty())
-			return;
-
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return;
-
-		k3d::mesh* const output = const_cast<k3d::mesh*>(m_output_mesh.internal_value());
-		if(!output)
-			return;
-
-		reshape_geometry(*input, *output);
-
-		m_output_mesh.changed_signal().emit();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		// If we don't have any input mesh, we're done ...
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
-
-		// Otherwise, we make a copy of the input mesh and modify the copy ...
-		k3d::mesh* const output = new k3d::mesh();
-		k3d::deep_copy(*input, *output);
-		k3d::replace_selection(m_mesh_selection.value(), *output);
-		reshape_geometry(*input, *output);
-
-		return output;
-	}
-
-	void reshape_geometry(const k3d::mesh& Source, k3d::mesh& Target)
+	void on_update_mesh(const k3d::mesh& Source, k3d::mesh& Target)
 	{
 		// Sanity checks ...
 		assert(Source.points.size() == Target.points.size());
@@ -105,19 +70,19 @@ public:
 
 		double target_radius = 0.0;
 		for(unsigned long i = 0; i != Source.points.size(); ++i)
-			target_radius = std::max(target_radius, Source.points[i]->position.Length());
+			target_radius = std::max(target_radius, k3d::to_vector(Source.points[i]->position).length());
 		if(0.0 == target_radius)
 			return;
 
 		for(unsigned long i = 0; i != Target.points.size(); ++i)
-			{
-				double radius = Source.points[i]->position.Length();
-				if(0.0 == radius)
-					radius = target_radius;
+		{
+			double radius = k3d::to_vector(Source.points[i]->position).length();
+			if(0.0 == radius)
+				radius = target_radius;
 
-				Target.points[i]->position =
-					k3d::mix(Source.points[i]->position, Source.points[i]->position * k3d::mix(radius, target_radius, sphere_factor) / radius, Target.points[i]->selection_weight);
-			}
+			Target.points[i]->position =
+				k3d::mix(Source.points[i]->position, Source.points[i]->position * k3d::mix(radius, target_radius, sphere_factor) / radius, Target.points[i]->selection_weight);
+		}
 	}
 
 	k3d::iplugin_factory& factory()
@@ -154,4 +119,3 @@ k3d::iplugin_factory& sphereize_points_factory()
 
 } // namespace libk3ddeformation
 
-
