@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 2005, Romain Behar
+// Copyright (c) 2005-2006, Romain Behar
 //
 // Contact: romainbehar@yahoo.com
 //
@@ -45,7 +45,7 @@ namespace detail
 class bevel_edge
 {
 public:
-	bevel_edge(const k3d::vector3& StartPosition, const k3d::vector3& EndPosition, k3d::point* Point) :
+	bevel_edge(const k3d::point3& StartPosition, const k3d::point3& EndPosition, k3d::point* Point) :
 		start_position(StartPosition),
 		end_position(EndPosition),
 		point(Point)
@@ -64,7 +64,7 @@ public:
 		else
 		{
 			// Offset is a length from start_position
-			const double length = (end_position - start_position).Length();
+			const double length = k3d::distance(end_position, start_position);
 			if(length)
 				edge_offset = Offset / length;
 		}
@@ -73,8 +73,8 @@ public:
 	}
 
 private:
-	k3d::vector3 start_position;
-	k3d::vector3 end_position;
+	k3d::point3 start_position;
+	k3d::point3 end_position;
 	k3d::point* point;
 };
 
@@ -83,7 +83,7 @@ typedef std::vector<bevel_edge> bevel_edges_t;
 class face_point
 {
 public:
-	face_point(const k3d::vector3& OldPosition, const k3d::vector3& InsetDirection, k3d::point& Point) :
+	face_point(const k3d::point3& OldPosition, const k3d::vector3& InsetDirection, k3d::point& Point) :
 		old_position(OldPosition),
 		inset_direction(InsetDirection),
 		point(&Point)
@@ -96,7 +96,7 @@ public:
 	}
 
 private:
-	k3d::vector3 old_position;
+	k3d::point3 old_position;
 	k3d::vector3 inset_direction;
 	k3d::point* point;
 };
@@ -118,8 +118,8 @@ typedef std::map<k3d::split_edge*, k3d::face*> edge_to_face_t;
 bool subdivide_edges(k3d::face& Face, k3d::mesh& Mesh, bevel_edges_t& BevelEdges, face_points_t& FacePoints, edge_to_face_t& EdgeToFace)
 {
 	// Calculate the face normal
-	k3d::vector3 face_normal = k3d::normal(Face);
-	if(0.0 == face_normal.Length2())
+	k3d::normal3 face_normal = k3d::normal(Face);
+	if(0.0 == face_normal.length2())
 		return false;
 
 	face_normal = k3d::normalize(face_normal);
@@ -142,8 +142,8 @@ bool subdivide_edges(k3d::face& Face, k3d::mesh& Mesh, bevel_edges_t& BevelEdges
 		edge = face_edges[i];
 		k3d::split_edge* previous_edge = face_edges[(i + n-1) % n];
 
-		const k3d::vector3 e1 = edge->vertex->position - previous_edge->vertex->position;
-		const k3d::vector3 e2 = edge->face_clockwise->vertex->position - edge->vertex->position;
+		const k3d::vector3 e1 = k3d::to_vector(edge->vertex->position - previous_edge->vertex->position);
+		const k3d::vector3 e2 = k3d::to_vector(edge->face_clockwise->vertex->position - edge->vertex->position);
 		const k3d::vector3 e3 = k3d::normalize(e1 ^ face_normal);
 		const k3d::vector3 e4 = k3d::normalize(e2 ^ face_normal);
 
@@ -197,8 +197,8 @@ bool subdivide_edges(k3d::face& Face, k3d::mesh& Mesh, bevel_edges_t& BevelEdges
 			k3d::split_edge* next_edge = subdivided_edge->companion->face_clockwise;
 
 			// Save original values
-			const k3d::vector3& start_position = subdivided_edge->vertex->position;
-			const k3d::vector3& end_position = subdivided_edge->face_clockwise->vertex->position;
+			const k3d::point3& start_position = subdivided_edge->vertex->position;
+			const k3d::point3& end_position = subdivided_edge->face_clockwise->vertex->position;
 
 			// Create new point
 			k3d::point* new_point = helpers::append_edge(subdivided_edge);
@@ -227,7 +227,7 @@ bool subdivide_edges(k3d::face& Face, k3d::mesh& Mesh, bevel_edges_t& BevelEdges
 }
 
 /// Connects two vertices belonging to the same face
-k3d::face* connect_vertices(k3d::face* Face, k3d::split_edge* Edge1, k3d::split_edge* Edge2, edge_to_face_t& EdgeToFace)
+k3d::face* connect_vertices(k3d::face* Face, k3d::split_edge* Edge1, k3d::split_edge* Edge2, edge_to_face_t& EdgeToFace, faces_t& SideFaces)
 {
 	// Make new edge loops
 	k3d::split_edge* new_edge1 = new k3d::split_edge(Edge1->vertex, Edge2);
@@ -246,10 +246,12 @@ k3d::face* connect_vertices(k3d::face* Face, k3d::split_edge* Edge1, k3d::split_
 	if(helpers::edge_in_face(Face->first_edge, Edge1))
 	{
 		new_face = new k3d::face(Edge2, Face->material);
+		SideFaces.push_back(Face);
 	}
 	else
 	{
 		new_face = new k3d::face(Edge1, Face->material);
+		SideFaces.push_back(new_face);
 	}
 
 	// Assign new face to edges
@@ -266,7 +268,7 @@ k3d::face* connect_vertices(k3d::face* Face, k3d::split_edge* Edge1, k3d::split_
 }
 
 // Connect subdivision vertices
-void bevel_faces(edge_to_face_t& EdgeToFace, faces_t& NewFaces)
+void bevel_faces(edge_to_face_t& EdgeToFace, faces_t& NewFaces, faces_t& SideFaces)
 {
 	for(edge_lists_t::iterator list = link_edges.begin(); list != link_edges.end(); ++list)
 	{
@@ -276,7 +278,7 @@ void bevel_faces(edge_to_face_t& EdgeToFace, faces_t& NewFaces)
 		{
 			k3d::split_edge* edge1 = edges[i];
 			k3d::split_edge* edge2 = edges[(i + 1) % n];
-			k3d::face* face = connect_vertices(EdgeToFace[edge1], edge1, edge2, EdgeToFace);
+			k3d::face* face = connect_vertices(EdgeToFace[edge1], edge1, edge2, EdgeToFace, SideFaces);
 			if(face)
 			{
 				NewFaces.push_back(face);
@@ -300,54 +302,39 @@ public:
 		base(Document),
 		m_offset(init_owner(*this) + init_name("offset") + init_label(_("Offset")) + init_description(_("Offset along external edges")) + init_value(0.3) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_inset(init_owner(*this) + init_name("inset") + init_label(_("Inset")) + init_description(_("Inset of bevelled face")) + init_value(0.3) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::distance))),
-		m_distance(init_owner(*this) + init_name("distance") + init_label(_("Distance")) + init_description(_("Use distance instead of edge offset")) + init_value(false))
+		m_distance(init_owner(*this) + init_name("distance") + init_label(_("Distance")) + init_description(_("Use distance instead of edge offset")) + init_value(false)),
+		m_select_side_faces(init_owner(*this) + init_name("select_side_faces") + init_label(_("Select side faces")) + init_description(_("Select side faces on output, doesn't change selection if off")) + init_value(true))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &bevel_faces_implementation::on_reset_geometry));
+		m_mesh_selection.changed_signal().connect(make_reset_mesh_slot());
 
-		m_mesh_selection.changed_signal().connect(sigc::mem_fun(*this, &bevel_faces_implementation::on_reset_geometry));
-
-		m_offset.changed_signal().connect(sigc::mem_fun(*this, &bevel_faces_implementation::on_reshape_geometry));
-		m_inset.changed_signal().connect(sigc::mem_fun(*this, &bevel_faces_implementation::on_reshape_geometry));
-		m_distance.changed_signal().connect(sigc::mem_fun(*this, &bevel_faces_implementation::on_reshape_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &bevel_faces_implementation::on_create_geometry));
+		m_offset.changed_signal().connect(make_update_mesh_slot());
+		m_inset.changed_signal().connect(make_update_mesh_slot());
+		m_distance.changed_signal().connect(make_update_mesh_slot());
+		m_select_side_faces.changed_signal().connect(make_update_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	/** \todo Improve the implementation so we don't have to do this */
+	k3d::iunknown* on_rewrite_hint(iunknown* const Hint)
 	{
-		m_output_mesh.reset();
+		// Force updates to re-allocate our mesh, for simplicity
+		return 0;
 	}
 
-	void on_reshape_geometry()
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		if(m_output_mesh.empty())
-			return;
-
-		reshape_geometry();
-
-		m_output_mesh.changed_signal().emit();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		// Get the input geometry
-		k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
-
 		// Clear previously cached data
 		m_bevel_edges.clear();
 		m_face_points.clear();
 		m_new_faces.clear();
+		m_side_faces.clear();
 
 		detail::link_edges.clear();
 
 		// Create output geometry
-		std::auto_ptr<k3d::mesh> output(new k3d::mesh());
-		k3d::deep_copy(*input, *output);
-		k3d::replace_selection(m_mesh_selection.value(), *output);
+		k3d::deep_copy(InputMesh, Mesh);
+		k3d::replace_selection(m_mesh_selection.value(), Mesh);
 
-		for(k3d::mesh::polyhedra_t::iterator polyhedron_i = output->polyhedra.begin(); polyhedron_i != output->polyhedra.end(); ++polyhedron_i)
+		for(k3d::mesh::polyhedra_t::iterator polyhedron_i = Mesh.polyhedra.begin(); polyhedron_i != Mesh.polyhedra.end(); ++polyhedron_i)
 		{
 			k3d::polyhedron& polyhedron = **polyhedron_i;
 
@@ -376,7 +363,7 @@ public:
 			for(k3d::polyhedron::faces_t::iterator selected_face = selected_faces.begin(); selected_face != selected_faces.end(); ++selected_face)
 			{
 				// Save original edges and create new points
-				if(detail::subdivide_edges(**selected_face, *output, m_bevel_edges, m_face_points, edge_to_face))
+				if(detail::subdivide_edges(**selected_face, Mesh, m_bevel_edges, m_face_points, edge_to_face))
 				{
 					m_new_faces.push_back(*selected_face);
 					break;
@@ -385,26 +372,15 @@ public:
 
 			// Join new points
 			detail::faces_t new_faces;
-			detail::bevel_faces(edge_to_face, new_faces);
+			detail::bevel_faces(edge_to_face, new_faces, m_side_faces);
 			polyhedron.faces.insert(polyhedron.faces.end(), new_faces.begin(), new_faces.end());
 			m_new_faces.insert(m_new_faces.end(), new_faces.begin(), new_faces.end());
 
 			assert_warning(k3d::is_valid(polyhedron));
 		}
-
-		// Update output selection
-		for(detail::faces_t::iterator face = m_new_faces.begin(); face != m_new_faces.end(); ++face)
-		{
-			(*face)->selection_weight = 1.0;
-			(*face)->visible_selection = true;
-		}
-
-		reshape_geometry();
-
-		return output.release();
 	}
 
-	void reshape_geometry()
+	void on_update_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
 		const double offset = m_offset.value();
 		const bool distance = m_distance.value();
@@ -413,9 +389,12 @@ public:
 			point->update(offset, distance);
 
 		const double inset = m_inset.value();
-
 		for(detail::face_points_t::iterator face_point = m_face_points.begin(); face_point != m_face_points.end(); ++face_point)
 			face_point->update(inset);
+
+		const double weight = m_select_side_faces.value() ? 1.0 : 0.0;
+		for(detail::faces_t::iterator face = m_side_faces.begin(); face != m_side_faces.end(); ++face)
+			(*face)->selection_weight = weight;
 	}
 
 	k3d::iplugin_factory& factory()
@@ -442,10 +421,12 @@ private:
 	detail::bevel_edges_t m_bevel_edges;
 	detail::face_points_t m_face_points;
 	detail::faces_t m_new_faces;
+	detail::faces_t m_side_faces;
 
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, measurement_property, with_serialization) m_offset;
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, measurement_property, with_serialization) m_inset;
 	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_distance;
+	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_select_side_faces;
 };
 
 /////////////////////////////////////////////////////////////////////////////
