@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 2005, Romain Behar
+// Copyright (c) 2005-2006, Romain Behar
 //
 // Contact: romainbehar@yahoo.com
 //
@@ -33,6 +33,8 @@
 
 #include "helpers.h"
 
+#include <set>
+
 namespace libk3dmesh
 {
 
@@ -209,44 +211,120 @@ bool collapse_selected_edge(k3d::polyhedron& Polyhedron, point_map_t& PointMap)
 } // namespace detail
 
 /////////////////////////////////////////////////////////////////////////////
-// collapse_edges_implementation
+// collapse_edges
 
-class collapse_edges_implementation :
+class collapse_edges :
 	public k3d::mesh_selection_sink<k3d::mesh_modifier<k3d::persistent<k3d::node> > >
 {
 	typedef k3d::mesh_selection_sink<k3d::mesh_modifier<k3d::persistent<k3d::node> > > base;
 
 public:
-	collapse_edges_implementation(k3d::idocument& Document) :
+	collapse_edges(k3d::idocument& Document) :
 		base(Document),
 		m_vertex(init_owner(*this) + init_name("vertex") + init_label(_("Vertex")) + init_description(_("Collapse destination : first or second vertex, or middle")) + init_value(CENTER) + init_enumeration(operator_values()))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &collapse_edges_implementation::on_reset_geometry));
-		m_vertex.changed_signal().connect(sigc::mem_fun(*this, &collapse_edges_implementation::on_reset_geometry));
-		m_mesh_selection.changed_signal().connect(sigc::mem_fun(*this, &collapse_edges_implementation::on_reset_geometry));
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &collapse_edges_implementation::on_create_geometry));
+		m_vertex.changed_signal().connect(make_reset_mesh_slot());
+		m_mesh_selection.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		m_output_mesh.reset();
+		k3d::deep_copy(InputMesh, Mesh);
+		k3d::replace_selection(m_mesh_selection.value(), Mesh);
+
+		const vertex_t destination_vertex = m_vertex.value();
+
+		// Keep track of replaced points
+		detail::point_map_t point_map;
+
+		// Collapse edges
+		for(k3d::mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
+		{
+			bool found_edge = false;
+			do
+			{
+				found_edge = detail::collapse_selected_edge(**polyhedron, point_map);
+			}
+			while(found_edge);
+		}
+
+		// Update point positions
+		typedef std::vector<k3d::point*> point_list_t;
+		typedef std::map<k3d::point*, point_list_t> collapsed_points_t;
+		collapsed_points_t collapsed_points;
+		for(detail::point_map_t::iterator pair = point_map.begin(); pair != point_map.end(); ++pair)
+		{
+			// List original positions for each final point
+			collapsed_points_t::iterator list = collapsed_points.find(pair->second);
+			if(list == collapsed_points.end())
+			{
+				point_list_t first_position;
+				first_position.push_back(pair->first);
+
+				collapsed_points.insert(std::make_pair(pair->second, first_position));
+			}
+			else
+			{
+				list->second.push_back(pair->first);
+			}
+		}
+
+		for(collapsed_points_t::iterator pair = collapsed_points.begin(); pair != collapsed_points.end(); ++pair)
+		{
+			point_list_t& list = pair->second;
+			k3d::point* end = pair->first;
+
+			if(list.size() == 1)
+			{
+				k3d::point* start = list.front();
+
+				switch(destination_vertex)
+				{
+					case FIRST_VERTEX:
+						end->position = start->position;
+						break;
+
+					case LAST_VERTEX:
+						// End point position is already in end point
+						break;
+
+					case CENTER:
+						end->position += start->position;
+						end->position /= 2;
+						break;
+
+					default:
+						assert_not_reached();
+				}
+			}
+			else
+			{
+				// More than one point were collapsed to the destination, set centroid
+				k3d::point3 sum = end->position;
+				for(point_list_t::const_iterator vertex = list.begin(); vertex != list.end(); ++vertex)
+					sum += (*vertex)->position;
+
+				end->position = sum / static_cast<double>((list.size() + 1));
+			}
+		}
+
+		// Replace removed points in geometric primitives
+		k3d::for_each_component(Mesh, replace_removed_points(point_map));
+
+		// Delete removed points and select points resulting from collapsed edges
+		for(detail::point_map_t::iterator point = point_map.begin(); point != point_map.end(); ++point)
+		{
+			Mesh.points.erase(std::remove(Mesh.points.begin(), Mesh.points.end(), point->first), Mesh.points.end());
+
+			point->second->selection_weight = 1.0;
+		}
+
+		for(k3d::mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
+			assert_warning(k3d::is_valid(**polyhedron));
 	}
 
-	k3d::mesh* on_create_geometry()
+	void on_update_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		// Get the input geometry ...
-		k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
-
-		// Create output geometry ...
-		k3d::mesh* const output = new k3d::mesh();
-		k3d::deep_copy(*input, *output);
-		k3d::replace_selection(m_mesh_selection.value(), *output);
-
-		update_geometry(*output);
-
-		return output;
 	}
 
 	class replace_removed_points
@@ -344,100 +422,6 @@ public:
 		detail::point_map_t::iterator point;
 	};
 
-	void update_geometry(k3d::mesh& Mesh)
-	{
-		const vertex_t destination_vertex = m_vertex.value();
-
-		// Keep track of replaced points
-		detail::point_map_t point_map;
-
-		// Collapse edges
-		for(k3d::mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
-		{
-			bool found_edge = false;
-			do
-			{
-				found_edge = detail::collapse_selected_edge(**polyhedron, point_map);
-			}
-			while(found_edge);
-		}
-
-		// Update point positions
-		typedef std::vector<k3d::point*> point_list_t;
-		typedef std::map<k3d::point*, point_list_t> collapsed_points_t;
-		collapsed_points_t collapsed_points;
-		for(detail::point_map_t::iterator pair = point_map.begin(); pair != point_map.end(); ++pair)
-		{
-			// List original positions for each final point
-			collapsed_points_t::iterator list = collapsed_points.find(pair->second);
-			if(list == collapsed_points.end())
-			{
-				point_list_t first_position;
-				first_position.push_back(pair->first);
-
-				collapsed_points.insert(std::make_pair(pair->second, first_position));
-			}
-			else
-			{
-				list->second.push_back(pair->first);
-			}
-		}
-
-		for(collapsed_points_t::iterator pair = collapsed_points.begin(); pair != collapsed_points.end(); ++pair)
-		{
-			point_list_t& list = pair->second;
-			k3d::point* end = pair->first;
-
-			if(list.size() == 1)
-			{
-				k3d::point* start = list.front();
-
-				switch(destination_vertex)
-				{
-					case FIRST_VERTEX:
-						end->position = start->position;
-						break;
-
-					case LAST_VERTEX:
-						// End point position is already in end point
-						break;
-
-					case CENTER:
-						end->position += start->position;
-						end->position /= 2;
-						break;
-
-					default:
-						assert_not_reached();
-				}
-			}
-			else
-			{
-				// More than one point were collapsed to the destination, set centroid
-				k3d::vector3 sum = end->position;
-				for(point_list_t::const_iterator vertex = list.begin(); vertex != list.end(); ++vertex)
-					sum += (*vertex)->position;
-
-				end->position = sum / static_cast<double>((list.size() + 1));
-			}
-		}
-
-		// Replace removed points in geometric primitives
-		k3d::for_each_component(Mesh, replace_removed_points(point_map));
-
-		// Delete removed points and select points resulting from collapsed edges
-		for(detail::point_map_t::iterator point = point_map.begin(); point != point_map.end(); ++point)
-		{
-			Mesh.points.erase(std::remove(Mesh.points.begin(), Mesh.points.end(), point->first), Mesh.points.end());
-
-			point->second->selection_weight = 1.0;
-			point->second->visible_selection = false;
-		}
-
-		for(k3d::mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
-			assert_warning(k3d::is_valid(**polyhedron));
-	}
-
 	k3d::iplugin_factory& factory()
 	{
 		return get_factory();
@@ -446,7 +430,7 @@ public:
 	static k3d::iplugin_factory& get_factory()
 	{
 		static k3d::plugin_factory<
-			k3d::document_plugin<collapse_edges_implementation>,
+			k3d::document_plugin<collapse_edges>,
 				k3d::interface_list<k3d::imesh_source,
 				k3d::interface_list<k3d::imesh_sink > > > factory(
 					k3d::uuid(0xa5252071, 0xa3484315, 0x9c9daf1c, 0x786042a3),
@@ -517,42 +501,115 @@ private:
 };
 
 /////////////////////////////////////////////////////////////////////////////
-// collapse_faces_implementation
+// collapse_faces
 
-class collapse_faces_implementation :
+class collapse_faces :
 	public k3d::mesh_selection_sink<k3d::mesh_modifier<k3d::persistent<k3d::node> > >
 {
 	typedef k3d::mesh_selection_sink<k3d::mesh_modifier<k3d::persistent<k3d::node> > > base;
 
 public:
-	collapse_faces_implementation(k3d::idocument& Document) :
+	collapse_faces(k3d::idocument& Document) :
 		base(Document)
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &collapse_faces_implementation::on_reset_geometry));
-		m_mesh_selection.changed_signal().connect(sigc::mem_fun(*this, &collapse_faces_implementation::on_reset_geometry));
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &collapse_faces_implementation::on_create_geometry));
+		m_mesh_selection.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	/** \todo Improve the implementation so we don't have to do this */
+	k3d::iunknown* on_rewrite_hint(iunknown* const Hint)
 	{
-		m_output_mesh.reset();
+		// Force updates to re-allocate our mesh, for simplicity
+		return 0;
 	}
 
-	k3d::mesh* on_create_geometry()
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		// Get the input geometry ...
-		k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
+		k3d::deep_copy(InputMesh, Mesh);
+		k3d::replace_selection(m_mesh_selection.value(), Mesh);
 
-		// Create output geometry ...
-		k3d::mesh* const output = new k3d::mesh();
-		k3d::deep_copy(*input, *output);
-		k3d::replace_selection(m_mesh_selection.value(), *output);
+		// Change face selection to an edge selection, then apply CollapseEdges
+		for(k3d::mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
+		{
+			for(k3d::polyhedron::faces_t::iterator face = (*polyhedron)->faces.begin(); face != (*polyhedron)->faces.end(); ++face)
+			{
+				const double weight = (*face)->selection_weight ? 1.0 : 0.0;
 
-		update_geometry(*output);
+				k3d::split_edge* edge = (*face)->first_edge;
+				do
+				{
+					edge->selection_weight = weight;
 
-		return output;
+					edge = edge->face_clockwise;
+				}
+				while(edge != (*face)->first_edge);
+			}
+		}
+
+		// Keep track of replaced points
+		detail::point_map_t point_map;
+
+		// Collapse edges
+		for(k3d::mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
+		{
+			bool found_edge = false;
+			do
+			{
+				found_edge = detail::collapse_selected_edge(**polyhedron, point_map);
+			}
+			while(found_edge);
+		}
+
+		// Update point positions
+		typedef std::vector<k3d::point*> point_list_t;
+		typedef std::map<k3d::point*, point_list_t> collapsed_points_t;
+		collapsed_points_t collapsed_points;
+		for(detail::point_map_t::iterator pair = point_map.begin(); pair != point_map.end(); ++pair)
+		{
+			// List original positions for each final point
+			collapsed_points_t::iterator list = collapsed_points.find(pair->second);
+			if(list == collapsed_points.end())
+			{
+				point_list_t first_position;
+				first_position.push_back(pair->first);
+
+				collapsed_points.insert(std::make_pair(pair->second, first_position));
+			}
+			else
+			{
+				list->second.push_back(pair->first);
+			}
+		}
+
+		for(collapsed_points_t::iterator pair = collapsed_points.begin(); pair != collapsed_points.end(); ++pair)
+		{
+			point_list_t& list = pair->second;
+			k3d::point* end = pair->first;
+
+			// More than one point were collapsed to the destination, set centroid
+			k3d::point3 sum = end->position;
+			for(point_list_t::const_iterator vertex = list.begin(); vertex != list.end(); ++vertex)
+				sum += (*vertex)->position;
+
+			end->position = sum / static_cast<double>((list.size() + 1));
+		}
+
+		// Replace removed points in geometric primitives
+		k3d::for_each_component(Mesh, replace_removed_points(point_map));
+
+		// Delete removed points and select points resulting from collapsed edges
+		for(detail::point_map_t::iterator point = point_map.begin(); point != point_map.end(); ++point)
+		{
+			Mesh.points.erase(std::remove(Mesh.points.begin(), Mesh.points.end(), point->first), Mesh.points.end());
+
+			point->second->selection_weight = 1.0;
+		}
+
+		for(k3d::mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
+			assert_warning(k3d::is_valid(**polyhedron));
+	}
+
+	void on_update_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
+	{
 	}
 
 	class replace_removed_points
@@ -650,90 +707,6 @@ public:
 		detail::point_map_t::iterator point;
 	};
 
-	void update_geometry(k3d::mesh& Mesh)
-	{
-		// Change face selection to an edge selection, then apply CollapseEdges
-		for(k3d::mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
-		{
-			for(k3d::polyhedron::faces_t::iterator face = (*polyhedron)->faces.begin(); face != (*polyhedron)->faces.end(); ++face)
-			{
-				const double weight = (*face)->selection_weight ? 1.0 : 0.0;
-
-				k3d::split_edge* edge = (*face)->first_edge;
-				do
-				{
-					edge->selection_weight = weight;
-
-					edge = edge->face_clockwise;
-				}
-				while(edge != (*face)->first_edge);
-			}
-		}
-
-		// Keep track of replaced points
-		detail::point_map_t point_map;
-
-		// Collapse edges
-		for(k3d::mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
-		{
-			bool found_edge = false;
-			do
-			{
-				found_edge = detail::collapse_selected_edge(**polyhedron, point_map);
-			}
-			while(found_edge);
-		}
-
-		// Update point positions
-		typedef std::vector<k3d::point*> point_list_t;
-		typedef std::map<k3d::point*, point_list_t> collapsed_points_t;
-		collapsed_points_t collapsed_points;
-		for(detail::point_map_t::iterator pair = point_map.begin(); pair != point_map.end(); ++pair)
-		{
-			// List original positions for each final point
-			collapsed_points_t::iterator list = collapsed_points.find(pair->second);
-			if(list == collapsed_points.end())
-			{
-				point_list_t first_position;
-				first_position.push_back(pair->first);
-
-				collapsed_points.insert(std::make_pair(pair->second, first_position));
-			}
-			else
-			{
-				list->second.push_back(pair->first);
-			}
-		}
-
-		for(collapsed_points_t::iterator pair = collapsed_points.begin(); pair != collapsed_points.end(); ++pair)
-		{
-			point_list_t& list = pair->second;
-			k3d::point* end = pair->first;
-
-			// More than one point were collapsed to the destination, set centroid
-			k3d::vector3 sum = end->position;
-			for(point_list_t::const_iterator vertex = list.begin(); vertex != list.end(); ++vertex)
-				sum += (*vertex)->position;
-
-			end->position = sum / static_cast<double>((list.size() + 1));
-		}
-
-		// Replace removed points in geometric primitives
-		k3d::for_each_component(Mesh, replace_removed_points(point_map));
-
-		// Delete removed points and select points resulting from collapsed edges
-		for(detail::point_map_t::iterator point = point_map.begin(); point != point_map.end(); ++point)
-		{
-			Mesh.points.erase(std::remove(Mesh.points.begin(), Mesh.points.end(), point->first), Mesh.points.end());
-
-			point->second->selection_weight = 1.0;
-			point->second->visible_selection = false;
-		}
-
-		for(k3d::mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
-			assert_warning(k3d::is_valid(**polyhedron));
-	}
-
 	k3d::iplugin_factory& factory()
 	{
 		return get_factory();
@@ -742,7 +715,7 @@ public:
 	static k3d::iplugin_factory& get_factory()
 	{
 		static k3d::plugin_factory<
-			k3d::document_plugin<collapse_faces_implementation>,
+			k3d::document_plugin<collapse_faces>,
 			k3d::interface_list<k3d::imesh_source,
 			k3d::interface_list<k3d::imesh_sink > > > factory(
 				k3d::uuid(0x13f8a224, 0x5e284ac4, 0xb5b58ebd, 0x10f65bb6),
@@ -760,12 +733,12 @@ public:
 
 k3d::iplugin_factory& collapse_edges_factory()
 {
-	return collapse_edges_implementation::get_factory();
+	return collapse_edges::get_factory();
 }
 
 k3d::iplugin_factory& collapse_faces_factory()
 {
-	return collapse_faces_implementation::get_factory();
+	return collapse_faces::get_factory();
 }
 
 } // namespace libk3dmesh
