@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -35,6 +35,8 @@
 
 #include "helpers.h"
 
+#include <set>
+
 namespace libk3dmesh
 {
 
@@ -45,9 +47,9 @@ namespace detail
 class new_point
 {
 public:
-	new_point(const k3d::vector3& OldPosition, const k3d::vector3& Direction, const k3d::vector3& InsetDirection, const double Ratio, k3d::point& Point) :
+	new_point(const k3d::point3& OldPosition, const k3d::normal3& Direction, const k3d::vector3& InsetDirection, const double Ratio, k3d::point& Point) :
 		old_position(OldPosition),
-		direction(Direction),
+		direction(k3d::to_vector(Direction)),
 		inset_direction(InsetDirection),
 		ratio(Ratio),
 		point(&Point)
@@ -60,7 +62,7 @@ public:
 	}
 
 private:
-	k3d::vector3 old_position;
+	k3d::point3 old_position;
 	k3d::vector3 direction;
 	k3d::vector3 inset_direction;
 	double ratio;
@@ -105,11 +107,11 @@ struct indexed_face_t
 };
 typedef std::map<k3d::split_edge*, indexed_face_t> edge_face_map_t;
 typedef std::vector<k3d::split_edge*> face_t;
-typedef std::map<k3d::point*, k3d::vector3> vertex_normals_t;
-void extrude_face(face_t& Face, k3d::face::holes_t& FaceHoles, k3d::imaterial* Material, const unsigned long Segments, k3d::vector3& Direction, vertex_normals_t& VertexNormals, const bool Inside, const bool BackFace, edge_face_map_t& InteriorEdges, k3d::mesh& Mesh, k3d::polyhedron& Polyhedron, new_points_t& NewPoints, k3d::polyhedron::faces_t& BackFaces, k3d::polyhedron::faces_t& SideFaces)
+typedef std::map<k3d::point*, k3d::normal3> vertex_normals_t;
+void extrude_face(face_t& Face, k3d::face::holes_t& FaceHoles, k3d::imaterial* Material, const unsigned long Segments, k3d::normal3& Direction, vertex_normals_t& VertexNormals, const bool Inside, const bool BackFace, edge_face_map_t& InteriorEdges, k3d::mesh& Mesh, k3d::polyhedron& Polyhedron, new_points_t& NewPoints, k3d::polyhedron::faces_t& BackFaces, k3d::polyhedron::faces_t& SideFaces)
 {
 	// Do nothing on zero-length direction
-	if(!Direction.Length())
+	if(!Direction.length2())
 		return;
 
 	// Create a new point for each original, save old and new points
@@ -119,7 +121,7 @@ void extrude_face(face_t& Face, k3d::face::holes_t& FaceHoles, k3d::imaterial* M
 	typedef std::vector<k3d::point*> points_t;
 	points_t old_points;
 	points_t new_points;
-	std::vector<k3d::vector3> point_normals;
+	std::vector<k3d::normal3> point_normals;
 
 	// Map extruded edges to their companions
 	typedef std::map<k3d::split_edge*, k3d::split_edge*> edge_map_t;
@@ -154,20 +156,20 @@ void extrude_face(face_t& Face, k3d::face::holes_t& FaceHoles, k3d::imaterial* M
 	points_t& source_points = Inside ? old_points : new_points;
 	points_t& target_points = Inside ? new_points : old_points;
 
+	typedef std::vector<k3d::normal3> normals_t;
+	normals_t normals;
 	typedef std::vector<k3d::vector3> vectors_t;
-	vectors_t normals;
 	vectors_t insets;
 
 	const unsigned long edge_count = old_points.size();
 	for(unsigned long i = 0; i != edge_count; ++i)
 	{
-		const k3d::vector3 e1 = source_points[i]->position - source_points[(i+edge_count-1)%edge_count]->position;
-		const k3d::vector3 e2 = source_points[(i+1)%edge_count]->position - source_points[i]->position;
-
+		const k3d::vector3 e1 = k3d::to_vector(source_points[i]->position - source_points[(i+edge_count-1)%edge_count]->position);
+		const k3d::vector3 e2 = k3d::to_vector(source_points[(i+1)%edge_count]->position - source_points[i]->position);
 		const k3d::vector3 e3 = k3d::normalize(e1 ^ Direction);
 		const k3d::vector3 e4 = k3d::normalize(e2 ^ Direction);
 
-		k3d::vector3 point_normal = point_normals.size() ? point_normals[i] : Direction;
+		k3d::normal3 point_normal = point_normals.size() ? point_normals[i] : Direction;
 		k3d::vector3 inset_direction = (-1 / cos(acos(e3 * e4) / 2)) * k3d::normalize(e3 + e4);
 
 		if(!Inside)
@@ -217,7 +219,6 @@ void extrude_face(face_t& Face, k3d::face::holes_t& FaceHoles, k3d::imaterial* M
 		// Make sure new face doesn't start at side1 or side2 for segments creation (see below)
 		k3d::face* const new_face = new k3d::face(start, Material);
 		new_face->selection_weight = 1.0;
-		new_face->visible_selection = true;
 
 		Polyhedron.faces.push_back(new_face);
 		SideFaces.push_back(new_face);
@@ -288,13 +289,13 @@ void extrude_face(face_t& Face, k3d::face::holes_t& FaceHoles, k3d::imaterial* M
 			// Create and save back face
 			k3d::face* const new_face = new k3d::face(back_edges.front(), Material);
 			new_face->selection_weight = true;
-			new_face->visible_selection = true;
 
 			Polyhedron.faces.push_back(new_face);
 			BackFaces.push_back(new_face);
 
 			// Duplicate holes
-			const k3d::vector3 null_normal(0, 0, 0);
+			const k3d::vector3 null_vector(0, 0, 0);
+			const k3d::normal3 null_normal(0, 0, 0);
 			for(k3d::face::holes_t::iterator hole = FaceHoles.begin(); hole != FaceHoles.end(); ++hole)
 			{
 				// Copy original hole's edges
@@ -315,7 +316,7 @@ void extrude_face(face_t& Face, k3d::face::holes_t& FaceHoles, k3d::imaterial* M
 
 					k3d::point* new_vertex = new k3d::point(vertex->position);
 					Mesh.points.push_back(new_vertex);
-					NewPoints.push_back(new_point(vertex->position, null_normal, null_normal, 1.0, *new_vertex));
+					NewPoints.push_back(new_point(vertex->position, null_normal, null_vector, 1.0, *new_vertex));
 
 					k3d::split_edge* new_edge = new k3d::split_edge(new_vertex);
 					new_edges.push_back(new_edge);
@@ -376,7 +377,6 @@ void extrude_face(face_t& Face, k3d::face::holes_t& FaceHoles, k3d::imaterial* M
 
 					k3d::face* hole_face = new k3d::face(hole_edge1, Material);
 					hole_face->selection_weight = 1.0;
-					hole_face->visible_selection = true;
 
 					Polyhedron.faces.push_back(hole_face);
 					SideFaces.push_back(hole_face);
@@ -391,7 +391,7 @@ void extrude_face(face_t& Face, k3d::face::holes_t& FaceHoles, k3d::imaterial* M
 	if(Segments > 1)
 		{
 			const double segment_number = static_cast<double>(Segments);
-			k3d::vector3 null_direction(0, 0, 0);
+			k3d::point3 null_direction(0, 0, 0);
 
 			// Save all side1 edges from extrusion faces
 			edges_t side1_edges;
@@ -423,7 +423,6 @@ void extrude_face(face_t& Face, k3d::face::holes_t& FaceHoles, k3d::imaterial* M
 					k3d::split_edge* edge2 = edge1->face_clockwise->face_clockwise->face_clockwise;
 					k3d::face* new_face = segment_connect_vertices(Material, edge1, edge2);
 					new_face->selection_weight = 1.0;
-					new_face->visible_selection = true;
 
 					Polyhedron.faces.push_back(new_face);
 					SideFaces.push_back(new_face);
@@ -579,7 +578,7 @@ void extrude_regions(k3d::polyhedron::faces_t& Faces, const unsigned long Segmen
 		}
 
 		// Compute either average region normal or vertex normals, collect holes and get a material
-		k3d::vector3 region_normal(0, 0, 0);
+		k3d::normal3 region_normal(0, 0, 0);
 		vertex_normals_t vertex_normals;
 		k3d::face::holes_t region_holes;
 		k3d::imaterial* material = 0;
@@ -588,8 +587,7 @@ void extrude_regions(k3d::polyhedron::faces_t& Faces, const unsigned long Segmen
 			region_holes.insert(region_holes.end(), (*face)->holes.begin(), (*face)->holes.end());
 			material = (*face)->material;
 
-			k3d::vector3 face_normal = k3d::normal(**face);
-			face_normal.Normalize();
+			k3d::normal3 face_normal = k3d::normalize(k3d::normal(**face));
 			region_normal += face_normal;
 
 			if(!GroupNormals)
@@ -619,7 +617,7 @@ void extrude_regions(k3d::polyhedron::faces_t& Faces, const unsigned long Segmen
 		// Average region normal and normalize vertex normals
 		region_normal /= static_cast<double>(region_faces.size());
 		for(vertex_normals_t::iterator normal = vertex_normals.begin(); normal != vertex_normals.end(); ++normal)
-			normal->second.Normalize();
+			normal->second = k3d::normalize(normal->second);
 
 		// Extrude each independant region
 		return_if_fail(border_edges.size());
@@ -734,8 +732,8 @@ struct extrude_faces
 	void operator()(k3d::face* Face)
 	{
 		// Calculate face normal
-		k3d::vector3 face_normal = k3d::normal(*Face);
-		if(0.0 == face_normal.Length2())
+		k3d::normal3 face_normal = k3d::normal(*Face);
+		if(0.0 == face_normal.length2())
 			return;
 
 		face_t face_edges;
@@ -783,15 +781,15 @@ struct extrude_faces
 } // namespace detail
 
 /////////////////////////////////////////////////////////////////////////////
-// extrude_faces_implementation
+// extrude_faces
 
-class extrude_faces_implementation :
+class extrude_faces :
 	public k3d::mesh_selection_sink<k3d::mesh_modifier<k3d::persistent<k3d::node> > >
 {
 	typedef k3d::mesh_selection_sink<k3d::mesh_modifier<k3d::persistent<k3d::node> > > base;
 
 public:
-	extrude_faces_implementation(k3d::idocument& Document) :
+	extrude_faces(k3d::idocument& Document) :
 		base(Document),
 		m_distance(init_owner(*this) + init_name("distance") + init_label(_("Distance")) + init_description(_("Distance between original and new faces")) + init_value(1.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_segments(init_owner(*this) + init_name("segments") + init_label(_("Segments")) + init_description(_("Segment number between original and new faces")) + init_value(1) + init_constraint(constraint::minimum(1L)) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
@@ -802,58 +800,29 @@ public:
 		m_back_face(init_owner(*this) + init_name("back_face") + init_label(_("Back face")) + init_description(_("Add back face when possible")) + init_value(true)),
 		m_select_side_faces(init_owner(*this) + init_name("select_side_faces") + init_label(_("Select side faces")) + init_description(_("Select side faces on output")) + init_value(false))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &extrude_faces_implementation::on_reset_geometry));
-		m_mesh_selection.changed_signal().connect(sigc::mem_fun(*this, &extrude_faces_implementation::on_reset_geometry));
+		m_mesh_selection.changed_signal().connect(make_reset_mesh_slot());
 
-		m_distance.changed_signal().connect(sigc::mem_fun(*this, &extrude_faces_implementation::on_reshape_geometry));
-		m_segments.changed_signal().connect(sigc::mem_fun(*this, &extrude_faces_implementation::on_reset_geometry));
-		m_inset.changed_signal().connect(sigc::mem_fun(*this, &extrude_faces_implementation::on_reshape_geometry));
-		m_direction.changed_signal().connect(sigc::mem_fun(*this, &extrude_faces_implementation::on_reset_geometry));
-		m_region.changed_signal().connect(sigc::mem_fun(*this, &extrude_faces_implementation::on_reset_geometry));
-		m_group_normals.changed_signal().connect(sigc::mem_fun(*this, &extrude_faces_implementation::on_reset_geometry));
-		m_back_face.changed_signal().connect(sigc::mem_fun(*this, &extrude_faces_implementation::on_reset_geometry));
-		m_select_side_faces.changed_signal().connect(sigc::mem_fun(*this, &extrude_faces_implementation::on_visible_selection));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &extrude_faces_implementation::on_create_geometry));
+		m_distance.changed_signal().connect(make_update_mesh_slot());
+		m_segments.changed_signal().connect(make_reset_mesh_slot());
+		m_inset.changed_signal().connect(make_update_mesh_slot());
+		m_direction.changed_signal().connect(make_reset_mesh_slot());
+		m_region.changed_signal().connect(make_reset_mesh_slot());
+		m_group_normals.changed_signal().connect(make_reset_mesh_slot());
+		m_back_face.changed_signal().connect(make_reset_mesh_slot());
+		m_select_side_faces.changed_signal().connect(make_update_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	/** \todo Improve the implementation so we don't have to do this */
+	k3d::iunknown* on_rewrite_hint(iunknown* const Hint)
 	{
-		m_new_points.clear();
-		m_output_mesh.reset();
+		// Force updates to re-allocate our mesh, for simplicity
+		return 0;
 	}
 
-	void on_reshape_geometry()
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		if(m_output_mesh.empty())
-			return;
-
-		reshape_geometry();
-
-		m_output_mesh.changed_signal().emit();
-	}
-
-	void on_visible_selection()
-	{
-		if(m_output_mesh.empty())
-			return;
-
-		visible_selection();
-
-		m_output_mesh.changed_signal().emit();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		// Get the input geometry ...
-		k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
-
-		// Create output geometry ...
-		k3d::mesh* const output = new k3d::mesh();
-		k3d::deep_copy(*input, *output);
-		k3d::replace_selection(m_mesh_selection.value(), *output);
+		k3d::deep_copy(InputMesh, Mesh);
+		k3d::replace_selection(m_mesh_selection.value(), Mesh);
 
 		// Clear data caches
 		m_new_points.clear();
@@ -868,7 +837,7 @@ public:
 
 		// For each polyhedron ...
 		k3d::polyhedron::faces_t back_faces;
-		for(k3d::mesh::polyhedra_t::iterator polyhedron = output->polyhedra.begin(); polyhedron != output->polyhedra.end(); ++polyhedron)
+		for(k3d::mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
 		{
 			// Get the set of all selected faces ...
 			k3d::copy_if((*polyhedron)->faces.begin(), (*polyhedron)->faces.end(), std::inserter(m_selected_faces, m_selected_faces.end()), k3d::is_selected());
@@ -879,50 +848,34 @@ public:
 			if(region)
 			{
 				// Extrude neighbouring faces as a single one
-				detail::extrude_regions(m_selected_faces, segments, (direction == INSIDE), group_normals, back_face, *output, **polyhedron, m_new_points, back_faces, m_side_faces);
+				detail::extrude_regions(m_selected_faces, segments, (direction == INSIDE), group_normals, back_face, Mesh, **polyhedron, m_new_points, back_faces, m_side_faces);
 			}
 			else
 			{
 				// Extrude each face
-				std::for_each(m_selected_faces.begin(), m_selected_faces.end(), detail::extrude_faces(segments, (direction == INSIDE), back_face, *output, **polyhedron, m_new_points, back_faces, m_side_faces));
+				std::for_each(m_selected_faces.begin(), m_selected_faces.end(), detail::extrude_faces(segments, (direction == INSIDE), back_face, Mesh, **polyhedron, m_new_points, back_faces, m_side_faces));
 			}
 
 			assert_warning(is_valid(**polyhedron));
 		}
 
 		m_selected_faces.insert(m_selected_faces.end(), back_faces.begin(), back_faces.end());
-		std::for_each(m_selected_faces.begin(), m_selected_faces.end(), k3d::selection::set(true, 1.0));
-
-		// Update new items selection
-		visible_selection();
-
-		// Position all the new geometry we've created ...
-		reshape_geometry();
-
-		return output;
+		std::for_each(m_selected_faces.begin(), m_selected_faces.end(), k3d::selection::set_weight(1.0));
 	}
 
-	void reshape_geometry()
+	void on_update_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
 		const double distance = m_distance.value();
 		const double inset = m_inset.value();
 
 		for(detail::new_points_t::iterator new_point = m_new_points.begin(); new_point != m_new_points.end(); ++new_point)
 			new_point->update(distance, inset);
-	}
-
-	void visible_selection()
-	{
-		for(k3d::polyhedron::faces_t::iterator face = m_selected_faces.begin(); face != m_selected_faces.end(); ++face)
-			(*face)->visible_selection = true;
 
+		// Update new items selection
 		const bool select_side_faces = m_select_side_faces.value();
 		const double side_weight = select_side_faces ? 1.0 : 0.0;
 		for(k3d::polyhedron::faces_t::iterator face = m_side_faces.begin(); face != m_side_faces.end(); ++face)
-		{
 			(*face)->selection_weight = side_weight;
-			(*face)->visible_selection = select_side_faces;
-		}
 	}
 
 	k3d::iplugin_factory& factory()
@@ -933,7 +886,7 @@ public:
 	static k3d::iplugin_factory& get_factory()
 	{
 		static k3d::plugin_factory<
-			k3d::document_plugin<extrude_faces_implementation>,
+			k3d::document_plugin<extrude_faces>,
 			k3d::interface_list<k3d::imesh_source,
 			k3d::interface_list<k3d::imesh_sink > > > factory(
 				k3d::uuid(0xdbd23c4e, 0x7119478c, 0x9ab2ee83, 0x5b56d527),
@@ -1014,7 +967,7 @@ private:
 
 k3d::iplugin_factory& extrude_faces_factory()
 {
-	return extrude_faces_implementation::get_factory();
+	return extrude_faces::get_factory();
 }
 
 } // namespace libk3dmesh
