@@ -47,44 +47,29 @@ public:
 		m_normalize(init_owner(*this) + init_name("normalize") + init_label(_("Normalize")) + init_description(_("Normalize normals")) + init_value(false)),
 		m_scale(init_owner(*this) + init_name("normals_scale") + init_label(_("Normals scale")) + init_description(_("Scale normals")) + init_value(1.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &face_normals_implementation::on_reset_geometry));
-
-		m_normalize.changed_signal().connect(sigc::mem_fun(*this, &face_normals_implementation::on_reset_geometry));
-		m_scale.changed_signal().connect(sigc::mem_fun(*this, &face_normals_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &face_normals_implementation::on_create_geometry));
+		m_normalize.changed_signal().connect(make_reset_mesh_slot());
+		m_scale.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	/** \todo Improve the implementation so we don't have to do this */
+	k3d::iunknown* on_rewrite_hint(iunknown* const Hint)
 	{
-		m_output_mesh.reset();
+		// Force updates to re-allocate our mesh, for simplicity
+		return 0;
 	}
 
-	k3d::mesh* on_create_geometry()
-	{
-		// If we don't have any input mesh, we're done ...
-		const k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
-
-		// Otherwise, we convert input mesh polyhedra to blobbies ...
-		k3d::mesh* const output = new k3d::mesh();
-		update_geometry(*input, *output);
-
-		return output;
-	}
-
-	void update_geometry(const k3d::mesh& input, k3d::mesh& output)
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
 		const bool normalize = m_normalize.value();
 		const double scale = m_scale.value();
 
 		// Create output geometry
 		k3d::linear_curve_group* const group = new k3d::linear_curve_group();
-		output.linear_curve_groups.push_back(group);
+		Mesh.linear_curve_groups.push_back(group);
 
 		// Build face normals
-		for(k3d::mesh::polyhedra_t::const_iterator polyhedron = input.polyhedra.begin(); polyhedron != input.polyhedra.end(); ++polyhedron)
+		for(k3d::mesh::polyhedra_t::const_iterator polyhedron = InputMesh.polyhedra.begin(); polyhedron != InputMesh.polyhedra.end(); ++polyhedron)
+		{
 			for(k3d::polyhedron::faces_t::const_iterator face = (*polyhedron)->faces.begin(); face != (*polyhedron)->faces.end(); ++face)
 			{
 				k3d::split_edge* first = (*face)->first_edge;
@@ -92,7 +77,7 @@ public:
 					// Skip empty faces
 					continue;
 
-				k3d::vector3 v_sum = first->vertex->position;
+				k3d::point3 v_sum = first->vertex->position;
 				unsigned long n_vertices = 1;
 
 				k3d::split_edge* current_edge = first->face_clockwise;
@@ -108,28 +93,33 @@ public:
 
 				// Get barycentric point and face normal
 				v_sum /= static_cast<double>(n_vertices);
-				k3d::vector3 normal = k3d::normal((*face)->first_edge);
+				k3d::normal3 normal = k3d::normal((*face)->first_edge);
 				if(normalize)
-					normal.Normalize();
+					normal = k3d::normalize(normal);
 				normal *= scale;
 
 				// Create normal in output geometry
 				k3d::linear_curve* const curve = new k3d::linear_curve();
 
-				const k3d::vector3 base = v_sum;
-				const k3d::vector3 tip = v_sum + normal;
+				const k3d::point3 base = v_sum;
+				const k3d::point3 tip = v_sum + k3d::to_vector(normal);
 
 				k3d::point* const base_p = new k3d::point(base);
 				k3d::point* const tip_p = new k3d::point(tip);
 
-				output.points.push_back(base_p);
-				output.points.push_back(tip_p);
+				Mesh.points.push_back(base_p);
+				Mesh.points.push_back(tip_p);
 
 				curve->control_points.push_back(base_p);
 				curve->control_points.push_back(tip_p);
 
 				group->curves.push_back(curve);
 			}
+		}
+	}
+
+	void on_update_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
