@@ -63,30 +63,17 @@ public:
 		base(Document),
 		m_color_number(init_owner(*this) + init_name("color_number") + init_label(_("Color number")) + init_description(_("Color number in the virtual color list")) + init_value(1) + init_precision(0) + init_step_increment(1) + init_constraint(constraint::minimum(1L)) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &filter_faces_by_color_implementation::on_reset_geometry));
-		m_color_number.changed_signal().connect(sigc::mem_fun(*this, &filter_faces_by_color_implementation::on_reset_geometry));
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &filter_faces_by_color_implementation::on_create_geometry));
+		m_color_number.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	/** \todo Improve the implementation so we don't have to do this */
+	k3d::iunknown* on_rewrite_hint(iunknown* const Hint)
 	{
-		m_output_mesh.reset();
+		// Force updates to re-allocate our mesh, for simplicity
+		return 0;
 	}
 
-	k3d::mesh* on_create_geometry()
-	{
-		// If we don't have any input mesh, we're done ...
-		const k3d::mesh* const input = m_input_mesh.value();
-		return_val_if_fail(input, 0);
-
-		// Otherwise, we convert input mesh polyhedra to blobbies ...
-		k3d::mesh* const output = new k3d::mesh();
-		update_geometry(*input, *output);
-
-		return output;
-	}
-
-	void update_geometry(const k3d::mesh& Input, k3d::mesh& Output)
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
 		// Find Nth color
 		const unsigned long color_number = m_color_number.value();
@@ -95,35 +82,35 @@ public:
 		bool found = false;
 		k3d::color selected_color;
 
-		for(k3d::mesh::polyhedra_t::const_iterator polyhedron = Input.polyhedra.begin(); polyhedron != Input.polyhedra.end(); ++polyhedron)
+		for(k3d::mesh::polyhedra_t::const_iterator polyhedron = InputMesh.polyhedra.begin(); polyhedron != InputMesh.polyhedra.end(); ++polyhedron)
+		{
+			for(k3d::polyhedron::faces_t::const_iterator face = (*polyhedron)->faces.begin(); face != (*polyhedron)->faces.end(); ++face)
 			{
-				for(k3d::polyhedron::faces_t::const_iterator face = (*polyhedron)->faces.begin(); face != (*polyhedron)->faces.end(); ++face)
+				k3d::parameters_t::const_iterator color = (*face)->uniform_data.find("Cs");
+				if(color != (*face)->uniform_data.end())
+				{
+					k3d::color face_color = boost::any_cast<k3d::color>(color->second);
+					if(color_map.find(face_color) != color_map.end())
+						// Color was already indexed
+						continue;
+					else
 					{
-						k3d::parameters_t::const_iterator color = (*face)->uniform_data.find("Cs");
-						if(color != (*face)->uniform_data.end())
-							{
-								k3d::color face_color = boost::any_cast<k3d::color>(color->second);
-								if(color_map.find(face_color) != color_map.end())
-									// Color was already indexed
-									continue;
-								else
-									{
-										// Save color
-										color_map[face_color] = current_color++;
-										if(current_color == color_number)
-											{
-												found = true;
-												selected_color = face_color;
-												break;
-											}
-									}
-							}
+						// Save color
+						color_map[face_color] = current_color++;
+						if(current_color == color_number)
+						{
+							found = true;
+							selected_color = face_color;
+							break;
+						}
 					}
-
-				if(found)
-					break;
+				}
 			}
 
+			if(found)
+				break;
+		}
+
 		// If specified color wasn't found, return
 		if(!found)
 			return;
@@ -131,56 +118,60 @@ public:
 		// Copy faces of the same color
 		k3d::polyhedron* output_polyhedron = new k3d::polyhedron();
 		return_if_fail(output_polyhedron);
-		Output.polyhedra.push_back(output_polyhedron);
+		Mesh.polyhedra.push_back(output_polyhedron);
 
 		typedef std::map<k3d::point*, k3d::point*> point_map_t;
 		point_map_t point_map;
-		for(k3d::mesh::polyhedra_t::const_iterator polyhedron = Input.polyhedra.begin(); polyhedron != Input.polyhedra.end(); ++polyhedron)
+		for(k3d::mesh::polyhedra_t::const_iterator polyhedron = InputMesh.polyhedra.begin(); polyhedron != InputMesh.polyhedra.end(); ++polyhedron)
+		{
+			for(k3d::polyhedron::faces_t::const_iterator face = (*polyhedron)->faces.begin(); face != (*polyhedron)->faces.end(); ++face)
 			{
-				for(k3d::polyhedron::faces_t::const_iterator face = (*polyhedron)->faces.begin(); face != (*polyhedron)->faces.end(); ++face)
+				k3d::parameters_t::const_iterator color = (*face)->uniform_data.find("Cs");
+				if(color != (*face)->uniform_data.end())
+				{
+					k3d::color face_color = boost::any_cast<k3d::color>(color->second);
+					if(face_color != selected_color)
+						continue;
+
+					// Copy face ...
+					k3d::split_edge* first_edge = (*face)->first_edge;
+					k3d::split_edge* current_edge = first_edge;
+
+					std::vector<k3d::split_edge*> edges;
+					do
 					{
-						k3d::parameters_t::const_iterator color = (*face)->uniform_data.find("Cs");
-						if(color != (*face)->uniform_data.end())
-							{
-								k3d::color face_color = boost::any_cast<k3d::color>(color->second);
-								if(face_color != selected_color)
-									continue;
+						k3d::point* point = current_edge->vertex;
+						k3d::point* new_point;
+						point_map_t::const_iterator point_equivalence = point_map.find(point);
+						if(point_equivalence != point_map.end())
+						{
+							new_point = point_equivalence->second;
+						}
+						else
+						{
+							new_point = new k3d::point(point->position);
+							Mesh.points.push_back(new_point);
+							return_if_fail(new_point);
+						}
 
-								// Copy face ...
-								k3d::split_edge* first_edge = (*face)->first_edge;
-								k3d::split_edge* current_edge = first_edge;
-
-								std::vector<k3d::split_edge*> edges;
-								do
-									{
-										k3d::point* point = current_edge->vertex;
-										k3d::point* new_point;
-										point_map_t::const_iterator point_equivalence = point_map.find(point);
-										if(point_equivalence != point_map.end())
-											{
-												new_point = point_equivalence->second;
-											}
-										else
-											{
-												new_point = new k3d::point(point->position);
-												Output.points.push_back(new_point);
-												return_if_fail(new_point);
-											}
-
-										edges.push_back(new k3d::split_edge(new_point));
-										current_edge = current_edge->face_clockwise;
-									}
-								while(current_edge != first_edge);
-
-								k3d::loop_edges(edges.begin(), edges.end());
-
-								k3d::face* const new_face = new k3d::face(**face);
-								new_face->first_edge = *edges.begin();
-
-								output_polyhedron->faces.push_back(new_face);
-							}
+						edges.push_back(new k3d::split_edge(new_point));
+						current_edge = current_edge->face_clockwise;
 					}
+					while(current_edge != first_edge);
+
+					k3d::loop_edges(edges.begin(), edges.end());
+
+					k3d::face* const new_face = new k3d::face(**face);
+					new_face->first_edge = *edges.begin();
+
+					output_polyhedron->faces.push_back(new_face);
+				}
 			}
+		}
+	}
+
+	void on_update_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
