@@ -28,6 +28,8 @@
 #include <k3dsdk/module.h>
 #include <k3dsdk/utility.h>
 
+#include <set>
+
 namespace libk3dmesh
 {
 
@@ -43,31 +45,172 @@ public:
 	join_points_implementation(k3d::idocument& Document) :
 		base(Document)
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &join_points_implementation::on_reset_geometry));
-		m_mesh_selection.changed_signal().connect(sigc::mem_fun(*this, &join_points_implementation::on_reset_geometry));
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &join_points_implementation::on_create_geometry));
+		m_mesh_selection.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	/** \todo Improve the implementation so we don't have to do this */
+	k3d::iunknown* on_rewrite_hint(iunknown* const Hint)
 	{
-		m_output_mesh.reset();
+		// Force updates to re-allocate our mesh, for simplicity
+		return 0;
 	}
 
-	k3d::mesh* on_create_geometry()
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		// Get the input geometry ...
-		k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
+		k3d::deep_copy(InputMesh, Mesh);
+		k3d::replace_selection(m_mesh_selection.value(), Mesh);
 
-		// Create output geometry ...
-		k3d::mesh* const output = new k3d::mesh();
-		k3d::deep_copy(*input, *output);
-		k3d::replace_selection(m_mesh_selection.value(), *output);
+		// We keep track of the geometry we're going to delete ...
+		std::set<k3d::point*> points;
+		std::set<k3d::face*> faces;
+		std::set<k3d::split_edge*> edges;
 
-		update_geometry(*output);
+		// Get selected points
+		k3d::point* point = 0;
+		// sum of opsitions
+		k3d::point3 total_position(0.0,0.0,0.0);
+		for(k3d::mesh::points_t::iterator p = Mesh.points.begin(); p != Mesh.points.end(); ++p)
+		{
+			if((*p)->selection_weight)
+			{
+				total_position += (*p)->position;
+				if(!point)
+					point = *p;
+				else
+					points.insert(*p);
+			}
+		}
 
-		return output;
+		// abort if nothing was selected
+		if (points.empty())
+			return;
+
+		point->position = total_position / points.size();
+
+		// Delete selected faces, taking their edges along with them ...
+		for(k3d::mesh::polyhedra_t::iterator p = Mesh.polyhedra.begin(); p != Mesh.polyhedra.end(); ++p)
+		{
+			k3d::polyhedron& polyhedron = **p;
+
+			for(k3d::polyhedron::faces_t::iterator f = polyhedron.faces.begin(); f != polyhedron.faces.end(); ++f)
+			{
+				k3d::face* face = *f;
+				face->selection_weight = 0.0;
+				for(k3d::split_edge* edge = face->first_edge; edge; edge = edge->face_clockwise)
+				{
+					edge->selection_weight = 0.0;
+					if(edge->vertex->selection_weight)
+						edges.insert(edge);
+					if(edge->face_clockwise == face->first_edge)
+						break;
+				}
+			}
+		}
+
+		point->selection_weight = 0.0;
+
+		for(std::set<k3d::split_edge*>::iterator edge = edges.begin(); edge != edges.end(); ++edge)
+			(*edge)->vertex = point;
+
+		for(std::set<k3d::split_edge*>::iterator e = edges.begin(); e != edges.end(); ++e)
+		{
+			k3d::split_edge* edge = *e;
+			if(edge->vertex == edge->face_clockwise->vertex)
+			{
+				edge->selection_weight = 1.0;
+			}
+		}
+
+		edges.clear();
+
+		// Delete selected edges, updating their owning faces and adjacent edges as needed.
+		for(k3d::mesh::polyhedra_t::iterator p = Mesh.polyhedra.begin(); p != Mesh.polyhedra.end(); ++p)
+		{
+			k3d::polyhedron& polyhedron = **p;
+
+			for(k3d::polyhedron::faces_t::iterator f = polyhedron.faces.begin(); f != polyhedron.faces.end(); )
+			{
+				k3d::face* face = *f;
+
+				std::vector<k3d::split_edge*> remaining_edges;
+				for(k3d::split_edge* edge = face->first_edge; edge; edge = edge->face_clockwise)
+				{
+					if(edge->selection_weight)
+					{
+						edges.insert(edge);
+					}
+					else
+					{
+						remaining_edges.push_back(edge);
+					}
+
+					if(edge->face_clockwise == face->first_edge)
+						break;
+				}
+
+				// If there aren't any edges left over for this face zap it, and its little holes too ...
+				if(remaining_edges.empty())
+				{
+					f = polyhedron.faces.erase(f);
+					faces.insert(face);
+					edges.clear();
+				}
+				else
+				{
+					// handle holes...
+					for(k3d::face::holes_t::iterator hole = face->holes.begin(); hole != face->holes.end(); )
+					{
+						std::vector<k3d::split_edge*> remaining_hole_edges;
+						for(k3d::split_edge* edge = *hole; edge; edge = edge->face_clockwise)
+						{
+							if(edge->selection_weight)
+							{
+								edges.insert(edge);
+							}
+							else
+							{
+								remaining_hole_edges.push_back(edge);
+							}
+
+							if(edge->face_clockwise == *hole)
+								break;
+						}
+
+					// Fix hole with left-out edges
+						if(!remaining_hole_edges.empty())
+						{
+							*hole = remaining_hole_edges.front();
+							k3d::loop_edges(remaining_hole_edges.begin(), remaining_hole_edges.end());
+							++hole;
+						}
+						else
+						{
+							hole = face->holes.erase(hole);
+						}
+					}
+
+					face->first_edge = remaining_edges.front();
+					k3d::loop_edges(remaining_edges.begin(), remaining_edges.end());
+					std::for_each(edges.begin(), edges.end(), k3d::delete_object());
+					edges.clear();
+
+					++f;
+				}
+			}
+		}
+
+		// Delete points ...
+		delete_geometry(Mesh.points, points, is_selected());
+
+		// Delete faces ...
+		std::for_each(faces.begin(), faces.end(), k3d::delete_object());
+
+		// Leave the only remaining point selected
+		point->selection_weight = 1.0;
+	}
+
+	void on_update_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
+	{
 	}
 
 	/// Functor object that returns true iff the given geometry is selected
@@ -98,169 +241,6 @@ public:
 		Source.erase(std::remove_if(Source.begin(), Source.end(), Functor), Source.end());
 	}
 
-	void update_geometry(k3d::mesh& Mesh)
-	{
-		// We keep track of the geometry we're going to delete ...
-		std::set<k3d::point*> points;
-		std::set<k3d::face*> faces;
-		std::set<k3d::split_edge*> edges;
-
-		// Delete selected faces, taking their edges along with them ...
-		for(k3d::mesh::polyhedra_t::iterator p = Mesh.polyhedra.begin(); p != Mesh.polyhedra.end(); ++p)
-		{
-			k3d::polyhedron& polyhedron = **p;
-
-			for(k3d::polyhedron::faces_t::iterator f = polyhedron.faces.begin(); f != polyhedron.faces.end(); ++f)
-			{
-				k3d::face* face = *f;
-				face->visible_selection = false;
-				face->selection_weight = 0.0;
-				for(k3d::split_edge* edge = face->first_edge; edge; edge = edge->face_clockwise)
-				{
-					edge->visible_selection = false;
-					edge->selection_weight = 0.0;
-					if(edge->vertex->selection_weight)
-						edges.insert(edge);
-					if(edge->face_clockwise == face->first_edge)
-						break;
-				}
-			}
-		}
-
-		k3d::point* point = 0;
-		for(k3d::mesh::points_t::iterator p = Mesh.points.begin(); p != Mesh.points.end(); ++p)
-		{
-			if((*p)->selection_weight)
-			{
-				if(!point)
-					point = *p;
-				else
-					points.insert(*p);
-			}
-		}
-
-		point->visible_selection = false;
-		point->selection_weight = 0.0;
-
-		for(std::set<k3d::split_edge*>::iterator edge = edges.begin(); edge != edges.end(); ++edge)
-			(*edge)->vertex = point;
-
-		for(std::set<k3d::split_edge*>::iterator e = edges.begin(); e != edges.end(); ++e)
-		{
-			k3d::split_edge* edge = *e;
-			if(edge->vertex == edge->face_clockwise->vertex)
-			{
-				edge->visible_selection = true;
-				edge->selection_weight = 1.0;
-			}
-		}
-
-		edges.clear();
-
-		// Delete selected edges, updating their owning faces and adjacent edges as needed.
-		for(k3d::mesh::polyhedra_t::iterator p = Mesh.polyhedra.begin(); p != Mesh.polyhedra.end(); ++p)
-		{
-			k3d::polyhedron& polyhedron = **p;
-
-			for(k3d::polyhedron::faces_t::iterator f = polyhedron.faces.begin(); f != polyhedron.faces.end(); )
-			{
-				k3d::face* face = *f;
-
-				std::vector<k3d::split_edge*> remaining_edges;
-				for(k3d::split_edge* edge = face->first_edge; edge; edge = edge->face_clockwise)
-				{
-					if(edge->selection_weight)
-					{
-						if(edge->companion)
-							edge->companion->companion = 0;
-
-						edges.insert(edge);
-					}
-					else
-					{
-						remaining_edges.push_back(edge);
-					}
-
-					if(edge->face_clockwise == face->first_edge)
-						break;
-				}
-
-				for(k3d::face::holes_t::iterator hole = face->holes.begin(); hole != face->holes.end(); )
-				{
-					std::vector<k3d::split_edge*> remaining_hole_edges;
-					for(k3d::split_edge* edge = *hole; edge; edge = edge->face_clockwise)
-					{
-						if(edge->selection_weight)
-						{
-							if(edge->companion)
-								edge->companion->companion = 0;
-
-							edges.insert(edge);
-						}
-						else
-						{
-							remaining_hole_edges.push_back(edge);
-						}
-
-						if(edge->face_clockwise == *hole)
-							break;
-					}
-
-					// If there aren't any edges left over for this hole, get rid of it ...
-					if(remaining_hole_edges.empty())
-					{
-						hole = face->holes.erase(hole);
-					}
-					else
-					{
-						*hole = remaining_hole_edges.front();
-						k3d::loop_edges(remaining_hole_edges.begin(), remaining_hole_edges.end());
-						++hole;
-					}
-				}
-
-				// If there aren't any edges left over for this face zap it, and its little holes too ...
-				if(remaining_edges.empty())
-				{
-					f = polyhedron.faces.erase(f);
-					faces.insert(face);
-
-					for(k3d::face::holes_t::iterator hole = face->holes.begin(); hole != face->holes.end(); ++hole)
-					{
-						for(k3d::split_edge* edge = *hole; edge; edge = edge->face_clockwise)
-						{
-							if(edge->companion)
-								edge->companion->companion = 0;
-
-							edges.insert(edge);
-
-							if(edge->face_clockwise == *hole)
-								break;
-						}
-					}
-				}
-				else
-				{
-					face->first_edge = remaining_edges.front();
-					k3d::loop_edges(remaining_edges.begin(), remaining_edges.end());
-
-					++f;
-				}
-			}
-		}
-
-		// Delete points ...
-		delete_geometry(Mesh.points, points, is_selected());
-
-		// Make it happen ...
-		std::for_each(faces.begin(), faces.end(), k3d::delete_object());
-		std::for_each(edges.begin(), edges.end(), k3d::delete_object());
-
-		// Leave the only remaining point selected
-		point->visible_selection = true;
-		point->selection_weight = 1.0;
-	}
-
 	k3d::iplugin_factory& factory()
 	{
 		return get_factory();
@@ -274,7 +254,7 @@ public:
 			k3d::interface_list<k3d::imesh_sink > > > factory(
 				k3d::uuid(0x915ba4d4, 0xd4154a12, 0x938bec97, 0x60f819f3),
 				"JoinPoints",
-				"Joins points at the position of the first point selected",
+				"Merge selected points into one point, positioned at the average position of the selected points.",
 				"Objects",
 				k3d::iplugin_factory::STABLE);
 
