@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 2004-2005, Romain Behar
+// Copyright (c) 2004-2006, Romain Behar
 //
 // Contact: romainbehar@yahoo.com
 //
@@ -32,6 +32,10 @@
 #include <k3dsdk/utility.h>
 #include <k3dsdk/vectors.h>
 
+#include "helpers.h"
+
+#include <set>
+
 namespace libk3dmesh
 {
 
@@ -42,7 +46,7 @@ namespace detail
 class middle_point
 {
 public:
-	middle_point(const k3d::vector3& StartPosition, const k3d::vector3& EndPosition, k3d::point& Point) :
+	middle_point(const k3d::point3& StartPosition, const k3d::point3& EndPosition, k3d::point& Point) :
 		start_position(StartPosition),
 		end_position(EndPosition),
 		point(&Point)
@@ -55,8 +59,8 @@ public:
 	}
 
 private:
-	k3d::vector3 start_position;
-	k3d::vector3 end_position;
+	k3d::point3 start_position;
+	k3d::point3 end_position;
 	k3d::point* point;
 };
 
@@ -85,7 +89,7 @@ struct center_to_points
 			return;
 
 		// Create center point
-		k3d::vector3 face_barycenter = k3d::vector3(0, 0, 0);
+		k3d::point3 face_barycenter = k3d::point3(0, 0, 0);
 		k3d::point* center_point = new k3d::point(face_barycenter);
 		mesh->points.push_back(center_point);
 
@@ -136,6 +140,8 @@ struct center_to_points
 			else
 			{
 				k3d::face* face = new k3d::face(edge0, Face->material);
+				face->selection_weight = 1.0;
+
 				new_faces.push_back(face);
 				polyhedron.faces.push_back(face);
 			}
@@ -175,7 +181,7 @@ struct center_to_midpoints
 		// Create barycenter and midpoints
 		std::vector<k3d::point*> new_points;
 		std::vector<k3d::split_edge*> companions;
-		k3d::vector3 face_barycenter = k3d::vector3(0, 0, 0);
+		k3d::point3 face_barycenter = k3d::point3(0, 0, 0);
 		std::vector<k3d::split_edge*> removed_edges;
 
 		bool used_original_face = false;
@@ -191,9 +197,9 @@ struct center_to_midpoints
 
 			if(is_before_midpoint == new_midpoints.end() && is_midpoint == new_midpoints.end())
 			{
-				const k3d::vector3 first_vertex = edge->vertex->position;
-				const k3d::vector3 last_vertex = edge->face_clockwise->vertex->position;
-				const k3d::vector3 midposition = (first_vertex + last_vertex) * 0.5;
+				const k3d::point3 first_vertex = edge->vertex->position;
+				const k3d::point3 last_vertex = edge->face_clockwise->vertex->position;
+				const k3d::point3 midposition = (first_vertex + last_vertex) * 0.5;
 				k3d::point* midpoint = new k3d::point(midposition);
 				new_midpoints.insert(midpoint);
 
@@ -203,21 +209,21 @@ struct center_to_midpoints
 				new_points.push_back(midpoint);
 				middle_points.push_back(middle_point(first_vertex, last_vertex, *midpoint));
 
-				// Split companion in two
-				if(edge->companion)
-					{
-						k3d::split_edge* second_edge = new k3d::split_edge(midpoint);
-						k3d::split_edge* first_edge = edge->companion;
-						second_edge->face_clockwise = first_edge->face_clockwise;
-						first_edge->face_clockwise = second_edge;
-						companions.push_back(second_edge);
-						companions.push_back(first_edge);
-					}
+				// Split companion in two (second test checks for faces that form a loop)
+				if(edge->companion && !helpers::edge_in_face(edge->companion, Face->first_edge))
+				{
+					k3d::split_edge* second_edge = new k3d::split_edge(midpoint);
+					k3d::split_edge* first_edge = edge->companion;
+					second_edge->face_clockwise = first_edge->face_clockwise;
+					first_edge->face_clockwise = second_edge;
+					companions.push_back(second_edge);
+					companions.push_back(first_edge);
+				}
 				else
-					{
-						companions.push_back(0);
-						companions.push_back(0);
-					}
+				{
+					companions.push_back(0);
+					companions.push_back(0);
+				}
 
 				face_barycenter += edge->vertex->position;
 				++original_vertices;
@@ -275,6 +281,8 @@ struct center_to_midpoints
 			else
 			{
 				k3d::face* face = new k3d::face(edge0, Face->material);
+				face->selection_weight = 1.0;
+
 				new_faces.push_back(face);
 				polyhedron.faces.push_back(face);
 			}
@@ -334,9 +342,9 @@ struct midpoints
 
 			if(is_before_midpoint == new_midpoints.end() && is_midpoint == new_midpoints.end())
 			{
-				const k3d::vector3 first_vertex = edge->vertex->position;
-				const k3d::vector3 last_vertex = edge->face_clockwise->vertex->position;
-				const k3d::vector3 midposition = (first_vertex + last_vertex) * 0.5;
+				const k3d::point3 first_vertex = edge->vertex->position;
+				const k3d::point3 last_vertex = edge->face_clockwise->vertex->position;
+				const k3d::point3 midposition = (first_vertex + last_vertex) * 0.5;
 				k3d::point* midpoint = new k3d::point(midposition);
 				new_midpoints.insert(midpoint);
 
@@ -346,8 +354,8 @@ struct midpoints
 				new_points.push_back(midpoint);
 				middle_points.push_back(middle_point(first_vertex, last_vertex, *midpoint));
 
-				// Split companion in two
-				if(edge->companion)
+				// Split companion in two (second test checks for faces that form a loop)
+				if(edge->companion && !helpers::edge_in_face(edge->companion, Face->first_edge))
 				{
 					k3d::split_edge* second_edge = new k3d::split_edge(midpoint);
 					k3d::split_edge* first_edge = edge->companion;
@@ -395,6 +403,8 @@ struct midpoints
 			middle_face.push_back(edge2);
 
 			k3d::face* face = new k3d::face(edge0, Face->material);
+			face->selection_weight = 1.0;
+
 			new_faces.push_back(face);
 			polyhedron.faces.push_back(face);
 		}
@@ -430,64 +440,44 @@ private:
 } // namespace detail
 
 /////////////////////////////////////////////////////////////////////////////
-// subdivide_faces_implementation
+// subdivide_faces
 
-class subdivide_faces_implementation :
+class subdivide_faces :
 	public k3d::mesh_selection_sink<k3d::mesh_modifier<k3d::persistent<k3d::node> > >
 {
 	typedef k3d::mesh_selection_sink<k3d::mesh_modifier<k3d::persistent<k3d::node> > > base;
 
 public:
-	subdivide_faces_implementation(k3d::idocument& Document) :
+	subdivide_faces(k3d::idocument& Document) :
 		base(Document),
 		m_subdivision_type(init_owner(*this) + init_name("subdivision_type") + init_label(_("Subdivision type")) + init_description(_("Join : center to midpoints, midpoints, center to points")) + init_value(CENTERTOMIDPOINTS) + init_enumeration(subdivision_values())),
 		m_middle_offset(init_owner(*this) + init_name("middle_offset") + init_label(_("Middle offset")) + init_description(_("Middle offset, for midpoints only")) + init_value(0.0) + init_precision(2) + init_step_increment(0.05) + init_units(typeid(k3d::measurement::distance)))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &subdivide_faces_implementation::on_reset_geometry));
+		m_mesh_selection.changed_signal().connect(make_reset_mesh_slot());
+		m_subdivision_type.changed_signal().connect(make_reset_mesh_slot());
 
-		m_mesh_selection.changed_signal().connect(sigc::mem_fun(*this, &subdivide_faces_implementation::on_reset_geometry));
-
-		m_subdivision_type.changed_signal().connect(sigc::mem_fun(*this, &subdivide_faces_implementation::on_reset_geometry));
-		m_middle_offset.changed_signal().connect(sigc::mem_fun(*this, &subdivide_faces_implementation::on_reshape_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &subdivide_faces_implementation::on_create_geometry));
+		m_middle_offset.changed_signal().connect(make_update_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	/** \todo Improve the implementation so we don't have to do this */
+	k3d::iunknown* on_rewrite_hint(iunknown* const Hint)
 	{
-		m_middle_points.clear();
-		m_output_mesh.reset();
+		// Force updates to re-allocate our mesh, for simplicity
+		return 0;
 	}
 
-	void on_reshape_geometry()
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		if(m_output_mesh.empty())
-			return;
-
-		reshape_geometry();
-
-		m_output_mesh.changed_signal().emit();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		// Get the input geometry ...
-		k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
-
-		// Clear previously cached data
 		m_middle_points.clear();
 		m_new_faces.clear();
 
 		// Create output geometry ...
-		k3d::mesh* const output = new k3d::mesh();
-		k3d::deep_copy(*input, *output);
-		k3d::replace_selection(m_mesh_selection.value(), *output);
+		k3d::deep_copy(InputMesh, Mesh);
+		k3d::replace_selection(m_mesh_selection.value(), Mesh);
 
 		const subdivision_t subdivision_type = m_subdivision_type.value();
 
-		for(k3d::mesh::polyhedra_t::iterator polyhedron_i = output->polyhedra.begin(); polyhedron_i != output->polyhedra.end(); ++polyhedron_i)
+		for(k3d::mesh::polyhedra_t::iterator polyhedron_i = Mesh.polyhedra.begin(); polyhedron_i != Mesh.polyhedra.end(); ++polyhedron_i)
 		{
 			k3d::polyhedron& polyhedron = **polyhedron_i;
 
@@ -502,16 +492,16 @@ public:
 			switch(subdivision_type)
 			{
 				case CENTERTOPOINTS:
-					std::for_each(selected_faces.begin(), selected_faces.end(), detail::center_to_points(output, m_new_faces, polyhedron));
-					break;
+					std::for_each(selected_faces.begin(), selected_faces.end(), detail::center_to_points(&Mesh, m_new_faces, polyhedron));
+				break;
 
 				case CENTERTOMIDPOINTS:
-					std::for_each(selected_faces.begin(), selected_faces.end(), detail::center_to_midpoints(output, m_new_faces, m_middle_points, polyhedron));
-					break;
+					std::for_each(selected_faces.begin(), selected_faces.end(), detail::center_to_midpoints(&Mesh, m_new_faces, m_middle_points, polyhedron));
+				break;
 
 				case CONTIGUOUSMIDPOINTS:
-					std::for_each(selected_faces.begin(), selected_faces.end(), detail::midpoints(output, m_new_faces, m_middle_points, polyhedron));
-					break;
+					std::for_each(selected_faces.begin(), selected_faces.end(), detail::midpoints(&Mesh, m_new_faces, m_middle_points, polyhedron));
+				break;
 
 				default:
 					assert_not_reached();
@@ -519,18 +509,9 @@ public:
 
 			assert_warning(is_valid(polyhedron));
 		}
-
-		// Update output selection
-		for(k3d::polyhedron::faces_t::iterator face = m_new_faces.begin(); face != m_new_faces.end(); ++face)
-			(*face)->visible_selection = true;
-
-		// Position all the new geometry we've created ...
-		reshape_geometry();
-
-		return output;
 	}
 
-	void reshape_geometry()
+	void on_update_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
 		const double offset = m_middle_offset.value();
 
@@ -546,7 +527,7 @@ public:
 	static k3d::iplugin_factory& get_factory()
 	{
 		static k3d::plugin_factory<
-			k3d::document_plugin<subdivide_faces_implementation>,
+			k3d::document_plugin<subdivide_faces>,
 			k3d::interface_list<k3d::imesh_source,
 			k3d::interface_list<k3d::imesh_sink > > > factory(
 				k3d::uuid(0xb9d4d815, 0x241c473d, 0xa22d3523, 0x406fa390),
@@ -627,7 +608,7 @@ private:
 
 k3d::iplugin_factory& subdivide_faces_factory()
 {
-	return subdivide_faces_implementation::get_factory();
+	return subdivide_faces::get_factory();
 }
 
 } // namespace libk3dmesh
