@@ -36,40 +36,33 @@ namespace libk3dnurbs
 {
 
 /////////////////////////////////////////////////////////////////////////////
-// nurbs_cylinder_implementation
+// nurbs_cylinder
 
-class nurbs_cylinder_implementation :
+class nurbs_cylinder :
 	public k3d::material_client<k3d::mesh_source<k3d::persistent<k3d::node> > >
 {
 	typedef k3d::material_client<k3d::mesh_source<k3d::persistent<k3d::node> > > base;
 
 public:
-	nurbs_cylinder_implementation(k3d::idocument& Document) :
+	nurbs_cylinder(k3d::idocument& Document) :
 		base(Document),
 		m_radius(init_owner(*this) + init_name("radius") + init_label(_("radius")) + init_description(_("Radius")) + init_value(5.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_zmax(init_owner(*this) + init_name("zmax") + init_label(_("zmax")) + init_description(_("ZMax")) + init_value(5.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_zmin(init_owner(*this) + init_name("zmin") + init_label(_("zmin")) + init_description(_("ZMin")) + init_value(-5.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_thetamax(init_owner(*this) + init_name("thetamax") + init_label(_("thetamax")) + init_description(_("End angle")) + init_value(k3d::radians(360.0)) + init_precision(2) + init_step_increment(k3d::radians(1.0)) + init_units(typeid(k3d::measurement::angle))),
-		m_u_segments(init_owner(*this) + init_name("u_segments") + init_label(_("u_segments")) + init_description(_("Radial Segments")) + init_value(4) + init_constraint(constraint::minimum(1L)) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
+		m_u_segments(init_owner(*this) + init_name("u_segments") + init_label(_("u_segments")) + init_description(_("Radial Segments")) + init_value(4) + init_constraint(constraint::minimum(3L)) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
 		m_v_segments(init_owner(*this) + init_name("v_segments") + init_label(_("v_segments")) + init_description(_("Radial Segments")) + init_value(2) + init_constraint(constraint::minimum(1L)) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_material.changed_signal().connect(sigc::mem_fun(*this, &nurbs_cylinder_implementation::on_reset_geometry));
-		m_radius.changed_signal().connect(sigc::mem_fun(*this, &nurbs_cylinder_implementation::on_reset_geometry));
-		m_zmax.changed_signal().connect(sigc::mem_fun(*this, &nurbs_cylinder_implementation::on_reset_geometry));
-		m_zmin.changed_signal().connect(sigc::mem_fun(*this, &nurbs_cylinder_implementation::on_reset_geometry));
-		m_thetamax.changed_signal().connect(sigc::mem_fun(*this, &nurbs_cylinder_implementation::on_reset_geometry));
-		m_u_segments.changed_signal().connect(sigc::mem_fun(*this, &nurbs_cylinder_implementation::on_reset_geometry));
-		m_v_segments.changed_signal().connect(sigc::mem_fun(*this, &nurbs_cylinder_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &nurbs_cylinder_implementation::on_create_geometry));
+		m_material.changed_signal().connect(make_reset_mesh_slot());
+		m_radius.changed_signal().connect(make_reset_mesh_slot());
+		m_zmax.changed_signal().connect(make_reset_mesh_slot());
+		m_zmin.changed_signal().connect(make_reset_mesh_slot());
+		m_thetamax.changed_signal().connect(make_reset_mesh_slot());
+		m_u_segments.changed_signal().connect(make_reset_mesh_slot());
+		m_v_segments.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
-	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
 		const double radius = m_radius.value();
 		const double zmax = m_zmax.value();
@@ -78,18 +71,17 @@ public:
 		const unsigned long u_segments = m_u_segments.value();
 		const unsigned long v_segments = m_v_segments.value();
 
-		k3d::mesh* const mesh = new k3d::mesh();
 		k3d::nupatch* const nupatch = new k3d::nupatch();
-		mesh->nupatches.push_back(nupatch);
+		Mesh.nupatches.push_back(nupatch);
 		
 		nupatch->material = m_material.value();
 		nupatch->u_order = 3;
 		nupatch->v_order = 2;
 
 		std::vector<double> weights;
-		std::vector<k3d::vector3> control_points;
-		k3d::nurbs_arc(k3d::vector3(1, 0, 0), k3d::vector3(0, 1, 0), 0, thetamax, u_segments, nupatch->u_knots, weights, control_points);
-		return_val_if_fail(weights.size() == control_points.size(), 0);
+		std::vector<k3d::point3> control_points;
+		k3d::nurbs_arc(k3d::point3(1, 0, 0), k3d::point3(0, 1, 0), 0, thetamax, u_segments, nupatch->u_knots, weights, control_points);
+		return_if_fail(weights.size() == control_points.size());
 
 		nupatch->v_knots.insert(nupatch->v_knots.end(), 2, 0);
 		for(unsigned long i = 1; i != v_segments; ++i)
@@ -97,18 +89,21 @@ public:
 		nupatch->v_knots.insert(nupatch->v_knots.end(), 2, v_segments);
 
 		for(unsigned long i = 0; i <= v_segments; ++i)
-			{
-				const k3d::vector3 offset = k3d::mix(zmin, zmax, static_cast<double>(i) / static_cast<double>(v_segments)) * k3d::vector3(0, 0, 1);
+		{
+			const k3d::point3 offset = k3d::mix(zmin, zmax, static_cast<double>(i) / static_cast<double>(v_segments)) * k3d::point3(0, 0, 1);
 
-				for(unsigned long j = 0; j != control_points.size(); ++j)
-					{
-						mesh->points.push_back(new k3d::point((radius * control_points[j]) + offset));
-						nupatch->control_points.push_back(k3d::nupatch::control_point(mesh->points.back(), weights[j]));
-					}
+			for(unsigned long j = 0; j != control_points.size(); ++j)
+			{
+				Mesh.points.push_back(new k3d::point((radius * control_points[j]) + offset));
+				nupatch->control_points.push_back(k3d::nupatch::control_point(Mesh.points.back(), weights[j]));
 			}
+		}
 
 		assert_warning(is_valid(*nupatch));
-		return mesh;
+	}
+
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -118,7 +113,7 @@ public:
 
 	static k3d::iplugin_factory& get_factory()
 	{
-		static k3d::plugin_factory<k3d::document_plugin<nurbs_cylinder_implementation>, k3d::interface_list<k3d::imesh_source > > factory(
+		static k3d::plugin_factory<k3d::document_plugin<nurbs_cylinder>, k3d::interface_list<k3d::imesh_source > > factory(
 		k3d::uuid(0x756958ae, 0x6932432f, 0x854adf09, 0x262c2da2),
 			"NurbsCylinder",
 			_("Generates a NURBS cylinder"),
@@ -142,9 +137,8 @@ private:
 
 k3d::iplugin_factory& nurbs_cylinder_factory()
 {
-	return nurbs_cylinder_implementation::get_factory();
+	return nurbs_cylinder::get_factory();
 }
 
 } // namespace libk3dnurbs
 
-
