@@ -37,15 +37,15 @@ namespace libk3dnurbs
 {
 
 /////////////////////////////////////////////////////////////////////////////
-// nurbs_grid_implementation
+// nurbs_grid
 
-class nurbs_grid_implementation :
+class nurbs_grid :
 	public k3d::material_client<k3d::mesh_source<k3d::persistent<k3d::node> > >
 {
 	typedef k3d::material_client<k3d::mesh_source<k3d::persistent<k3d::node> > > base;
 
 public:
-	nurbs_grid_implementation(k3d::idocument& Document) :
+	nurbs_grid(k3d::idocument& Document) :
 		base(Document),
 		m_u_order(init_owner(*this) + init_name("u_order") + init_label(_("u_order")) + init_description(_("U Order")) + init_value(4) + init_constraint(constraint::minimum(2L)) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
 		m_v_order(init_owner(*this) + init_name("v_order") + init_label(_("v_order")) + init_description(_("V Order")) + init_value(4) + init_constraint(constraint::minimum(2L)) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
@@ -54,74 +54,22 @@ public:
 		m_width(init_owner(*this) + init_name("width") + init_label(_("width")) + init_description(_("Width")) + init_value(10.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_height(init_owner(*this) + init_name("height") + init_label(_("height")) + init_description(_("Height")) + init_value(10.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance)))
 	{
-		m_material.changed_signal().connect(sigc::mem_fun(*this, &nurbs_grid_implementation::on_reset_geometry));
-
-		m_u_order.changed_signal().connect(sigc::mem_fun(*this, &nurbs_grid_implementation::on_reset_geometry));
-		m_v_order.changed_signal().connect(sigc::mem_fun(*this, &nurbs_grid_implementation::on_reset_geometry));
-		m_u_segments.changed_signal().connect(sigc::mem_fun(*this, &nurbs_grid_implementation::on_reset_geometry));
-		m_v_segments.changed_signal().connect(sigc::mem_fun(*this, &nurbs_grid_implementation::on_reset_geometry));
-		m_width.changed_signal().connect(sigc::mem_fun(*this, &nurbs_grid_implementation::on_reset_geometry));
-		m_height.changed_signal().connect(sigc::mem_fun(*this, &nurbs_grid_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &nurbs_grid_implementation::on_create_geometry));
+		m_material.changed_signal().connect(make_reset_mesh_slot());
+		m_u_order.changed_signal().connect(make_reset_mesh_slot());
+		m_v_order.changed_signal().connect(make_reset_mesh_slot());
+		m_u_segments.changed_signal().connect(make_reset_mesh_slot());
+		m_v_segments.changed_signal().connect(make_reset_mesh_slot());
+		m_width.changed_signal().connect(make_reset_mesh_slot());
+		m_height.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		k3d::mesh* const mesh = new k3d::mesh();
-
 		k3d::nupatch* const patch = new k3d::nupatch();
-		mesh->nupatches.push_back(patch);
+		Mesh.nupatches.push_back(patch);
 
 		patch->material = m_material.value();
 
-/*
-		// For testing purposes - create a rational Bezier patch
-		patch->u_order = 4;
-		patch->v_order = 2;
-
-		mesh->points.push_back(new k3d::point(-3, 6, 0));
-		mesh->points.push_back(new k3d::point(3, 6, 0));
-		mesh->points.push_back(new k3d::point(3, 0, 0));
-		mesh->points.push_back(new k3d::point(-3, 0, 0));
-		mesh->points.push_back(new k3d::point(-3, -6, 0));
-		mesh->points.push_back(new k3d::point(3, -6, 0));
-
-		mesh->points.push_back(new k3d::point(-3, 6, 6));
-		mesh->points.push_back(new k3d::point(3, 6, 6));
-		mesh->points.push_back(new k3d::point(3, 0, 6));
-		mesh->points.push_back(new k3d::point(-3, 0, 6));
-		mesh->points.push_back(new k3d::point(-3, -6, 6));
-		mesh->points.push_back(new k3d::point(3, -6, 6));
-
-		patch->control_points.push_back(k3d::nupatch::control_point(mesh->points[0], 1));
-		patch->control_points.push_back(k3d::nupatch::control_point(mesh->points[1], 10));
-		patch->control_points.push_back(k3d::nupatch::control_point(mesh->points[2], 10));
-		patch->control_points.push_back(k3d::nupatch::control_point(mesh->points[3], 1));
-		patch->control_points.push_back(k3d::nupatch::control_point(mesh->points[4], 1));
-		patch->control_points.push_back(k3d::nupatch::control_point(mesh->points[5], 1));
-
-		patch->control_points.push_back(k3d::nupatch::control_point(mesh->points[0+6], 1));
-		patch->control_points.push_back(k3d::nupatch::control_point(mesh->points[1+6], 1));
-		patch->control_points.push_back(k3d::nupatch::control_point(mesh->points[2+6], 1));
-		patch->control_points.push_back(k3d::nupatch::control_point(mesh->points[3+6], 1));
-		patch->control_points.push_back(k3d::nupatch::control_point(mesh->points[4+6], 1));
-		patch->control_points.push_back(k3d::nupatch::control_point(mesh->points[5+6], 1));
-
-		patch->u_knots.insert(patch->u_knots.end(), 4, 0);
-		patch->u_knots.insert(patch->u_knots.end(), 1);
-		patch->u_knots.insert(patch->u_knots.end(), 1);
-		patch->u_knots.insert(patch->u_knots.end(), 4, 3);
-
-		patch->v_knots.insert(patch->v_knots.end(), 2, 0);
-		patch->v_knots.insert(patch->v_knots.end(), 2, 1);
-*/
-
 		const unsigned long u_order = m_u_order.value();
 		const unsigned long v_order = m_v_order.value();
 		const unsigned long u_segments = m_u_segments.value();
@@ -150,23 +98,24 @@ public:
 		patch->v_knots.insert(patch->v_knots.end(), v_order, v_segments);
 
 		for(unsigned long row = 0; row != v_points; ++row)
+		{
+			const double row_percent = 0.5 - (static_cast<double>(row) / static_cast<double>(v_points-1));
+
+			for(unsigned long column = 0; column != u_points; ++column)
 			{
-				const double row_percent = 0.5 - (static_cast<double>(row) / static_cast<double>(v_points-1));
+				const double column_percent = (static_cast<double>(column) / static_cast<double>(u_points-1)) - 0.5;
 
-				for(unsigned long column = 0; column != u_points; ++column)
-					{
-						const double column_percent = (static_cast<double>(column) / static_cast<double>(u_points-1)) - 0.5;
-
-						k3d::point* point = new k3d::point(width * column_percent, height * row_percent, 0);
-						mesh->points.push_back(point);
-						patch->control_points.push_back(k3d::nupatch::control_point(point, 1));
-					}
+				k3d::point* point = new k3d::point(width * column_percent, height * row_percent, 0);
+				Mesh.points.push_back(point);
+				patch->control_points.push_back(k3d::nupatch::control_point(point, 1));
 			}
+		}
 
-		// Do some sanity checking just for the heck of it ...
 		assert_warning(k3d::is_valid(*patch));
+	}
 
-		return mesh;
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -176,7 +125,7 @@ public:
 
 	static k3d::iplugin_factory& get_factory()
 	{
-		static k3d::plugin_factory<k3d::document_plugin<nurbs_grid_implementation>, k3d::interface_list<k3d::imesh_source > > factory(
+		static k3d::plugin_factory<k3d::document_plugin<nurbs_grid>, k3d::interface_list<k3d::imesh_source > > factory(
 			k3d::uuid(0x5aac4e72, 0xf9b04b61, 0xf8b1bdbc, 0x851cf62e),
 			"NurbsGrid",
 			_("Generates a NURBS grid"),
@@ -200,9 +149,8 @@ private:
 
 k3d::iplugin_factory& nurbs_grid_factory()
 {
-	return nurbs_grid_implementation::get_factory();
+	return nurbs_grid::get_factory();
 }
 
 } // namespace libk3dnurbs
 
-
