@@ -36,15 +36,15 @@ namespace libk3dnurbs
 {
 
 /////////////////////////////////////////////////////////////////////////////
-// nurbs_hyperboloid_implementation
+// nurbs_hyperboloid
 
-class nurbs_hyperboloid_implementation :
+class nurbs_hyperboloid :
 	public k3d::material_client<k3d::mesh_source<k3d::persistent<k3d::node> > >
 {
 	typedef k3d::material_client<k3d::mesh_source<k3d::persistent<k3d::node> > > base;
 
 public:
-	nurbs_hyperboloid_implementation(k3d::idocument& Document) :
+	nurbs_hyperboloid(k3d::idocument& Document) :
 		base(Document),
 		m_x1(init_owner(*this) + init_name("x1") + init_label(_("x1")) + init_description(_("x1")) + init_value(5.0) + init_precision( 2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_y1(init_owner(*this) + init_name("y1") + init_label(_("y1")) + init_description(_("y1")) + init_value(-5.0) + init_precision( 2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
@@ -54,24 +54,17 @@ public:
 		m_z2(init_owner(*this) + init_name("z2") + init_label(_("z2")) + init_description(_("z2")) + init_value(5.0) + init_precision( 2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_thetamax(init_owner(*this) + init_name("thetamax") + init_label(_("thetamax")) + init_description(_("thetamax")) + init_value(k3d::radians(360.0)) + init_precision( 2) + init_step_increment(k3d::radians(1.0)) + init_units(typeid(k3d::measurement::angle)))
 	{
-		m_material.changed_signal().connect(sigc::mem_fun(*this, &nurbs_hyperboloid_implementation::on_reset_geometry));
-		m_x1.changed_signal().connect(sigc::mem_fun(*this, &nurbs_hyperboloid_implementation::on_reset_geometry));
-		m_y1.changed_signal().connect(sigc::mem_fun(*this, &nurbs_hyperboloid_implementation::on_reset_geometry));
-		m_z1.changed_signal().connect(sigc::mem_fun(*this, &nurbs_hyperboloid_implementation::on_reset_geometry));
-		m_x2.changed_signal().connect(sigc::mem_fun(*this, &nurbs_hyperboloid_implementation::on_reset_geometry));
-		m_y2.changed_signal().connect(sigc::mem_fun(*this, &nurbs_hyperboloid_implementation::on_reset_geometry));
-		m_z2.changed_signal().connect(sigc::mem_fun(*this, &nurbs_hyperboloid_implementation::on_reset_geometry));
-		m_thetamax.changed_signal().connect(sigc::mem_fun(*this, &nurbs_hyperboloid_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &nurbs_hyperboloid_implementation::on_create_geometry));
+		m_material.changed_signal().connect(make_reset_mesh_slot());
+		m_x1.changed_signal().connect(make_reset_mesh_slot());
+		m_y1.changed_signal().connect(make_reset_mesh_slot());
+		m_z1.changed_signal().connect(make_reset_mesh_slot());
+		m_x2.changed_signal().connect(make_reset_mesh_slot());
+		m_y2.changed_signal().connect(make_reset_mesh_slot());
+		m_z2.changed_signal().connect(make_reset_mesh_slot());
+		m_thetamax.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
-	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
 		const double x1 = m_x1.value();
 		const double y1 = m_y1.value();
@@ -87,42 +80,44 @@ public:
 		const double radius1 = sqrt(x1 * x1 + y1 * y1);
 		const double radius2 = sqrt(x2 * x2 + y2 * y2);
 
-		const k3d::vector3 offset1 = z1 * k3d::vector3(0, 0, 1);
-		const k3d::vector3 offset2 = z2 * k3d::vector3(0, 0, 1);
+		const k3d::point3 offset1 = z1 * k3d::point3(0, 0, 1);
+		const k3d::point3 offset2 = z2 * k3d::point3(0, 0, 1);
 
-		k3d::mesh* const mesh = new k3d::mesh();
 		k3d::nupatch* const nupatch = new k3d::nupatch();
-		mesh->nupatches.push_back(nupatch);
+		Mesh.nupatches.push_back(nupatch);
 		
 		nupatch->material = m_material.value();
 		nupatch->u_order = 3;
 		nupatch->v_order = 2;
 
 		std::vector<double> weights1;
-		std::vector<k3d::vector3> arc_points1;
-		k3d::nurbs_arc(k3d::vector3(1, 0, 0), k3d::vector3(0, 1, 0), thetamin1, thetamin1 + thetamax, 4, nupatch->u_knots, weights1, arc_points1);
+		std::vector<k3d::point3> arc_points1;
+		k3d::nurbs_arc(k3d::point3(1, 0, 0), k3d::point3(0, 1, 0), thetamin1, thetamin1 + thetamax, 4, nupatch->u_knots, weights1, arc_points1);
 		
 		std::vector<double> weights2;
-		std::vector<k3d::vector3> arc_points2;
-		k3d::nurbs_arc(k3d::vector3(1, 0, 0), k3d::vector3(0, 1, 0), thetamin2, thetamin2 + thetamax, 4, nupatch->u_knots, weights2, arc_points2);
+		std::vector<k3d::point3> arc_points2;
+		k3d::nurbs_arc(k3d::point3(1, 0, 0), k3d::point3(0, 1, 0), thetamin2, thetamin2 + thetamax, 4, nupatch->u_knots, weights2, arc_points2);
 
 		nupatch->v_knots.insert(nupatch->v_knots.end(), 2, 0);
 		nupatch->v_knots.insert(nupatch->v_knots.end(), 2, 1);
 
 		for(unsigned long j = 0; j != arc_points1.size(); ++j)
-			{
-				mesh->points.push_back(new k3d::point(radius1 * arc_points1[j] + offset1));
-				nupatch->control_points.push_back(k3d::nupatch::control_point(mesh->points.back(), weights1[j]));
-			}
+		{
+			Mesh.points.push_back(new k3d::point(radius1 * arc_points1[j] + offset1));
+			nupatch->control_points.push_back(k3d::nupatch::control_point(Mesh.points.back(), weights1[j]));
+		}
 		
 		for(unsigned long j = 0; j != arc_points2.size(); ++j)
-			{
-				mesh->points.push_back(new k3d::point(radius2 * arc_points2[j] + offset2));
-				nupatch->control_points.push_back(k3d::nupatch::control_point(mesh->points.back(), weights2[j]));
-			}
+		{
+			Mesh.points.push_back(new k3d::point(radius2 * arc_points2[j] + offset2));
+			nupatch->control_points.push_back(k3d::nupatch::control_point(Mesh.points.back(), weights2[j]));
+		}
 
 		assert_warning(is_valid(*nupatch));
-		return mesh;
+	}
+
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -132,7 +127,7 @@ public:
 
 	static k3d::iplugin_factory& get_factory()
 	{
-		static k3d::plugin_factory<k3d::document_plugin<nurbs_hyperboloid_implementation>, k3d::interface_list<k3d::imesh_source > > factory(
+		static k3d::plugin_factory<k3d::document_plugin<nurbs_hyperboloid>, k3d::interface_list<k3d::imesh_source > > factory(
 		k3d::uuid(0xb653277c, 0x7bba4505, 0xbabcd65d, 0x591ecedf),
 			"NurbsHyperboloid",
 			_("Generates a NURBS hyperboloid"),
@@ -157,9 +152,8 @@ private:
 
 k3d::iplugin_factory& nurbs_hyperboloid_factory()
 {
-	return nurbs_hyperboloid_implementation::get_factory();
+	return nurbs_hyperboloid::get_factory();
 }
 
 } // namespace libk3dnurbs
 
-
