@@ -36,40 +36,33 @@ namespace libk3dnurbs
 {
 
 /////////////////////////////////////////////////////////////////////////////
-// nurbs_sphere_implementation
+// nurbs_sphere
 
-class nurbs_sphere_implementation :
+class nurbs_sphere :
 	public k3d::material_client<k3d::mesh_source<k3d::persistent<k3d::node> > >
 {
 	typedef k3d::material_client<k3d::mesh_source<k3d::persistent<k3d::node> > > base;
 
 public:
-	nurbs_sphere_implementation(k3d::idocument& Document) :
+	nurbs_sphere(k3d::idocument& Document) :
 		base(Document),
 		m_radius(init_owner(*this) + init_name("radius") + init_label(_("radius")) + init_description(_("Radius")) + init_value(5.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_zmax(init_owner(*this) + init_name("zmax") + init_label(_("zmax")) + init_description(_("ZMax")) + init_value(1.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_zmin(init_owner(*this) + init_name("zmin") + init_label(_("zmin")) + init_description(_("ZMin")) + init_value(-1.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
 		m_thetamax(init_owner(*this) + init_name("thetamax") + init_label(_("thetamax")) + init_description(_("thetamax")) + init_value(k3d::radians(360.0)) + init_precision(2) + init_step_increment(k3d::radians(1.0)) + init_units(typeid(k3d::measurement::angle))),
-		m_u_segments(init_owner(*this) + init_name("u_segments") + init_label(_("u_segments")) + init_description(_("Radial Segments")) + init_value(4) + init_constraint(constraint::minimum(1L)) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
-		m_v_segments(init_owner(*this) + init_name("v_segments") + init_label(_("v_segments")) + init_description(_("Radial Segments")) + init_value(2) + init_constraint(constraint::minimum(1L)) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar)))
+		m_u_segments(init_owner(*this) + init_name("u_segments") + init_label(_("u_segments")) + init_description(_("Radial Segments")) + init_value(4) + init_constraint(constraint::minimum(3L)) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
+		m_v_segments(init_owner(*this) + init_name("v_segments") + init_label(_("v_segments")) + init_description(_("Radial Segments")) + init_value(2) + init_constraint(constraint::minimum(2L)) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_material.changed_signal().connect(sigc::mem_fun(*this, &nurbs_sphere_implementation::on_reset_geometry));
-		m_radius.changed_signal().connect(sigc::mem_fun(*this, &nurbs_sphere_implementation::on_reset_geometry));
-		m_zmax.changed_signal().connect(sigc::mem_fun(*this, &nurbs_sphere_implementation::on_reset_geometry));
-		m_zmin.changed_signal().connect(sigc::mem_fun(*this, &nurbs_sphere_implementation::on_reset_geometry));
-		m_thetamax.changed_signal().connect(sigc::mem_fun(*this, &nurbs_sphere_implementation::on_reset_geometry));
-		m_u_segments.changed_signal().connect(sigc::mem_fun(*this, &nurbs_sphere_implementation::on_reset_geometry));
-		m_v_segments.changed_signal().connect(sigc::mem_fun(*this, &nurbs_sphere_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &nurbs_sphere_implementation::on_create_geometry));
+		m_material.changed_signal().connect(make_reset_mesh_slot());
+		m_radius.changed_signal().connect(make_reset_mesh_slot());
+		m_zmax.changed_signal().connect(make_reset_mesh_slot());
+		m_zmin.changed_signal().connect(make_reset_mesh_slot());
+		m_thetamax.changed_signal().connect(make_reset_mesh_slot());
+		m_u_segments.changed_signal().connect(make_reset_mesh_slot());
+		m_v_segments.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
-	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
 		const double radius = m_radius.value();
 		const double zmax = m_zmax.value();
@@ -81,37 +74,39 @@ public:
 		const double phimin = (zmin > -1) ? asin(zmin) : -k3d::pi_over_2();
 		const double phimax = (zmax < 1) ? asin(zmax) : k3d::pi_over_2();
 
-		k3d::mesh* const mesh = new k3d::mesh();
 		k3d::nupatch* const nupatch = new k3d::nupatch();
-		mesh->nupatches.push_back(nupatch);
+		Mesh.nupatches.push_back(nupatch);
 		
 		nupatch->material = m_material.value();
 		nupatch->u_order = 3;
 		nupatch->v_order = 3;
 
 		std::vector<double> v_weights;
-		std::vector<k3d::vector3> v_arc_points;
-		k3d::nurbs_arc(k3d::vector3(0, 1, 0), k3d::vector3(0, 0, 1), phimin, phimax, v_segments, nupatch->v_knots, v_weights, v_arc_points);
+		std::vector<k3d::point3> v_arc_points;
+		k3d::nurbs_arc(k3d::point3(0, 1, 0), k3d::point3(0, 0, 1), phimin, phimax, v_segments, nupatch->v_knots, v_weights, v_arc_points);
 
 		std::vector<double> u_weights;
-		std::vector<k3d::vector3> u_arc_points;
-		k3d::nurbs_arc(k3d::vector3(1, 0, 0), k3d::vector3(0, 1, 0), 0, thetamax, u_segments, nupatch->u_knots, u_weights, u_arc_points);
+		std::vector<k3d::point3> u_arc_points;
+		k3d::nurbs_arc(k3d::point3(1, 0, 0), k3d::point3(0, 1, 0), 0, thetamax, u_segments, nupatch->u_knots, u_weights, u_arc_points);
 
 		for(unsigned long v = 0; v != v_arc_points.size(); ++v)
-			{
-				const k3d::vector3 offset = radius * v_arc_points[v][2] * k3d::vector3(0, 0, 1);
-				const double radius2 = radius * v_arc_points[v][1];
-				const double v_weight = v_weights[v];
+		{
+			const k3d::point3 offset = radius * v_arc_points[v][2] * k3d::point3(0, 0, 1);
+			const double radius2 = radius * v_arc_points[v][1];
+			const double v_weight = v_weights[v];
 
-				for(unsigned long u = 0; u != u_arc_points.size(); ++u)
-					{
-						mesh->points.push_back(new k3d::point(radius2 * u_arc_points[u] + offset));
-						nupatch->control_points.push_back(k3d::nupatch::control_point(mesh->points.back(), v_weight * u_weights[u]));
-					}
+			for(unsigned long u = 0; u != u_arc_points.size(); ++u)
+			{
+				Mesh.points.push_back(new k3d::point(radius2 * u_arc_points[u] + offset));
+				nupatch->control_points.push_back(k3d::nupatch::control_point(Mesh.points.back(), v_weight * u_weights[u]));
 			}
+		}
 
 		assert_warning(is_valid(*nupatch));
-		return mesh;
+	}
+
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -121,7 +116,7 @@ public:
 
 	static k3d::iplugin_factory& get_factory()
 	{
-		static k3d::plugin_factory<k3d::document_plugin<nurbs_sphere_implementation>, k3d::interface_list<k3d::imesh_source > > factory(
+		static k3d::plugin_factory<k3d::document_plugin<nurbs_sphere>, k3d::interface_list<k3d::imesh_source > > factory(
 		k3d::uuid(0x082eeef0, 0x837f4277, 0xa7c21ff1, 0x839b286e),
 			"NurbsSphere",
 			_("Generates a NURBS sphere"),
@@ -145,7 +140,7 @@ private:
 
 k3d::iplugin_factory& nurbs_sphere_factory()
 {
-	return nurbs_sphere_implementation::get_factory();
+	return nurbs_sphere::get_factory();
 }
 
 } // namespace libk3dnurbs
