@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -19,12 +19,14 @@
 
 /** \file
 		\author Tim Shead (tshead@k-3d.com)
+		\author Romain Behar (romainbehar@yahoo.com)
 */
 
 #include <k3dsdk/classes.h>
 #include <k3dsdk/data.h>
 #include <k3dsdk/frames.h>
 #include <k3dsdk/geometry.h>
+#include <k3dsdk/gl.h>
 #include <k3dsdk/i18n.h>
 #include <k3dsdk/icamera.h>
 #include <k3dsdk/icrop_window.h>
@@ -39,7 +41,6 @@
 #include <k3dsdk/property_group_collection.h>
 #include <k3dsdk/renderman.h>
 #include <k3dsdk/time_source.h>
-#include <k3dsdk/gl.h>
 
 #include <iostream>
 
@@ -119,6 +120,135 @@ private:
 	const k3d::gl::select_state& m_select_state;
 };
 
+void gl_reset(const k3d::color BackgroundColor, const double PointSize)
+{
+	// Clear background ...
+	glClearDepth(1.0);
+	glClearColor(
+		static_cast<GLfloat>(BackgroundColor.red),
+		static_cast<GLfloat>(BackgroundColor.green),
+		static_cast<GLfloat>(BackgroundColor.blue),
+		0.0f);
+	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+
+	// Set point size ...
+	glPointSize(static_cast<GLfloat>(PointSize));
+
+	// Setup stippling for path normals / bezier control lines ...
+	glLineStipple(2, 0xaaaa);
+	glDisable(GL_LINE_STIPPLE);
+	glDisable(GL_POLYGON_STIPPLE);
+}
+
+void gl_setup_textures()
+{
+	// Setup texture matrix
+	glMatrixMode(GL_TEXTURE);
+	glLoadIdentity();
+}
+
+void gl_draw_2d_widgets(k3d::icamera& Camera, const k3d::rectangle& WindowRect, const k3d::rectangle& CameraRect, const bool DrawFrustum, const bool DrawCropWindow, const bool DrawSafeZone, const bool DrawAimPoint, k3d::idocument& Document)
+{
+	// Set ourselves up to draw 2D overlays ...
+	glMatrixMode(GL_PROJECTION);
+	glLoadIdentity();
+	glOrtho(WindowRect.left, WindowRect.right, WindowRect.bottom, WindowRect.top, -1, 1);
+
+	glMatrixMode(GL_MODELVIEW);
+	glLoadIdentity();
+
+	if(DrawFrustum)
+	{
+		glColor4d(0, 0, 0, 1);
+		glDisable(GL_LIGHTING);
+
+		glBegin(GL_LINE_LOOP);
+			glVertex2d(CameraRect.left, CameraRect.top);
+			glVertex2d(CameraRect.right, CameraRect.top);
+			glVertex2d(CameraRect.right, CameraRect.bottom);
+			glVertex2d(CameraRect.left, CameraRect.bottom);
+		glEnd();
+	}
+
+	// Optionally show the extents of the crop window
+	if(DrawCropWindow)
+	{
+		const double left = boost::any_cast<double>(k3d::get_value(Document.dag(), Camera.crop_window().crop_left()));
+		const double right = boost::any_cast<double>(k3d::get_value(Document.dag(), Camera.crop_window().crop_right()));
+		const double bottom = boost::any_cast<double>(k3d::get_value(Document.dag(), Camera.crop_window().crop_bottom()));
+		const double top = boost::any_cast<double>(k3d::get_value(Document.dag(), Camera.crop_window().crop_top()));
+
+		glColor4d(0, 0, 0.5, 1);
+		glDisable(GL_LIGHTING);
+
+		glBegin(GL_LINE_LOOP);
+			glVertex2d(k3d::mix(CameraRect.left, CameraRect.right, left), k3d::mix(CameraRect.top, CameraRect.bottom, top));
+			glVertex2d(k3d::mix(CameraRect.left, CameraRect.right, right), k3d::mix(CameraRect.top, CameraRect.bottom, top));
+			glVertex2d(k3d::mix(CameraRect.left, CameraRect.right, right), k3d::mix(CameraRect.top, CameraRect.bottom, bottom));
+			glVertex2d(k3d::mix(CameraRect.left, CameraRect.right, left), k3d::mix(CameraRect.top, CameraRect.bottom, bottom));
+		glEnd();
+	}
+
+	// Optionally draw a safe zone for video ...
+	if(DrawSafeZone)
+	{
+		glColor4d(0, 0, 0, 1);
+		glDisable(GL_LIGHTING);
+
+		glBegin(GL_LINE_LOOP);
+			glVertex2d(CameraRect.left * 0.9, CameraRect.top * 0.9);
+			glVertex2d(CameraRect.right * 0.9, CameraRect.top * 0.9);
+			glVertex2d(CameraRect.right * 0.9, CameraRect.bottom * 0.9);
+			glVertex2d(CameraRect.left * 0.9, CameraRect.bottom * 0.9);
+		glEnd();
+	}
+
+	// Optionally draw the "aimpoint" (crosshairs aligned with the center of the viewport)
+	if(DrawAimPoint)
+	{
+		glColor4d(0, 0, 0, 1);
+		glDisable(GL_LIGHTING);
+
+		glBegin(GL_LINES);
+			glVertex2d(CameraRect.left * 0.05, 0);
+			glVertex2d(CameraRect.right * 0.05, 0);
+			glVertex2d(0, CameraRect.top * 0.05);
+			glVertex2d(0, CameraRect.bottom * 0.05);
+		glEnd();
+	}
+}
+
+void gl_setup_lights(const bool Headlight)
+{
+	// Setup lights ...
+	glEnable(GL_LIGHTING);
+	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
+	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
+
+	// Make sure all lights are turned off initially ...
+	GLint maxlights = 0;
+	glGetIntegerv(GL_MAX_LIGHTS, &maxlights);
+	for(GLint i = 0; i < maxlights; ++i)
+		glDisable(GLenum(GL_LIGHT0 + i));
+
+	// Setup the headlight ...
+	if(Headlight)
+	{
+		// Enable the light ...
+		glEnable(GL_LIGHT0);
+
+		// Setup color ...
+		const GLfloat color[4] = { 1.0f, 1.0f, 1.0f, 1.0f };
+		glLightfv(GL_LIGHT0, GL_AMBIENT, color);
+		glLightfv(GL_LIGHT0, GL_DIFFUSE, color);
+		glLightfv(GL_LIGHT0, GL_SPECULAR, color);
+
+		// Setup light direction ...
+		const GLfloat position[4] = { 0.0f, 0.0f, 1.0f, 0.0f };
+		glLightfv(GL_LIGHT0, GL_POSITION, position);
+	}
+}
+
 } // namespace detail
 
 /////////////////////////////////////////////////////////////////////////////
@@ -135,7 +265,7 @@ class render_engine :
 public:
 	render_engine(k3d::idocument& Document) :
 		base(Document),
-		m_point_size(init_owner(*this) + init_name("point_size") + init_label(_("Point Size")) + init_description(_("OpengGL point size")) + init_value(4) + init_precision(1) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
+		m_point_size(init_owner(*this) + init_name("point_size") + init_label(_("Point Size")) + init_description(_("OpengGL point size")) + init_value(4) + init_constraint(constraint::minimum(0.0)) + init_precision(1) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
 		m_background_color(init_owner(*this) + init_name("background_color") + init_label(_("Background Color")) + init_description(_("Background color")) + init_value(k3d::color(0.8, 0.8, 0.8))),
 		m_fog(init_owner(*this) + init_name("fog") + init_label(_("Fog")) + init_description(_("Fog")) + init_value(false)),
 		m_fog_near(init_owner(*this) + init_name("fog_near") + init_label(_("Fog Near")) + init_description(_("Fog near distance")) + init_value(0.0) + init_constraint(constraint::minimum(0.0)) + init_precision(1) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::distance))),
@@ -152,8 +282,7 @@ public:
 		m_draw_bicubic_patches(init_owner(*this) + init_name("draw_bicubic_patches") + init_label(_("Draw Bicubic Patches")) + init_description(_("Draw bicubic patches")) + init_value(true)),
 		m_draw_nupatches(init_owner(*this) + init_name("draw_nupatches") + init_label(_("Draw NURBS Patches")) + init_description(_("Draw NURBS patches")) + init_value(true)),
 		m_draw_blobbies(init_owner(*this) + init_name("draw_blobbies") + init_label(_("Draw Blobbies")) + init_description(_("Draw RenderMan blobbies")) + init_value(true)),
-		m_draw_face_orientations(init_owner(*this) + init_name("draw_face_orientations") + init_label(_("Draw Face Orientations")) + init_description(_("Face orientation starts from first edge")) + init_value(false)),
-		m_draw_two_sided(init_owner(*this) + init_name("draw_two_sided") + init_label(_("Draw Two Sided")) + init_description(_("Draw two sided")) + init_value(false)),
+		m_draw_two_sided(init_owner(*this) + init_name("draw_two_sided") + init_label(_("Draw Two Sided")) + init_description(_("Draw two sided")) + init_value(true)),
 		m_draw_safe_zone(init_owner(*this) + init_name("draw_safe_zone") + init_label(_("Draw Safe Zone")) + init_description(_("Draw Safe Zone")) + init_value(false)),
 		m_draw_aimpoint(init_owner(*this) + init_name("draw_aimpoint") + init_label(_("Draw Aim Point")) + init_description(_("Draw center screen cross")) + init_value(true)),
 		m_draw_crop_window(init_owner(*this) + init_name("draw_crop_window") + init_label(_("Draw Crop Window")) + init_description(_("Draw bounding rectangle for output rendering")) + init_value(true)),
@@ -170,7 +299,6 @@ public:
 		visibility_group.properties.push_back(&static_cast<k3d::iproperty&>(m_draw_bicubic_patches));
 		visibility_group.properties.push_back(&static_cast<k3d::iproperty&>(m_draw_nupatches));
 		visibility_group.properties.push_back(&static_cast<k3d::iproperty&>(m_draw_blobbies));
-		visibility_group.properties.push_back(&static_cast<k3d::iproperty&>(m_draw_face_orientations));
 		visibility_group.properties.push_back(&static_cast<k3d::iproperty&>(m_draw_two_sided));
 		visibility_group.properties.push_back(&static_cast<k3d::iproperty&>(m_draw_safe_zone));
 		visibility_group.properties.push_back(&static_cast<k3d::iproperty&>(m_draw_aimpoint));
@@ -179,29 +307,33 @@ public:
 
 		register_property_group(visibility_group);
 
-		m_point_size.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_background_color.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_fog.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_fog_near.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_fog_far.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_headlight.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_show_lights.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_points.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_edges.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_faces.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_linear_curves.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_cubic_curves.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_nucurves.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_bilinear_patches.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_bicubic_patches.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_nupatches.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_blobbies.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_face_orientations.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_two_sided.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_safe_zone.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_aimpoint.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_crop_window.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
-		m_draw_frustum.changed_signal().connect(sigc::bind(m_redraw_request_signal.make_slot(), k3d::gl::irender_engine::ASYNCHRONOUS));
+		m_point_size.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_background_color.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_fog.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_fog_near.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_fog_far.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_headlight.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_show_lights.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_points.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_edges.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_faces.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_linear_curves.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_cubic_curves.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_nucurves.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_bilinear_patches.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_bicubic_patches.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_nupatches.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_blobbies.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_two_sided.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_safe_zone.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_aimpoint.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_crop_window.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+		m_draw_frustum.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_redraw));
+	}
+
+	void on_redraw(k3d::iunknown*)
+	{
+		m_redraw_request_signal.emit(k3d::gl::irender_engine::ASYNCHRONOUS);
 	}
 
 	k3d::iplugin_factory& factory()
@@ -329,209 +461,11 @@ public:
 
 	void redraw(k3d::icamera& Camera, const unsigned long PixelWidth, const unsigned long PixelHeight, const unsigned long FontListBase, GLdouble ViewMatrix[16], GLdouble ProjectionMatrix[16], GLint Viewport[4])
 	{
-		// If width or height are zero, we're done ...
-		if(!PixelWidth || !PixelHeight)
+		k3d::gl::render_state state(Camera);
+		k3d::rectangle unused(0, 0, 0, 0);
+		if(!draw_scene(Camera, PixelWidth, PixelHeight, FontListBase, ViewMatrix, ProjectionMatrix, Viewport, false, unused, state))
 			return;
 
-		// Clear background ...
-		glClearDepth(1.0);
-		const k3d::color background_color(m_background_color.value());
-		glClearColor(
-			static_cast<GLfloat>(background_color.red),
-			static_cast<GLfloat>(background_color.green),
-			static_cast<GLfloat>(background_color.blue),
-			0.0f);
-		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-
-		// Set point size ...
-		glPointSize(m_point_size.value());
-
-		// Setup stippling for path normals / bezier control lines ...
-		glLineStipple(2, 0xaaaa);
-		glDisable(GL_LINE_STIPPLE);
-		glDisable(GL_POLYGON_STIPPLE);
-
-		// Setup culling ...
-		glFrontFace(GL_CW);
-		glCullFace(GL_BACK);
-
-		// Setup dithering ...
-		glDisable(GL_DITHER);
-
-		// Setup antialiasing ...
-		glDisable(GL_LINE_SMOOTH);
-		glDisable(GL_BLEND);
-		glLineWidth(1.0f);
-
-		// Disable stencil ...
-		glDisable(GL_STENCIL_TEST);
-		glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
-		glStencilMask(0x00);
-
-		// Set Z buffer options ...
-		glDepthMask(GL_TRUE);
-		glDepthFunc(GL_LESS);
-		glEnable(GL_DEPTH_TEST);
-
-		// Normalization ...
-		glShadeModel(GL_SMOOTH);
-		glEnable(GL_NORMALIZE);
-		glEnable(GL_AUTO_NORMAL);
-
-		// Prepare texture options ...
-		glDisable(GL_TEXTURE_2D);
-
-		// Setup texture matrix
-		glMatrixMode(GL_TEXTURE);
-		glLoadIdentity();
-
-		// Cache render state to pass to clients ...
-		k3d::gl::render_state state;
-		state.draw_points = m_draw_points.value();
-		state.draw_edges = m_draw_edges.value();
-		state.draw_faces = m_draw_faces.value();
-		state.draw_linear_curves = m_draw_linear_curves.value();
-		state.draw_cubic_curves = m_draw_cubic_curves.value();
-		state.draw_nucurves = m_draw_nucurves.value();
-		state.draw_bilinear_patches = m_draw_bilinear_patches.value();
-		state.draw_bicubic_patches = m_draw_bicubic_patches.value();
-		state.draw_nupatches = m_draw_nupatches.value();
-		state.draw_blobbies = m_draw_blobbies.value();
-		state.draw_face_orientations = m_draw_face_orientations.value();
-		state.draw_two_sided = m_draw_two_sided.value();
-		state.gl_ascii_font_list_base = FontListBase;
-
-		// Setup viewport ...
-		glViewport(0, 0, PixelWidth, PixelHeight);
-		glGetIntegerv(GL_VIEWPORT, state.gl_viewport);
-		glGetIntegerv(GL_VIEWPORT, Viewport);
-
-		k3d::rectangle window_rect(0, 0, 0, 0);
-		k3d::rectangle camera_rect(0, 0, 0, 0);
-		double near = 0;
-		double far = 0;
-		bool orthographic = false;
-		calculate_projection(Camera, PixelWidth, PixelHeight, window_rect, camera_rect, near, far, orthographic);
-
-		// Set ourselves up to draw 2D overlays ...
-		glMatrixMode(GL_PROJECTION);
-		glLoadIdentity();
-		glOrtho(window_rect.left, window_rect.right, window_rect.bottom, window_rect.top, -1, 1);
-
-		glMatrixMode(GL_MODELVIEW);
-		glLoadIdentity();
-
-		if(m_draw_frustum.value())
-		{
-			glColor4d(0, 0, 0, 1);
-			glDisable(GL_LIGHTING);
-
-			glBegin(GL_LINE_LOOP);
-				glVertex2d(camera_rect.left, camera_rect.top);
-				glVertex2d(camera_rect.right, camera_rect.top);
-				glVertex2d(camera_rect.right, camera_rect.bottom);
-				glVertex2d(camera_rect.left, camera_rect.bottom);
-			glEnd();
-		}
-
-		// Optionally show the extents of the crop window
-		if(m_draw_crop_window.value())
-		{
-			const double left = boost::any_cast<double>(k3d::get_value(document().dag(), Camera.crop_window().crop_left()));
-			const double right = boost::any_cast<double>(k3d::get_value(document().dag(), Camera.crop_window().crop_right()));
-			const double bottom = boost::any_cast<double>(k3d::get_value(document().dag(), Camera.crop_window().crop_bottom()));
-			const double top = boost::any_cast<double>(k3d::get_value(document().dag(), Camera.crop_window().crop_top()));
-
-			glColor4d(0, 0, 0.5, 1);
-			glDisable(GL_LIGHTING);
-
-			glBegin(GL_LINE_LOOP);
-				glVertex2d(k3d::mix(camera_rect.left, camera_rect.right, left), k3d::mix(camera_rect.top, camera_rect.bottom, top));
-				glVertex2d(k3d::mix(camera_rect.left, camera_rect.right, right), k3d::mix(camera_rect.top, camera_rect.bottom, top));
-				glVertex2d(k3d::mix(camera_rect.left, camera_rect.right, right), k3d::mix(camera_rect.top, camera_rect.bottom, bottom));
-				glVertex2d(k3d::mix(camera_rect.left, camera_rect.right, left), k3d::mix(camera_rect.top, camera_rect.bottom, bottom));
-			glEnd();
-		}
-
-		// Optionally draw a safe zone for video ...
-		if(m_draw_safe_zone.value())
-		{
-			glColor4d(0, 0, 0, 1);
-			glDisable(GL_LIGHTING);
-
-			glBegin(GL_LINE_LOOP);
-				glVertex2d(camera_rect.left * 0.9, camera_rect.top * 0.9);
-				glVertex2d(camera_rect.right * 0.9, camera_rect.top * 0.9);
-				glVertex2d(camera_rect.right * 0.9, camera_rect.bottom * 0.9);
-				glVertex2d(camera_rect.left * 0.9, camera_rect.bottom * 0.9);
-			glEnd();
-		}
-
-		// Optionally draw the "aimpoint" (crosshairs aligned with the center of the viewport)
-		if(m_draw_aimpoint.value())
-		{
-			glColor4d(0, 0, 0, 1);
-			glDisable(GL_LIGHTING);
-
-			glBegin(GL_LINES);
-				glVertex2d(camera_rect.left * 0.05, 0);
-				glVertex2d(camera_rect.right * 0.05, 0);
-				glVertex2d(0, camera_rect.top * 0.05);
-				glVertex2d(0, camera_rect.bottom * 0.05);
-			glEnd();
-		}
-
-		// Setup projection ...
-		glMatrixMode(GL_PROJECTION);
-		glLoadIdentity();
-		if(orthographic)
-			glOrtho(window_rect.left, window_rect.right, window_rect.bottom, window_rect.top, near, far);
-		else
-			glFrustum(window_rect.left, window_rect.right, window_rect.bottom, window_rect.top, near, far);
-		glGetFloatv(GL_PROJECTION_MATRIX, state.gl_projection_matrix);
-		glGetDoublev(GL_PROJECTION_MATRIX, ProjectionMatrix);
-
-		// Setup modelview matrix
-		glMatrixMode(GL_MODELVIEW);
-		glLoadIdentity();
-
-		// Setup lights ...
-		glEnable(GL_LIGHTING);
-		glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
-		glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
-
-		// Make sure all lights are turned off initially ...
-		GLint maxlights = 0;
-		glGetIntegerv(GL_MAX_LIGHTS, &maxlights);
-		for(GLint i = 0; i < maxlights; ++i)
-			glDisable(GLenum(GL_LIGHT0 + i));
-
-		// Setup the headlight ...
-		if(m_headlight.value())
-		{
-			// Enable the light ...
-			glEnable(GL_LIGHT0);
-
-			// Setup color ...
-			const GLfloat color[4] = { 1.0f, 1.0f, 1.0f, 1.0f };
-			glLightfv(GL_LIGHT0, GL_AMBIENT, color);
-			glLightfv(GL_LIGHT0, GL_DIFFUSE, color);
-			glLightfv(GL_LIGHT0, GL_SPECULAR, color);
-
-			// Setup light direction ...
-			const GLfloat position[4] = { 0.0f, 0.0f, 1.0f, 0.0f };
-			glLightfv(GL_LIGHT0, GL_POSITION, position);
-		}
-
-		const k3d::matrix4 transform_matrix = boost::any_cast<k3d::matrix4>(get_value(document().dag(), Camera.transformation().transform_source_output()));
-		const k3d::angle_axis orientation(k3d::euler_angles(transform_matrix, k3d::euler_angles::ZXYstatic));
-		const k3d::vector3 position(k3d::extract_translation(transform_matrix));
-
-		glScaled(1.0, 1.0, -1.0);
-		glRotated(-k3d::degrees(orientation.angle), orientation.axis[0], orientation.axis[1], orientation.axis[2]);
-		glTranslated(-position[0], -position[1], -position[2]);
-		glGetDoublev(GL_MODELVIEW_MATRIX, ViewMatrix);
-
 		// Setup fog ...
 		if(m_fog.value())
 		{
@@ -560,8 +494,7 @@ public:
 
 		std::for_each(document().nodes().collection().begin(), document().nodes().collection().end(), detail::draw(state));
 
-
-/* I really hate to lose this feedback, but the GLU NURBS routines generate large numbers of errors, which ruins its utility :-(
+/* I really hate to loose this feedback, but the GLU NURBS routines generate large numbers of errors, which ruins its utility :-(
 		for(GLenum gl_error = glGetError(); gl_error != GL_NO_ERROR; gl_error = glGetError())
 			k3d::log() << error << "OpenGL error: " << reinterpret_cast<const char*>(gluErrorString(gl_error)) << std::endl;
 */
@@ -573,10 +506,28 @@ public:
 	}
 
 	void select(const k3d::gl::select_state& SelectState, k3d::icamera& Camera, const unsigned long PixelWidth, const unsigned long PixelHeight, const unsigned long FontListBase, const k3d::rectangle& Region, GLdouble ViewMatrix[16], GLdouble ProjectionMatrix[16], GLint Viewport[4])
+	{
+		k3d::gl::render_state state(Camera);
+		if(!draw_scene(Camera, PixelWidth, PixelHeight, FontListBase, ViewMatrix, ProjectionMatrix, Viewport, true, Region, state))
+			return;
+
+		// Clear background ...
+		glClear(GL_DEPTH_BUFFER_BIT);
+
+		glDisable(GL_LIGHTING);
+
+		std::for_each(document().nodes().collection().begin(), document().nodes().collection().end(), detail::draw_selection(state, SelectState));
+	}
+
+private:
+	bool draw_scene(k3d::icamera& Camera, const unsigned long PixelWidth, const unsigned long PixelHeight, const unsigned long FontListBase, GLdouble ViewMatrix[16], GLdouble ProjectionMatrix[16], GLint Viewport[4], const bool Select, const k3d::rectangle& SelectionRegion, k3d::gl::render_state& RenderState)
 	{
 		// If width or height are zero, we're done ...
 		if(!PixelWidth || !PixelHeight)
-			return;
+			return false;
+
+		if(!Select)
+			detail::gl_reset(m_background_color.value(), m_point_size.value());
 
 		// Setup culling ...
 		glFrontFace(GL_CW);
@@ -601,31 +552,33 @@ public:
 		glEnable(GL_DEPTH_TEST);
 
 		// Normalization ...
-		glShadeModel(GL_FLAT);
-		glDisable(GL_NORMALIZE);
-		glDisable(GL_AUTO_NORMAL);
+		glShadeModel(Select ? GL_FLAT : GL_SMOOTH);
+		glEnable(GL_NORMALIZE);
+		glEnable(GL_AUTO_NORMAL);
 
+		// Prepare texture options ...
 		glDisable(GL_TEXTURE_2D);
 
+		if(!Select)
+			detail::gl_setup_textures();
+
 		// Cache render state to pass to clients ...
-		k3d::gl::render_state state;
-		state.draw_points = m_draw_points.value();
-		state.draw_edges = m_draw_edges.value();
-		state.draw_faces = m_draw_faces.value();
-		state.draw_linear_curves = m_draw_linear_curves.value();
-		state.draw_cubic_curves = m_draw_cubic_curves.value();
-		state.draw_nucurves = m_draw_nucurves.value();
-		state.draw_bilinear_patches = m_draw_bilinear_patches.value();
-		state.draw_bicubic_patches = m_draw_bicubic_patches.value();
-		state.draw_nupatches = m_draw_nupatches.value();
-		state.draw_blobbies = m_draw_blobbies.value();
-		state.draw_face_orientations = m_draw_face_orientations.value();
-		state.draw_two_sided = m_draw_two_sided.value();
-		state.gl_ascii_font_list_base = FontListBase;
+		RenderState.draw_points = m_draw_points.value();
+		RenderState.draw_edges = m_draw_edges.value();
+		RenderState.draw_faces = m_draw_faces.value();
+		RenderState.draw_linear_curves = m_draw_linear_curves.value();
+		RenderState.draw_cubic_curves = m_draw_cubic_curves.value();
+		RenderState.draw_nucurves = m_draw_nucurves.value();
+		RenderState.draw_bilinear_patches = m_draw_bilinear_patches.value();
+		RenderState.draw_bicubic_patches = m_draw_bicubic_patches.value();
+		RenderState.draw_nupatches = m_draw_nupatches.value();
+		RenderState.draw_blobbies = m_draw_blobbies.value();
+		RenderState.draw_two_sided = m_draw_two_sided.value();
+		RenderState.gl_ascii_font_list_base = FontListBase;
 
 		// Setup viewport ...
 		glViewport(0, 0, PixelWidth, PixelHeight);
-		glGetIntegerv(GL_VIEWPORT, state.gl_viewport);
+		glGetIntegerv(GL_VIEWPORT, RenderState.gl_viewport);
 		glGetIntegerv(GL_VIEWPORT, Viewport);
 
 		k3d::rectangle window_rect(0, 0, 0, 0);
@@ -635,34 +588,113 @@ public:
 		bool orthographic = false;
 		calculate_projection(Camera, PixelWidth, PixelHeight, window_rect, camera_rect, near, far, orthographic);
 
-		// First, setup the projection matrix as we would normally do for drawing (so we can store the projection matrix, unaffected by the pick matrix) ...
+		if(!Select)
+			detail::gl_draw_2d_widgets(Camera, window_rect, camera_rect, m_draw_frustum.value(), m_draw_crop_window.value(), m_draw_safe_zone.value(), m_draw_aimpoint.value(), document());
+
+		// Setup projection ...
 		glMatrixMode(GL_PROJECTION);
 		glLoadIdentity();
 		if(orthographic)
-			glOrtho(window_rect.left, window_rect.right, window_rect.bottom, window_rect.top, near, far);
+		{
+			const k3d::matrix4 transform_matrix = boost::any_cast<k3d::matrix4>(get_value(document().dag(), Camera.transformation().transform_source_output()));
+			const k3d::point3 world_position = transform_matrix * k3d::point3(0, 0, 0);
+			const k3d::point3 world_target = boost::any_cast<k3d::point3>(Camera.world_target().property_value());
+			const double distance = k3d::distance(world_position, world_target);
+
+			const double window_aspect = (window_rect.right - window_rect.left) / (window_rect.top - window_rect.bottom);
+			const double window_tan_fov = (window_rect.top - window_rect.bottom) * 0.5 / near;
+			const double window_size = distance * window_tan_fov;
+
+			RenderState.orthographic = true;
+			
+			RenderState.gl_window_frustum_left = -window_size * window_aspect;
+			RenderState.gl_window_frustum_right = window_size * window_aspect;
+			RenderState.gl_window_frustum_top = window_size;
+			RenderState.gl_window_frustum_bottom = -window_size;
+			RenderState.gl_window_frustum_near = near;
+			RenderState.gl_window_frustum_far = far;
+			
+			const double camera_aspect = (camera_rect.right - camera_rect.left) / (camera_rect.top - camera_rect.bottom);
+
+			if(camera_aspect > window_aspect)
+			{
+				RenderState.gl_camera_frustum_left = RenderState.gl_window_frustum_left;
+				RenderState.gl_camera_frustum_right = RenderState.gl_window_frustum_right;
+				RenderState.gl_camera_frustum_top = RenderState.gl_window_frustum_top * window_aspect / camera_aspect;
+				RenderState.gl_camera_frustum_bottom = RenderState.gl_window_frustum_bottom * window_aspect / camera_aspect;
+				RenderState.gl_camera_frustum_near = near;
+				RenderState.gl_camera_frustum_far = far;
+			}
+			else
+			{
+				RenderState.gl_camera_frustum_left = RenderState.gl_window_frustum_left * camera_aspect / window_aspect;
+				RenderState.gl_camera_frustum_right = RenderState.gl_window_frustum_right * camera_aspect / window_aspect;
+				RenderState.gl_camera_frustum_top = RenderState.gl_window_frustum_top;
+				RenderState.gl_camera_frustum_bottom = RenderState.gl_window_frustum_bottom;
+				RenderState.gl_camera_frustum_near = near;
+				RenderState.gl_camera_frustum_far = far;
+			}
+
+			glOrtho(-window_size * window_aspect, window_size * window_aspect, -window_size, window_size, near, far);
+
+			if(Select)
+			{
+				// Setup the projection matrix with the pick matrix ...
+				glLoadIdentity();
+
+				const double width  = SelectionRegion.width();
+				const double height = SelectionRegion.height();
+				gluPickMatrix(SelectionRegion.left + (width * 0.5), RenderState.gl_viewport[3] - (SelectionRegion.top + (height * 0.5)), width, height, RenderState.gl_viewport);
+
+				glOrtho(-window_size * window_aspect, window_size * window_aspect, -window_size, window_size, near, far);
+			}
+		}
 		else
+		{
+			RenderState.orthographic = false;
+			
+			RenderState.gl_window_frustum_left = window_rect.left;
+			RenderState.gl_window_frustum_right = window_rect.right;
+			RenderState.gl_window_frustum_top = window_rect.top;
+			RenderState.gl_window_frustum_bottom = window_rect.bottom;
+			RenderState.gl_window_frustum_near = near;
+			RenderState.gl_window_frustum_far = far;
+
+			RenderState.gl_camera_frustum_left = camera_rect.left;
+			RenderState.gl_camera_frustum_right = camera_rect.right;
+			RenderState.gl_camera_frustum_top = camera_rect.top;
+			RenderState.gl_camera_frustum_bottom = camera_rect.bottom;
+			RenderState.gl_camera_frustum_near = near;
+			RenderState.gl_camera_frustum_far = far;
+
 			glFrustum(window_rect.left, window_rect.right, window_rect.bottom, window_rect.top, near, far);
-		glGetFloatv(GL_PROJECTION_MATRIX, state.gl_projection_matrix);
+
+			if(Select)
+			{
+				// Setup the projection matrix with the pick matrix ...
+				glLoadIdentity();
+
+				const double width  = SelectionRegion.width();
+				const double height = SelectionRegion.height();
+				gluPickMatrix(SelectionRegion.left + (width * 0.5), RenderState.gl_viewport[3] - (SelectionRegion.top + (height * 0.5)), width, height, RenderState.gl_viewport);
+
+				glFrustum(window_rect.left, window_rect.right, window_rect.bottom, window_rect.top, near, far);
+			}
+		}
+
+		glGetFloatv(GL_PROJECTION_MATRIX, RenderState.gl_projection_matrix);
 		glGetDoublev(GL_PROJECTION_MATRIX, ProjectionMatrix);
 
-		// Second, setup the projection matrix with the pick matrix ...
-		glLoadIdentity();
-
-		const double width  = Region.width();
-		const double height = Region.height();
-		gluPickMatrix(Region.left + (width * 0.5), state.gl_viewport[3] - (Region.top + (height * 0.5)), width, height, state.gl_viewport);
-		if(orthographic)
-			glOrtho(window_rect.left, window_rect.right, window_rect.bottom, window_rect.top, near, far);
-		else
-			glFrustum(window_rect.left, window_rect.right, window_rect.bottom, window_rect.top, near, far);
-
 		// Setup modelview matrix
 		glMatrixMode(GL_MODELVIEW);
 		glLoadIdentity();
 
+		if(!Select)
+			detail::gl_setup_lights(m_headlight.value());
+
 		const k3d::matrix4 transform_matrix = boost::any_cast<k3d::matrix4>(get_value(document().dag(), Camera.transformation().transform_source_output()));
 		const k3d::angle_axis orientation(k3d::euler_angles(transform_matrix, k3d::euler_angles::ZXYstatic));
-		const k3d::vector3 position(k3d::extract_translation(transform_matrix));
+		const k3d::point3 position(k3d::position(transform_matrix));
 
 		glScaled(1.0, 1.0, -1.0);
 		glRotated(-k3d::degrees(orientation.angle), orientation.axis[0], orientation.axis[1], orientation.axis[2]);
@@ -670,23 +702,12 @@ public:
 
 		glGetDoublev(GL_MODELVIEW_MATRIX, ViewMatrix);
 
-		// Clear background ...
-		glClear(GL_DEPTH_BUFFER_BIT);
-
-		glDisable(GL_LIGHTING);
-
-		std::for_each(document().nodes().collection().begin(), document().nodes().collection().end(), detail::draw_selection(state, SelectState));
-
-/* I really hate to lose this feedback, but the glu NURBS routines generate large numbers of errors, which ruins its utility :-(
-		for(GLenum gl_error = glGetError(); gl_error != GL_NO_ERROR; gl_error = glGetError())
-			k3d::log() << error << "OpenGL error: " << reinterpret_cast<const char*>(gluErrorString(gl_error)) << std::endl;
-*/
+		return true;
 	}
 
-private:
 	redraw_request_signal_t m_redraw_request_signal;
 
-	k3d_data(long, immutable_name, change_signal, with_undo, local_storage, no_constraint, measurement_property, with_serialization) m_point_size;
+	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, measurement_property, with_serialization) m_point_size;
 	k3d_data(k3d::color, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_background_color;
 	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_fog;
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, with_constraint, measurement_property, with_serialization) m_fog_near;
@@ -703,7 +724,6 @@ private:
 	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_draw_bicubic_patches;
 	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_draw_nupatches;
 	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_draw_blobbies;
-	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_draw_face_orientations;
 	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_draw_two_sided;
 	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_draw_safe_zone;
 	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_draw_aimpoint;
@@ -712,17 +732,12 @@ private:
 };
 
 /////////////////////////////////////////////////////////////////////////////
-// viewport_factory
+// render_engine_factory
 
-k3d::iplugin_factory& viewport_factory()
+k3d::iplugin_factory& render_engine_factory()
 {
 	return render_engine::get_factory();
 }
 
 } // namespace libk3dopengl
 
-K3D_MODULE_START(libk3dopengl, k3d::uuid(0xae701c78, 0x095243c1, 0xa3739654, 0xe43c80db), Registry)
-	Registry.register_factory(libk3dopengl::viewport_factory());
-K3D_MODULE_END
-
-
