@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 2004-2005, Romain Behar
+// Copyright (c) 2004-2006, Romain Behar
 //
 // Contact: romainbehar@yahoo.com
 //
@@ -25,16 +25,18 @@
 #include <k3dsdk/classes.h>
 #include <k3dsdk/color.h>
 #include <k3dsdk/frames.h>
+#include <k3dsdk/fstream.h>
 #include <k3dsdk/gl.h>
 #include <k3dsdk/i18n.h>
-#include <k3dsdk/ianimation_render_engine.h>
 #include <k3dsdk/icamera.h>
+#include <k3dsdk/icamera_animation_render_engine.h>
+#include <k3dsdk/icamera_preview_render_engine.h>
+#include <k3dsdk/icamera_still_render_engine.h>
 #include <k3dsdk/imaterial.h>
 #include <k3dsdk/imaterial_client.h>
 #include <k3dsdk/imesh_source.h>
 #include <k3dsdk/iprojection.h>
 #include <k3dsdk/irender_farm.h>
-#include <k3dsdk/istill_render_engine.h>
 #include <k3dsdk/itransform_source.h>
 #include <k3dsdk/measurement.h>
 #include <k3dsdk/mesh.h>
@@ -42,14 +44,13 @@
 #include <k3dsdk/node.h>
 #include <k3dsdk/persistent.h>
 #include <k3dsdk/property.h>
+#include <k3dsdk/render_farm.h>
 #include <k3dsdk/resolutions.h>
 #include <k3dsdk/time_source.h>
 #include <k3dsdk/transform.h>
 
-#include <k3dsdk/fstream.h>
 #include <boost/filesystem/path.hpp>
 
-#include <fstream>
 #include <iomanip>
 #include <iterator>
 
@@ -77,7 +78,7 @@ inline bool operator < (const k3d::color& a, const k3d::color& b)
 namespace libk3dpov
 {
 
-void pov_point(const k3d::vector3& point, std::ostream& stream)
+void pov_point(const k3d::point3& point, std::ostream& stream)
 {
 	stream << "<" << point[0] << ", " << point[1] << ", " << point[2] << ">";
 }
@@ -87,8 +88,9 @@ void pov_point(const k3d::vector3& point, std::ostream& stream)
 
 class render_engine :
 	public k3d::persistent<k3d::node>,
-	public k3d::istill_render_engine,
-	public k3d::ianimation_render_engine
+	public k3d::icamera_preview_render_engine,
+	public k3d::icamera_still_render_engine,
+	public k3d::icamera_animation_render_engine
 {
 	typedef k3d::persistent<k3d::node> base;
 
@@ -102,7 +104,7 @@ public:
 		m_resolution.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_resolution_changed));
 	}
 
-	void on_resolution_changed()
+	void on_resolution_changed(k3d::iunknown*)
 	{
 		const std::string new_resolution = m_resolution.value();
 
@@ -120,10 +122,10 @@ public:
 		assert_not_reached();
 	}
 
-	bool render_preview(k3d::icamera& Camera)
+	bool render_camera_preview(k3d::icamera& Camera)
 	{
 		// Start a new render job ...
-		k3d::irender_job& job = k3d::application().render_farm().create_job("k3d-preview");
+		k3d::irender_job& job = k3d::render_farm().create_job("k3d-preview");
 
 		// Add a single render frame to the job ...
 		k3d::irender_frame& frame = job.create_frame("frame");
@@ -139,18 +141,18 @@ public:
 		return_val_if_fail(render(Camera, frame, outputimagepath, true), false);
 
 		// Start the job running ...
-		k3d::application().render_farm().start_job(job);
+		k3d::render_farm().start_job(job);
 
 		return true;
 	}
 
-	bool render_frame(k3d::icamera& Camera, const boost::filesystem::path& OutputImage, const bool ViewImage)
+	bool render_camera_frame(k3d::icamera& Camera, const boost::filesystem::path& OutputImage, const bool ViewImage)
 	{
 		// Sanity checks ...
 		return_val_if_fail(!OutputImage.empty(), false);
 
 		// Start a new render job ...
-		k3d::irender_job& job = k3d::application().render_farm().create_job("k3d-render-frame");
+		k3d::irender_job& job = k3d::render_farm().create_job("k3d-render-frame");
 
 		// Add a single render frame to the job ...
 		k3d::irender_frame& frame = job.create_frame("frame");
@@ -170,12 +172,12 @@ public:
 		return_val_if_fail(render(Camera, frame, outputimagepath, false), false);
 
 		// Start the job running ...
-		k3d::application().render_farm().start_job(job);
+		k3d::render_farm().start_job(job);
 
 		return true;
 	}
 
-	bool render_animation(k3d::icamera& Camera, const boost::filesystem::path& OutputImages, const bool ViewCompletedImages)
+	bool render_camera_animation(k3d::icamera& Camera, const boost::filesystem::path& OutputImages, const bool ViewCompletedImages)
 	{
 		// Sanity checks ...
 		return_val_if_fail(!OutputImages.empty(), false);
@@ -199,7 +201,7 @@ public:
 		return_val_if_fail(frames.max_frame() >= end_frame, false);
 
 		// Start a new render job ...
-		k3d::irender_job& job = k3d::application().render_farm().create_job("k3d-render-animation");
+		k3d::irender_job& job = k3d::render_farm().create_job("k3d-render-animation");
 
 		// For each frame to be rendered ...
 		for(long view_frame = start_frame; view_frame < end_frame; ++view_frame)
@@ -233,7 +235,7 @@ public:
 		}
 
 		// Start the job running ...
-		k3d::application().render_farm().start_job(job);
+		k3d::render_farm().start_job(job);
 
 		return true;
 	}
@@ -247,8 +249,9 @@ public:
 	{
 		static k3d::plugin_factory<
 			k3d::document_plugin<render_engine>,
-			k3d::interface_list<k3d::ianimation_render_engine,
-			k3d::interface_list<k3d::istill_render_engine > > > factory(
+			k3d::interface_list<k3d::icamera_preview_render_engine,
+			k3d::interface_list<k3d::icamera_still_render_engine,
+			k3d::interface_list<k3d::icamera_animation_render_engine> > > > factory(
 				k3d::uuid(0x7982ee23, 0x854a43e7, 0x874c8ad7, 0x5d0b95d2),
 				"POVEngine",
 				"POV Render Engine",
@@ -332,7 +335,7 @@ private:
 /*
 			if((*object)->factory().class_id() == k3d::classes::Sphere())
 			{
-				const k3d::vector3 sphere_center = k3d::node_to_world_matrix(**object) * k3d::vector3(0, 0, 0);
+				const k3d::point3 sphere_center = k3d::node_to_world_matrix(**object) * k3d::point3(0, 0, 0);
 				const boost::any sphere_radius(k3d::get_value(**object, "radius"));
 				if(typeid(double) == sphere_radius.type())
 				{
@@ -404,7 +407,7 @@ private:
 				for(k3d::mesh::points_t::const_iterator point = all_points.begin(); point != all_points.end(); ++point)
 				{
 					point_map.insert(std::make_pair(*point, point_map.size()));
-					const k3d::vector3 position = mesh_matrix * (*point)->position;
+					const k3d::point3 position = mesh_matrix * (*point)->position;
 					stream << "  "; pov_point(position, stream); stream << "," << std::endl;
 				}
 				stream << " }" << std::endl;
@@ -478,7 +481,7 @@ private:
 		// Setup the camera ...
 		const k3d::matrix4 transform_matrix = boost::any_cast<k3d::matrix4>(get_value(document().dag(), Camera.transformation().transform_source_output()));
 		const k3d::euler_angles camera_angles = k3d::euler_angles(transform_matrix, k3d::euler_angles::XYZstatic);
-		const k3d::vector3 camera_position(k3d::extract_translation(transform_matrix));
+		const k3d::point3 camera_position(k3d::position(transform_matrix));
 
 		const double pixel_width = static_cast<double>(m_pixel_width.value());
 		const double pixel_height = static_cast<double>(m_pixel_height.value());
