@@ -965,142 +965,137 @@ public:
 		m_type(init_owner(*this) + init_name("type") + init_label(_("Primitive")) + init_description(_("Primitive type (teapot, teacup or teaspoon")) + init_value(TEAPOT) + init_enumeration(type_values())),
 		m_size(init_owner(*this) + init_name("size") + init_label(_("Size")) + init_description(_("Primitive size (scale)")) + init_value(1.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_material.changed_signal().connect(sigc::mem_fun(*this, &newell_primitive_implementation::on_reset_geometry));
-
-		m_type.changed_signal().connect(sigc::mem_fun(*this, &newell_primitive_implementation::on_reset_geometry));
-		m_size.changed_signal().connect(sigc::mem_fun(*this, &newell_primitive_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &newell_primitive_implementation::on_create_geometry));
+		m_material.changed_signal().connect(make_reset_mesh_slot());
+		m_type.changed_signal().connect(make_reset_mesh_slot());
+		m_size.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		k3d::mesh* const mesh = new k3d::mesh();
-
 		const double size = m_size.value();
 		k3d::imaterial* const material = m_material.value();
 
 		// Creates Newell's primitives using his bezier patches ...
 		switch(m_type.value())
+		{
+			case TEAPOT:
 			{
-				case TEAPOT:
-					{
-						// Min : -3.0, max : 3.525 -> 20/6.525 = 3.065
-						for(unsigned long i = 0; i < 306; i++)
-							mesh->points.push_back(new k3d::point(size * k3d::vector3(TeapotPoints[i][0], TeapotPoints[i][1], TeapotPoints[i][2])));
+				// Min : -3.0, max : 3.525 -> 20/6.525 = 3.065
+				for(unsigned long i = 0; i < 306; i++)
+					Mesh.points.push_back(new k3d::point(size * k3d::point3(TeapotPoints[i][0], TeapotPoints[i][1], TeapotPoints[i][2])));
 
-						// Rim
-						for(unsigned long i = 0; i < 4; i++)
-							{
-								k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
-								patch->material = material;
+				// Rim
+				for(unsigned long i = 0; i < 4; i++)
+				{
+					k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
+					patch->material = material;
 
-								for(unsigned long j = 0; j < 16; j++)
-									patch->control_points[j] = mesh->points[TeapotPatches[i][j]-1];
+					for(unsigned long j = 0; j < 16; j++)
+						patch->control_points[j] = Mesh.points[TeapotPatches[i][j]-1];
 
-								mesh->bicubic_patches.push_back(patch);
-							}
-						// Body
-						for(unsigned long i = 4; i < 12; i++)
-							{
-								k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
-								patch->material = material;
+					Mesh.bicubic_patches.push_back(patch);
+				}
+				// Body
+				for(unsigned long i = 4; i < 12; i++)
+				{
+					k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
+					patch->material = material;
 
-								for(unsigned long j = 0; j < 16; j++)
-									patch->control_points[j] = mesh->points[TeapotPatches[i][j]-1];
+					for(unsigned long j = 0; j < 16; j++)
+						patch->control_points[j] = Mesh.points[TeapotPatches[i][j]-1];
 
-								mesh->bicubic_patches.push_back(patch);
-							}
-						// Handle
-						for(unsigned long i = 12; i < 16; i++)
-							{
-								k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
-								patch->material = material;
+					Mesh.bicubic_patches.push_back(patch);
+				}
+				// Handle
+				for(unsigned long i = 12; i < 16; i++)
+				{
+					k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
+					patch->material = material;
 
-								for(unsigned long j = 0; j < 16; j++)
-									patch->control_points[j] = mesh->points[TeapotPatches[i][j]-1];
+					for(unsigned long j = 0; j < 16; j++)
+						patch->control_points[j] = Mesh.points[TeapotPatches[i][j]-1];
 
-								mesh->bicubic_patches.push_back(patch);
-							}
-						// Spout
-						for(unsigned long i = 16; i < 20; i++)
-							{
-								k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
-								patch->material = material;
+					Mesh.bicubic_patches.push_back(patch);
+				}
+				// Spout
+				for(unsigned long i = 16; i < 20; i++)
+				{
+					k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
+					patch->material = material;
 
-								for(unsigned long j = 0; j < 16; j++)
-									patch->control_points[j] = mesh->points[TeapotPatches[i][j]-1];
+					for(unsigned long j = 0; j < 16; j++)
+						patch->control_points[j] = Mesh.points[TeapotPatches[i][j]-1];
 
-								mesh->bicubic_patches.push_back(patch);
-							}
-						// Lid
-						for(unsigned long i = 20; i < 28; i++)
-							{
-								k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
-								patch->material = material;
+					Mesh.bicubic_patches.push_back(patch);
+				}
+				// Lid
+				for(unsigned long i = 20; i < 28; i++)
+				{
+					k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
+					patch->material = material;
 
-								for(unsigned long j = 0; j < 16; j++)
-									patch->control_points[j] = mesh->points[TeapotPatches[i][j]-1];
+					for(unsigned long j = 0; j < 16; j++)
+						patch->control_points[j] = Mesh.points[TeapotPatches[i][j]-1];
 
-								mesh->bicubic_patches.push_back(patch);
-							}
-						// Bottom
-						for(unsigned long i = 28; i < 32; i++)
-							{
-								k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
-								patch->material = material;
+					Mesh.bicubic_patches.push_back(patch);
+				}
+				// Bottom
+				for(unsigned long i = 28; i < 32; i++)
+				{
+					k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
+					patch->material = material;
 
-								for(unsigned long j = 0; j < 16; j++)
-									patch->control_points[j] = mesh->points[TeapotPatches[i][j]-1];
+					for(unsigned long j = 0; j < 16; j++)
+						patch->control_points[j] = Mesh.points[TeapotPatches[i][j]-1];
 
-								mesh->bicubic_patches.push_back(patch);
-							}
-					}
-					break;
-				case TEACUP :
-					{
-						// Min : -1.0, max : 1.0
-						for(unsigned long i = 0; i < 251; i++)
-							mesh->points.push_back(new k3d::point(size * k3d::vector3(TeacupPoints[i][0], TeacupPoints[i][1], TeacupPoints[i][2])));
+					Mesh.bicubic_patches.push_back(patch);
+				}
 
-						for(unsigned long i = 0; i < 26; i++)
-							{
-								k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
-								patch->material = material;
-
-								for(unsigned long j = 0; j < 16; j++)
-									patch->control_points[j] = mesh->points[TeacupPatches[i][j]-1];
-
-								mesh->bicubic_patches.push_back(patch);
-							}
-					}
-					break;
-				case TEASPOON :
-					{
-						// Min : -1.0, max : 0.221 -> 20/1.221 = 16.38
-						for(unsigned long i = 0; i < 256; i++)
-							mesh->points.push_back(new k3d::point(size * k3d::vector3(TeaspoonPoints[i][0], TeaspoonPoints[i][1], TeaspoonPoints[i][2])));
-
-						for(unsigned long i = 0; i < 16; i++)
-							{
-								k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
-								patch->material = material;
-
-								for(unsigned long j = 0; j < 16; j++)
-									patch->control_points[j] = mesh->points[TeaspoonPatches[i][j]-1];
-
-								mesh->bicubic_patches.push_back(patch);
-							}
-					}
-					break;
+				break;
 			}
+			case TEACUP :
+			{
+				// Min : -1.0, max : 1.0
+				for(unsigned long i = 0; i < 251; i++)
+					Mesh.points.push_back(new k3d::point(size * k3d::point3(TeacupPoints[i][0], TeacupPoints[i][2], TeacupPoints[i][1])));
 
-		return mesh;
+				for(unsigned long i = 0; i < 26; i++)
+				{
+					k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
+					patch->material = material;
+
+					for(unsigned long j = 0; j < 16; j++)
+						patch->control_points[j] = Mesh.points[TeacupPatches[i][j]-1];
+
+					Mesh.bicubic_patches.push_back(patch);
+				}
+
+				break;
+			}
+			case TEASPOON :
+			{
+				// Min : -1.0, max : 0.221 -> 20/1.221 = 16.38
+				for(unsigned long i = 0; i < 256; i++)
+					Mesh.points.push_back(new k3d::point(size * k3d::point3(TeaspoonPoints[i][1], TeaspoonPoints[i][0], TeaspoonPoints[i][2])));
+
+				for(unsigned long i = 0; i < 16; i++)
+				{
+					k3d::bicubic_patch* const patch = new k3d::bicubic_patch();
+					patch->material = material;
+
+					for(unsigned long j = 0; j < 16; j++)
+						patch->control_points[j] = Mesh.points[TeaspoonPatches[i][j]-1];
+
+					Mesh.bicubic_patches.push_back(patch);
+				}
+
+				break;
+			}
+		}
+	}
+
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -1132,17 +1127,17 @@ private:
 	friend std::ostream& operator << (std::ostream& Stream, const primitive_t& Value)
 	{
 		switch(Value)
-			{
-				case TEAPOT:
-					Stream << "teapot";
-					break;
-				case TEACUP:
-					Stream << "teacup";
-					break;
-				case TEASPOON:
-					Stream << "teaspoon";
-					break;
-			}
+		{
+			case TEAPOT:
+				Stream << "teapot";
+				break;
+			case TEACUP:
+				Stream << "teacup";
+				break;
+			case TEASPOON:
+				Stream << "teaspoon";
+				break;
+		}
 		return Stream;
 	}
 
@@ -1193,4 +1188,3 @@ k3d::iplugin_factory& newell_primitive_factory()
 
 } // namespace libk3dprimitives
 
-
