@@ -134,7 +134,7 @@ void sphere(const unsigned long u_segments, const unsigned long v_segments, cons
 			x = radius * k3d::sign(x) * std::pow(std::abs(x), inv_u_power);
 			y = radius * k3d::sign(y) * std::pow(std::abs(y), inv_u_power);
 
-			Mesh->points[u_segments - u]->position = k3d::vector3(x, y, 0);
+			Mesh->points[u_segments - u]->position = k3d::point3(x, y, 0);
 		}
 
 		return;
@@ -292,7 +292,7 @@ void sphere(const unsigned long u_segments, const unsigned long v_segments, cons
 			x = varying_radius * k3d::sign(x) * std::pow(std::abs(x), inv_u_power);
 			y = varying_radius * k3d::sign(y) * std::pow(std::abs(y), inv_u_power);
 
-			points[v][u]->position = k3d::vector3(x, y, z);
+			points[v][u]->position = k3d::point3(x, y, z);
 		}
 	}
 }
@@ -355,7 +355,7 @@ void sphereized_cylinder(const unsigned long u_segments, const unsigned long v_s
 			x = varying_radius * k3d::sign(x) * std::pow(std::abs(x), inv_u_power);
 			y = varying_radius * k3d::sign(y) * std::pow(std::abs(y), inv_u_power);
 
-			points[v][u]->position = k3d::vector3(x, y, z);
+			points[v][u]->position = k3d::point3(x, y, z);
 		}
 	}
 }
@@ -380,23 +380,16 @@ public:
 		m_v_power(init_owner(*this) + init_name("v_power") + init_label(_("V power")) + init_description(_("Length Power")) + init_value(1.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::scalar))),
 		m_type(init_owner(*this) + init_name("type") + init_label(_("Primitive")) + init_description(_("Primitive type (sphere, quad only sphere or sphereized cylinder")) + init_value(SPHERE) + init_enumeration(type_values()))
 	{
-		m_material.changed_signal().connect(sigc::mem_fun(*this, &poly_sphere_implementation::on_reset_geometry));
-		m_u_segments.changed_signal().connect(sigc::mem_fun(*this, &poly_sphere_implementation::on_reset_geometry));
-		m_v_segments.changed_signal().connect(sigc::mem_fun(*this, &poly_sphere_implementation::on_reset_geometry));
-		m_radius.changed_signal().connect(sigc::mem_fun(*this, &poly_sphere_implementation::on_reset_geometry));
-		m_u_power.changed_signal().connect(sigc::mem_fun(*this, &poly_sphere_implementation::on_reset_geometry));
-		m_v_power.changed_signal().connect(sigc::mem_fun(*this, &poly_sphere_implementation::on_reset_geometry));
-		m_type.changed_signal().connect(sigc::mem_fun(*this, &poly_sphere_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &poly_sphere_implementation::on_create_geometry));
+		m_material.changed_signal().connect(make_reset_mesh_slot());
+		m_u_segments.changed_signal().connect(make_reset_mesh_slot());
+		m_v_segments.changed_signal().connect(make_reset_mesh_slot());
+		m_radius.changed_signal().connect(make_reset_mesh_slot());
+		m_u_power.changed_signal().connect(make_reset_mesh_slot());
+		m_v_power.changed_signal().connect(make_reset_mesh_slot());
+		m_type.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
-	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
 		const unsigned long u_segments = m_u_segments.value();
 		const unsigned long v_segments = m_v_segments.value();
@@ -408,31 +401,30 @@ public:
 		const double inv_u_power = u_power ? 1.0 / u_power : 1.0;
 		const double inv_v_power = v_power ? 1.0 / v_power : 1.0;
 
-
-		std::auto_ptr<k3d::mesh> mesh(new k3d::mesh());
-
-		mesh->polyhedra.push_back(new k3d::polyhedron());
-		k3d::polyhedron& polyhedron = *mesh->polyhedra.back();
+		Mesh.polyhedra.push_back(new k3d::polyhedron());
+		k3d::polyhedron& polyhedron = *Mesh.polyhedra.back();
 
 		switch(m_type.value())
 		{
 			case SPHERE:
-				detail::sphere(u_segments, v_segments, radius, inv_u_power, inv_v_power, false, material, mesh.get(), polyhedron);
+				detail::sphere(u_segments, v_segments, radius, inv_u_power, inv_v_power, false, material, &Mesh, polyhedron);
 			break;
 			case QUAD_ONLY_SPHERE:
-				detail::sphere(u_segments, v_segments, radius, inv_u_power, inv_v_power, true, material, mesh.get(), polyhedron);
+				detail::sphere(u_segments, v_segments, radius, inv_u_power, inv_v_power, true, material, &Mesh, polyhedron);
 			break;
 			case SPHEREIZED_CYLINDER:
-				detail::sphereized_cylinder(u_segments, v_segments, radius, inv_u_power, inv_v_power, material, mesh.get(), polyhedron);
+				detail::sphereized_cylinder(u_segments, v_segments, radius, inv_u_power, inv_v_power, material, &Mesh, polyhedron);
 			break;
 			default:
 				assert_not_reached();
 		}
 
-		return_val_if_fail(is_valid(polyhedron), 0);
+		assert_warning(is_valid(polyhedron));
 		assert_warning(is_solid(polyhedron));
+	}
 
-		return mesh.release();
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
