@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 2002-2005, Romain Behar
+// Copyright (c) 2002-2006, Romain Behar
 //
 // Contact: romainbehar@yahoo.com
 //
@@ -1443,21 +1443,13 @@ public:
 		m_dual(init_owner(*this) + init_name("dual") + init_label(_("Dual")) + init_description(_("Polyhedron's dual")) + init_value(false)),
 		m_size(init_owner(*this) + init_name("size") + init_label(_("Size")) + init_description(_("Polyhedron size (scale)")) + init_value(5.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_material.changed_signal().connect(sigc::mem_fun(*this, &polyhedron_implementation::on_reset_geometry));
-
-		m_number.changed_signal().connect(sigc::mem_fun(*this, &polyhedron_implementation::on_reset_geometry));
-		m_dual.changed_signal().connect(sigc::mem_fun(*this, &polyhedron_implementation::on_reset_geometry));
-		m_size.changed_signal().connect(sigc::mem_fun(*this, &polyhedron_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &polyhedron_implementation::on_create_geometry));
+		m_material.changed_signal().connect(make_reset_mesh_slot());
+		m_number.changed_signal().connect(make_reset_mesh_slot());
+		m_dual.changed_signal().connect(make_reset_mesh_slot());
+		m_size.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
-	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
 		k3d::imaterial* const material = m_material.value();
 		const unsigned long number = m_number.value();
@@ -1465,50 +1457,46 @@ public:
 		const double size= m_size.value();
 		const std::string name = dual ? kaleido::uniform[number - 1].dual : kaleido::uniform[number - 1].name;
 
-		std::auto_ptr<k3d::mesh> mesh(new k3d::mesh());
-
 		LoadPolyhedron(number, dual);
 
 		// Add geometry to the mesh ...
 		for(unsigned long i = 0; i != m_PolyhedronPoints.size(); ++i)
-			mesh->points.push_back(new k3d::point(m_PolyhedronPoints[i] * m_PolyhedronScaling * size));
+			Mesh.points.push_back(new k3d::point(m_PolyhedronPoints[i] * m_PolyhedronScaling * size));
 
-		mesh->polyhedra.push_back(new k3d::polyhedron());
-		k3d::polyhedron& polyhedron = *mesh->polyhedra.back();
+		Mesh.polyhedra.push_back(new k3d::polyhedron());
+		k3d::polyhedron& polyhedron = *Mesh.polyhedra.back();
 
 		unsigned long polygon_index = 0;
 		for(unsigned long i = 0; i != m_PolyhedronPaths.size(); ++i)
-			{
-				if(!m_PolyhedronPaths[i].size())
-					continue;
+		{
+			if(!m_PolyhedronPaths[i].size())
+				continue;
 
-				std::vector<k3d::split_edge*> edges;
-				for(unsigned long j = 0; j != m_PolyhedronPaths[i].size(); ++j)
-					edges.push_back(new k3d::split_edge(mesh->points[m_PolyhedronPaths[i][j]]));
+			std::vector<k3d::split_edge*> edges;
+			for(unsigned long j = 0; j != m_PolyhedronPaths[i].size(); ++j)
+				edges.push_back(new k3d::split_edge(Mesh.points[m_PolyhedronPaths[i][j]]));
 
-				k3d::loop_edges(edges.begin(), edges.end());
+			k3d::loop_edges(edges.begin(), edges.end());
 
-				// Set material ...
-				unsigned long color_index = *m_polygon_colors.begin();
-				// Trick to handle special case #39 and #57
-				if(polygon_index < m_polygon_colors.size())
-					color_index = m_polygon_colors[polygon_index];
+			// Set material ...
+			unsigned long color_index = *m_polygon_colors.begin();
+			// Trick to handle special case #39 and #57
+			if(polygon_index < m_polygon_colors.size())
+				color_index = m_polygon_colors[polygon_index];
 
-				k3d::face* new_face = new k3d::face(edges.front(), material);
-				return_val_if_fail(new_face, 0);
-				new_face->uniform_data["Cs"] = detail::colors[color_index];
-				polyhedron.faces.push_back(new_face);
+			k3d::face* new_face = new k3d::face(edges.front(), material);
+			new_face->uniform_data["Cs"] = detail::colors[color_index];
+			polyhedron.faces.push_back(new_face);
 
-				polygon_index++;
-			}
-
-		return_val_if_fail(is_valid(polyhedron), 0);
-		//assert_warning(is_solid(polyhedron));
-
-		// Set companions
+			polygon_index++;
+		}
 		k3d::set_companions(polyhedron);
 
-		return mesh.release();
+		assert_warning(is_valid(polyhedron));
+	}
+
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -1538,7 +1526,7 @@ private:
 	//k3d_data_property(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint) m_previous_materials;
 
 	/// Special variables and methods to generate polyhedra ...
-	typedef std::vector<k3d::vector3> PolyhedronPoints;
+	typedef std::vector<k3d::point3> PolyhedronPoints;
 	PolyhedronPoints m_PolyhedronPoints;
 
 	std::string m_PolyhedronProperties;
@@ -1575,28 +1563,28 @@ private:
 		// Write the polyhedron geometric properties string
 		m_PolyhedronProperties = "";
 		if(Dual && (polyhedron->hemi || polyhedron->index == kaleido::last_uniform - 1))
+		{
+			m_PolyhedronProperties += "infinite dual";
+
+			// Infinite duals have to scaled to a unit sphere
+			double maxlength = 0.0;
+			for(unsigned long i = 0; i < m_PolyhedronPoints.size(); i++)
 			{
-				m_PolyhedronProperties += "infinite dual";
-
-				// Infinite duals have to scaled to a unit sphere
-				double maxlength = 0.0;
-				for(unsigned long i = 0; i < m_PolyhedronPoints.size(); i++)
-					{
-						double length = m_PolyhedronPoints[i].Length();
-						if(length > maxlength)
-							maxlength = length;
-					}
-
-				m_PolyhedronScaling /= maxlength;
+				double length = k3d::to_vector(m_PolyhedronPoints[i]).length();
+				if(length > maxlength)
+					maxlength = length;
 			}
 
+			m_PolyhedronScaling /= maxlength;
+		}
+
 		// Free the polyhedron data ...
 		delete polyhedron;
 	}
 
 	void PolyhedronPoint(kaleido::Kvector v)
 	{
-		m_PolyhedronPoints.push_back(k3d::vector3(v.x, v.y, v.z));
+		m_PolyhedronPoints.push_back(k3d::point3(v.x, v.y, v.z));
 	}
 
 	void PolyhedronFace()
@@ -1928,4 +1916,3 @@ k3d::iplugin_factory& polyhedron_factory()
 
 } // namespace libk3dprimitives
 
-
