@@ -37,7 +37,7 @@
 namespace k3d
 {
 
-inline bool operator < (const k3d::vector3& a, const k3d::vector3& b)
+inline bool operator < (const k3d::point3& a, const k3d::point3& b)
 {
 	if(a[0] < b[0])
 		return true;
@@ -128,15 +128,15 @@ public:
 	void operator ^=(double n)
 	{
 		if(f > 0)
-			{
-				double x0 = pow(f, n);
-				double x1 = n * x0/f;
-				double x2 = (n-1)*x1/f;
-				fuv = x1*fuv + x2*fu*fv;
-				fu = x1*fu;
-				fv = x1*fv;
-				f = x0;
-			}
+		{
+			double x0 = pow(f, n);
+			double x1 = n * x0/f;
+			double x2 = (n-1)*x1/f;
+			fuv = x1*fuv + x2*fu*fv;
+			fu = x1*fu;
+			fv = x1*fv;
+			f = x0;
+		}
 	}
 	void Annihilate(int index)
 	{
@@ -373,24 +373,24 @@ TwoJet D(const ThreeJet x, int index)
 {
 	TwoJet result;
 	if(index == 0)
-		{
-			result.f = x.fu;
-			result.fu = x.fuu;
-			result.fv = x.fuv;
-			result.fuv = x.fuuv;
-		}
+	{
+		result.f = x.fu;
+		result.fu = x.fuu;
+		result.fv = x.fuv;
+		result.fuv = x.fuuv;
+	}
 	else if(index == 1)
-		{
-			result.f = x.fv;
-			result.fu = x.fuv;
-			result.fv = x.fvv;
-			result.fuv = x.fuvv;
-		}
+	{
+		result.f = x.fv;
+		result.fu = x.fuv;
+		result.fv = x.fvv;
+		result.fuv = x.fuvv;
+	}
 	else
-		{
-			result.f = result.fu = result.fv =
-			result.fuv = 0;
-		}
+	{
+		result.f = result.fu = result.fv =
+		result.fuv = 0;
+	}
 
 	return result;
 }
@@ -399,15 +399,15 @@ ThreeJet Annihilate(const ThreeJet x, int index)
 {
 	ThreeJet result = ThreeJet(x.f,0,0);
 	if(index == 0)
-		{
-			result.fv = x.fv;
-			result.fvv = x.fvv;
-		}
+	{
+		result.fv = x.fv;
+		result.fvv = x.fvv;
+	}
 	else if(index == 1)
-		{
-			result.fu = x.fu;
-			result.fuu = x.fuu;
-		}
+	{
+		result.fu = x.fu;
+		result.fuu = x.fuu;
+	}
 
 	return result;
 }
@@ -927,14 +927,14 @@ TwoJetVec UnCorrugate(ThreeJet u, ThreeJet v, double t, int numStrips)
 
 // ----------------------------------------
 
-typedef std::map<k3d::vector3, k3d::point*> point_map_t;
+typedef std::map<k3d::point3, k3d::point*> point_map_t;
 point_map_t point_map;
 k3d::imaterial* current_material;
 double current_radius;
 
 k3d::point* get_spline_point(k3d::mesh* mesh, TwoJetVec p, double ps, double pus, double pvs, double puvs)
 {
-	k3d::vector3 v;
+	k3d::point3 v;
 	v[0] = (p.x.f)*ps + (p.x.df_du()*pus + p.x.df_dv()*pvs)/3.0 + p.x.d2f_dudv()*puvs/9.0;
 	v[1] = (p.y.f)*ps + (p.y.df_du()*pus + p.y.df_dv()*pvs)/3.0 + p.y.d2f_dudv()*puvs/9.0;
 	v[2] = (p.z.f)*ps + (p.z.df_du()*pus + p.z.df_dv()*pvs)/3.0 + p.z.d2f_dudv()*puvs/9.0;
@@ -944,15 +944,15 @@ k3d::point* get_spline_point(k3d::mesh* mesh, TwoJetVec p, double ps, double pus
 	k3d::point* point;
 	point_map_t::const_iterator mapped_point = point_map.find(v);
 	if(mapped_point != point_map.end())
-		{
-			point = mapped_point->second;
-		}
+	{
+		point = mapped_point->second;
+	}
 	else
-		{
-			point = new k3d::point(v * current_radius);
-			point_map[v] = point;
-			mesh->points.push_back(point);
-		}
+	{
+		point = new k3d::point(v * current_radius);
+		point_map[v] = point;
+		mesh->points.push_back(point);
+	}
 
 	return point;
 }
@@ -1039,60 +1039,60 @@ void get_eversion(k3d::mesh* mesh, bool polygonal, SurfaceTimeFunction* func, do
 	//double* speedv = (double*)calloc(ucount+1, sizeof(double));
 	//double** speedu = (double**)calloc(ucount+1, sizeof(double*));
 	for(int j = 0; j <= ucount; j++)
+	{
+		double u = umin + j*delta_u;
+		values[j] = (TwoJetVec*)calloc(vcount+1, sizeof(TwoJetVec));
+		//speedu[j] = (double*)calloc(vcount+1, sizeof(double));
+		//speedv[j] = calcSpeedV((*func)(ThreeJet(u, 1, 0), ThreeJet(0, 0, 1), t, numStrips));
+		double speed_v = calcSpeedV((*func)(ThreeJet(u, 1, 0), ThreeJet(0, 0, 1), t, numStrips));
+		if(speed_v == 0)
+		//if(speedv[j] == 0)
 		{
-			double u = umin + j*delta_u;
-			values[j] = (TwoJetVec*)calloc(vcount+1, sizeof(TwoJetVec));
-			//speedu[j] = (double*)calloc(vcount+1, sizeof(double));
+			// Perturb a bit, hoping to avoid degeneracy
+			u += (u < 1) ? 1e-9 : -1e-9;
 			//speedv[j] = calcSpeedV((*func)(ThreeJet(u, 1, 0), ThreeJet(0, 0, 1), t, numStrips));
-			double speed_v = calcSpeedV((*func)(ThreeJet(u, 1, 0), ThreeJet(0, 0, 1), t, numStrips));
-			if(speed_v == 0)
-			//if(speedv[j] == 0)
-				{
-					// Perturb a bit, hoping to avoid degeneracy
-					u += (u < 1) ? 1e-9 : -1e-9;
-					//speedv[j] = calcSpeedV((*func)(ThreeJet(u, 1, 0), ThreeJet(0, 0, 1), t, numStrips));
-				}
-
-			for(int k = 0; k <= vcount; k++)
-				{
-					double v = vmin + k*delta_v;
-					values[j][k] = (*func)(ThreeJet(u, 1, 0), ThreeJet(v, 0, 1), t, numStrips);
-					//speedu[j][k] = calcSpeedU(values[j][k]);
-				}
 		}
 
+		for(int k = 0; k <= vcount; k++)
+		{
+			double v = vmin + k*delta_v;
+			values[j][k] = (*func)(ThreeJet(u, 1, 0), ThreeJet(v, 0, 1), t, numStrips);
+			//speedu[j][k] = calcSpeedU(values[j][k]);
+		}
+	}
+
 	if(!polygonal)
-		{
-			// Bezier patches
-			for(int j = 0; j < ucount; ++j)
-				//double u = umin + j*delta_u;
-				for(int k = 0; k < vcount; ++k)
-					//double v = vmin + k*delta_v;
-					get_patch(mesh, values[j][k], values[j][k+1], values[j+1][k], values[j+1][k+1], delta_u, delta_v, umin + j*delta_u, umin + (j+1)*delta_u, vmin + k*delta_v, vmin + (k+1)*delta_v);
-		}
+	{
+		// Bezier patches
+		for(int j = 0; j < ucount; ++j)
+			//double u = umin + j*delta_u;
+			for(int k = 0; k < vcount; ++k)
+				//double v = vmin + k*delta_v;
+				get_patch(mesh, values[j][k], values[j][k+1], values[j+1][k], values[j+1][k+1], delta_u, delta_v, umin + j*delta_u, umin + (j+1)*delta_u, vmin + k*delta_v, vmin + (k+1)*delta_v);
+	}
 	else
+	{
+		// Quadrilaterals
+		for(int j = 0; j <= ucount; ++j)
 		{
-			// Quadrilaterals
-			for(int j = 0; j <= ucount; ++j)
-				{
-					geometry_vector_t point_vector;
-					for(int k = 0; k <= vcount; ++k)
-						{
-							eversion_point new_point;
-							get_polygon_point(values[j][k], new_point);
-							point_vector.push_back(new_point);
-						}
+			geometry_vector_t point_vector;
+			for(int k = 0; k <= vcount; ++k)
+			{
+				eversion_point new_point;
+				get_polygon_point(values[j][k], new_point);
+				point_vector.push_back(new_point);
+			}
 
-					points.push_back(point_vector);
-				}
+			points.push_back(point_vector);
 		}
+	}
 
 	// Clean up
 	for(int j = 0; j <= ucount; j++)
-		{
-			free(values[j]);
-			//free(speedu[j]);
-		}
+	{
+		free(values[j]);
+		//free(speedu[j]);
+	}
 
 	free(values);
 	//free(speedu);
@@ -1131,10 +1131,10 @@ void generate_strip(
 {
 	point_map.clear();
 	if(bendtime >= 0.0)
-		{
-			get_eversion(mesh, polygonal, BendIn, u_min, u_max, u_count, v_min, v_max, v_count, bendtime, points, numStrips);
-			return;
-		}
+	{
+		get_eversion(mesh, polygonal, BendIn, u_min, u_max, u_count, v_min, v_max, v_count, bendtime, points, numStrips);
+		return;
+	}
 
 	// time = (time - howfar) / chunk
 
@@ -1180,28 +1180,20 @@ public:
 		m_cylinder_test(init_owner(*this) + init_name("cylinder_test") + init_label(_("Cylinder test")) + init_description(_("Cylinder bending into sphere")) + init_value(false)),
 		m_polygonal(init_owner(*this) + init_name("polygonal") + init_label(_("Polygonal")) + init_description(_("Output polygons (true) or patches (false)")) + init_value(false))
 	{
-		m_material.changed_signal().connect(sigc::mem_fun(*this, &sphere_eversion_implementation::on_reset_geometry));
-
-		m_time.changed_signal().connect(sigc::mem_fun(*this, &sphere_eversion_implementation::on_reset_geometry));
-		m_radius.changed_signal().connect(sigc::mem_fun(*this, &sphere_eversion_implementation::on_reset_geometry));
-		m_latitudinal_patches.changed_signal().connect(sigc::mem_fun(*this, &sphere_eversion_implementation::on_reset_geometry));
-		m_longitudinal_patches.changed_signal().connect(sigc::mem_fun(*this, &sphere_eversion_implementation::on_reset_geometry));
-		m_strips.changed_signal().connect(sigc::mem_fun(*this, &sphere_eversion_implementation::on_reset_geometry));
-		m_display_strips.changed_signal().connect(sigc::mem_fun(*this, &sphere_eversion_implementation::on_reset_geometry));
-		m_half_strips.changed_signal().connect(sigc::mem_fun(*this, &sphere_eversion_implementation::on_reset_geometry));
-		m_lower_hemisphere.changed_signal().connect(sigc::mem_fun(*this, &sphere_eversion_implementation::on_reset_geometry));
-		m_cylinder_test.changed_signal().connect(sigc::mem_fun(*this, &sphere_eversion_implementation::on_reset_geometry));
-		m_polygonal.changed_signal().connect(sigc::mem_fun(*this, &sphere_eversion_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &sphere_eversion_implementation::on_create_geometry));
+		m_material.changed_signal().connect(make_reset_mesh_slot());
+		m_time.changed_signal().connect(make_reset_mesh_slot());
+		m_radius.changed_signal().connect(make_reset_mesh_slot());
+		m_latitudinal_patches.changed_signal().connect(make_reset_mesh_slot());
+		m_longitudinal_patches.changed_signal().connect(make_reset_mesh_slot());
+		m_strips.changed_signal().connect(make_reset_mesh_slot());
+		m_display_strips.changed_signal().connect(make_reset_mesh_slot());
+		m_half_strips.changed_signal().connect(make_reset_mesh_slot());
+		m_lower_hemisphere.changed_signal().connect(make_reset_mesh_slot());
+		m_cylinder_test.changed_signal().connect(make_reset_mesh_slot());
+		m_polygonal.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_reset_geometry()
-	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
+	void on_create_mesh(k3d::mesh& Mesh)
 	{
 		const double time = m_time.value();
 		const double radius = m_radius.value();
@@ -1229,144 +1221,140 @@ public:
 		);
 
 		if(polygonal)
+		{
+			// Create polygonal strip mesh ...
+			std::vector< std::vector<k3d::point*> > points;
+			for(unsigned long j = 0; j <= latitudinal; ++j)
 			{
-				// Create polygonal strip mesh ...
-				std::vector< std::vector<k3d::point*> > points;
-				for(unsigned long j = 0; j <= latitudinal; ++j)
-					{
-						std::vector<k3d::point*> vpoints;
-						for(unsigned long k = 0; k <= longitudinal; ++k)
-							{
-								double* p = sphere_points[j][k].vertex;
-								double x = p[0] * radius;
-								double y = p[1] * radius;
-								double z = p[2] * radius;
+				std::vector<k3d::point*> vpoints;
+				for(unsigned long k = 0; k <= longitudinal; ++k)
+				{
+					double* p = sphere_points[j][k].vertex;
+					double x = p[0] * radius;
+					double y = p[1] * radius;
+					double z = p[2] * radius;
 
-								// Fix handedness
-								k3d::point* point = new k3d::point(x, y, -z);
-								return_val_if_fail(point, 0);
+					// Fix handedness
+					k3d::point* point = new k3d::point(x, y, -z);
 
-								// Set normal
-								p = sphere_points[j][k].normal;
-								x = p[0];
-								y = p[1];
-								z = p[2];
-								point->vertex_data["N"] = k3d::ri::normal(k3d::vector3(x, y, -z));
+					// Set normal
+					p = sphere_points[j][k].normal;
+					x = p[0];
+					y = p[1];
+					z = p[2];
+					point->vertex_data["N"] = k3d::ri::normal(x, y, -z);
 
-								strip_mesh->points.push_back(point);
-								vpoints.push_back(point);
-							}
+					strip_mesh->points.push_back(point);
+					vpoints.push_back(point);
+				}
 
-						points.push_back(vpoints);
-					}
-
-				strip_mesh->polyhedra.push_back(new k3d::polyhedron());
-				k3d::polyhedron& polyhedron = *strip_mesh->polyhedra.back();
-
-				for(unsigned long j = 0; j < latitudinal; ++j)
-					{
-						k3d::point* p1 = 0;
-						k3d::point* p2 = 0;
-						k3d::point* p3 = 0;
-						bool strip_toggle = false;
-
-						for(unsigned long k = 0; k <= longitudinal; ++k)
-						for(unsigned long n = 0; n < 2; n++)
-						{
-							p1 = p2;
-							p2 = p3;
-							p3 = points[j+n][k];
-							if(!p1)
-								continue;
-
-							std::vector<k3d::split_edge*> edges;
-							edges.push_back(new k3d::split_edge(p1));
-							edges.push_back(new k3d::split_edge(p2));
-							edges.push_back(new k3d::split_edge(p3));
-							if(strip_toggle)
-								std::reverse(edges.begin(), edges.end());
-							strip_toggle = !strip_toggle;
-
-							k3d::loop_edges(edges.begin(), edges.end());
-
-							k3d::face* new_face = new k3d::face(edges.front(), detail::current_material);
-							return_val_if_fail(new_face, 0);
-							polyhedron.faces.push_back(new_face);
-						}
-					}
-
-				return_val_if_fail(is_valid(polyhedron), 0);
-				//assert_warning(is_solid(polyhedron));
+				points.push_back(vpoints);
 			}
 
+			strip_mesh->polyhedra.push_back(new k3d::polyhedron());
+			k3d::polyhedron& polyhedron = *strip_mesh->polyhedra.back();
+
+			for(unsigned long j = 0; j < latitudinal; ++j)
+			{
+				k3d::point* p1 = 0;
+				k3d::point* p2 = 0;
+				k3d::point* p3 = 0;
+				bool strip_toggle = false;
+
+				for(unsigned long k = 0; k <= longitudinal; ++k)
+				for(unsigned long n = 0; n < 2; n++)
+				{
+					p1 = p2;
+					p2 = p3;
+					p3 = points[j+n][k];
+					if(!p1)
+						continue;
+
+					std::vector<k3d::split_edge*> edges;
+					edges.push_back(new k3d::split_edge(p1));
+					edges.push_back(new k3d::split_edge(p2));
+					edges.push_back(new k3d::split_edge(p3));
+					if(strip_toggle)
+						std::reverse(edges.begin(), edges.end());
+					strip_toggle = !strip_toggle;
+
+					k3d::loop_edges(edges.begin(), edges.end());
+
+					k3d::face* new_face = new k3d::face(edges.front(), detail::current_material);
+					polyhedron.faces.push_back(new_face);
+				}
+			}
+
+			assert_warning(is_valid(polyhedron));
+		}
+
 		// Duplicate strip to strip number
 		unsigned long original_points = strip_mesh->points.size();
 
-		k3d::mesh* const output = new k3d::mesh();
-
 		double strip_rotation = 2*k3d::pi() / static_cast<double>(strip_number);
 		for(unsigned long n = 0; n < display_strips_number && n < strip_number; n++)
+		{
+			k3d::deep_copy(*strip_mesh, Mesh);
+
+			k3d::matrix4 rotation(k3d::rotation3D(k3d::point3(0, 0, static_cast<double>(n) * strip_rotation)));
+			if(n > 0)
+				for(k3d::mesh::points_t::iterator p = Mesh.points.end() - original_points; p != Mesh.points.end(); p++)
+				{
+					(*p)->position = rotation * (*p)->position;
+					if(polygonal)
+						(*p)->vertex_data["N"] = k3d::ri::normal(rotation * boost::any_cast<k3d::ri::normal>((*p)->vertex_data["N"]));
+				}
+
+			if(!lower_hemisphere)
+				continue;
+
+			if(!polygonal)
 			{
-				k3d::deep_copy(*strip_mesh, *output);
-
-				k3d::matrix4 rotation(k3d::rotation3D(k3d::vector3(0, 0, static_cast<double>(n) * strip_rotation)));
+				k3d::deep_copy(*strip_mesh, Mesh);
+				k3d::matrix4 rotation2(k3d::rotation3D(k3d::point3(k3d::pi(), 0, 0)));
 				if(n > 0)
-					for(k3d::mesh::points_t::iterator p = output->points.end() - original_points; p != output->points.end(); p++)
-						{
-							(*p)->position = rotation * (*p)->position;
-							if(polygonal)
-								(*p)->vertex_data["N"] = k3d::ri::normal(rotation * boost::any_cast<k3d::ri::normal>((*p)->vertex_data["N"]));
-						}
-
-				if(!lower_hemisphere)
-					continue;
-
-				if(!polygonal)
-					{
-						k3d::deep_copy(*strip_mesh, *output);
-						k3d::matrix4 rotation2(k3d::rotation3D(k3d::vector3(k3d::pi(), 0, 0)));
-						if(n > 0)
-							rotation = rotation2 * rotation;
-						else
-							rotation = rotation2;
-
-						for(k3d::mesh::points_t::iterator p = output->points.end() - original_points; p != output->points.end(); p++)
-							(*p)->position = rotation * (*p)->position;
-					}
+					rotation = rotation2 * rotation;
 				else
-					{
-						k3d::deep_copy(*strip_mesh, *output);
-						for(k3d::mesh::points_t::iterator p = output->points.end() - original_points; p != output->points.end(); p++)
-							{
-								k3d::vector3 position = rotation * (*p)->position;
-								k3d::ri::normal normal = k3d::ri::normal(rotation * boost::any_cast<k3d::ri::normal>((*p)->vertex_data["N"]));
-
-								position.n[2] = -position.n[2];
-								normal.n[2] = -normal.n[2];
-
-								(*p)->position = position;
-								(*p)->vertex_data["N"] = k3d::ri::normal(normal);
-							}
-
-						// Invert triangles
-						k3d::polyhedron& polyhedron = *output->polyhedra.back();
-						for(k3d::polyhedron::faces_t::iterator face = polyhedron.faces.begin(); face != polyhedron.faces.end(); face++)
-							{
-								k3d::split_edge* first = (*face)->first_edge;
-								k3d::split_edge* second = first->face_clockwise;
-								k3d::split_edge* third = second->face_clockwise;
-
-								second->face_clockwise = first;
-								third->face_clockwise = second;
-								first->face_clockwise = third;
-							}
-					}
+					rotation = rotation2;
 
+				for(k3d::mesh::points_t::iterator p = Mesh.points.end() - original_points; p != Mesh.points.end(); p++)
+					(*p)->position = rotation * (*p)->position;
 			}
+			else
+			{
+				k3d::deep_copy(*strip_mesh, Mesh);
+				for(k3d::mesh::points_t::iterator p = Mesh.points.end() - original_points; p != Mesh.points.end(); p++)
+				{
+					k3d::point3 position = rotation * (*p)->position;
+					k3d::ri::normal normal = k3d::ri::normal(rotation * boost::any_cast<k3d::ri::normal>((*p)->vertex_data["N"]));
+
+					position.n[2] = -position.n[2];
+					normal.n[2] = -normal.n[2];
+
+					(*p)->position = position;
+					(*p)->vertex_data["N"] = k3d::ri::normal(normal);
+				}
+
+				// Invert triangles
+				k3d::polyhedron& polyhedron = *Mesh.polyhedra.back();
+				for(k3d::polyhedron::faces_t::iterator face = polyhedron.faces.begin(); face != polyhedron.faces.end(); face++)
+				{
+					k3d::split_edge* first = (*face)->first_edge;
+					k3d::split_edge* second = first->face_clockwise;
+					k3d::split_edge* third = second->face_clockwise;
+
+					second->face_clockwise = first;
+					third->face_clockwise = second;
+					first->face_clockwise = third;
+				}
+			}
+		}
 
 		delete strip_mesh;
+	}
 
-		return output;
+	void on_update_mesh(k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -1410,4 +1398,3 @@ k3d::iplugin_factory& sphere_eversion_factory()
 
 } // namespace libk3dprimitives
 
-
