@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2004, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -55,12 +55,16 @@ namespace libk3dpython
 const std::string magic_token("#python");
 
 /// Ensures that Py_Initialize() is called before any other Python API functions
-struct initialize_python
+class initialize_python
 {
+public:
 	initialize_python()
 	{
 		if(!Py_IsInitialized())
+		{
 			PYTHON_INITIALIZE();
+			init_object_model();
+		}
 	}
 };
 
@@ -70,28 +74,20 @@ class engine :
 {
 public:
 	engine() :
-		m_interpreter(0),
-		m_module(0)
+		m_local_dict(PyDict_New())
 	{
-		if(!Py_IsInitialized())
-			PYTHON_INITIALIZE();
-
-		m_interpreter = Py_NewInterpreter();
-		PyThreadState_Swap(0);
 	}
 
 	virtual ~engine()
 	{
-		PyThreadState_Swap(m_interpreter);
-		Py_EndInterpreter(m_interpreter);
-		PyThreadState_Swap(0);
+		Py_DECREF(m_local_dict);
 	}
 
 	k3d::iplugin_factory& factory()
 	{
 		return get_factory();
 	}
-	
+
 	static k3d::iplugin_factory& get_factory()
 	{
 		static k3d::plugin_factory<k3d::application_plugin<engine>, k3d::interface_list<k3d::iscript_engine> > factory(
@@ -108,42 +104,36 @@ public:
 		return "Python";
 	}
 
-	bool can_execute(std::istream& Script)
+	bool can_execute(const std::string& Script)
 	{
-		Script.seekg(0, std::ios::beg);
-		std::string buffer;
-		std::getline(Script, buffer);
-		Script.seekg(0, std::ios::beg);
-	
-		return buffer.substr(0, magic_token.size()) == magic_token;
+		return Script.substr(0, magic_token.size()) == magic_token;
 	}
 
-	bool execute(const std::string& ScriptName, std::istream& Script, context_t& Context)
+	bool execute(const std::string& ScriptName, const std::string& Script, context_t& Context)
 	{
-		return_val_if_fail(ScriptName.size(), false);
-
-		PyThreadState_Swap(m_interpreter);
-
-		if(!m_module)
-			m_module = init_object_model();
-
-		init_context(m_module, Context);
-		
-		Script.seekg(0, std::ios::beg);
-		std::stringstream buffer;
-		buffer << Script.rdbuf();
+		set_context(Context, m_local_dict);
 
 		// The embedded python interpreter cannot handle DOS line-endings, see http://sourceforge.net/tracker/?group_id=5470&atid=105470&func=detail&aid=1167922
-		std::string script = buffer.str();
+		std::string script = Script;
 		script.erase(std::remove(script.begin(), script.end(), '\r'), script.end());
 
-		const int returncode = PyRun_SimpleString(const_cast<char*>(script.c_str()));
+		PyDict_Update(m_local_dict, PyObject_GetAttrString(PyImport_AddModule("__main__"), "__dict__"));
+		
+		PyObject* const result = PyRun_String(const_cast<char*>(script.c_str()), Py_file_input, m_local_dict, m_local_dict);
+		if(result)
+		{
+			Py_DECREF(result);
+			if(Py_FlushLine())
+				PyErr_Clear();
+		}
+		else
+		{
+	                PyErr_Print();
+		}
 
-		update_context(m_module, Context);
-
-		PyThreadState_Swap(0);
-				
-		return (returncode == 0) ? true : false;
+		get_context(m_local_dict, Context);
+		
+		return result ? true : false;
 	}
 
 	bool halt()
@@ -180,8 +170,7 @@ public:
 
 private:
 	initialize_python m_initialize_python;
-	PyThreadState* m_interpreter;
-	PyObject* m_module;
+	PyObject* const m_local_dict;
 };
 
 } // namespace libk3dpython
@@ -190,4 +179,3 @@ K3D_MODULE_START(libk3dpython, k3d::uuid(0x9d1e8e42, 0x3dc7422c, 0xbe1c3215, 0x9
 	Registry.register_factory(libk3dpython::engine::get_factory());
 K3D_MODULE_END
 
-
