@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 2004-2005, Romain Behar
+// Copyright (c) 2004-2006, Romain Behar
 //
 // Contact: romainbehar@yahoo.com
 //
@@ -24,12 +24,12 @@
 
 #include <k3dsdk/basic_math.h>
 #include <k3dsdk/i18n.h>
-#include <k3dsdk/node.h>
-#include <k3dsdk/persistent.h>
-#include <k3dsdk/property.h>
 #include <k3dsdk/measurement.h>
 #include <k3dsdk/mesh_modifier.h>
 #include <k3dsdk/module.h>
+#include <k3dsdk/node.h>
+#include <k3dsdk/persistent.h>
+#include <k3dsdk/property.h>
 #include <k3dsdk/utility.h>
 
 #include "MxQSlim.h"
@@ -61,7 +61,7 @@ MxStdModel* triangulate_mesh(const k3d::mesh& Mesh)
 	for(k3d::mesh::points_t::const_iterator point = all_points.begin(); point != all_points.end(); ++point)
 	{
 		point_map.insert(std::make_pair(*point, point_map.size()));
-		const k3d::vector3 position = (*point)->position;
+		const k3d::point3 position = (*point)->position;
 		Vec3 pos(-position[0], position[1], position[2]);
 		model->add_vertex(pos);
 	}
@@ -102,48 +102,33 @@ public:
 		m_compactness_ratio(init_owner(*this) + init_name("compactness_ratio") + init_label(_("Compactness ratio")) + init_description(_("Compactness ratio")) + init_value(k3d::radians(0.0)) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::angle))),
 		m_meshing_penalty(init_owner(*this) + init_name("meshing_penalty") + init_label(_("Meshing penalty")) + init_description(_("Penalty for bad meshes")) + init_value(1.0) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &quadric_decimation::on_reset_geometry));
-		m_face_number.changed_signal().connect(sigc::mem_fun(*this, &quadric_decimation::on_reset_geometry));
-		m_contraction_type.changed_signal().connect(sigc::mem_fun(*this, &quadric_decimation::on_reset_geometry));
-		m_placement_policy.changed_signal().connect(sigc::mem_fun(*this, &quadric_decimation::on_reset_geometry));
-		m_quadric_weighting.changed_signal().connect(sigc::mem_fun(*this, &quadric_decimation::on_reset_geometry));
-		m_boundary_weight.changed_signal().connect(sigc::mem_fun(*this, &quadric_decimation::on_reset_geometry));
-		m_compactness_ratio.changed_signal().connect(sigc::mem_fun(*this, &quadric_decimation::on_reset_geometry));
-		m_meshing_penalty.changed_signal().connect(sigc::mem_fun(*this, &quadric_decimation::on_reset_geometry));
-		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &quadric_decimation::on_create_geometry));
-
-		new_input = true;
+		m_face_number.changed_signal().connect(make_reset_mesh_slot());
+		m_contraction_type.changed_signal().connect(make_reset_mesh_slot());
+		m_placement_policy.changed_signal().connect(make_reset_mesh_slot());
+		m_quadric_weighting.changed_signal().connect(make_reset_mesh_slot());
+		m_boundary_weight.changed_signal().connect(make_reset_mesh_slot());
+		m_compactness_ratio.changed_signal().connect(make_reset_mesh_slot());
+		m_meshing_penalty.changed_signal().connect(make_reset_mesh_slot());
 	}
 
-	void on_new_input()
+	/** \todo Improve the implementation so we don't have to do this */
+	k3d::iunknown* on_rewrite_hint(iunknown* const Hint)
 	{
-		new_input = true;
-		on_reset_geometry();
+		// Force updates to re-allocate our mesh, for simplicity
+		return 0;
 	}
 
-	void on_reset_geometry()
+	void on_create_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
 	{
-		m_output_mesh.reset();
-	}
-
-	k3d::mesh* on_create_geometry()
-	{
-		// Get the input geometry ...
-		k3d::mesh* const input = m_input_mesh.value();
-		if(!input)
-			return 0;
+		if(!InputMesh.polyhedra.size() || !InputMesh.polyhedra.front()->faces.size())
+			return;
 
 		// Load mesh into MxStdModel
-		MxStdModel* m = detail::triangulate_mesh(*input);
-		return_val_if_fail(m, 0);
+		MxStdModel* m = detail::triangulate_mesh(InputMesh);
+		return_if_fail(m);
 
-		// Set current face number in case of new input
-		if(new_input)
-		{
-			const unsigned long face_number = m->face_count();
-			k3d::set_value(m_face_number, face_number);
-			new_input = false;
-		}
+		const unsigned long face_number = m->face_count();
+		k3d::set_value(m_face_number, face_number);
 
 		// Init
 		MxQSlim* slim = 0;
@@ -203,24 +188,21 @@ public:
 		// Compact vertex array so only valid vertices remain
 		m->compact_vertices();
 
-		// Create output geometry ...
-		k3d::mesh* const output = new k3d::mesh();
-
 		std::map<unsigned long, k3d::point*> point_map;
 		for(unsigned int v = 0; v < m->vert_count(); v++)
 		{
-			k3d::vector3 position(m->vertex(v));
+			k3d::point3 position(m->vertex(v));
 			position[0] = -position[0];
 
 			k3d::point* point = new k3d::point(position);
-			output->points.push_back(point);
+			Mesh.points.push_back(point);
 
 			point_map[v] = point;
 		}
 
 		k3d::polyhedron* polyhedron = new k3d::polyhedron();
-		return_val_if_fail(polyhedron, 0);
-		output->polyhedra.push_back(polyhedron);
+		Mesh.polyhedra.push_back(polyhedron);
+		k3d::imaterial* material = InputMesh.polyhedra.front()->faces.front()->material;
 		for(unsigned int f = 0; f < m->face_count(); f++)
 		{
 			if(m->face_is_valid(f))
@@ -239,14 +221,17 @@ public:
 				edge3->face_clockwise = edge2;
 				edge2->face_clockwise = edge1;
 
-				k3d::face* const face = new k3d::face(edge1, 0);
-				return_val_if_fail(face, 0);
-
+				k3d::face* const face = new k3d::face(edge1, material);
 				polyhedron->faces.push_back(face);
 			}
 		}
 
-		return output;
+		// Set edge companions
+		k3d::set_companions(*polyhedron);
+	}
+
+	void on_update_mesh(const k3d::mesh& InputMesh, k3d::mesh& Mesh)
+	{
 	}
 
 	k3d::iplugin_factory& factory()
@@ -270,9 +255,6 @@ public:
 	}
 
 private:
-	/// Stores whether current input mesh has changed
-	bool new_input;
-
 	/// Enumerates contraction functions
 	typedef enum
 	{
