@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -39,25 +39,25 @@ namespace libk3drenderman
 {
 
 /////////////////////////////////////////////////////////////////////////////
-// array_2d_implementation
+// array_2d
 
-class array_2d_implementation :
+class array_2d :
 	public k3d::material_client<k3d::gl::drawable<k3d::ri::renderable<k3d::transformable<k3d::persistent<k3d::node> > > > >
 {
 	typedef k3d::material_client<k3d::gl::drawable<k3d::ri::renderable<k3d::transformable<k3d::persistent<k3d::node> > > > > base;
 
 public:
-	array_2d_implementation(k3d::idocument& Document) :
+	array_2d(k3d::idocument& Document) :
 		base(Document),
 		m_source(init_owner(*this) + init_name("source") + init_label(_("Source")) + init_description(_("Source")) + init_value<k3d::ri::irenderable*>(0)),
 		m_layout(init_owner(*this) + init_name("layout") + init_label(_("Layout")) + init_description(_("Layout")) + init_value<k3d::itransform_array_2d*>(0)),
 		m_count1(init_owner(*this) + init_name("count1") + init_label(_("Count 1")) + init_description(_("Dimension 1 Count")) + init_value(5) + init_constraint(constraint::minimum(0L)) + init_precision(0) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar))),
 		m_count2(init_owner(*this) + init_name("count2") + init_label(_("Count 2")) + init_description(_("Dimension 2 Count")) + init_value(5) + init_constraint(constraint::minimum(0L)) + init_precision(0) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_source.changed_signal().connect(sigc::mem_fun(*this, &array_2d_implementation::async_redraw_all));
-		m_layout.changed_signal().connect(sigc::mem_fun(*this, &array_2d_implementation::async_redraw_all));
-		m_count1.changed_signal().connect(sigc::mem_fun(*this, &array_2d_implementation::async_redraw_all));
-		m_count2.changed_signal().connect(sigc::mem_fun(*this, &array_2d_implementation::async_redraw_all));
+		m_source.changed_signal().connect(make_async_redraw_slot());
+		m_layout.changed_signal().connect(make_async_redraw_slot());
+		m_count1.changed_signal().connect(make_async_redraw_slot());
+		m_count2.changed_signal().connect(make_async_redraw_slot());
 	}
 
 	void draw(const k3d::gl::render_state& State)
@@ -78,7 +78,7 @@ public:
 		{
 			for(unsigned long j = 0; j != count2; ++j)
 			{
-				glVertex3dv(k3d::vector3(0, 0, 0) * layout->get_element(i, count1, j, count2));
+				k3d::gl::vertex3d(k3d::point3(0, 0, 0) * layout->get_element(i, count1, j, count2));
 			}
 		}
 		glEnd();
@@ -130,10 +130,10 @@ public:
 
 		// Make sure we don't enter an infinite loop trying to render ourself ...
 		if(renderable == this)
-			{
-				k3d::log() << error << factory().name() << " [" << name() << "] cannot instance itself" << std::endl;
-				return;
-			}
+		{
+			k3d::log() << error << factory().name() << " [" << name() << "] cannot instance itself" << std::endl;
+			return;
+		}
 
 		const k3d::ri::object_handle handle = State.engine.RiObjectBegin();
 			k3d::ri::render_state state(State);
@@ -163,7 +163,7 @@ public:
 	static k3d::iplugin_factory& get_factory()
 	{
 		static k3d::plugin_factory<
-			k3d::document_plugin<array_2d_implementation> > factory(
+			k3d::document_plugin<array_2d> > factory(
 				k3d::uuid(0x5a7881eb, 0xf8e14075, 0x977fe1fb, 0x2036f517),
 				"RenderManArray2D",
 				_("Renders a two-dimensional array of geometric object instances"),
@@ -185,9 +185,8 @@ private:
 
 k3d::iplugin_factory& array_2d_factory()
 {
-	return array_2d_implementation::get_factory();
+	return array_2d::get_factory();
 }
 
 } // namespace libk3drenderman
 
-
