@@ -46,16 +46,16 @@ namespace
 {
 
 /////////////////////////////////////////////////////////////////////////////
-// background_sphere_implementation
+// background_sphere
 
-class background_sphere_implementation :
+class background_sphere :
 	public k3d::material_client<k3d::persistent<k3d::node> >,
 	public k3d::ri::irenderable
 {
 	typedef k3d::material_client<k3d::persistent<k3d::node> > base;
 
 public:
-	background_sphere_implementation(k3d::idocument& Document) :
+	background_sphere(k3d::idocument& Document) :
 		base(Document),
 		m_distance(init_owner(*this) + init_name("distance") + init_label(_("distance")) + init_description(_("Distance")) + init_value(0.99) + init_constraint(k3d::data::constraint::minimum(0.0, k3d::data::constraint::maximum(1.0)))),
 		m_render_final(init_owner(*this) + init_name("render_final") + init_label(_("render_final")) + init_description(_("Visible in the final rendered image")) + init_value(true))
@@ -77,31 +77,31 @@ public:
 		k3d::iperspective* const perspective = dynamic_cast<k3d::iperspective*>(&State.projection);
 		k3d::iorthographic* const orthographic = dynamic_cast<k3d::iorthographic*>(&State.projection);
 		if(!perspective && !orthographic)
-			{
-				k3d::log() << error << k3d_file_reference << ": unknown projection type" << std::endl;
-				return;
-			}
+		{
+			k3d::log() << error << k3d_file_reference << ": unknown projection type" << std::endl;
+			return;
+		}
 	
 		double radius = 0;
 		if(perspective)
-			{
-				const double near = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->near()));
-				const double far = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->far()));
-				radius = k3d::mix(near, far, m_distance.value());
-			}
+		{
+			const double near = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->near()));
+			const double far = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->far()));
+			radius = k3d::mix(near, far, m_distance.value());
+		}
 
 		if(orthographic)
-			{
-				const double near = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->near()));
-				const double far = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->far()));
-				radius = k3d::mix(near, far, m_distance.value());
-			}
+		{
+			const double near = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->near()));
+			const double far = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->far()));
+			radius = k3d::mix(near, far, m_distance.value());
+		}
 
 		// Get data from the camera ...
-		const k3d::vector3 camera_coords = State.camera_matrix * k3d::vector3(0, 0, 0);
+		const k3d::point3 camera_coords = State.camera_matrix * k3d::point3(0, 0, 0);
 
 		State.engine.RiAttributeBegin();
-		State.engine.RiAttributeV("identifier", k3d::ri::parameter_list(1, k3d::ri::parameter("name", k3d::ri::UNIFORM, name())));
+		State.engine.RiAttributeV("identifier", k3d::ri::parameter_list(1, k3d::ri::parameter("name", k3d::ri::UNIFORM, k3d::ri::string(name()))));
 		State.engine.RiIdentity();
 		State.engine.RiTranslate(camera_coords[0], camera_coords[1], camera_coords[2]);
 		State.engine.RiRotate(90.0, 1.0, 0.0, 0.0);
@@ -119,7 +119,7 @@ public:
 
 	static k3d::iplugin_factory& get_factory()
 	{
-		static k3d::plugin_factory<k3d::document_plugin<background_sphere_implementation> >factory(
+		static k3d::plugin_factory<k3d::document_plugin<background_sphere> >factory(
 			k3d::uuid(0x00000001, 0x00000000, 0x00000000, 0x00000019),
 			"RenderManBackgroundSphere",
 			_("Places a shader on a sphere, centered on the camera and aligned with world coordinates"),
@@ -141,7 +141,7 @@ namespace libk3drenderman
 
 k3d::iplugin_factory& background_sphere_factory()
 {
-	return background_sphere_implementation::get_factory();
+	return background_sphere::get_factory();
 }
 
 } // namespace libk3drenderman
