@@ -24,41 +24,34 @@
 
 #include <k3dsdk/classes.h>
 #include <k3dsdk/i18n.h>
-#include <k3dsdk/iapplication.h>
 #include <k3dsdk/imaterial.h>
-#include <k3dsdk/igl.h>
 #include <k3dsdk/node.h>
 #include <k3dsdk/persistent.h>
 #include <k3dsdk/measurement.h>
 #include <k3dsdk/module.h>
 #include <k3dsdk/renderman.h>
 #include <k3dsdk/vectors.h>
-#include <k3dsdk/gl.h>
 
 namespace libk3drenderman
 {
 
 /////////////////////////////////////////////////////////////////////////////
-// material_implementation
-
-const GLfloat m_BlackData[4] = {0.0f, 0.0f, 0.0f, 1.0f};
-const GLfloat m_WhiteData[4] = {1.0f, 1.0f, 1.0f, 1.0f};
+// material
 
 const std::string shadowtype_none = "None";
 const std::string shadowtype_opaque = "Opaque";
 const std::string shadowtype_opacity = "Opacity";
 const std::string shadowtype_shaded = "Shaded";
 
-class material_implementation :
+class material :
 	public k3d::persistent<k3d::node> ,
 	public k3d::imaterial,
-	public k3d::gl::imaterial,
 	public k3d::ri::imaterial
 {
 	typedef k3d::persistent<k3d::node>  base;
 
 public:
-	material_implementation(k3d::idocument& Document) :
+	material(k3d::idocument& Document) :
 		base(Document),
 		m_surface_shader(init_owner(*this) + init_name("surface_shader") + init_label(_("Surface Shader")) + init_description(_("Surface shader")) + init_value<k3d::ri::isurface_shader*>(0)),
 		m_displacement_shader(init_owner(*this) + init_name("displacement_shader") + init_label(_("Displacement Shader")) + init_description(_("Displacement shader")) + init_value<k3d::ri::idisplacement_shader*>(0)),
@@ -70,45 +63,28 @@ public:
 		m_Color(init_owner(*this) + init_name("color") + init_label(_("Color")) + init_description(_("Color")) + init_value(k3d::color(1, 1, 1))),
 		m_Opacity(init_owner(*this) + init_name("opacity") + init_label(_("Opacity")) + init_description(_("Opacity")) + init_value(k3d::color(1, 1, 1)))
 	{
-		deleted_signal().connect(sigc::mem_fun(*this, &material_implementation::on_deleted));
-	
-		m_Color.changed_signal().connect(sigc::mem_fun(*this, &material_implementation::on_async_redraw_all));
+		deleted_signal().connect(sigc::mem_fun(*this, &material::on_deleted));
 
 		m_RiTexture = 0;
 	}
 
-	void on_async_redraw_all()
-	{
-		k3d::gl::redraw_all(document(), k3d::gl::irender_engine::ASYNCHRONOUS);
-	}
-
 	void on_deleted()
 	{
 	}
 
-	void setup_gl_material()
+	k3d::gl::imaterial* gl_material()
 	{
-		// Cache some current values for speed ...
-		k3d::color color = m_Color.value();
-		m_DiffuseData[0] = (GLfloat)color.red;
-		m_DiffuseData[1] = (GLfloat)color.green;
-		m_DiffuseData[2] = (GLfloat)color.blue;
-		m_DiffuseData[3] = 1.0f;
+		return 0;
+	}
 
-		glDepthFunc(GL_LESS);
-		glEnable(GL_DEPTH_TEST);
-		glDisable(GL_TEXTURE_2D);
+	k3d::ri::imaterial* ri_material()
+	{
+		return this;
+	}
 
-		glDepthMask(GL_TRUE);
-		glDisable(GL_BLEND);
-
-		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, m_BlackData);
-		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, m_DiffuseData);
-		glColor4fv(m_DiffuseData);
-		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, m_WhiteData);
-		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, m_BlackData);
-//		glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 64.0f);
-		glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, m_BlackData);
+	k3d::yafray::imaterial* yafray_material()
+	{
+		return 0;
 	}
 
 	void setup_renderman_material(const k3d::ri::render_state& State)
@@ -152,10 +128,9 @@ public:
 
 	static k3d::iplugin_factory& get_factory()
 	{
-		static k3d::plugin_factory<k3d::document_plugin<material_implementation>,
+		static k3d::plugin_factory<k3d::document_plugin<material>,
 				k3d::interface_list<k3d::imaterial,
-				k3d::interface_list<k3d::gl::imaterial,
-				k3d::interface_list<k3d::ri::imaterial> > > > factory(
+				k3d::interface_list<k3d::ri::imaterial> > > factory(
 			k3d::classes::RenderManMaterial(),
 			"RenderManMaterial",
 			_("A RenderMan surface / displacement material"),
@@ -177,10 +152,6 @@ private:
 	k3d_data(k3d::color, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_Color;
 	k3d_data(k3d::color, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_Opacity;
 
-	GLfloat m_AmbientData[4];
-	GLfloat m_DiffuseData[4];
-	GLfloat m_SpecularData[4];
-
 	k3d::ri::itexture* m_RiTexture;
 };
 
@@ -189,7 +160,7 @@ private:
 
 k3d::iplugin_factory& material_factory()
 {
-	return material_implementation::get_factory();
+	return material::get_factory();
 }
 
 } // namespace libk3drenderman
