@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2004, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -23,14 +23,16 @@
 
 #include <k3dsdk/classes.h>
 #include <k3dsdk/frames.h>
+#include <k3dsdk/fstream.h>
+#include <k3dsdk/gl.h>
 #include <k3dsdk/i18n.h>
-#include <k3dsdk/ianimation_render_engine.h>
 #include <k3dsdk/icamera.h>
+#include <k3dsdk/icamera_animation_render_engine.h>
+#include <k3dsdk/icamera_preview_render_engine.h>
+#include <k3dsdk/icamera_still_render_engine.h>
 #include <k3dsdk/icrop_window.h>
-#include <k3dsdk/ipreview_render_engine.h>
 #include <k3dsdk/iprojection.h>
 #include <k3dsdk/irender_farm.h>
-#include <k3dsdk/istill_render_engine.h>
 #include <k3dsdk/itransform_source.h>
 #include <k3dsdk/iuser_interface.h>
 #include <k3dsdk/measurement.h>
@@ -40,15 +42,15 @@
 #include <k3dsdk/persistent.h>
 #include <k3dsdk/property.h>
 #include <k3dsdk/property_group_collection.h>
+#include <k3dsdk/render_farm.h>
 #include <k3dsdk/renderman.h>
 #include <k3dsdk/resolutions.h>
+#include <k3dsdk/shader_cache.h>
 #include <k3dsdk/shaders.h>
 #include <k3dsdk/string_cast.h>
 #include <k3dsdk/time_source.h>
+#include <k3dsdk/user_interface.h>
 #include <k3dsdk/version.h>
-#include <k3dsdk/gl.h>
-
-#include <k3dsdk/fstream.h>
 
 #include <iomanip>
 
@@ -133,9 +135,9 @@ private:
 
 class render_engine :
 	public k3d::persistent<k3d::node>,
-	public k3d::ipreview_render_engine,
-	public k3d::istill_render_engine,
-	public k3d::ianimation_render_engine,
+	public k3d::icamera_preview_render_engine,
+	public k3d::icamera_still_render_engine,
+	public k3d::icamera_animation_render_engine,
 	public k3d::property_group_collection
 {
 	typedef k3d::persistent<k3d::node> base;
@@ -157,12 +159,12 @@ public:
 		m_grid_size(init_owner(*this) + init_name("grid_size") + init_label(_("Grid Size")) + init_description(_("Grid Size")) + init_value(256) + init_constraint(constraint::minimum(1L)) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
 		m_eye_splits(init_owner(*this) + init_name("eye_splits") + init_label(_("Eye Splits")) + init_description(_("Eye Splits")) + init_value(10) + init_constraint(constraint::minimum(1L)) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
 		m_texture_memory(init_owner(*this) + init_name("texture_memory") + init_label(_("Texture Memory")) + init_description(_("Texture Memory")) + init_value(1024) + init_constraint(constraint::minimum(1L)) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
-		m_render_alpha(init_owner(*this) + init_name("render_alpha") + init_label(_("Render Alpha")) + init_description(_("Render Alpha")) + init_value(true)),
-		m_pixel_xsamples(init_owner(*this) + init_name("pixel_xsamples") + init_label(_("Pixel X Samples")) + init_description(_("Pixel X Samples")) + init_value(1.0) + init_constraint(constraint::minimum(std::numeric_limits<double>::epsilon())) + init_precision(2) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar))),
-		m_pixel_ysamples(init_owner(*this) + init_name("pixel_ysamples") + init_label(_("Pixel Y Samples")) + init_description(_("Pixel Y Samples")) + init_value(1.0) + init_constraint(constraint::minimum(std::numeric_limits<double>::epsilon())) + init_precision(2) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar))),
-		m_pixel_filter(init_owner(*this) + init_name("pixel_filter") + init_label(_("Pixel Filter")) + init_description(_("Pixel Filter")) + init_value(k3d::ri::RI_GAUSSIAN()) + init_values(pixel_filter_values())),
-		m_pixel_filter_width(init_owner(*this) + init_name("pixel_filter_width") + init_label(_("Pixel Filter Width")) + init_description(_("Pixel Filter Width")) + init_value(2.0) + init_constraint(constraint::minimum(std::numeric_limits<double>::epsilon())) + init_precision(2) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar))),
-		m_pixel_filter_height(init_owner(*this) + init_name("pixel_filter_height") + init_label(_("Pixel Filter Height")) + init_description(_("Pixel Filter Height")) + init_value(2.0) + init_constraint(constraint::minimum(std::numeric_limits<double>::epsilon())) + init_precision(2) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar))),
+		m_render_alpha(init_owner(*this) + init_name("render_alpha") + init_label(_("Render Alpha")) + init_description(_("Render Alpha")) + init_value(false)),
+		m_pixel_xsamples(init_owner(*this) + init_name("pixel_xsamples") + init_label(_("Pixel X Samples")) + init_description(_("The smaller the faster, the higher the better (anti-aliasing)")) + init_value(3.0) + init_constraint(constraint::minimum(1.0)) + init_precision(2) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar))),
+		m_pixel_ysamples(init_owner(*this) + init_name("pixel_ysamples") + init_label(_("Pixel Y Samples")) + init_description(_("The smaller the faster, the higher the better (anti-aliasing)")) + init_value(3.0) + init_constraint(constraint::minimum(1.0)) + init_precision(2) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar))),
+		m_pixel_filter(init_owner(*this) + init_name("pixel_filter") + init_label(_("Pixel Filter")) + init_description(_("Anti-aliasing type")) + init_value(k3d::ri::RI_GAUSSIAN()) + init_values(pixel_filter_values())),
+		m_pixel_filter_width(init_owner(*this) + init_name("pixel_filter_width") + init_label(_("Pixel Filter Width")) + init_description(_("Filter width in X")) + init_value(2.0) + init_constraint(constraint::minimum(std::numeric_limits<double>::epsilon())) + init_precision(2) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar))),
+		m_pixel_filter_height(init_owner(*this) + init_name("pixel_filter_height") + init_label(_("Pixel Filter Height")) + init_description(_("Filter width in Y")) + init_value(2.0) + init_constraint(constraint::minimum(std::numeric_limits<double>::epsilon())) + init_precision(2) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar))),
 		m_exposure(init_owner(*this) + init_name("exposure") + init_label(_("Exposure")) + init_description(_("Exposure")) + init_value(1.0) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::scalar))),
 		m_gamma(init_owner(*this) + init_name("gamma") + init_label(_("Gamma")) + init_description(_("Gamma")) + init_value(1.0) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::scalar))),
 		m_dof(init_owner(*this) + init_name("dof") + init_label(_("DOF")) + init_description(_("Depth of Field")) + init_value(false)),
@@ -186,7 +188,7 @@ public:
 		output_group.properties.push_back(&static_cast<k3d::iproperty&>(m_default_exterior_shader));
 		output_group.properties.push_back(&static_cast<k3d::iproperty&>(m_imager_shader));
 		output_group.properties.push_back(&static_cast<k3d::iproperty&>(m_render_alpha));
-		
+
 		k3d::iproperty_group_collection::group sampling_group("Sampling");
 		sampling_group.properties.push_back(&static_cast<k3d::iproperty&>(m_bucket_width));
 		sampling_group.properties.push_back(&static_cast<k3d::iproperty&>(m_bucket_height));
@@ -208,7 +210,7 @@ public:
 		lens_group.properties.push_back(&static_cast<k3d::iproperty&>(m_fstop));
 		lens_group.properties.push_back(&static_cast<k3d::iproperty&>(m_focal_length));
 		lens_group.properties.push_back(&static_cast<k3d::iproperty&>(m_focus_plane));
-		
+
 		register_property_group(output_group);
 		register_property_group(sampling_group);
 		register_property_group(lens_group);
@@ -216,7 +218,7 @@ public:
 		m_resolution.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_resolution_changed));
 	}
 
-	void on_resolution_changed()
+	void on_resolution_changed(k3d::iunknown*)
 	{
 		const std::string new_resolution = m_resolution.value();
 
@@ -230,17 +232,17 @@ public:
 			m_pixel_height.set_value(resolution->height);
 			return;
 		}
-	
+
 		assert_not_reached();
 	}
-	
-	bool render_preview(k3d::icamera& Camera)
+
+	bool render_camera_preview(k3d::icamera& Camera)
 	{
 		// Keep track of compiled shaders ...
 		k3d::ri::render_engine::shaders_t shaders;
 
 		// Start a new render job ...
-		k3d::irender_job& job = k3d::application().render_farm().create_job("k3d-preview");
+		k3d::irender_job& job = k3d::render_farm().create_job("k3d-preview");
 
 		// Add a single render frame to the job ...
 		k3d::irender_frame& frame = job.create_frame("frame");
@@ -253,12 +255,12 @@ public:
 		return_val_if_fail(render(Camera, frame, outputimagepath, true, shaders), false);
 
 		// Start the job running ...
-		k3d::application().render_farm().start_job(job);
+		k3d::render_farm().start_job(job);
 
 		return true;
 	}
 
-	bool render_frame(k3d::icamera& Camera, const boost::filesystem::path& OutputImage, const bool ViewImage)
+	bool render_camera_frame(k3d::icamera& Camera, const boost::filesystem::path& OutputImage, const bool ViewImage)
 	{
 		// Sanity checks ...
 		return_val_if_fail(!OutputImage.empty(), false);
@@ -267,7 +269,7 @@ public:
 		k3d::ri::render_engine::shaders_t shaders;
 
 		// Start a new render job ...
-		k3d::irender_job& job = k3d::application().render_farm().create_job("k3d-render-frame");
+		k3d::irender_job& job = k3d::render_farm().create_job("k3d-render-frame");
 
 		// Add a single render frame to the job ...
 		k3d::irender_frame& frame = job.create_frame("frame");
@@ -287,12 +289,12 @@ public:
 		return_val_if_fail(render(Camera, frame, outputimagepath, false, shaders), false);
 
 		// Start the job running ...
-		k3d::application().render_farm().start_job(job);
+		k3d::render_farm().start_job(job);
 
 		return true;
 	}
 
-	bool render_animation(k3d::icamera& Camera, const boost::filesystem::path& OutputImages, const bool ViewCompletedImages)
+	bool render_camera_animation(k3d::icamera& Camera, const boost::filesystem::path& OutputImages, const bool ViewCompletedImages)
 	{
 		// Sanity checks ...
 		return_val_if_fail(!OutputImages.empty(), false);
@@ -308,10 +310,10 @@ public:
 		const double start_time = boost::any_cast<double>(k3d::get_value(document().dag(), *start_time_property));
 		const double end_time = boost::any_cast<double>(k3d::get_value(document().dag(), *end_time_property));
 		const double frame_rate = boost::any_cast<double>(k3d::get_value(document().dag(), *frame_rate_property));
-		
+
 		const long start_frame = static_cast<long>(k3d::round(frame_rate * start_time));
 		const long end_frame = static_cast<long>(k3d::round(frame_rate * end_time));
-		
+
 		k3d::frames frames(OutputImages, start_frame, end_frame);
 		return_val_if_fail(frames.max_frame() >= end_frame, false);
 
@@ -319,7 +321,7 @@ public:
 		k3d::ri::render_engine::shaders_t shaders;
 
 		// Start a new render job ...
-		k3d::irender_job& job = k3d::application().render_farm().create_job("k3d-render-animation");
+		k3d::irender_job& job = k3d::render_farm().create_job("k3d-render-animation");
 
 		// For each frame to be rendered ...
 		for(long view_frame = start_frame; view_frame < end_frame; ++view_frame)
@@ -353,7 +355,7 @@ public:
 		}
 
 		// Start the job running ...
-		k3d::application().render_farm().start_job(job);
+		k3d::render_farm().start_job(job);
 
 		return true;
 	}
@@ -367,9 +369,9 @@ public:
 	{
 		static k3d::plugin_factory<
 			k3d::document_plugin<render_engine>,
-				 k3d::interface_list<k3d::ianimation_render_engine,
-				 k3d::interface_list<k3d::istill_render_engine,
-				 k3d::interface_list<k3d::ipreview_render_engine> > > > factory(
+				 k3d::interface_list<k3d::icamera_animation_render_engine,
+				 k3d::interface_list<k3d::icamera_still_render_engine,
+				 k3d::interface_list<k3d::icamera_preview_render_engine> > > > factory(
 					k3d::classes::RenderManEngine(),
 					"RenderManEngine",
 					_("RenderMan Render Engine"),
@@ -390,7 +392,7 @@ private:
 		k3d::iproperty* const time_property = k3d::get_time(document());
 		k3d::iwritable_property* const writable_time_property = dynamic_cast<k3d::iwritable_property*>(time_property);
 		return_val_if_fail(frame_rate_property && time_property && writable_time_property, false);
-		
+
 		const double frame_delta = 1.0 / boost::any_cast<double>(k3d::get_value(document().dag(), *frame_rate_property));
 		const double frame_time = boost::any_cast<double>(k3d::get_value(document().dag(), *time_property));
 
@@ -418,7 +420,11 @@ private:
 		engine.RiComment("Setup file search paths");
 
 		k3d::ri::parameter_list searchpath_options;
-		searchpath_options.push_back(k3d::ri::parameter("shader", k3d::ri::UNIFORM, k3d::application().shader_cache_path().native_file_string() + ":&"));
+		searchpath_options.push_back(
+			k3d::ri::parameter(
+				"shader",
+				k3d::ri::UNIFORM,
+				k3d::ri::string(k3d::shader_cache_path().native_file_string() + ":&")));
 		engine.RiOptionV("searchpath", searchpath_options);
 
 		// Perform frame-start operations for textures
@@ -447,7 +453,7 @@ private:
 		engine.RiNewline();
 		engine.RiComment("Setup user options");
 		k3d::ri::set_options(*this, engine);
-		
+
 		// Set the display type ...
 		engine.RiNewline();
 		engine.RiComment("Setup Display");
@@ -467,7 +473,7 @@ private:
 		const k3d::ri::unsigned_integer pixel_width = m_pixel_width.value();
 		const k3d::ri::unsigned_integer pixel_height = m_pixel_height.value();
 		const k3d::ri::real pixel_aspect_ratio = m_pixel_aspect_ratio.value();
-		
+
 		engine.RiFormat(pixel_width, pixel_height, pixel_aspect_ratio);
 
 		// Set pixel sampling rates ...
@@ -503,8 +509,8 @@ private:
 		const double crop_bottom = boost::any_cast<double>(get_value(document().dag(), Camera.crop_window().crop_bottom()));
 
 		if(crop_left >= crop_right || crop_top >= crop_bottom)
-			k3d::application().user_interface().message("Current crop window settings will produce an empty image", "RenderMan:");
-		
+			k3d::user_interface().message("Current crop window settings will produce an empty image", "RenderMan:");
+
 		engine.RiCropWindow(crop_left, crop_right, crop_top, crop_bottom);
 
 		// Setup our motion-blur sampling loop ...
@@ -513,7 +519,7 @@ private:
 			samples.push_back(1.0);
 
 		const bool motion_blur_camera = m_motion_blur.value();
-		std::vector<k3d::vector3> motion_blur_camera_position_samples;
+		std::vector<k3d::point3> motion_blur_camera_position_samples;
 		std::vector<k3d::angle_axis> motion_blur_camera_orientation_samples;
 
 		for(boost::uint32_t sample_index = 0; sample_index < samples.size(); ++sample_index)
@@ -526,7 +532,7 @@ private:
 
 			const k3d::matrix4 transform_matrix = boost::any_cast<k3d::matrix4>(get_value(document().dag(), Camera.transformation().transform_source_output()));
 			const k3d::angle_axis orientation(k3d::euler_angles(transform_matrix, k3d::euler_angles::ZXYstatic));
-			const k3d::vector3 position(k3d::extract_translation(transform_matrix));
+			const k3d::point3 position(k3d::position(transform_matrix));
 
 			motion_blur_camera_position_samples.push_back(position);
 			motion_blur_camera_orientation_samples.push_back(orientation);
@@ -538,7 +544,7 @@ private:
 			{
 				engine.RiNewline();
 				engine.RiComment("Setup viewing transformation");
-				
+
 				if(k3d::iperspective* const perspective = dynamic_cast<k3d::iperspective*>(&Camera.projection()))
 				{
 					const double left = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->left()));
@@ -547,9 +553,10 @@ private:
 					const double bottom = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->bottom()));
 					const double near = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->near()));
 					const double far = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->far()));
+					return_val_if_fail(near > 0, false);
 
 					engine.RiProjectionV("perspective");
-					engine.RiScreenWindow(left, right, bottom, top);
+					engine.RiScreenWindow(left / near, right / near, bottom / near, top / near);
 					engine.RiClipping(near, far);
 				}
 				else if(k3d::iorthographic* const orthographic = dynamic_cast<k3d::iorthographic*>(&Camera.projection()))
@@ -560,9 +567,22 @@ private:
 					const double bottom = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->bottom()));
 					const double near = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->near()));
 					const double far = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->far()));
+					return_val_if_fail(near > 0, false);
+
+					const k3d::matrix4 transform_matrix = boost::any_cast<k3d::matrix4>(get_value(document().dag(), Camera.transformation().transform_source_output()));
+					const k3d::point3 world_position = transform_matrix * k3d::point3(0, 0, 0);
+					const k3d::point3 world_target = boost::any_cast<k3d::point3>(Camera.world_target().property_value());
+
+					const double distance = k3d::distance(world_position, world_target);
+					const double height = top - bottom;
+					return_val_if_fail(height, false);
+
+					const double aspect = (right - left) / height;
+					const double tan_fov = height * 0.5 / near;
+					const double size = distance * tan_fov;
 
 					engine.RiProjectionV("orthographic");
-					engine.RiScreenWindow(left, right, bottom, top);
+					engine.RiScreenWindow(-size * aspect, size * aspect, -size, size);
 					engine.RiClipping(near, far);
 				}
 
@@ -576,7 +596,7 @@ private:
 						engine.RiRotate(-k3d::degrees(a[0]), 0.0f, 0.0f, 1.0f);
 					}
 					engine.RiMotionEnd();
-					
+
 					engine.RiMotionBeginV(state.sample_times);
 					for(unsigned int i = 0; i < motion_blur_camera_orientation_samples.size(); ++i)
 					{
@@ -594,7 +614,7 @@ private:
 						engine.RiRotate(-k3d::degrees(a[2]), 0.0f, 1.0f, 0.0f);
 					}
 					engine.RiMotionEnd();
-					
+
 					engine.RiMotionBeginV(state.sample_times);
 					for(unsigned int i = 0; i < motion_blur_camera_position_samples.size(); ++i)
 						engine.RiTranslate(-motion_blur_camera_position_samples[i][0], -motion_blur_camera_position_samples[i][1], -motion_blur_camera_position_samples[i][2]);
@@ -609,11 +629,11 @@ private:
 					q = k3d::quaternion(motion_blur_camera_orientation_samples[0]);
 					a = k3d::euler_angles(q, k3d::euler_angles::ZXYstatic);
 					engine.RiRotate(-k3d::degrees(a[1]), 1.0f, 0.0f, 0.0f);
-					
+
 					q = k3d::quaternion(motion_blur_camera_orientation_samples[0]);
 					a = k3d::euler_angles(q, k3d::euler_angles::ZXYstatic);
 					engine.RiRotate(-k3d::degrees(a[2]), 0.0f, 1.0f, 0.0f);
-					
+
 					engine.RiTranslate(-motion_blur_camera_position_samples[0][0], -motion_blur_camera_position_samples[0][1], -motion_blur_camera_position_samples[0][2]);
 				}
 
@@ -626,7 +646,7 @@ private:
 					m_default_exterior_shader.value()->setup_renderman_exterior_shader(state);
 				if(m_imager_shader.value())
 					m_imager_shader.value()->setup_renderman_imager_shader(state);
-					
+
 				// Begin the world ...
 				engine.RiNewline();
 				engine.RiWorldBegin();
@@ -653,28 +673,18 @@ private:
 
 		// Synchronize shaders ...
 		const k3d::ri::render_engine::shaders_t shaders = engine.shaders();
-		for(k3d::ri::render_engine::shaders_t::const_iterator shader_name = shaders.begin(); shader_name != shaders.end(); ++shader_name)
+		for(k3d::ri::render_engine::shaders_t::const_iterator shader = shaders.begin(); shader != shaders.end(); ++shader)
 		{
 			// If this shader has already been synchronized, we're done ...
-			if(Shaders.count(*shader_name))
+			if(Shaders.count(*shader))
 				continue;
 
-			// Find a matching shader ...
-			for(k3d::sl::shaders_t::const_iterator shader = k3d::application().shaders().begin(); shader != k3d::application().shaders().end(); ++shader)
-			{
-				// No match ...
-				if(shader->name != *shader_name)
-					continue;
-
-				// Compile that bad-boy!
-				if(!k3d::compile_shader(boost::filesystem::path(shader->file_path, boost::filesystem::native), "ri", m_render_engine.value()))
-					k3d::log() << error << k3d::string_cast(boost::format(_("Error compiling shader %1%")) % shader->name) << std::endl;
-
-				break;
-			}
+			// Compile that bad-boy!
+			if(!k3d::compile_shader(*shader, "ri", m_render_engine.value()))
+				k3d::log() << error << k3d::string_cast(boost::format(_("Error compiling shader %1%")) % shader->native_file_string()) << std::endl;
 
 			// Keep track of this shader ...
-			Shaders.insert(*shader_name);
+			Shaders.insert(*shader);
 		}
 
 		return true;
