@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2004, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -42,16 +42,16 @@ namespace libk3drenderman
 {
 
 /////////////////////////////////////////////////////////////////////////////
-// shadow_map_implementation
+// shadow_map
 
-class shadow_map_implementation :
+class shadow_map :
 	public k3d::persistent<k3d::node>,
 	public k3d::ri::itexture
 {
 	typedef k3d::persistent<k3d::node> base;
 
 public:
-	shadow_map_implementation(k3d::idocument& Document) :
+	shadow_map(k3d::idocument& Document) :
 		base(Document),
 		m_camera(init_owner(*this) + init_name("camera") + init_label(_("Camera")) + init_description(_("Shadow Map Camera")) + init_value<k3d::icamera*>(0)),
 		m_create_shadow_map(init_owner(*this) + init_name("create") + init_label(_("create")) + init_description(_("Create shadow map during rendering")) + init_value(true)),
@@ -61,27 +61,27 @@ public:
 		m_pixel_height(init_owner(*this) + init_name("pixel_height") + init_label(_("pixel_height")) + init_description(_("Output pixel height")) + init_value(256) + init_constraint(constraint::minimum(1L)) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
 		m_pixel_aspect_ratio(init_owner(*this) + init_name("pixel_aspect_ratio") + init_label(_("pixel_aspect_ratio")) + init_description(_("Output pixel aspect ratio")) + init_value(1.0) + init_constraint(constraint::minimum(std::numeric_limits<double>::epsilon())) + init_precision(2) + init_step_increment(0.01) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_resolution.changed_signal().connect(sigc::mem_fun(*this, &shadow_map_implementation::on_resolution_changed));
+		m_resolution.changed_signal().connect(sigc::mem_fun(*this, &shadow_map::on_resolution_changed));
 	}
 
-	void on_resolution_changed()
+	void on_resolution_changed(k3d::iunknown*)
 	{
 		const std::string new_resolution = m_resolution.value();
 
 		const k3d::resolutions_t& resolutions = k3d::resolutions();
 		for(k3d::resolutions_t::const_iterator resolution = resolutions.begin(); resolution != resolutions.end(); ++resolution)
-			{
-				if(resolution->name != new_resolution)
-					continue;
+		{
+			if(resolution->name != new_resolution)
+				continue;
+
+			m_pixel_width.set_value(resolution->width);
+			m_pixel_height.set_value(resolution->height);
+			return;
+		}
 
-				m_pixel_width.set_value(resolution->width);
-				m_pixel_height.set_value(resolution->height);
-				return;
-			}
-	
 		assert_not_reached();
 	}
-	
+
 	void setup_renderman_texture(k3d::irender_frame& Frame, k3d::ri::irender_engine& Engine)
 	{
 		m_shadow_map_path = boost::filesystem::path();
@@ -91,7 +91,7 @@ public:
 
 		k3d::icamera* const camera = m_camera.value();
 		return_if_fail(camera);
-		
+
 		Engine.RiFrameBegin(0);
 
 		m_shadow_map_path = Frame.add_output_file("shadowmap.z");
@@ -113,31 +113,45 @@ public:
 		Engine.RiComment("Setup shadowmap viewing transformations");
 
 		if(k3d::iperspective* const perspective = dynamic_cast<k3d::iperspective*>(&camera->projection()))
-			{
-				const double left = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->left()));
-				const double right = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->right()));
-				const double top = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->top()));
-				const double bottom = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->bottom()));
-				const double near = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->near()));
-				const double far = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->far()));
+		{
+			const double left = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->left()));
+			const double right = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->right()));
+			const double top = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->top()));
+			const double bottom = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->bottom()));
+			const double near = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->near()));
+			const double far = boost::any_cast<double>(k3d::get_value(document().dag(), perspective->far()));
+			return_if_fail(near > 0);
 
-				Engine.RiProjectionV("perspective");
-				Engine.RiScreenWindow(left, right, bottom, top);
-				Engine.RiClipping(near, far);
-			}
+			Engine.RiProjectionV("perspective");
+			Engine.RiScreenWindow(left / near, right / near, bottom / near, top / near);
+			Engine.RiClipping(near, far);
+		}
 		else if(k3d::iorthographic* const orthographic = dynamic_cast<k3d::iorthographic*>(&camera->projection()))
-			{
-				const double left = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->left()));
-				const double right = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->right()));
-				const double top = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->top()));
-				const double bottom = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->bottom()));
-				const double near = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->near()));
-				const double far = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->far()));
+		{
+			const double left = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->left()));
+			const double right = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->right()));
+			const double top = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->top()));
+			const double bottom = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->bottom()));
+			const double near = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->near()));
+			const double far = boost::any_cast<double>(k3d::get_value(document().dag(), orthographic->far()));
+			return_if_fail(near > 0);
 
-				Engine.RiProjectionV("orthographic");
-				Engine.RiScreenWindow(left, right, bottom, top);
-				Engine.RiClipping(near, far);
-			}
+			const k3d::matrix4 transform_matrix = boost::any_cast<k3d::matrix4>(get_value(document().dag(), camera->transformation().transform_source_output()));
+			const k3d::point3 world_position = transform_matrix * k3d::point3(0, 0, 0);
+			const k3d::point3 world_target = boost::any_cast<k3d::point3>(camera->world_target().property_value());
+
+			const double distance = k3d::distance(world_position, world_target);
+			const double height = top - bottom;
+			return_if_fail(height);
+
+			const double aspect = (right - left) / height;
+			const double tan_fov = height * 0.5 / near;
+			const double size = distance * tan_fov;
+
+			Engine.RiProjectionV("orthographic");
+			Engine.RiScreenWindow(-size * aspect, size * aspect, -size, size);
+			Engine.RiClipping(near, far);
+		}
 
 		// Setup the camera viewing transform ...
 		const k3d::matrix4 transform_matrix = boost::any_cast<k3d::matrix4>(get_value(document().dag(), camera->transformation().transform_source_output()));
@@ -148,11 +162,11 @@ public:
 		// Render objects ...
 		k3d::ri::render_state state(Frame, Engine, camera->projection(), k3d::ri::SHADOW_MAP, k3d::ri::sample_times_t(1, 0.0), 0, transform_matrix);
 		for(k3d::nodes_t::const_iterator object = document().nodes().collection().begin(); object != document().nodes().collection().end(); ++object)
-			{
-				k3d::ri::irenderable* const renderable = dynamic_cast<k3d::ri::irenderable*>(*object);
-				if(renderable)
-					renderable->renderman_render(state);
-			}
+		{
+			k3d::ri::irenderable* const renderable = dynamic_cast<k3d::ri::irenderable*>(*object);
+			if(renderable)
+				renderable->renderman_render(state);
+		}
 
 		Engine.RiWorldEnd();
 		Engine.RiFrameEnd();
@@ -171,13 +185,13 @@ public:
 	static k3d::iplugin_factory& get_factory()
 	{
 		static k3d::plugin_factory<
-			k3d::document_plugin<shadow_map_implementation>,
-			k3d::interface_list<k3d::ri::itexture> > factory(
-			k3d::uuid(0x9bce6e2b, 0x083c4353, 0x829052e9, 0xf98c7023),
-			"RenderManShadowMap",
-			_("A shadow_map source within the 3D workspace"),
-			"RenderMan",
-			k3d::iplugin_factory::STABLE);
+			k3d::document_plugin<shadow_map>,
+				k3d::interface_list<k3d::ri::itexture> > factory(
+				k3d::uuid(0x9bce6e2b, 0x083c4353, 0x829052e9, 0xf98c7023),
+				"RenderManShadowMap",
+				_("A shadow_map source within the 3D workspace"),
+				"RenderMan",
+				k3d::iplugin_factory::STABLE);
 
 		return factory;
 	}
@@ -199,9 +213,8 @@ private:
 
 k3d::iplugin_factory& shadow_map_factory()
 {
-	return shadow_map_implementation::get_factory();
+	return shadow_map::get_factory();
 }
 
 } // namespace libk3drenderman
 
-
