@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2004, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -22,11 +22,11 @@
 		\author Romain Behar (romainbehar@yahoo.com)
 */
 
-#include <k3dsdk/i18n.h>
 #include <k3dsdk/axis.h>
 #include <k3dsdk/bitmap_modifier.h>
 #include <k3dsdk/classes.h>
 #include <k3dsdk/file_filter.h>
+#include <k3dsdk/i18n.h>
 #include <k3dsdk/ibitmap_write_format.h>
 #include <k3dsdk/irenderman.h>
 #include <k3dsdk/irender_farm.h>
@@ -44,16 +44,16 @@ namespace libk3drenderman
 {
 
 /////////////////////////////////////////////////////////////////////////////
-// texture_map_implementation
+// texture_map
 
-class texture_map_implementation :
+class texture_map :
 	public k3d::bitmap_modifier<k3d::persistent<k3d::node> >,
 	public k3d::ri::itexture
 {
 	typedef k3d::bitmap_modifier<k3d::persistent<k3d::node> > base;
 
 public:
-	texture_map_implementation(k3d::idocument& Document) :
+	texture_map(k3d::idocument& Document) :
 		base(Document),
 		m_cache_path(k3d::system::generate_temp_file()),
 		m_refresh_cache(true),
@@ -63,20 +63,19 @@ public:
 		m_swidth(init_owner(*this) + init_name("swidth") + init_label(_("swidth")) + init_description(_("Filter S Width")) + init_value(2.0) + init_constraint(constraint::minimum(std::numeric_limits<double>::epsilon())) + init_precision(2) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar))),
 		m_twidth(init_owner(*this) + init_name("twidth") + init_label(_("twidth")) + init_description(_("Filter T Width")) + init_value(2.0) + init_constraint(constraint::minimum(std::numeric_limits<double>::epsilon())) + init_precision(2) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		m_input.changed_signal().connect(sigc::mem_fun(*this, &texture_map_implementation::on_reset_bitmap));
-		m_output.need_data_signal().connect(sigc::mem_fun(*this, &texture_map_implementation::on_create_bitmap));
+		m_input.changed_signal().connect(sigc::mem_fun(*this, &texture_map::on_reset_bitmap));
 	}
 
-	~texture_map_implementation()
+	~texture_map()
 	{
 		if(!m_cache_path.empty())
 			boost::filesystem::remove(m_cache_path);
 	}
-	
+
 	void setup_renderman_texture(k3d::irender_frame& Frame, k3d::ri::irender_engine& Engine)
 	{
 		return_if_fail(!m_cache_path.empty());
-		
+
 		m_ri_image_path = boost::filesystem::path();
 		m_ri_texture_path = boost::filesystem::path();
 
@@ -87,19 +86,19 @@ public:
 
 		m_ri_image_path = Frame.add_input_file("texture");
 		return_if_fail(!m_ri_image_path.empty());
-		
+
 		m_ri_texture_path = Frame.add_input_file("texture");
 		return_if_fail(!m_ri_texture_path.empty());
 
 		if(m_refresh_cache || !boost::filesystem::exists(m_cache_path))
-			{
-				m_refresh_cache = false;
+		{
+			m_refresh_cache = false;
+
+			k3d::ibitmap_write_format* const filter = k3d::file_filter<k3d::ibitmap_write_format>(k3d::classes::TIFFWriter());
+			return_if_fail(filter);
+			return_if_fail(filter->write_file(m_cache_path, *input));
+		}
 
-				k3d::ibitmap_write_format* const filter = k3d::file_filter<k3d::ibitmap_write_format>(k3d::classes::TIFFWriter());
-				return_if_fail(filter);
-				return_if_fail(filter->write_file(m_cache_path, *input));
-			}
-		
 		boost::filesystem::copy_file(m_cache_path, m_ri_image_path);
 		Engine.RiMakeTextureV(m_ri_image_path.native_file_string(), m_ri_texture_path.native_file_string(), m_swrap.value(), m_twrap.value(), m_filter.value(), m_swidth.value(), m_twidth.value());
 	}
@@ -109,23 +108,20 @@ public:
 		return m_ri_texture_path;
 	}
 
-	void on_reset_bitmap()
+	void on_reset_bitmap(k3d::iunknown* const Hint)
 	{
 		m_refresh_cache = true;
-		m_output.reset();
+		m_output.reset(0, Hint);
 	}
-	
-	k3d::bitmap* on_create_bitmap()
+
+	void on_create_bitmap(k3d::bitmap& Bitmap)
 	{
 		// If we don't have any input bitmap, we're done ...
 		const k3d::bitmap* const input = m_input.value();
 		if(!input)
-			return 0;
+			return;
 
-		// Otherwise, we make a copy of the input bitmap and modify the copy ...
-		k3d::bitmap* const output = new k3d::bitmap(*input);
-
-		return output;
+		Bitmap = *input;
 	}
 
 	k3d::iplugin_factory& factory()
@@ -136,7 +132,7 @@ public:
 	static k3d::iplugin_factory& get_factory()
 	{
 		static k3d::plugin_factory<
-			k3d::document_plugin<texture_map_implementation>,
+			k3d::document_plugin<texture_map>,
 			k3d::interface_list<k3d::ibitmap_source,
 			k3d::interface_list<k3d::ibitmap_sink,
 			k3d::interface_list<k3d::ri::itexture> > > > factory(
@@ -158,7 +154,7 @@ private:
 	boost::filesystem::path m_ri_image_path;
 	/// Stores the absolute path to the version of this texture_map converted by the renderer (via RiMakeTexture())
 	boost::filesystem::path m_ri_texture_path;
-	
+
 	k3d_data(std::string, immutable_name, change_signal, with_undo, local_storage, no_constraint, list_property, with_serialization) m_swrap;
 	k3d_data(std::string, immutable_name, change_signal, with_undo, local_storage, no_constraint, list_property, with_serialization) m_twrap;
 	k3d_data(std::string, immutable_name, change_signal, with_undo, local_storage, no_constraint, list_property, with_serialization) m_filter;
@@ -169,25 +165,26 @@ private:
 	{
 		static k3d::ilist_property<std::string>::values_t values;
 		if(values.empty())
-			{
-				values.push_back(k3d::ri::RI_BLACK());
-				values.push_back(k3d::ri::RI_CLAMP());
-				values.push_back(k3d::ri::RI_PERIODIC());
-			}
+		{
+			values.push_back(k3d::ri::RI_BLACK());
+			values.push_back(k3d::ri::RI_CLAMP());
+			values.push_back(k3d::ri::RI_PERIODIC());
+		}
 		return values;
 	}
-	
+
 	const k3d::ilist_property<std::string>::values_t& filter_values()
 	{
 		static k3d::ilist_property<std::string>::values_t values;
 		if(values.empty())
-			{
-				values.push_back(k3d::ri::RI_GAUSSIAN());
-				values.push_back(k3d::ri::RI_BOX());
-				values.push_back(k3d::ri::RI_TRIANGLE());
-				values.push_back(k3d::ri::RI_CATMULL_ROM());
-				values.push_back(k3d::ri::RI_SINC());
-			}
+		{
+			values.push_back(k3d::ri::RI_GAUSSIAN());
+			values.push_back(k3d::ri::RI_BOX());
+			values.push_back(k3d::ri::RI_TRIANGLE());
+			values.push_back(k3d::ri::RI_CATMULL_ROM());
+			values.push_back(k3d::ri::RI_SINC());
+		}
+
 		return values;
 	}
 };
@@ -197,7 +194,7 @@ private:
 
 k3d::iplugin_factory& texture_map_factory()
 {
-	return texture_map_implementation::get_factory();
+	return texture_map::get_factory();
 }
 
 } // namespace libk3drenderman
