@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -22,14 +22,13 @@
 		\author Romain Behar <romainbehar@yahoo.com>
 */
 
-#include "ilight.h"
-
 #include <k3dsdk/color.h>
 #include <k3dsdk/gl.h>
 #include <k3dsdk/i18n.h>
+#include <k3dsdk/iyafray.h>
+#include <k3dsdk/module.h>
 #include <k3dsdk/node.h>
 #include <k3dsdk/persistent.h>
-#include <k3dsdk/module.h>
 #include <k3dsdk/selection.h>
 #include <k3dsdk/transform.h>
 #include <k3dsdk/transformable.h>
@@ -44,7 +43,7 @@ namespace libk3dyafray
 template<typename base_t>
 class light :
 	public base_t,
-	public yafray::ilight
+	public k3d::yafray::ilight
 {
 public:
 	light(k3d::idocument& Document) :
@@ -106,6 +105,7 @@ public:
 	drawable_light(k3d::idocument& Document) :
 		base(Document)
 	{
+		base::m_selection_weight.changed_signal().connect(base::make_async_redraw_slot());
 	}
 
 	void on_gl_draw(const k3d::gl::render_state& State)
@@ -116,7 +116,7 @@ public:
 		glDisable(GL_TEXTURE_1D);
 		glDisable(GL_TEXTURE_2D);
 
-		k3d::gl::color3d(base_t::get_visible_selection() ? k3d::color(1, 1, 1) : emitting ? k3d::color(1, 1, 0) : k3d::color(0, 0, 0));
+		k3d::gl::color3d(base_t::get_selection_weight() ? k3d::color(1, 1, 1) : emitting ? k3d::color(1, 1, 0) : k3d::color(0, 0, 0));
 		glLineWidth(1.0f);
 		glDisable(GL_LINE_STIPPLE);
 
@@ -135,7 +135,7 @@ private:
 	{
 		glBegin(GL_LINES);
 
-		k3d::vector3 coords(0, 0, 0);
+		k3d::point3 coords(0, 0, 0);
 
 		glVertex3d(coords[0] + 1.0, coords[1], coords[2]);
 		glVertex3d(coords[0] - 1.0, coords[1], coords[2]);
@@ -190,8 +190,15 @@ class point_light :
 public:
 	point_light(k3d::idocument& Document) :
 		base(Document),
+		m_quadric(gluNewQuadric()),
 		m_power(init_owner(*this) + init_name("power") + init_label(_("power")) + init_description(_("Power")) + init_value(400.0))
 	{
+		m_selection_weight.changed_signal().connect(make_async_redraw_slot());
+	}
+
+	~point_light()
+	{
+		gluDeleteQuadric(m_quadric);
 	}
 
 	void setup_light(std::ostream& Stream)
@@ -199,7 +206,7 @@ public:
 		if(!m_emit.value())
 			return;
 
-		const k3d::vector3 from = k3d::world_position(*this);
+		const k3d::point3 from = k3d::world_position(*this);
 		const k3d::color color = m_color.value();
 
 		Stream << "<light type=\"pointlight\" name=\"" << name() << "\" power=\"" << m_power.value() << "\" cast_shadows=\"" << (m_cast_shadows.value() ? "yes" : "no") << "\">" << std::endl;
@@ -208,6 +215,30 @@ public:
 		Stream << "</light>" << std::endl;
 	}
 
+	void on_gl_draw(const k3d::gl::render_state& State)
+	{
+		k3d::gl::store_attributes attributes;
+
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_AMBIENT, k3d::color(0, 0, 0));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_DIFFUSE, get_selection_weight() ? k3d::color(1, 1, 1) : k3d::color(0.4, 0.4, 0.4));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_SPECULAR, k3d::color(0, 0, 0));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_EMISSION, k3d::color(0, 0, 0));
+
+		glEnable(GL_LIGHTING);
+		draw_geometry();
+	}
+
+	void on_gl_select(const k3d::gl::render_state& State, const k3d::gl::select_state& SelectState)
+	{
+		k3d::gl::store_attributes attributes;
+
+		glDisable(GL_LIGHTING);
+
+		k3d::gl::push_selection_token(this);
+		draw_geometry();
+		k3d::gl::pop_selection_token();
+	}
+
 	k3d::iplugin_factory& factory()
 	{
 		return get_factory();
@@ -219,16 +250,24 @@ public:
 			k3d::document_plugin<point_light>,
 			k3d::interface_list<k3d::itransform_source,
 			k3d::interface_list<k3d::itransform_sink > > > factory(
-			k3d::uuid(0xd693bb64, 0xd73943ce, 0x80852061, 0x24fd242e),
-			"YafrayPointLight",
-			_("Yafray Point Light"),
-			"Yafray Lights",
-			k3d::iplugin_factory::STABLE);
+				k3d::uuid(0xd693bb64, 0xd73943ce, 0x80852061, 0x24fd242e),
+				"YafrayPointLight",
+				_("Yafray Point Light"),
+				"Yafray Lights",
+				k3d::iplugin_factory::STABLE);
 
 		return factory;
 	}
 
 private:
+	void draw_geometry()
+	{
+		gluSphere(m_quadric, 0.5, 8, 8);
+	}
+
+	/// Stores a GLU quadric object for drawing the manipulators
+	GLUquadricObj* const m_quadric;
+
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_power;
 };
 
@@ -243,8 +282,15 @@ class sun_light :
 public:
 	sun_light(k3d::idocument& Document) :
 		base(Document),
+		m_quadric(gluNewQuadric()),
 		m_power(init_owner(*this) + init_name("power") + init_label(_("power")) + init_description(_("Power")) + init_value(40.0))
 	{
+		m_selection_weight.changed_signal().connect(make_async_redraw_slot());
+	}
+
+	~sun_light()
+	{
+		gluDeleteQuadric(m_quadric);
 	}
 
 	void setup_light(std::ostream& Stream)
@@ -252,7 +298,7 @@ public:
 		if(!m_emit.value())
 			return;
 
-		const k3d::vector3 from = k3d::world_position(*this);
+		const k3d::point3 from = k3d::world_position(*this);
 		const k3d::color color = m_color.value();
 
 		Stream << "<light type=\"sunlight\" name=\"" << name() << "\" power=\"" << m_power.value() << "\" cast_shadows=\"" << (m_cast_shadows.value() ? "yes" : "no") << "\">" << std::endl;
@@ -261,6 +307,30 @@ public:
 		Stream << "</light>" << std::endl;
 	}
 
+	void on_gl_draw(const k3d::gl::render_state& State)
+	{
+		k3d::gl::store_attributes attributes;
+
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_AMBIENT, k3d::color(0, 0, 0));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_DIFFUSE, get_selection_weight() ? k3d::color(1, 1, 0) : k3d::color(0.4, 0.4, 0.4));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_SPECULAR, k3d::color(0, 0, 0));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_EMISSION, k3d::color(0, 0, 0));
+
+		glEnable(GL_LIGHTING);
+		draw_geometry();
+	}
+
+	void on_gl_select(const k3d::gl::render_state& State, const k3d::gl::select_state& SelectState)
+	{
+		k3d::gl::store_attributes attributes;
+
+		glDisable(GL_LIGHTING);
+
+		k3d::gl::push_selection_token(this);
+		draw_geometry();
+		k3d::gl::pop_selection_token();
+	}
+
 	k3d::iplugin_factory& factory()
 	{
 		return get_factory();
@@ -272,16 +342,40 @@ public:
 			k3d::document_plugin<sun_light>,
 			k3d::interface_list<k3d::itransform_source,
 			k3d::interface_list<k3d::itransform_sink > > > factory(
-			k3d::uuid(0x9a48777d, 0xa68345b5, 0xacb9fc07, 0x8af3e7f6),
-			"YafraySunLight",
-			_("Yafray Sun Light"),
-			"Yafray Lights",
-			k3d::iplugin_factory::STABLE);
+				k3d::uuid(0x9a48777d, 0xa68345b5, 0xacb9fc07, 0x8af3e7f6),
+				"YafraySunLight",
+				_("Yafray Sun Light"),
+				"Yafray Lights",
+				k3d::iplugin_factory::STABLE);
 
 		return factory;
 	}
 
 private:
+	void draw_geometry()
+	{
+		gluSphere(m_quadric, 0.2, 8, 8);
+
+		const k3d::point3 from = m_input_matrix.value() * k3d::point3(0, 0, 0);
+		const k3d::point3 to = k3d::point3(0, 0, 0);
+		const k3d::vector3 spherical = k3d::spherical(k3d::to_vector(to - from));
+		glPushMatrix();
+		k3d::gl::push_matrix(k3d::inverse(k3d::extract_rotation(m_input_matrix.value())) * rotation3D(k3d::quaternion(k3d::euler_angles(0, -spherical[2], spherical[1], k3d::euler_angles::ZXYstatic))));
+
+		gluQuadricDrawStyle(m_quadric, GLU_FILL);
+		gluQuadricNormals(m_quadric, GLU_SMOOTH);
+		gluCylinder(m_quadric, 0.1, 0.001, 1, 8, 1);
+		glPushMatrix();
+		k3d::gl::push_matrix(k3d::translation3D(k3d::point3(0, 0, 0.5)));
+		gluCylinder(m_quadric, 0.3, 0.001, 0.5, 8, 1);
+		glPopMatrix();
+
+		glPopMatrix();
+	}
+
+	/// Stores a GLU quadric object for drawing the manipulators
+	GLUquadricObj* const m_quadric;
+
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_power;
 };
 
@@ -296,6 +390,7 @@ class soft_light :
 public:
 	soft_light(k3d::idocument& Document) :
 		base(Document),
+		m_quadric(gluNewQuadric()),
 		m_power(init_owner(*this) + init_name("power") + init_label(_("power")) + init_description(_("Power")) + init_value(400.0)),
 		m_radius(init_owner(*this) + init_name("radius") + init_label(_("radius")) + init_description(_("Softness")) + init_value(5.0)),
 		m_resolution(init_owner(*this) + init_name("resolution") + init_label(_("resolution")) + init_description(_("Shadowmap resolution")) + init_value(256) + init_constraint(k3d::data::constraint::minimum(0L, k3d::data::constraint::maximum(4096L)))),
@@ -304,6 +399,12 @@ public:
 		m_glow_type(init_owner(*this) + init_name("glow_type") + init_label(_("glow_type")) + init_description(_("Glow type")) + init_value(0)),
 		m_glow_offset(init_owner(*this) + init_name("glow_offset") + init_label(_("glow_offset")) + init_description(_("Glow offset")) + init_value(0))
 	{
+		m_selection_weight.changed_signal().connect(make_async_redraw_slot());
+	}
+
+	~soft_light()
+	{
+		gluDeleteQuadric(m_quadric);
 	}
 
 	void setup_light(std::ostream& Stream)
@@ -311,7 +412,7 @@ public:
 		if(!m_emit.value())
 			return;
 
-		const k3d::vector3 from = k3d::world_position(*this);
+		const k3d::point3 from = k3d::world_position(*this);
 		const k3d::color color = m_color.value();
 
 		Stream << "<light type=\"softlight\" name=\"" << name() << "\" power=\"" << m_power.value() << "\" res=\"" << m_resolution.value()  << "\" radius=\"" << m_radius.value() << "\" bias=\"" << m_bias.value() << "\" glow_intensity=\"" << m_glow_intensity.value() << "\" glow_type=\"" << m_glow_type.value() << "\" glow_offset=\"" << m_glow_offset.value() << "\">" << std::endl;
@@ -320,6 +421,30 @@ public:
 		Stream << "</light>" << std::endl;
 	}
 
+	void on_gl_draw(const k3d::gl::render_state& State)
+	{
+		k3d::gl::store_attributes attributes;
+
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_AMBIENT, k3d::color(0, 0, 0));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_DIFFUSE, get_selection_weight() ? k3d::color(0, 1, 1) : k3d::color(0.4, 0.4, 0.4));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_SPECULAR, k3d::color(0, 0, 0));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_EMISSION, k3d::color(0, 0, 0));
+
+		glEnable(GL_LIGHTING);
+		draw_geometry();
+	}
+
+	void on_gl_select(const k3d::gl::render_state& State, const k3d::gl::select_state& SelectState)
+	{
+		k3d::gl::store_attributes attributes;
+
+		glDisable(GL_LIGHTING);
+
+		k3d::gl::push_selection_token(this);
+		draw_geometry();
+		k3d::gl::pop_selection_token();
+	}
+
 	k3d::iplugin_factory& factory()
 	{
 		return get_factory();
@@ -331,16 +456,24 @@ public:
 			k3d::document_plugin<soft_light>,
 			k3d::interface_list<k3d::itransform_source,
 			k3d::interface_list<k3d::itransform_sink > > > factory(
-			k3d::uuid(0x2fcaffb5, 0xed294a0d, 0x82133a8f, 0x48df4988),
-			"YafraySoftLight",
-			_("Yafray Soft Light"),
-			"Yafray Lights",
-			k3d::iplugin_factory::STABLE);
+				k3d::uuid(0x2fcaffb5, 0xed294a0d, 0x82133a8f, 0x48df4988),
+				"YafraySoftLight",
+				_("Yafray Soft Light"),
+				"Yafray Lights",
+				k3d::iplugin_factory::STABLE);
 
 		return factory;
 	}
 
 private:
+	void draw_geometry()
+	{
+		gluSphere(m_quadric, 0.5, 8, 8);
+	}
+
+	/// Stores a GLU quadric object for drawing the manipulators
+	GLUquadricObj* const m_quadric;
+
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_power;
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_radius;
 	k3d_data(long, immutable_name, change_signal, with_undo, local_storage, with_constraint, writable_property, with_serialization) m_resolution;
@@ -361,6 +494,7 @@ class spot_light :
 public:
 	spot_light(k3d::idocument& Document) :
 		base(Document),
+		m_quadric(gluNewQuadric()),
 		m_power(init_owner(*this) + init_name("power") + init_label(_("power")) + init_description(_("Power")) + init_value(400.0)),
 		m_size(init_owner(*this) + init_name("size") + init_label(_("size")) + init_description(_("Angle")) + init_value(k3d::radians(30.0))),
 		m_nblend(init_owner(*this) + init_name("nblend") + init_label(_("nblend")) + init_description(_("Edge softness")) + init_value(5.0)),
@@ -368,13 +502,18 @@ public:
 	{
 	}
 
+	~spot_light()
+	{
+		gluDeleteQuadric(m_quadric);
+	}
+
 	void setup_light(std::ostream& Stream)
 	{
 		if(!m_emit.value())
 			return;
 
-		const k3d::vector3 from = k3d::world_position(*this);
-		const k3d::vector3 to = from + k3d::node_to_world_matrix(*this) * k3d::normal3(0, 0, 1);
+		const k3d::point3 from = k3d::world_position(*this);
+		const k3d::point3 to = from + k3d::node_to_world_matrix(*this) * k3d::vector3(0, 0, 1);
 		const k3d::color color = m_color.value();
 		const double power = m_power.value();
 		const double size = k3d::degrees(m_size.value());
@@ -392,27 +531,24 @@ public:
 	{
 		k3d::gl::store_attributes attributes;
 
-		const bool emitting = m_emit.value();
-
-		glDisable(GL_LIGHTING);
-		glDisable(GL_TEXTURE_1D);
-		glDisable(GL_TEXTURE_2D);
-
-		if(emitting)
-			glColor3d(1, 1, 1);
-		else
-			glColor3d(0, 0, 0);
-
-		glLineWidth(1.0f);
-		glDisable(GL_LINE_STIPPLE);
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_AMBIENT, k3d::color(0, 0, 0));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_DIFFUSE, get_selection_weight() ? k3d::color(1, 1, 1) : k3d::color(0.4, 0.4, 0.4));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_SPECULAR, k3d::color(0, 0, 0));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_EMISSION, k3d::color(0, 0, 0));
 
+		glEnable(GL_LIGHTING);
 		draw_geometry();
 	}
 
 	void on_gl_select(const k3d::gl::render_state& State, const k3d::gl::select_state& SelectState)
 	{
 		k3d::gl::store_attributes attributes;
+
+		glDisable(GL_LIGHTING);
+
+		k3d::gl::push_selection_token(this);
 		draw_geometry();
+		k3d::gl::pop_selection_token();
 	}
 
 	k3d::iplugin_factory& factory()
@@ -426,11 +562,11 @@ public:
 			k3d::document_plugin<spot_light>,
 			k3d::interface_list<k3d::itransform_source,
 			k3d::interface_list<k3d::itransform_sink > > > factory(
-			k3d::uuid(0x5e363371, 0xf8464895, 0x99f0ddf0, 0x4e26ee4a),
-			"YafraySpotLight",
-			_("Yafray Spot Light"),
-			"Yafray Lights",
-			k3d::iplugin_factory::STABLE);
+				k3d::uuid(0x5e363371, 0xf8464895, 0x99f0ddf0, 0x4e26ee4a),
+				"YafraySpotLight",
+				_("Yafray Spot Light"),
+				"Yafray Lights",
+				k3d::iplugin_factory::STABLE);
 
 		return factory;
 	}
@@ -438,22 +574,14 @@ public:
 private:
 	void draw_geometry()
 	{
-		glBegin(GL_LINES);
-
-		k3d::vector3 coords(0, 0, 0);
-
-		glVertex3d(coords[0], coords[1], coords[2]);
-		glVertex3d(coords[0] - 0.2, coords[1] + 0.2, coords[2] + 1.0);
-		glVertex3d(coords[0], coords[1], coords[2]);
-		glVertex3d(coords[0] + 0.2, coords[1] + 0.2, coords[2] + 1.0);
-		glVertex3d(coords[0], coords[1], coords[2]);
-		glVertex3d(coords[0] + 0.2, coords[1] - 0.2, coords[2] + 1.0);
-		glVertex3d(coords[0], coords[1], coords[2]);
-		glVertex3d(coords[0] - 0.2, coords[1] - 0.2, coords[2] + 1.0);
-
-		glEnd();
+		gluQuadricDrawStyle(m_quadric, GLU_FILL);
+		gluQuadricNormals(m_quadric, GLU_SMOOTH);
+		gluCylinder(m_quadric, 0.001, 0.2, 1, 8, 1);
 	}
 
+	/// Stores a GLU quadric object for drawing the manipulators
+	GLUquadricObj* const m_quadric;
+
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_power;
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_size;
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_nblend;
@@ -464,13 +592,14 @@ private:
 // hemi_light
 
 class hemi_light :
-	public colored_light<light<k3d::persistent<k3d::node> > >
+	public drawable_light<colored_light<light<k3d::transformable<k3d::persistent<k3d::node> > > > >
 {
-	typedef colored_light<light<k3d::persistent<k3d::node> > > base;
+	typedef drawable_light<colored_light<light<k3d::transformable<k3d::persistent<k3d::node> > > > > base;
 
 public:
 	hemi_light(k3d::idocument& Document) :
 		base(Document),
+		m_quadric(gluNewQuadric()),
 		m_power(init_owner(*this) + init_name("power") + init_label(_("power")) + init_description(_("Power")) + init_value(2.0)),
 		m_samples(init_owner(*this) + init_name("samples") + init_label(_("samples")) + init_description(_("Samples")) + init_value(256) + init_constraint(k3d::data::constraint::minimum(0L, k3d::data::constraint::maximum(1024L)))),
 		m_use_QMC(init_owner(*this) + init_name("use_QMC") + init_label(_("use_QMC")) + init_description(_("Use QMC")) + init_value(false)),
@@ -480,6 +609,11 @@ public:
 		m_color.internal_value() = k3d::color(0.9, 0.9, 1.0);
 	}
 
+	~hemi_light()
+	{
+		gluDeleteQuadric(m_quadric);
+	}
+
 	void setup_light(std::ostream& Stream)
 	{
 		if(!m_emit.value())
@@ -492,6 +626,31 @@ public:
 		Stream << "</light>" << std::endl;
 	}
 
+	void on_gl_draw(const k3d::gl::render_state& State)
+	{
+		k3d::gl::store_attributes attributes;
+
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_AMBIENT, k3d::color(0, 0, 0));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_DIFFUSE, get_selection_weight() ? k3d::color(1, 1, 1) : k3d::color(0.4, 0.4, 0.4));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_SPECULAR, k3d::color(0, 0, 0));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_EMISSION, k3d::color(0, 0, 0));
+
+		glEnable(GL_LIGHTING);
+		draw_geometry();
+	}
+
+	void on_gl_select(const k3d::gl::render_state& State, const k3d::gl::select_state& SelectState)
+	{
+		k3d::gl::store_attributes attributes;
+
+		glDisable(GL_LIGHTING);
+		glDisable(GL_CULL_FACE);
+
+		k3d::gl::push_selection_token(this);
+		draw_geometry();
+		k3d::gl::pop_selection_token();
+	}
+
 	k3d::iplugin_factory& factory()
 	{
 		return get_factory();
@@ -510,6 +669,33 @@ public:
 	}
 
 private:
+	void draw_geometry()
+	{
+		const GLdouble radius = 0.5;
+		const GLint sides = 16;
+		const double stacks = 4;
+
+		const double angle = k3d::pi_over_2() / stacks;
+
+		for(double n = 0; n < stacks; ++n)
+		{
+			const double angle1 = angle * n;
+			const double angle2 = angle * (n + 1);
+			const double z1 = sin(angle1) * radius;
+			const double z2 = sin(angle2) * radius;
+			const double r1 = cos(angle1) * radius;
+			const double r2 = cos(angle2) * radius;
+
+			glPushMatrix();
+			glTranslated(0, 0, z1);
+			gluCylinder(m_quadric, r1, r2 ? r2 : 0.001, z2 - z1, sides, 1);
+			glPopMatrix();
+		}
+	}
+
+	/// Stores a GLU quadric object for drawing the manipulators
+	GLUquadricObj* const m_quadric;
+
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_power;
 	k3d_data(long, immutable_name, change_signal, with_undo, local_storage, with_constraint, writable_property, with_serialization) m_samples;
 	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_use_QMC;
@@ -527,11 +713,18 @@ class area_light :
 public:
 	area_light(k3d::idocument& Document) :
 		base(Document),
+		m_quadric(gluNewQuadric()),
 		m_power(init_owner(*this) + init_name("power") + init_label(_("power")) + init_description(_("Power")) + init_value(20.0)),
 		m_samples(init_owner(*this) + init_name("samples") + init_label(_("samples")) + init_description(_("Samples")) + init_value(256) + init_constraint(k3d::data::constraint::minimum(0L, k3d::data::constraint::maximum(1024L)))),
 		m_psamples(init_owner(*this) + init_name("psamples") + init_label(_("psamples")) + init_description(_("PSamples")) + init_value(256) + init_constraint(k3d::data::constraint::minimum(0L, k3d::data::constraint::maximum(1024L)))),
 		m_dummy(init_owner(*this) + init_name("dummy") + init_label(_("dummy")) + init_description(_("Dummy (global photon)")) + init_value(false))
 	{
+		m_selection_weight.changed_signal().connect(make_async_redraw_slot());
+	}
+
+	~area_light()
+	{
+		gluDeleteQuadric(m_quadric);
 	}
 
 	void setup_light(std::ostream& Stream)
@@ -540,12 +733,12 @@ public:
 			return;
 
 		const k3d::color color = m_color.value();
-		
+
 		const k3d::matrix4 transformation = k3d::node_to_world_matrix(*this);
-		const k3d::vector3 a = transformation * k3d::vector3(-0.5, 0.5, 0);
-		const k3d::vector3 b = transformation * k3d::vector3(0.5, 0.5, 0);
-		const k3d::vector3 c = transformation * k3d::vector3(0.5, -0.5, 0);
-		const k3d::vector3 d = transformation * k3d::vector3(-0.5, -0.5, 0);
+		const k3d::point3 a = transformation * k3d::point3(-0.5, 0.5, 0);
+		const k3d::point3 b = transformation * k3d::point3(0.5, 0.5, 0);
+		const k3d::point3 c = transformation * k3d::point3(0.5, -0.5, 0);
+		const k3d::point3 d = transformation * k3d::point3(-0.5, -0.5, 0);
 
 		Stream << "<light type=\"arealight\" name=\"" << name() << "\" power=\"" << m_power.value() << "\" samples=\"" << m_samples.value() << "\" psamples=\"" << m_psamples.value() << "\" dummy=\"" << (m_dummy.value() ? "on" : "off") << "\">" << std::endl;
 		Stream << "	<a  x =\"" << -a[0] << "\" y =\"" << a[1] << "\" z =\"" << a[2] << "\"/>" << std::endl;
@@ -574,13 +767,20 @@ public:
 		glLineWidth(1.0f);
 		glDisable(GL_LINE_STIPPLE);
 
-		draw_geometry();
+		glEnable(GL_LIGHTING);
+		draw_geometry(get_selection_weight() ? k3d::color(1, 0, 0) : k3d::color(0.4, 0.4, 0.4));
 	}
 
 	void on_gl_select(const k3d::gl::render_state& State, const k3d::gl::select_state& SelectState)
 	{
 		k3d::gl::store_attributes attributes;
-		draw_geometry();
+
+		glDisable(GL_LIGHTING);
+		glDisable(GL_CULL_FACE);
+
+		k3d::gl::push_selection_token(this);
+		draw_geometry(k3d::color(1, 1, 1));
+		k3d::gl::pop_selection_token();
 	}
 
 	k3d::iplugin_factory& factory()
@@ -601,11 +801,11 @@ public:
 	}
 
 private:
-	void draw_geometry()
+	void draw_geometry(const k3d::color ArrowColor)
 	{
 		glBegin(GL_LINES);
 
-		k3d::vector3 coords(0, 0, 0);
+		k3d::point3 coords(0, 0, 0);
 
 		glVertex3d(coords[0] - 0.5, coords[1] + 0.5, coords[2]);
 		glVertex3d(coords[0] + 0.5, coords[1] + 0.5, coords[2]);
@@ -620,8 +820,23 @@ private:
 		glVertex3d(coords[0] - 0.5, coords[1] + 0.5, coords[2]);
 
 		glEnd();
+
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_AMBIENT, k3d::color(0, 0, 0));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_DIFFUSE, ArrowColor);
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_SPECULAR, k3d::color(0, 0, 0));
+		k3d::gl::material(GL_FRONT_AND_BACK, GL_EMISSION, k3d::color(0, 0, 0));
+		gluQuadricDrawStyle(m_quadric, GLU_FILL);
+		gluQuadricNormals(m_quadric, GLU_SMOOTH);
+		gluCylinder(m_quadric, 0.1, 0.001, 1, 8, 1);
+		glPushMatrix();
+		k3d::gl::push_matrix(k3d::translation3D(k3d::point3(0, 0, 0.5)));
+		gluCylinder(m_quadric, 0.3, 0.001, 0.5, 8, 1);
+		glPopMatrix();
 	}
 
+	/// Stores a GLU quadric object for drawing the manipulators
+	GLUquadricObj* const m_quadric;
+
 	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_power;
 	k3d_data(long, immutable_name, change_signal, with_undo, local_storage, with_constraint, writable_property, with_serialization) m_samples;
 	k3d_data(long, immutable_name, change_signal, with_undo, local_storage, with_constraint, writable_property, with_serialization) m_psamples;
@@ -702,8 +917,8 @@ public:
 
 		const k3d::color color = m_color.value();
 
-		const k3d::vector3 from = k3d::world_position(*this);
-		const k3d::vector3 to = from + k3d::node_to_world_matrix(*this) * k3d::normal3(0, 0, 1);
+		const k3d::point3 from = k3d::world_position(*this);
+		const k3d::point3 to = from + k3d::node_to_world_matrix(*this) * k3d::vector3(0, 0, 1);
 
 		Stream << "<light type=\"photonlight\" name=\"" << name() << "\" power=\"" << m_power.value() << "\" photons=\"" << m_photons.value() << "\" depth=\"" << m_depth.value() << "\" mindepth=\"" << m_mindepth.value() << "\" bias=\"" << m_bias.value() << "\" search=\"" << m_search.value() << "\" angle=\"" << m_angle.value() << "\" mode=\"" << (m_mode.value() == DIFFUSE ? "diffuse" : "caustic") << "\" fixedradius=\"" << m_fixedradius.value() << "\" cluster=\"" << m_cluster.value() << "\" use_QMC=\"" << (m_use_QMC.value() ? "on" : "off") << "\">" << std::endl;
 		Stream << "	<from  x =\"" << -from[0] << "\" y =\"" << from[1] << "\" z =\"" << from[2] << "\"/>" << std::endl;
@@ -761,7 +976,7 @@ private:
 	{
 		glBegin(GL_LINES);
 
-		k3d::vector3 coords(0, 0, 0);
+		k3d::point3 coords(0, 0, 0);
 
 		glVertex3d(coords[0], coords[1], coords[2]);
 		glVertex3d(coords[0] - 0.2, coords[1] + 0.2, coords[2] + 1.0);
@@ -784,14 +999,14 @@ private:
 	friend std::ostream& operator << (std::ostream& Stream, const mode_t& Value)
 	{
 		switch(Value)
-			{
-				case DIFFUSE:
-					Stream << "diffuse";
-					break;
-				case CAUSTIC:
-					Stream << "caustic";
-					break;
-			}
+		{
+			case DIFFUSE:
+				Stream << "diffuse";
+				break;
+			case CAUSTIC:
+				Stream << "caustic";
+				break;
+		}
 		return Stream;
 	}
 
@@ -874,4 +1089,3 @@ k3d::iplugin_factory& photon_light_factory()
 
 } // namespace libk3dyafray
 
-
