@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -22,9 +22,6 @@
 		\author Romain Behar <romainbehar@yahoo.com>
 */
 
-#include "ilight.h"
-#include "imaterial.h"
-
 #include <subdivision_surface/k3d_sds_binding.h>
 
 #include <k3dsdk/algebra.h>
@@ -33,30 +30,31 @@
 #include <k3dsdk/fstream.h>
 #include <k3dsdk/gl.h>
 #include <k3dsdk/i18n.h>
-#include <k3dsdk/ianimation_render_engine.h>
 #include <k3dsdk/icamera.h>
+#include <k3dsdk/icamera_animation_render_engine.h>
+#include <k3dsdk/icamera_preview_render_engine.h>
+#include <k3dsdk/icamera_still_render_engine.h>
 #include <k3dsdk/imaterial.h>
 #include <k3dsdk/imaterial_client.h>
 #include <k3dsdk/imesh_sink.h>
 #include <k3dsdk/imesh_source.h>
-#include <k3dsdk/ipreview_render_engine.h>
 #include <k3dsdk/iprojection.h>
 #include <k3dsdk/irender_farm.h>
-#include <k3dsdk/istill_render_engine.h>
 #include <k3dsdk/itransform_source.h>
+#include <k3dsdk/iyafray.h>
 #include <k3dsdk/measurement.h>
 #include <k3dsdk/mesh.h>
 #include <k3dsdk/module.h>
 #include <k3dsdk/node.h>
 #include <k3dsdk/persistent.h>
 #include <k3dsdk/property.h>
+#include <k3dsdk/render_farm.h>
 #include <k3dsdk/resolutions.h>
 #include <k3dsdk/time_source.h>
 #include <k3dsdk/transform.h>
 
 #include <boost/filesystem/path.hpp>
 
-#include <fstream>
 #include <iomanip>
 #include <iterator>
 #include <map>
@@ -69,9 +67,9 @@ namespace libk3dyafray
 
 class render_engine :
 	public k3d::persistent<k3d::node>,
-	public k3d::ipreview_render_engine,
-	public k3d::istill_render_engine,
-	public k3d::ianimation_render_engine
+	public k3d::icamera_preview_render_engine,
+	public k3d::icamera_still_render_engine,
+	public k3d::icamera_animation_render_engine
 {
 	typedef k3d::persistent<k3d::node> base;
 
@@ -92,13 +90,12 @@ public:
 		m_gamma(init_owner(*this) + init_name("gamma") + init_label(_("gamma")) + init_description(_("gamma")) + init_value(1)),
 		m_fog_density(init_owner(*this) + init_name("fog_density") + init_label(_("fog_density")) + init_description(_("fog_density")) + init_value(0.0)),
 		m_fog_color(init_owner(*this) + init_name("fog_color") + init_label(_("fog_color")) + init_description(_("Fog color")) + init_value(k3d::color(1, 1, 1))),
-		m_preview_sds(init_owner(*this) + init_name("preview_sds") + init_label(_("Preview SDS")) + init_description(_("Show SDS Surfaces")) + init_value(true)),
-		m_sds_level(init_owner(*this) + init_name("sds_level") + init_label(_("SDS Level")) + init_description(_("Subdivision Level")) + init_constraint(constraint::minimum(1)) + init_value(2) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar)))
+		m_preview_sds(init_owner(*this) + init_name("preview_sds") + init_label(_("Preview SDS")) + init_description(_("Show SDS Surfaces")) + init_value(true))
 	{
 		m_resolution.changed_signal().connect(sigc::mem_fun(*this, &render_engine::on_resolution_changed));
 	}
 
-	void on_resolution_changed()
+	void on_resolution_changed(k3d::iunknown*)
 	{
 		const std::string new_resolution = m_resolution.value();
 
@@ -116,10 +113,10 @@ public:
 		assert_not_reached();
 	}
 
-	bool render_preview(k3d::icamera& Camera)
+	bool render_camera_preview(k3d::icamera& Camera)
 	{
 		// Start a new render job ...
-		k3d::irender_job& job = k3d::application().render_farm().create_job("k3d-preview");
+		k3d::irender_job& job = k3d::render_farm().create_job("k3d-preview");
 
 		// Add a single render frame to the job ...
 		k3d::irender_frame& frame = job.create_frame("frame");
@@ -135,18 +132,18 @@ public:
 		return_val_if_fail(render(Camera, frame, outputimagepath, true), false);
 
 		// Start the job running ...
-		k3d::application().render_farm().start_job(job);
+		k3d::render_farm().start_job(job);
 
 		return true;
 	}
 
-	bool render_frame(k3d::icamera& Camera, const boost::filesystem::path& OutputImage, const bool ViewImage)
+	bool render_camera_frame(k3d::icamera& Camera, const boost::filesystem::path& OutputImage, const bool ViewImage)
 	{
 		// Sanity checks ...
 		return_val_if_fail(!OutputImage.empty(), false);
 
 		// Start a new render job ...
-		k3d::irender_job& job = k3d::application().render_farm().create_job("k3d-render-frame");
+		k3d::irender_job& job = k3d::render_farm().create_job("k3d-render-frame");
 
 		// Add a single render frame to the job ...
 		k3d::irender_frame& frame = job.create_frame("frame");
@@ -166,12 +163,12 @@ public:
 		return_val_if_fail(render(Camera, frame, outputimagepath, false), false);
 
 		// Start the job running ...
-		k3d::application().render_farm().start_job(job);
+		k3d::render_farm().start_job(job);
 
 		return true;
 	}
 
-	bool render_animation(k3d::icamera& Camera, const boost::filesystem::path& OutputImages, const bool ViewCompletedImages)
+	bool render_camera_animation(k3d::icamera& Camera, const boost::filesystem::path& OutputImages, const bool ViewCompletedImages)
 	{
 		// Sanity checks ...
 		return_val_if_fail(!OutputImages.empty(), false);
@@ -195,7 +192,7 @@ public:
 		return_val_if_fail(frames.max_frame() >= end_frame, false);
 
 		// Start a new render job ...
-		k3d::irender_job& job = k3d::application().render_farm().create_job("k3d-render-animation");
+		k3d::irender_job& job = k3d::render_farm().create_job("k3d-render-animation");
 
 		// For each frame to be rendered ...
 		for(long view_frame = start_frame; view_frame < end_frame; ++view_frame)
@@ -229,7 +226,7 @@ public:
 		}
 
 		// Start the job running ...
-		k3d::application().render_farm().start_job(job);
+		k3d::render_farm().start_job(job);
 
 		return true;
 	}
@@ -243,9 +240,9 @@ public:
 	{
 		static k3d::plugin_factory<
 			k3d::document_plugin<render_engine>,
-			k3d::interface_list<k3d::ianimation_render_engine,
-			k3d::interface_list<k3d::istill_render_engine,
-			k3d::interface_list<k3d::ipreview_render_engine> > > > factory(
+			k3d::interface_list<k3d::icamera_animation_render_engine,
+			k3d::interface_list<k3d::icamera_still_render_engine,
+			k3d::interface_list<k3d::icamera_preview_render_engine> > > > factory(
 				k3d::uuid(0xef38bf93, 0x66654f9f, 0x992ca91b, 0x62bae139),
 				"YafrayEngine",
 				_("Yafray Render Engine"),
@@ -256,12 +253,26 @@ public:
 	}
 
 private:
-	const std::string shader_name(k3d::iunknown& Object)
+	const std::string shader_name(k3d::inode& Node)
 	{
-		if(k3d::imaterial_client* const material_client = dynamic_cast<k3d::imaterial_client*>(&Object))
+		if(k3d::imaterial_client* const material_client = dynamic_cast<k3d::imaterial_client*>(&Node))
 		{
-			if(yafray::imaterial* const material = dynamic_cast<yafray::imaterial*>(material_client->material()))
-				return dynamic_cast<k3d::inode*>(material)->name();
+			if(k3d::imaterial* const material = material_client->material())
+			{
+				if(k3d::yafray::imaterial* const yafray_material = material->yafray_material())
+					return dynamic_cast<k3d::inode*>(yafray_material)->name();
+			}
+		}
+
+		return "default_shader";
+	}
+
+	const std::string shader_name(k3d::imaterial* Material)
+	{
+		if(Material)
+		{
+			if(k3d::yafray::imaterial* const yafray_material = Material->yafray_material())
+				return dynamic_cast<k3d::inode*>(yafray_material)->name();
 		}
 
 		return "default_shader";
@@ -291,12 +302,12 @@ private:
 		bool found_light = false;
 
 		// Get the document contents ...
-		const k3d::nodes_t objects(document().nodes().collection());
+		const k3d::nodes_t nodes(document().nodes().collection());
 
 		// Setup shaders ...
-		for(k3d::nodes_t::const_iterator object = objects.begin(); object != objects.end(); ++object)
+		for(k3d::nodes_t::const_iterator node = nodes.begin(); node != nodes.end(); ++node)
 		{
-			if(yafray::imaterial* const material = dynamic_cast<yafray::imaterial*>(*object))
+			if(k3d::yafray::imaterial* const material = dynamic_cast<k3d::yafray::imaterial*>(*node))
 				material->setup_material(stream);
 		}
 
@@ -315,9 +326,9 @@ private:
 		stream << "</shader>" << std::endl;
 
 		// Setup lights ...
-		for(k3d::nodes_t::const_iterator object = objects.begin(); object != objects.end(); ++object)
+		for(k3d::nodes_t::const_iterator node = nodes.begin(); node != nodes.end(); ++node)
 		{
-			if(yafray::ilight* const light = dynamic_cast<yafray::ilight*>(*object))
+			if(k3d::yafray::ilight* const light = dynamic_cast<k3d::yafray::ilight*>(*node))
 			{
 				found_light = true;
 				light->setup_light(stream);
@@ -334,16 +345,16 @@ private:
 		}
 
 		// Setup geometry
-		for(k3d::nodes_t::const_iterator object = objects.begin(); object != objects.end(); ++object)
+		for(k3d::nodes_t::const_iterator node = nodes.begin(); node != nodes.end(); ++node)
 		{
-			// Render sphere objects ...
-			if((*object)->factory().class_id() == k3d::classes::Sphere())
+			// Render sphere nodes ...
+			if((*node)->factory().class_id() == k3d::classes::Sphere())
 			{
-				const k3d::vector3 sphere_center = k3d::node_to_world_matrix(**object) * k3d::vector3(0, 0, 0);
-				const boost::any sphere_radius(k3d::get_value(**object, "radius"));
+				const k3d::point3 sphere_center = k3d::node_to_world_matrix(**node) * k3d::point3(0, 0, 0);
+				const boost::any sphere_radius(k3d::get_value(**node, "radius"));
 				if(typeid(double) == sphere_radius.type())
 				{
-					stream << "<object name=\"" << (*object)->name() << "\" shader_name=\"" << shader_name(**object) << "\">" << std::endl;
+					stream << "<object name=\"" << (*node)->name() << "\" shader_name=\"" << shader_name(**node) << "\">" << std::endl;
 					stream << "	<attributes>" << std::endl;
 					stream << "	</attributes>" << std::endl;
 					stream << "	<sphere radius=\"" << boost::any_cast<double>(sphere_radius) << "\">" << std::endl;
@@ -357,19 +368,38 @@ private:
 				continue;
 			}
 
-			// Render mesh objects ...
-			if((*object)->factory().class_id() == k3d::classes::MeshInstance())
+			// Render mesh nodes ...
+			if((*node)->factory().class_id() == k3d::classes::MeshInstance())
 			{
-				k3d::imesh_source* const mesh_source = dynamic_cast<k3d::imesh_source*>(*object);
+				if(!dynamic_cast<k3d::gl::idrawable*>(*node))
+					continue;
+
+				k3d::imesh_source* const mesh_source = dynamic_cast<k3d::imesh_source*>(*node);
 				if(!mesh_source)
 					continue;
 
-				k3d::mesh* const mesh = boost::any_cast<k3d::mesh*>(k3d::get_value(document().dag(), mesh_source->mesh_source_output()));
+				// Check for transformed output mesh
+				k3d::mesh* mesh = 0;
+				if(k3d::iproperty* property = get_property(**node, "transformed_output_mesh"))
+				{
+					mesh = boost::any_cast<k3d::mesh*>(property->property_value());
+				}
+				else
+				{
+					// Fall back to mesh source
+					k3d::iproperty& property = mesh_source->mesh_source_output();
+					mesh = boost::any_cast<k3d::mesh*>(property.property_value());
+				}
+
 				if(!mesh)
 					continue;
 
 				k3d::mesh out_mesh;
-				sds_filter(*mesh, out_mesh);
+				if (get_property(**node, "polyhedron_render_type"))
+				{
+					const std::string& render_type =  boost::any_cast<std::string>(k3d::get_internal_value(**node, "polyhedron_render_type"));
+					sds_filter(*mesh, render_type, out_mesh, boost::any_cast<int>(k3d::get_internal_value(**node, "sds_render_level")));
+				}
 
 				k3d::polyhedron::faces_t new_faces;
 				k3d::mesh::points_t new_points;
@@ -377,6 +407,8 @@ private:
 				for(k3d::mesh::polyhedra_t::const_iterator polyhedron = out_mesh.polyhedra.begin(); polyhedron != out_mesh.polyhedra.end(); ++polyhedron)
 					k3d::triangulate((*polyhedron)->faces, new_faces, new_points);
 
+				k3d::log() << debug << new_faces.size() << std::endl;
+
 				typedef std::multimap<k3d::imaterial*, k3d::face*> sorted_faces_t;
 				sorted_faces_t sorted_faces;
 				for(k3d::polyhedron::faces_t::iterator face = new_faces.begin(); face != new_faces.end(); ++face)
@@ -399,24 +431,27 @@ private:
 					double autosmooth_value = 90;
 					bool has_orco = false;
 
-					if(yafray::imaterial* const material = dynamic_cast<yafray::imaterial*>(begin->first))
+					if(begin->first)
 					{
-						material_name = dynamic_cast<k3d::inode*>(material)->name();
-						shadow = boost::any_cast<bool>(k3d::get_internal_value(*material, "shadow"));
-						emit_rad = boost::any_cast<bool>(k3d::get_internal_value(*material, "emit_rad"));
-						recv_rad = boost::any_cast<bool>(k3d::get_internal_value(*material, "recv_rad"));
-						caustics = boost::any_cast<bool>(k3d::get_internal_value(*material, "caustics"));
-						caus_IOR = boost::any_cast<double>(k3d::get_internal_value(*material, "caus_IOR"));
-						caus_rcolor = boost::any_cast<k3d::color>(k3d::get_internal_value(*material, "caus_rcolor"));
-						caus_tcolor = boost::any_cast<k3d::color>(k3d::get_internal_value(*material, "caus_tcolor"));
-						autosmooth = boost::any_cast<bool>(k3d::get_internal_value(*material, "mesh_autosmooth"));
-						autosmooth_value = boost::any_cast<double>(k3d::get_internal_value(*material, "mesh_autosmooth_value"));
-						has_orco = boost::any_cast<bool>(k3d::get_internal_value(*material, "has_orco"));
+						if(k3d::yafray::imaterial* const material = begin->first->yafray_material())
+						{
+							material_name = dynamic_cast<k3d::inode*>(material)->name();
+							shadow = boost::any_cast<bool>(k3d::get_internal_value(*material, "shadow"));
+							emit_rad = boost::any_cast<bool>(k3d::get_internal_value(*material, "emit_rad"));
+							recv_rad = boost::any_cast<bool>(k3d::get_internal_value(*material, "recv_rad"));
+							caustics = boost::any_cast<bool>(k3d::get_internal_value(*material, "caustics"));
+							caus_IOR = boost::any_cast<double>(k3d::get_internal_value(*material, "caus_IOR"));
+							caus_rcolor = boost::any_cast<k3d::color>(k3d::get_internal_value(*material, "caus_rcolor"));
+							caus_tcolor = boost::any_cast<k3d::color>(k3d::get_internal_value(*material, "caus_tcolor"));
+							autosmooth = boost::any_cast<bool>(k3d::get_internal_value(*material, "mesh_autosmooth"));
+							autosmooth_value = boost::any_cast<double>(k3d::get_internal_value(*material, "mesh_autosmooth_value"));
+							has_orco = boost::any_cast<bool>(k3d::get_internal_value(*material, "has_orco"));
+						}
 					}
 
 					stream
-						<< "<object name=\"" << (*object)->name() << k3d::string_cast(++unique_id)
-						<< "\" shader_name=\"" << material_name
+						<< "<object name=\"" << (*node)->name() << k3d::string_cast(++unique_id)
+						<< "\" shader_name=\"" << shader_name(begin->first)
 						<< "\" shadow=\"" << (shadow ? "on" : "off")
 						<< "\" emit_rad=\"" << (emit_rad ? "on" : "off")
 						<< "\" recv_rad=\"" << (recv_rad ? "on" : "off")
@@ -515,9 +550,9 @@ private:
 
 		// Setup the camera ...
 		const k3d::matrix4 camera_matrix = boost::any_cast<k3d::matrix4>(get_value(document().dag(), Camera.transformation().transform_source_output()));
-		const k3d::vector3 camera_position = k3d::extract_translation(camera_matrix);
-		const k3d::vector3 camera_to_vector = camera_matrix * k3d::vector3(0, 0, 1);
-		const k3d::vector3 camera_up_vector = camera_matrix * k3d::vector3(0, 1, 0);
+		const k3d::point3 camera_position = k3d::position(camera_matrix);
+		const k3d::point3 camera_to_vector = camera_matrix * k3d::point3(0, 0, 1);
+		const k3d::point3 camera_up_vector = camera_matrix * k3d::point3(0, 1, 0);
 
 		stream << "<camera name=\"camera\" resx=\"" << m_pixel_width.value() << "\" resy=\"" << m_pixel_height.value() << "\" focal=\"0.7\">" << std::endl;
 		stream << "	<from x=\"" << -camera_position[0] << "\" y=\"" << camera_position[1] << "\" z=\"" << camera_position[2] << "\"/>" << std::endl;
@@ -544,9 +579,9 @@ private:
 	}
 
 	/// Apply SDS if needed
-	void sds_filter(const k3d::mesh& Input, k3d::mesh& Output)
+	void sds_filter(const k3d::mesh& Input, const std::string& RenderType, k3d::mesh& Output, int Levels)
 	{
-		if (!m_preview_sds.value() || !(Input.polyhedra.size() > 0 && (*Input.polyhedra.begin())->type == k3d::polyhedron::CATMULL_CLARK_SUBDIVISION_MESH))
+		if (!m_preview_sds.value() || !(Input.polyhedra.size() > 0 && (RenderType == "catmull-clark")))
 		{
 			k3d::deep_copy(Input, Output);
 			return;
@@ -554,7 +589,7 @@ private:
 		k3d::sds::k3d_mesh_sds_cache sds_cache;
 
 		// Set levels -before- input
-		sds_cache.set_levels(m_sds_level.value());
+		sds_cache.set_levels(Levels);
 		sds_cache.set_input(&Input);
 		sds_cache.update();
 		sds_cache.output(&Output);
@@ -577,7 +612,6 @@ private:
 	k3d_data(k3d::color, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_fog_color;
 	// SDS options
 	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_preview_sds;
-	k3d_data(int, immutable_name, change_signal, with_undo, local_storage, with_constraint, measurement_property, with_serialization) m_sds_level;
 };
 
 k3d::iplugin_factory& render_engine_factory()
