@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -27,16 +27,17 @@
 #include <gtkmm/menu.h>
 #include <gtkmm/stock.h>
 
+#include "check_menu_item.h"
 #include "context_menu.h"
 #include "detail.h"
 #include "document_state.h"
-#include "file_selection.h"
-#include "modifiers.h"
+#include "file_chooser_dialog.h"
 #include "icons.h"
 #include "image_menu_item.h"
 #include "menu_item.h"
 #include "menus.h"
 #include "messages.h"
+#include "modifiers.h"
 #include "render.h"
 #include "viewport.h"
 #include "widget_manip.h"
@@ -91,54 +92,60 @@ public:
 			set_markup(_("<b><i>Viewport:</i></b>"))));
 		append(*Gtk::manage(m_viewport1));
 
+		items().push_back(*Gtk::manage(
+			new menu_item::control(Parent, "set_viewport_camera", _("Set Camera ...")) <<
+			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_set_viewport_camera))));
+
 		// Render-related functionality ...
-		m_viewport2 = new image_menu_item::control(Parent, "render_preview",
-				*Gtk::manage(new Gtk::Image(load_icon("render_preview", Gtk::ICON_SIZE_MENU))),
-				_("Render Preview")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_render_preview));
-		items().push_back(*Gtk::manage(m_viewport2));
+		Gtk::Menu* const render_submenu = new Gtk::Menu();
 
-		m_viewport3 = new image_menu_item::control(Parent, "render_frame",
-				*Gtk::manage(new Gtk::Image(load_icon("render_frame", Gtk::ICON_SIZE_MENU))),
-				_("Render Image")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_render_frame));
-		items().push_back(*Gtk::manage(m_viewport3));
+		render_submenu->items().push_back(*Gtk::manage(
+			new image_menu_item::control(Parent, "render_preview",
+			*Gtk::manage(new Gtk::Image(load_icon("render_preview", Gtk::ICON_SIZE_MENU))),
+			_("Render Preview")) <<
+				connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_render_preview))));
 
-		m_viewport4 = new image_menu_item::control(Parent, "render_animation",
-				*Gtk::manage(new Gtk::Image(load_icon("render_animation", Gtk::ICON_SIZE_MENU))),
-				_("Render Animation")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_render_animation));
-		items().push_back(*Gtk::manage(m_viewport4));
+		render_submenu->items().push_back(*Gtk::manage(
+			new image_menu_item::control(Parent, "render_frame",
+			*Gtk::manage(new Gtk::Image(load_icon("render_frame", Gtk::ICON_SIZE_MENU))),
+			_("Render Image")) <<
+				connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_render_frame))));
 
-		m_viewport5 = new Gtk::Menu_Helpers::SeparatorElem();
-		items().push_back(*m_viewport5);
+		render_submenu->items().push_back(*Gtk::manage(
+			new image_menu_item::control(Parent, "render_animation",
+			*Gtk::manage(new Gtk::Image(load_icon("render_animation", Gtk::ICON_SIZE_MENU))),
+			_("Render Animation")) <<
+				connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_render_animation))));
 
-		m_viewport6 = new menu_item::control(Parent, "set_viewport_camera", _("Set Camera ...")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_set_viewport_camera));
-		items().push_back(*Gtk::manage(m_viewport6));
+		items().push_back(Gtk::Menu_Helpers::MenuElem(_("Render"), *Gtk::manage(render_submenu)));
 
-		m_viewport7 = new menu_item::control(Parent, "set_viewport_gl_engine", _("Set OpenGL Engine ...")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_set_viewport_gl_engine));
-		items().push_back(*Gtk::manage(m_viewport7));
+		// Set render engines
+		Gtk::Menu* const engines_submenu = new Gtk::Menu();
 
-		m_viewport8 = new menu_item::control(Parent, "set_viewport_preview_engine", _("Set Preview Engine ...")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_set_viewport_preview_engine));
-		items().push_back(*Gtk::manage(m_viewport8));
+		engines_submenu->items().push_back(*Gtk::manage(
+			new menu_item::control(Parent, "set_viewport_preview_engine", _("Set preview engine ...")) <<
+			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_set_viewport_preview_engine))));
 
-		m_viewport9 = new menu_item::control(Parent, "set_viewport_still_engine", _("Set Still Engine ...")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_set_viewport_still_engine));
-		items().push_back(*Gtk::manage(m_viewport9));
+		engines_submenu->items().push_back(*Gtk::manage(
+			new menu_item::control(Parent, "set_viewport_still_engine", _("Set still engine ...")) <<
+			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_set_viewport_still_engine))));
 
-		m_viewport10 = new menu_item::control(Parent, "set_viewport_animation_engine", _("Set Animation Engine ...")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_set_viewport_animation_engine));
-		items().push_back(*Gtk::manage(m_viewport10));
+		engines_submenu->items().push_back(*Gtk::manage(
+			new menu_item::control(Parent, "set_viewport_animation_engine", _("Set animation engine ...")) <<
+			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_set_viewport_animation_engine))));
 
-		m_viewport11 = new Gtk::Menu_Helpers::SeparatorElem();
-		items().push_back(*m_viewport11);
+		engines_submenu->items().push_back(*Gtk::manage(
+			new menu_item::control(Parent, "set_viewport_gl_engine", _("Set OpenGL Engine ...")) <<
+			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_set_viewport_gl_engine))));
+
+		items().push_back(Gtk::Menu_Helpers::MenuElem(_("Set render engine"), *Gtk::manage(engines_submenu)));
+
+		// Viewport section end
+		items().push_back(Gtk::Menu_Helpers::SeparatorElem());
 
 		// Functionality common to all nodes ...
 		m_delete_nodes = new menu_item::control(Parent, "delete", _("Delete")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_delete));
+			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_delete));
 		items().push_back(*Gtk::manage(m_delete_nodes));
 
 		// Persistence-related functionality ...
@@ -149,22 +156,22 @@ public:
 				new image_menu_item::control(Parent, "save_node",
 					*Gtk::manage(new Gtk::Image(Gtk::Stock::SAVE, Gtk::ICON_SIZE_MENU)),
 					_("Save Object")) <<
-				connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_save_node))));
+				connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_save_node))));
 			items().push_back(*Gtk::manage(
 				new image_menu_item::control(Parent, "load_node",
 					*Gtk::manage(new Gtk::Image(Gtk::Stock::OPEN, Gtk::ICON_SIZE_MENU)),
 					_("Load Object")) <<
-				connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_load_node))));
+				connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_load_node))));
 		}
 */
 
 		// Mesh-specific functionality ...
 		m_instantiate_meshes = new menu_item::control(Parent, "instantiate_meshes", _("Instantiate")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_instantiate));
+			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_instantiate));
 		items().push_back(*Gtk::manage(m_instantiate_meshes));
 
 		m_duplicate_meshes = new menu_item::control(Parent, "duplicate_meshes", _("Duplicate")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_duplicate));
+			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_duplicate));
 		items().push_back(*Gtk::manage(m_duplicate_meshes));
 
 		// Mesh modifiers menu
@@ -179,7 +186,7 @@ public:
 
 				m_mesh_modifiers->items().push_back(*Gtk::manage(
 					create_menu_item(Parent, "mesh_modifier_", factory) <<
-					connect_menu_item(sigc::bind(sigc::mem_fun(*this, &node_context_menu::on_context_modifier_meshes), &factory))));
+					connect_menu_item(sigc::bind(sigc::mem_fun(*this, &node_context_menu::on_modify_meshes), &factory))));
 			}
 			items().push_back(Gtk::Menu_Helpers::MenuElem(_("Mesh Modifier"), *manage(m_mesh_modifiers)));
 		}
@@ -196,7 +203,7 @@ public:
 
 				m_transform_modifiers->items().push_back(*Gtk::manage(
 					create_menu_item(Parent, "transform_modifier_", factory) <<
-					connect_menu_item(sigc::bind(sigc::mem_fun(*this, &node_context_menu::on_context_modifier_transform), &factory))));
+					connect_menu_item(sigc::bind(sigc::mem_fun(*this, &node_context_menu::on_modifier_transform), &factory))));
 			}
 			items().push_back(Gtk::Menu_Helpers::MenuElem(_("Transform Modifier"), *manage(m_transform_modifiers)));
 		}
@@ -206,19 +213,19 @@ public:
 
 		items().push_back(*Gtk::manage(
 			new menu_item::control(Parent, "hide_selection", _("Hide Selection")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_hide_selection))));
+			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_hide_selection))));
 
 		items().push_back(*Gtk::manage(
 			new menu_item::control(Parent, "show_selection", _("Show Selection")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_show_selection))));
+			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_show_selection))));
 
 		items().push_back(*Gtk::manage(
 			new menu_item::control(Parent, "hide_unselected", _("Hide Unselected")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_hide_unselected))));
+			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_hide_unselected))));
 
 		items().push_back(*Gtk::manage(
 			new menu_item::control(Parent, "show_all", _("Show All")) <<
-			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_context_show_all))));
+			connect_menu_item(sigc::mem_fun(*this, &node_context_menu::on_show_all))));
 
 		show_all();
 	}
@@ -229,16 +236,6 @@ public:
 		m_viewport = m_document_state.get_focus_viewport();
 		const bool display_viewport = (m_viewport && m_viewport->gl_engine());
 		m_viewport1->set_sensitive(display_viewport);
-		m_viewport2->set_sensitive(display_viewport);
-		m_viewport3->set_sensitive(display_viewport);
-		m_viewport4->set_sensitive(display_viewport);
-		//m_viewport5->show();
-		m_viewport6->set_sensitive(display_viewport);
-		m_viewport7->set_sensitive(display_viewport);
-		m_viewport8->set_sensitive(display_viewport);
-		m_viewport9->set_sensitive(display_viewport);
-		m_viewport10->set_sensitive(display_viewport);
-		//m_viewport11->show();
 
 		// Mesh specific functionality
 		unsigned long selected_node_counter = 0;
@@ -255,12 +252,12 @@ public:
 				++selected_transform_counter;
 		}
 
-		const bool display_delete = (selected_node_counter > 0);
-		m_delete_nodes->set_sensitive(display_delete);
+		const bool non_empty_selection = (selected_node_counter > 0);
+		m_delete_nodes->set_sensitive(non_empty_selection);
+		m_instantiate_meshes->set_sensitive(non_empty_selection);
+		m_duplicate_meshes->set_sensitive(non_empty_selection);
 
 		const bool display_mesh_modifiers = (selected_mesh_counter > 0);
-		m_instantiate_meshes->set_sensitive(display_mesh_modifiers);
-		m_duplicate_meshes->set_sensitive(display_mesh_modifiers);
 		m_mesh_modifiers->set_sensitive(display_mesh_modifiers);
 
 		const bool display_transform_modifiers = (selected_transform_counter > 0);
@@ -274,18 +271,51 @@ public:
 	}
 
 private:
-	void on_context_set_viewport_camera()
+	void on_set_viewport_camera()
 	{
 		return_if_fail(m_viewport);
 
-		k3d::icamera* const camera = pick_camera(m_document_state);
+		k3d::icamera* const camera = pick_camera(m_document_state, m_viewport->camera());
 		if(!camera)
 			return;
 
 		m_viewport->set_camera(camera);
 	}
 
-	void on_context_set_viewport_gl_engine()
+	void on_set_viewport_preview_engine()
+	{
+		return_if_fail(m_viewport);
+
+		k3d::icamera_preview_render_engine* const engine = pick_camera_preview_render_engine(m_document_state);
+		if(!engine)
+			return;
+
+		m_viewport->set_camera_preview_engine(engine);
+	}
+
+	void on_set_viewport_still_engine()
+	{
+		return_if_fail(m_viewport);
+
+		k3d::icamera_still_render_engine* const engine = pick_camera_still_render_engine(m_document_state);
+		if(!engine)
+			return;
+
+		m_viewport->set_camera_still_engine(engine);
+	}
+
+	void on_set_viewport_animation_engine()
+	{
+		return_if_fail(m_viewport);
+
+		k3d::icamera_animation_render_engine* const engine = pick_camera_animation_render_engine(m_document_state);
+		if(!engine)
+			return;
+
+		m_viewport->set_camera_animation_engine(engine);
+	}
+
+	void on_set_viewport_gl_engine()
 	{
 		return_if_fail(m_viewport);
 
@@ -296,52 +326,29 @@ private:
 		m_viewport->set_gl_engine(engine);
 	}
 
-	void on_context_set_viewport_preview_engine()
+	void on_render_preview()
 	{
 		return_if_fail(m_viewport);
 
-		k3d::ipreview_render_engine* const engine = pick_preview_render_engine(m_document_state);
-		if(!engine)
+		k3d::icamera* camera = m_viewport->camera();
+		if(!camera)
+			camera = pick_camera(m_document_state);
+		if(!camera)
 			return;
 
-		m_viewport->set_preview_engine(engine);
-	}
-
-	void on_context_set_viewport_still_engine()
-	{
-		return_if_fail(m_viewport);
-
-		k3d::istill_render_engine* const engine = pick_still_render_engine(m_document_state);
-		if(!engine)
+		k3d::icamera_preview_render_engine* render_engine = m_viewport->camera_preview_engine();
+		if(!render_engine)
+			render_engine = pick_camera_preview_render_engine(m_document_state);
+		if(!render_engine)
 			return;
 
-		m_viewport->set_still_engine(engine);
+		m_viewport->set_camera(camera);
+		m_viewport->set_camera_preview_engine(render_engine);
+
+		render_camera_preview(*camera, *render_engine);
 	}
 
-	void on_context_set_viewport_animation_engine()
-	{
-		return_if_fail(m_viewport);
-
-		k3d::ianimation_render_engine* const engine = pick_animation_render_engine(m_document_state);
-		if(!engine)
-			return;
-
-		m_viewport->set_animation_engine(engine);
-	}
-
-	void on_context_render_preview()
-	{
-		k3d::icamera* camera = m_viewport ? m_viewport->camera() : 0;
-		k3d::ipreview_render_engine* render_engine = m_viewport ? m_viewport->preview_engine() : 0;
-
-		if(render_preview(m_document_state, camera, render_engine))
-		{
-			m_viewport->set_camera(camera);
-			m_viewport->set_preview_engine(render_engine);
-		}
-	}
-
-	void on_context_render_frame()
+	void on_render_frame()
 	{
 		k3d::icamera* camera = m_viewport ? m_viewport->camera() : 0;
 		if(!camera)
@@ -349,22 +356,22 @@ private:
 		if(!camera)
 			return;
 
-		k3d::istill_render_engine* render_engine = m_viewport ? m_viewport->still_engine() : 0;
+		k3d::icamera_still_render_engine* render_engine = m_viewport ? m_viewport->camera_still_engine() : 0;
 		if(!render_engine)
-			render_engine = pick_still_render_engine(m_document_state);
+			render_engine = pick_camera_still_render_engine(m_document_state);
 		if(!render_engine)
 			return;
 
 		if(m_viewport)
 		{
 			m_viewport->set_camera(camera);
-			m_viewport->set_still_engine(render_engine);
+			m_viewport->set_camera_still_engine(render_engine);
 		}
 
-		render_frame(*camera, *render_engine);
+		render_camera_frame(*camera, *render_engine);
 	}
 
-	void on_context_render_animation()
+	void on_render_animation()
 	{
 		k3d::icamera* camera = m_viewport ? m_viewport->camera() : 0;
 		if(!camera)
@@ -372,23 +379,23 @@ private:
 		if(!camera)
 			return;
 
-		k3d::ianimation_render_engine* render_engine = m_viewport ? m_viewport->animation_engine() : 0;
+		k3d::icamera_animation_render_engine* render_engine = m_viewport ? m_viewport->camera_animation_engine() : 0;
 		if(!render_engine)
-			render_engine = pick_animation_render_engine(m_document_state);
+			render_engine = pick_camera_animation_render_engine(m_document_state);
 		if(!render_engine)
 			return;
 
 		if(m_viewport)
 		{
 			m_viewport->set_camera(camera);
-			m_viewport->set_animation_engine(render_engine);
+			m_viewport->set_camera_animation_engine(render_engine);
 		}
 
-		render_animation(m_document_state, *camera, *render_engine);
+		render_camera_animation(m_document_state, *camera, *render_engine);
 	}
 
 /*
-	void on_context_save_node()
+	void on_save_node()
 	{
 		k3d::ipersistent* const persistent = dynamic_cast<k3d::ipersistent*>(&m_node);
 		return_if_fail(persistent);
@@ -416,7 +423,7 @@ private:
 		file << k3d::xml::declaration() << xml;
 	}
 
-	void on_context_load_node()
+	void on_load_node()
 	{
 		k3d::ipersistent* const persistent = dynamic_cast<k3d::ipersistent*>(&m_node);
 		return_if_fail(persistent);
@@ -467,7 +474,7 @@ private:
 	}
 */
 
-	void on_context_delete()
+	void on_delete()
 	{
 		k3d::record_state_change_set changeset(m_document_state.document(), _("Delete nodes"));
 
@@ -478,28 +485,28 @@ private:
 	}
 
 	/// Instantiates selected meshes
-	void on_context_instantiate()
+	void on_instantiate()
 	{
 		instantiate_selected_nodes(m_document_state);
 	}
 
 	/// Duplicates selected meshes
-	void on_context_duplicate()
+	void on_duplicate()
 	{
 		duplicate_selected_nodes(m_document_state);
 	}
 
 	/// Modify selected meshes
-	void on_context_modifier_meshes(k3d::iplugin_factory* Modifier)
+	void on_modify_meshes(k3d::iplugin_factory* Modifier)
 	{
 		return_if_fail(Modifier);
 
-		k3d::nodes_t selected_nodes = m_document_state.selected_nodes();
+		k3d::inode* new_modifier = 0;
 
-		k3d::inode* new_modifier;
-		for(k3d::nodes_t::iterator node = selected_nodes.begin(); node != selected_nodes.end(); ++node)
+		const k3d::nodes_t selected_nodes = m_document_state.selected_nodes();
+		for(k3d::nodes_t::const_iterator node = selected_nodes.begin(); node != selected_nodes.end(); ++node)
 		{
-			new_modifier = modify_mesh(m_document_state.document(), **node, Modifier);
+			new_modifier = modify_mesh(m_document_state, **node, Modifier);
 			assert_warning(new_modifier);
 		}
 
@@ -511,7 +518,7 @@ private:
 	}
 
 	/// Modify selected transforms
-	void on_context_modifier_transform(k3d::iplugin_factory* Modifier)
+	void on_modifier_transform(k3d::iplugin_factory* Modifier)
 	{
 		return_if_fail(Modifier);
 
@@ -531,25 +538,25 @@ private:
 		k3d::gl::redraw_all(m_document_state.document(), k3d::gl::irender_engine::ASYNCHRONOUS);
 	}
 
-	void on_context_hide_selection()
+	void on_hide_selection()
 	{
 		k3d::record_state_change_set change_set(m_document_state.document(), _("Hide selection"));
 		m_document_state.hide_selection();
 	}
 
-	void on_context_show_selection()
+	void on_show_selection()
 	{
 		k3d::record_state_change_set change_set(m_document_state.document(), _("Show selection"));
 		m_document_state.show_selection();
 	}
 
-	void on_context_hide_unselected()
+	void on_hide_unselected()
 	{
 		k3d::record_state_change_set change_set(m_document_state.document(), _("Hide unselected"));
 		m_document_state.hide_unselected();
 	}
 
-	void on_context_show_all()
+	void on_show_all()
 	{
 		k3d::record_state_change_set change_set(m_document_state.document(), _("Show all"));
 		m_document_state.show_all_nodes();
@@ -557,18 +564,6 @@ private:
 
 	// Viewport items
 	Gtk::MenuItem* m_viewport1;
-	image_menu_item::control* m_viewport2;
-	image_menu_item::control* m_viewport3;
-	image_menu_item::control* m_viewport4;
-	Gtk::Menu_Helpers::SeparatorElem* m_viewport5;
-	menu_item::control* m_viewport6;
-	menu_item::control* m_viewport7;
-	menu_item::control* m_viewport8;
-	menu_item::control* m_viewport9;
-	menu_item::control* m_viewport10;
-	Gtk::Menu_Helpers::SeparatorElem* m_viewport11;
-
-	Gtk::MenuItem* m_node_label;
 
 	// Mesh functionality
 	menu_item::control* m_delete_nodes;
@@ -590,4 +585,3 @@ Gtk::Menu* create_context_menu(document_state& DocumentState, k3d::icommand_node
 
 } // namespace libk3dngui
 
-
