@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -17,11 +17,18 @@
 // License along with this program; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-/** \author Timothy M. Shead (tshead@k-3d.com)
+/**		\author Timothy M. Shead (tshead@k-3d.com)
 */
 
-#include <gtkmm/invisible.h>
+#include <gdkmm/screen.h>
+#include <gtkmm/dialog.h>
+#include <gtkmm/menu.h>
+#include <gtkmm/window.h>
 
+#include <gtk/gtkgl.h>
+#include <gtk/gtkmain.h>
+
+#include "application_state.h"
 #include "context_menu.h"
 #include "document_state.h"
 #include "knife_tool.h"
@@ -31,8 +38,11 @@
 #include "plug_tool.h"
 #include "render_region_tool.h"
 #include "rotate_tool.h"
+#include "safe_close_dialog.h"
 #include "scale_tool.h"
 #include "selection_tool.h"
+#include "snap_tool.h"
+#include "unsaved_document.h"
 #include "utility.h"
 #include "viewport.h"
 
@@ -44,6 +54,7 @@
 #include <k3dsdk/data.h>
 #include <k3dsdk/i18n.h>
 #include <k3dsdk/ianimation_render_engine.h>
+#include <k3dsdk/iapplication.h>
 #include <k3dsdk/icommand_tree.h>
 #include <k3dsdk/imesh_sink.h>
 #include <k3dsdk/imesh_source.h>
@@ -61,14 +72,9 @@
 #include <k3dsdk/time_source.h>
 #include <k3dsdk/transform.h>
 
-#include <gdkmm/screen.h>
-#include <gtkmm/menu.h>
-
-#include <gtk/gtkgl.h>
-#include <gtk/gtkmain.h>
-
 #include <iterator>
 #include <functional>
+#include <set>
 
 namespace libk3dngui
 {
@@ -261,21 +267,28 @@ struct select_faces
 						k3d::mesh_selection::record(token->id+1, weight),
 						Selection.faces);
 					return;
+
 				case k3d::selection::BILINEAR_PATCH:
 					k3d::mesh_selection::insert(
 						token->id,
 						k3d::mesh_selection::record(token->id+1, weight),
 						Selection.bilinear_patches);
+					return;
+
 				case k3d::selection::BICUBIC_PATCH:
 					k3d::mesh_selection::insert(
 						token->id,
 						k3d::mesh_selection::record(token->id+1, weight),
 						Selection.bicubic_patches);
+					return;
+
 				case k3d::selection::NUPATCH:
 					k3d::mesh_selection::insert(
 						token->id,
 						k3d::mesh_selection::record(token->id+1, weight),
 						Selection.nupatches);
+					return;
+
 				default:
 					break;
 			}
@@ -304,16 +317,10 @@ void select_components(const k3d::selection::records& Selection, const UpdatePol
 			}
 
 			node = n->first;
-			selection = k3d::mesh_selection::deselect_all();
+			selection = k3d::mesh_selection::select_null();
 			sink = dynamic_cast<k3d::imesh_selection_sink*>(node);
 			if(sink)
-			{
-				selection =
-					boost::any_cast<k3d::mesh_selection>(sink->mesh_selection_sink_input().property_value());
-
-				if(selection == k3d::mesh_selection::select_null() || selection == k3d::mesh_selection::select_all())
-					selection = k3d::mesh_selection::deselect_all();
-			}
+				selection = boost::any_cast<k3d::mesh_selection>(sink->mesh_selection_sink_input().property_value());
 		}
 
 		if(!sink)
@@ -350,6 +357,7 @@ void update_component_selection(const k3d::nodes_t& Nodes, const UpdatePolicyT&
 		UpdatePolicy(selection, mesh);
 
 		k3d::set_value(sink->mesh_selection_sink_input(), selection);
+		k3d::set_value(**node, "show_component_selection", VisibleSelection);
 	}
 }
 
@@ -590,9 +598,6 @@ struct convert_to_points
 		if(!Mesh)
 			return;
 
-		if(Selection == k3d::mesh_selection::select_null())
-			k3d::for_each_component(*Mesh, k3d::selection::set_weight(0.0));
-
 		k3d::for_each_component(*Mesh, helper());
 
 		k3d::store_selection(*Mesh, Selection);
@@ -711,9 +716,6 @@ struct convert_to_lines
 		if(!Mesh)
 			return;
 
-		if(Selection == k3d::mesh_selection::select_null())
-			k3d::for_each_component(*Mesh, k3d::selection::set_weight(0.0));
-
 		k3d::for_each_component(*Mesh, helper());
 		k3d::for_each_point(*Mesh, k3d::selection::set_weight(0.0));
 
@@ -729,7 +731,7 @@ struct convert_to_faces
 		template<typename T>
 		void operator()(T& Component)
 		{
-			Component.selection_weight = 0.0;
+			Component.selection_weight = 0;
 		}
 
 		void operator()(k3d::point& Component)
@@ -751,7 +753,7 @@ struct convert_to_faces
 			{
 				if(edge->selection_weight || (edge->companion && edge->companion->selection_weight) || (edge->vertex && edge->vertex->selection_weight))
 				{
-					Component.selection_weight = 1.0;
+					Component.selection_weight = 1;
 					return;
 				}
 
@@ -765,7 +767,7 @@ struct convert_to_faces
 				{
 					if(edge->selection_weight || (edge->companion && edge->companion->selection_weight) || (edge->vertex && edge->vertex->selection_weight))
 					{
-						Component.selection_weight = 1.0;
+						Component.selection_weight = 1;
 						return;
 					}
 
@@ -784,7 +786,7 @@ struct convert_to_faces
 			{
 				if((*control_point)->selection_weight)
 				{
-					Component.selection_weight = 1.0;
+					Component.selection_weight = 1;
 					return;
 				}
 			}
@@ -799,7 +801,7 @@ struct convert_to_faces
 			{
 				if((*control_point)->selection_weight)
 				{
-					Component.selection_weight = 1.0;
+					Component.selection_weight = 1;
 					return;
 				}
 			}
@@ -814,7 +816,7 @@ struct convert_to_faces
 			{
 				if(control_point->position->selection_weight)
 				{
-					Component.selection_weight = 1.0;
+					Component.selection_weight = 1;
 					return;
 				}
 			}
@@ -826,12 +828,9 @@ struct convert_to_faces
 		if(!Mesh)
 			return;
 
-		if(Selection == k3d::mesh_selection::select_null())
-			k3d::for_each_component(*Mesh, k3d::selection::set_weight(0.0));
-
 		k3d::for_each_component(*Mesh, helper());
-		k3d::for_each_point(*Mesh, k3d::selection::set_weight(0.0));
-		k3d::for_each_edge(*Mesh, k3d::selection::set_weight(0.0));
+		k3d::for_each_point(*Mesh, k3d::selection::set_weight(0));
+		k3d::for_each_edge(*Mesh, k3d::selection::set_weight(0));
 
 		k3d::store_selection(*Mesh, Selection);
 		deselect_gaps(Selection);
@@ -844,14 +843,15 @@ struct convert_to_faces
 // document_state::implementation
 
 class document_state::implementation :
-	public sigc::trackable,
-	public k3d::ipersistent
+	public sigc::trackable/*,
+	public k3d::ipersistent*/
 {
 public:
 	implementation(k3d::idocument& Document) :
 		m_document(Document),
 		m_gdkgl_share_list(0),
 		m_selection_mode(init_owner(*this) + init_name("selection_mode") + init_label(_("Selection Type")) + init_description(_("Sets selection mode (nodes, faces, edges, points, etc)")) + init_value(SELECT_NODES) + init_values(detail::selection_mode_values())),
+		m_last_selection_mode(SELECT_NODES),
 		m_active_tool(0),
 		m_selection_tool(0),
 		m_move_tool(0),
@@ -864,10 +864,14 @@ public:
 		m_context_menu_node("context_menu", dynamic_cast<k3d::icommand_node*>(&Document)),
 		m_context_menu(0)
 	{
+		/** \todo Restore UI serialization */
+/*
 		m_document.enable_plugin_serialization(get_class_id(), *this);
-
+*/
 		m_selection_mode.changed_signal().connect(sigc::mem_fun(*this, &implementation::on_selection_mode_changed));
 
+		// Process remove_nodes_signal
+		m_document.nodes().remove_nodes_signal().connect(sigc::mem_fun(*this, &implementation::on_nodes_removed));
 	}
 
 	~implementation()
@@ -889,6 +893,8 @@ public:
 		return m_document;
 	}
 
+	/** \todo Restore UI serialization */
+/*
 	void save(k3d::xml::element& Element, const k3d::ipersistent::save_context& Context)
 	{
 		save_command_node(dynamic_cast<k3d::icommand_node&>(m_document), Element.append(k3d::xml::element("nodes")), Context);
@@ -927,10 +933,11 @@ public:
 				persistent->load(*xml_node, Context);
 		}
 	}
+*/
 
-	/// Returns a "global" (to the document) gdkgl context that can be used to share display lists
 #ifndef K3D_PLATFORM_WIN32
 
+	/// Returns a "global" (to the document) gdkgl context that can be used to share display lists
 	GdkGLContext* gdkgl_share_list()
 	{
 		if(!m_gdkgl_share_list)
@@ -957,9 +964,27 @@ public:
 
 #else // !K3D_PLATFORM_WIN32
 
+	/// Returns a "global" (to the document) gdkgl context that can be used to share display lists
 	GdkGLContext* gdkgl_share_list()
 	{
-		return 0;
+		if(!m_gdkgl_share_list)
+		{
+			GdkGLConfig* const config = gdk_gl_config_new_by_mode(
+				static_cast<GdkGLConfigMode>(GDK_GL_MODE_RGBA | GDK_GL_MODE_DOUBLE | GDK_GL_MODE_DEPTH));
+			return_val_if_fail(config, 0);
+
+			Gtk::Window* const window = new Gtk::Window();
+			gtk_widget_set_gl_capability(GTK_WIDGET(window->gobj()), config, 0, true, GDK_GL_RGBA_TYPE);
+			window->show();
+			window->hide();
+
+			GdkGLContext* const context = gtk_widget_get_gl_context(GTK_WIDGET(window->gobj()));
+			return_val_if_fail(context, 0);
+
+			m_gdkgl_share_list = context;
+		}
+
+		return m_gdkgl_share_list;
 	}
 
 #endif // K3D_PLATFORM_WIN32
@@ -982,6 +1007,12 @@ public:
 		return m_view_tool_properties_signal;
 	}
 
+	/// Returns a signal that can be emitted to acknowledge of a document selection change
+	document_selection_change_signal_t& document_selection_change_signal()
+	{
+		return m_document_selection_change_signal;
+	}
+
 	/// Returns the active tool for the document
 	tool& active_tool()
 	{
@@ -994,7 +1025,7 @@ public:
 		m_active_tool->deactivate();
 		m_active_tool = &ActiveTool;
 		m_active_tool->activate();
-		m_active_tool_changed_signal.emit();
+		m_active_tool_changed_signal.emit(0);
 	}
 
 	active_tool_changed_signal_t& active_tool_changed_signal()
@@ -1013,14 +1044,22 @@ public:
 		m_selection_mode.set_value(Mode);
 	}
 
+	/// Called by the signal system anytime nodes are removed from the document
+	void on_nodes_removed(const k3d::inode_collection::nodes_t& Nodes)
+	{
+		selection_changed();
+	}
+
 	typedef std::set<k3d::mesh*> meshes_t;
 	typedef std::set<k3d::inode*> node_selection_t;
 
-	const bool is_selected(k3d::inode* Object)
+	const bool is_selected(k3d::inode* Node)
 	{
-		k3d::iselectable* selectable = dynamic_cast<k3d::iselectable*>(Object);
-		if(selectable && selectable->get_visible_selection())
-			return true;
+		if(k3d::iselectable* selectable = dynamic_cast<k3d::iselectable*>(Node))
+		{
+			if(selectable->get_selection_weight())
+				return true;
+		}
 
 		return false;
 	}
@@ -1044,7 +1083,22 @@ public:
 
 	void selection_changed()
 	{
+		// Switch to node selection mode when there's no selected mesh in the document
+		if(SELECT_NODES != m_selection_mode.value())
+		{
+			unsigned long selected_mesh_count = 0;
+
+			const k3d::nodes_t nodes = selected_nodes();
+			for(k3d::nodes_t::const_iterator node = nodes.begin(); node != nodes.end(); ++node)
+				if(dynamic_cast<k3d::imesh_source*>(*node))
+					++selected_mesh_count;
+
+			if(!selected_mesh_count)
+				set_selection_mode(SELECT_NODES);
+		}
+
 		m_active_tool->document_selection_changed();
+		m_document_selection_change_signal.emit();
 	}
 
 	void select_points(const k3d::selection::records& Selection)
@@ -1064,10 +1118,26 @@ public:
 
 	void select_nodes(const k3d::selection::records& Selection)
 	{
+		unsigned long selected_nodes = 0;
+		k3d::inode* selected_node = 0;
+
 		for(k3d::selection::records::const_iterator record = Selection.begin(); record != Selection.end(); ++record)
 		{
 			if(k3d::inode* const node = k3d::selection::get_node(*record))
+			{
 				k3d::selection::select(node);
+
+				++selected_nodes;
+				selected_node = node;
+			}
+		}
+
+		// Iff one node was selected, send "view node" signals
+		if(1 == selected_nodes && selected_node)
+		{
+			m_view_node_properties_signal.emit(selected_node);
+			m_view_node_history_signal.emit(selected_node);
+
 		}
 	}
 
@@ -1183,18 +1253,12 @@ public:
 	void deselect_all_nodes()
 	{
 		for(k3d::inode_collection::nodes_t::const_iterator node = m_document.nodes().collection().begin(); node != m_document.nodes().collection().end(); ++node)
-		{
-			if(k3d::iselectable* const selectable = dynamic_cast<k3d::iselectable*>(*node))
-			{
-				selectable->set_selection_weight(0.0);
-				selectable->set_visible_selection(false);
-			}
-		}
+			k3d::selection::deselect(*node);
 	}
 
 	void deselect_all_components()
 	{
-		detail::update_component_selection(m_document.nodes().collection(), detail::deselect_all(), false);
+		detail::update_component_selection(m_document.nodes().collection(), detail::deselect_all(), true);
 	}
 
 	void deselect_all()
@@ -1279,8 +1343,9 @@ public:
 		const k3d::nodes_t nodes = selected_nodes();
 		for(k3d::nodes_t::const_iterator node = nodes.begin(); node != nodes.end(); ++node)
 		{
-			k3d::set_value(**node, "viewport_visible", false);
-			k3d::set_value(**node, "render_final", false);
+			k3d::selection::select(*node);
+			k3d::quiet_set_value(**node, "viewport_visible", false);
+			k3d::quiet_set_value(**node, "render_final", false);
 		}
 	}
 
@@ -1289,8 +1354,8 @@ public:
 		const k3d::nodes_t nodes = selected_nodes();
 		for(k3d::nodes_t::const_iterator node = nodes.begin(); node != nodes.end(); ++node)
 		{
-			k3d::set_value(**node, "viewport_visible", true);
-			k3d::set_value(**node, "render_final", true);
+			k3d::quiet_set_value(**node, "viewport_visible", true);
+			k3d::quiet_set_value(**node, "render_final", true);
 		}
 	}
 
@@ -1300,8 +1365,8 @@ public:
 		{
 			if(!is_selected(*node))
 			{
-				k3d::set_value(**node, "viewport_visible", false);
-				k3d::set_value(**node, "render_final", false);
+				k3d::quiet_set_value(**node, "viewport_visible", false);
+				k3d::quiet_set_value(**node, "render_final", false);
 			}
 		}
 	}
@@ -1310,8 +1375,8 @@ public:
 	{
 		for(k3d::inode_collection::nodes_t::const_iterator node = m_document.nodes().collection().begin(); node != m_document.nodes().collection().end(); ++node)
 		{
-			k3d::set_value(**node, "viewport_visible", true);
-			k3d::set_value(**node, "render_final", true);
+			k3d::quiet_set_value(**node, "viewport_visible", true);
+			k3d::quiet_set_value(**node, "render_final", true);
 		}
 	}
 
@@ -1394,37 +1459,49 @@ public:
 		return node;
 	}
 
-	void popup_context_menu()
+	void popup_context_menu(const bool UserAction)
 	{
-		m_context_menu->popup(1, gtk_get_current_event_time());
+		if(UserAction)
+			m_context_menu->popup(1, gtk_get_current_event_time());
+		else
+			m_context_menu->popup(0, GDK_CURRENT_TIME);
 	}
 
-	/// Converts the current selection when node selection mode is chosen
+	/// Sets the current selection when node selection mode is chosen
 	void on_set_node_mode()
 	{
-		detail::update_component_selection(m_document.nodes().collection(), detail::select_null(), false);
+		detail::update_component_selection(selected_nodes(), detail::select_null(), false);
 	}
 
-	/// Converts the current selection when point selection mode is chosen
+	/// Sets the current selection when point selection mode is chosen
 	void on_set_point_mode()
 	{
-		detail::update_component_selection(m_document.nodes().collection(), detail::convert_to_points(), true);
+		if(SELECT_NODES == m_last_selection_mode)
+			detail::update_component_selection(selected_nodes(), detail::select_null(), true);
+		else
+			detail::update_component_selection(selected_nodes(), detail::convert_to_points(), true);
 	}
 
-	/// Converts the current selection when line selection mode is chosen
+	/// Sets the current selection when line selection mode is chosen
 	void on_set_line_mode()
 	{
-		detail::update_component_selection(m_document.nodes().collection(), detail::convert_to_lines(), true);
+		if(SELECT_NODES == m_last_selection_mode)
+			detail::update_component_selection(selected_nodes(), detail::select_null(), true);
+		else
+			detail::update_component_selection(selected_nodes(), detail::convert_to_lines(), true);
 	}
 
-	/// Converts the current selection when face selection mode is chosen
+	/// Sets the current selection when face selection mode is chosen
 	void on_set_face_mode()
 	{
-		detail::update_component_selection(m_document.nodes().collection(), detail::convert_to_faces(), true);
+		if(SELECT_NODES == m_last_selection_mode)
+			detail::update_component_selection(selected_nodes(), detail::select_null(), true);
+		else
+			detail::update_component_selection(selected_nodes(), detail::convert_to_faces(), true);
 	}
 
 	/// Called by the signal system when the selection mode changes
-	void on_selection_mode_changed()
+	void on_selection_mode_changed(k3d::iunknown*)
 	{
 		switch(m_selection_mode.value())
 		{
@@ -1442,11 +1519,14 @@ public:
 				break;
 		}
 
+		m_last_selection_mode = m_selection_mode.value();
+
 		selection_changed();
 	}
 
 	/// Stores a reference to the owning document
 	k3d::idocument& m_document;
+	sigc::signal0<bool, k3d::signal::cancelable> m_safe_to_close_signal;
 	/// Stores an unused off-screen pixmap so we can create a global gdkgl context for sharing display lists
 	Glib::RefPtr<Gdk::Pixmap> m_gdkgl_share_pixmap;
 	/// Stores a gdkgl context that can be used to share display lists for the entire document
@@ -1457,10 +1537,14 @@ public:
 	view_node_properties_signal_t m_view_node_properties_signal;
 	/// A signal that can be emitted to request display of the properties for a tool
 	view_tool_properties_signal_t m_view_tool_properties_signal;
+	/// A signal that can be emitted to acknowledge of a document selection change
+	document_selection_change_signal_t m_document_selection_change_signal;
 	/// A signal that will be emitted whenever the active tool changes
 	active_tool_changed_signal_t m_active_tool_changed_signal;
 	/// Stores the current document-wide selection mode
 	selection_mode_property_t m_selection_mode;
+	/// Stores the last document-wide selection mode
+	selection_mode_t m_last_selection_mode;
 
 	document_state::set_cursor_signal_t m_set_cursor_signal;
 	document_state::clear_cursor_signal_t m_clear_cursor_signal;
@@ -1477,11 +1561,25 @@ public:
 	tool* m_plug_tool;
 	tool* m_render_region_tool;
 	tool* m_knife_tool;
+	tool* m_snap_tool;
 
 	/// Placeholder that "groups" context menu command nodes together
 	k3d::command_node::implementation m_context_menu_node;
 	/// Document wide context menu
 	Gtk::Menu* m_context_menu;
+
+	/// Used to keep documents sorted in the "safe to close" dialog
+	struct sort_by_title
+	{
+		bool operator()(const safe_close_dialog::entry& LHS, const safe_close_dialog::entry& RHS)
+		{
+			return LHS.document->unsaved_document_title() < RHS.document->unsaved_document_title();
+		}
+	};
+
+	typedef sigc::signal<unsaved_document*> safe_close_signal_t;
+	/// Signal that will be emitted prior to safe shutdown to "gather" unsaved documents
+	safe_close_signal_t m_safe_close_signal;
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -1491,6 +1589,8 @@ document_state::document_state(k3d::idocument& Document) :
 	m_implementation(new implementation(Document)),
 	m_focus_viewport(0)
 {
+//	application_state::instance().connect_safe_to_close_signal(m_implementation->m_safe_to_close_signal.make_slot());
+
 	Document.close_signal().connect(sigc::mem_fun(*this, &document_state::on_document_close));
 
 	m_implementation->m_selection_tool = new libk3dngui::selection_tool(*this, "selection_tool");
@@ -1501,6 +1601,7 @@ document_state::document_state(k3d::idocument& Document) :
 	m_implementation->m_plug_tool = new libk3dngui::plug_tool(*this, "plug_tool");
 	m_implementation->m_render_region_tool = new libk3dngui::render_region_tool(*this, "render_region_tool");
 	m_implementation->m_knife_tool = new libk3dngui::knife_tool(*this, "knife_tool");
+	m_implementation->m_snap_tool = new libk3dngui::snap_tool(*this, "snap_tool");
 
 	m_implementation->m_active_tool = m_implementation->m_selection_tool;
 	m_implementation->m_active_tool->activate();
@@ -1524,6 +1625,57 @@ k3d::idocument& document_state::document()
 	return m_implementation->document();
 }
 
+sigc::connection document_state::connect_safe_close_signal(const sigc::slot<unsaved_document*>& Slot)
+{
+	return m_implementation->m_safe_close_signal.connect(Slot);
+}
+
+bool document_state::safe_close(Gtk::Window& Parent)
+{
+	if(application_state::instance().batch_mode())
+	{
+		k3d::application().close_document(document());
+		return true;
+	}
+	
+	safe_close_dialog::entries_t entries;
+
+	implementation::safe_close_signal_t::slot_list_type slots = m_implementation->m_safe_close_signal.slots();
+	for(implementation::safe_close_signal_t::slot_list_type::iterator slot = slots.begin(); slot != slots.end(); ++slot)
+	{
+		unsaved_document* doc = (*slot)();
+		if(doc && doc->unsaved_changes())
+			entries.push_back(safe_close_dialog::entry(doc));
+	}
+
+	if(!entries.empty())
+	{
+		switch(safe_close_dialog::run(Parent, entries))
+		{
+			case Gtk::RESPONSE_NONE:
+			case Gtk::RESPONSE_CANCEL:
+				return false;
+			case Gtk::RESPONSE_CLOSE:
+				k3d::application().close_document(document());
+				return true;
+			case Gtk::RESPONSE_OK:
+				for(safe_close_dialog::entries_t::const_iterator entry = entries.begin(); entry != entries.end(); ++entry)
+				{
+					if(entry->save)
+					{
+						if(!entry->document->save_unsaved_changes())
+							return false;
+					}
+				}
+				k3d::application().close_document(document());
+				return true;
+		}
+	}
+
+	k3d::application().close_document(document());
+	return true;
+}
+
 GdkGLContext* document_state::gdkgl_share_list()
 {
 	return m_implementation->gdkgl_share_list();
@@ -1544,6 +1696,11 @@ document_state::view_tool_properties_signal_t& document_state::view_tool_propert
 	return m_implementation->view_tool_properties_signal();
 }
 
+document_state::document_selection_change_signal_t& document_state::document_selection_change_signal()
+{
+	return m_implementation->document_selection_change_signal();
+}
+
 tool& document_state::active_tool()
 {
 	return m_implementation->active_tool();
@@ -1599,6 +1756,11 @@ tool& document_state::knife_tool()
 	return *m_implementation->m_knife_tool;
 }
 
+tool& document_state::snap_tool()
+{
+	return *m_implementation->m_snap_tool;
+}
+
 document_state::selection_mode_property_t& document_state::selection_mode()
 {
 	return m_implementation->selection_mode();
@@ -1619,9 +1781,9 @@ void document_state::select(const k3d::selection::records& Selection)
 	m_implementation->select(Selection);
 }
 
-const bool document_state::is_selected(k3d::inode* Object)
+const bool document_state::is_selected(k3d::inode* Node)
 {
-	return m_implementation->is_selected(Object);
+	return m_implementation->is_selected(Node);
 }
 
 const bool document_state::is_selected(const k3d::selection::record& Selection)
@@ -1704,9 +1866,9 @@ k3d::inode* document_state::create_node(k3d::iplugin_factory* const Factory)
 	return m_implementation->create_node(Factory);
 }
 
-void document_state::popup_context_menu()
+void document_state::popup_context_menu(const bool UserAction)
 {
-	m_implementation->popup_context_menu();
+	m_implementation->popup_context_menu(UserAction);
 }
 
 } // namespace libk3dngui
