@@ -30,15 +30,14 @@ namespace libk3dngui
 {
 
 class document_state;
-class user_interface;
 
 /// Opaque handle for the main per-document user interface window
 class main_document_window
 {
 public:
-	main_document_window(document_state& DocumentState, user_interface& UserInterface);
+	main_document_window(document_state& DocumentState);
 	~main_document_window();
-
+	
 private:
 	class implementation;
 	implementation* const m_implementation;
@@ -51,4 +50,3 @@ void populate_new_document(k3d::idocument& Document);
 
 #endif // NGUI_MAIN_DOCUMENT_WINDOW_H
 
-
