@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -17,6 +17,7 @@
 // License along with this program; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+#include "angle_axis_control.h"
 #include "aqsis_layer_chooser.h"
 #include "asynchronous_update.h"
 #include "bitmap_preview.h"
@@ -29,14 +30,13 @@
 #include "document_state.h"
 #include "entry.h"
 #include "enumeration_chooser.h"
-#include "file_selection.h"
+#include "file_chooser_dialog.h"
 #include "icons.h"
 #include "messages.h"
 #include "node_chooser.h"
 #include "node_properties.h"
-#include "orientation.h"
 #include "path_chooser.h"
-#include "position.h"
+#include "point_control.h"
 #include "property_button.h"
 #include "property_label.h"
 #include "render.h"
@@ -52,8 +52,12 @@
 
 #include <k3dsdk/frames.h>
 #include <k3dsdk/i18n.h>
+#include <k3dsdk/ianimation_render_engine.h>
 #include <k3dsdk/iaqsis.h>
 #include <k3dsdk/icamera.h>
+#include <k3dsdk/icamera_animation_render_engine.h>
+#include <k3dsdk/icamera_preview_render_engine.h>
+#include <k3dsdk/icamera_still_render_engine.h>
 #include <k3dsdk/ideletable.h>
 #include <k3dsdk/idocument.h>
 #include <k3dsdk/ienumeration_property.h>
@@ -61,18 +65,16 @@
 #include <k3dsdk/imeasurement_property.h>
 #include <k3dsdk/inode.h>
 #include <k3dsdk/iplugin_factory.h>
-#include <k3dsdk/ipreview_depict_engine.h>
 #include <k3dsdk/ipreview_render_engine.h>
 #include <k3dsdk/iproperty_group_collection.h>
 #include <k3dsdk/iscript_property.h>
 #include <k3dsdk/iselectable.h>
-#include <k3dsdk/istill_depict_engine.h>
 #include <k3dsdk/istill_render_engine.h>
 #include <k3dsdk/iuser_property.h>
-#include <k3dsdk/ianimation_render_engine.h>
 #include <k3dsdk/mesh_selection.h>
 #include <k3dsdk/options.h>
 #include <k3dsdk/property.h>
+#include <k3dsdk/renderman_types.h>
 #include <k3dsdk/state_change_set.h>
 #include <k3dsdk/string_cast.h>
 #include <k3dsdk/string_modifiers.h>
@@ -104,9 +106,10 @@ namespace node_properties
 /////////////////////////////////////////////////////////////////////////////
 // control::implementation
 
-struct control::implementation :
+class control::implementation :
 	public asynchronous_update
 {
+public:
 	implementation(document_state& DocumentState, k3d::icommand_node& Parent) :
 		m_document_state(DocumentState),
 		m_node(0),
@@ -118,15 +121,27 @@ struct control::implementation :
 		m_scrolled_window.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
 		m_scrolled_window.add(m_vbox);
 
+		// If only one node is selected, show its properties
+		const k3d::nodes_t nodes = m_document_state.selected_nodes();
+		if(1 == nodes.size())
+			on_view_node_properties(nodes.front());
+
+		m_document_state.document().close_signal().connect(sigc::mem_fun(*this, &implementation::on_document_closed));
 		m_document_state.view_node_properties_signal().connect(sigc::mem_fun(*this, &implementation::on_view_node_properties));
+
 		schedule_update();
 	}
 
-	bool on_view_node_properties(k3d::inode* const Object)
+	void on_document_closed()
 	{
-		if(Object != m_node)
+		block_updates();
+	}
+
+	bool on_view_node_properties(k3d::inode* const Node)
+	{
+		if(Node != m_node)
 		{
-			m_node = Object;
+			m_node = Node;
 
 			m_node_deleted_connection.disconnect();
 			m_node_name_change_connection.disconnect();
@@ -140,7 +155,7 @@ struct control::implementation :
 				m_node_name_change_connection = m_node->name_changed_signal().connect(sigc::mem_fun(*this, &implementation::update_label));
 				k3d::iproperty_collection* const property_collection = dynamic_cast<k3d::iproperty_collection*>(m_node);
 				if(property_collection)
-					m_node_properties_changed_connection = property_collection->properties_changed_signal().connect(sigc::mem_fun(*this, &implementation::on_node_properties_changed));
+					m_node_properties_changed_connection = property_collection->properties_changed_signal().connect(sigc::hide(sigc::mem_fun(*this, &implementation::on_node_properties_changed)));
 
 			}
 
@@ -187,31 +202,39 @@ struct control::implementation :
 
 		k3d::istate_recorder* const state_recorder = &m_document_state.document().state_recorder();
 
-		// Add controls for managing user properties ...
-		k3d::iproperty_collection* const property_collection = dynamic_cast<k3d::iproperty_collection*>(m_node);
-		if(property_collection)
+		// Add controls for cameras and camera render engines ...
+		if(dynamic_cast<k3d::icamera*>(m_node) || dynamic_cast<k3d::icamera_preview_render_engine*>(m_node))
 		{
-			{
-				button::control* const control =
-					new button::control(m_parent, "add_user_property", *Gtk::manage(new Gtk::Image(Gtk::Stock::ADD, Gtk::ICON_SIZE_BUTTON)))
-						<< connect_button(sigc::mem_fun(*this, &implementation::on_add_user_property))
-						<< set_tooltip(_("Add a user property to the node"));
+			button::control* const control =
+				new button::control(m_parent, "render_camera_preview", *Gtk::manage(new Gtk::Image(load_icon("render_preview", Gtk::ICON_SIZE_BUTTON))))
+					<< connect_button(sigc::mem_fun(*this, &implementation::on_render_camera_preview))
+					<< set_tooltip(_("Render Preview"));
 
-				toolbar_control->row(0).pack_start(*Gtk::manage(control), Gtk::PACK_SHRINK);
-			}
-
-			{
-				button::control* const control =
-					new button::control(m_parent, "add_renderman_property", *Gtk::manage(new Gtk::Image(Gtk::Stock::ADD, Gtk::ICON_SIZE_BUTTON)))
-						<< connect_button(sigc::mem_fun(*this, &implementation::on_add_renderman_property))
-						<< set_tooltip(_("Add a RenderMan property to the node"));
-
-				toolbar_control->row(0).pack_start(*manage(control), Gtk::PACK_SHRINK);
-			}
+			toolbar_control->row(0).pack_start(*Gtk::manage(control), Gtk::PACK_SHRINK);
 		}
 
-		// Add controls for cameras ...
-		if(dynamic_cast<k3d::icamera*>(m_node) || dynamic_cast<k3d::ipreview_render_engine*>(m_node))
+		if(dynamic_cast<k3d::icamera*>(m_node) || dynamic_cast<k3d::icamera_still_render_engine*>(m_node))
+		{
+			button::control* const control =
+				new button::control(m_parent, "render_camera_frame", *Gtk::manage(new Gtk::Image(load_icon("render_frame", Gtk::ICON_SIZE_BUTTON))))
+					<< connect_button(sigc::mem_fun(*this, &implementation::on_render_camera_frame))
+					<< set_tooltip(_("Render Frame"));
+
+			toolbar_control->row(0).pack_start(*Gtk::manage(control), Gtk::PACK_SHRINK);
+		}
+
+		if(dynamic_cast<k3d::icamera*>(m_node) || dynamic_cast<k3d::icamera_animation_render_engine*>(m_node))
+		{
+			button::control* const control =
+				new button::control(m_parent, "render_camera_animation", *Gtk::manage(new Gtk::Image(load_icon("render_animation", Gtk::ICON_SIZE_BUTTON))))
+					<< connect_button(sigc::mem_fun(*this, &implementation::on_render_camera_animation))
+					<< set_tooltip(_("Render Animation"));
+
+			toolbar_control->row(0).pack_start(*Gtk::manage(control), Gtk::PACK_SHRINK);
+		}
+
+		// Add controls for render engines
+		if(dynamic_cast<k3d::ipreview_render_engine*>(m_node))
 		{
 			button::control* const control =
 				new button::control(m_parent, "render_preview", *Gtk::manage(new Gtk::Image(load_icon("render_preview", Gtk::ICON_SIZE_BUTTON))))
@@ -221,7 +244,7 @@ struct control::implementation :
 			toolbar_control->row(0).pack_start(*Gtk::manage(control), Gtk::PACK_SHRINK);
 		}
 
-		if(dynamic_cast<k3d::icamera*>(m_node) || dynamic_cast<k3d::istill_render_engine*>(m_node))
+		if(dynamic_cast<k3d::istill_render_engine*>(m_node))
 		{
 			button::control* const control =
 				new button::control(m_parent, "render_frame", *Gtk::manage(new Gtk::Image(load_icon("render_frame", Gtk::ICON_SIZE_BUTTON))))
@@ -231,7 +254,7 @@ struct control::implementation :
 			toolbar_control->row(0).pack_start(*Gtk::manage(control), Gtk::PACK_SHRINK);
 		}
 
-		if(dynamic_cast<k3d::icamera*>(m_node) || dynamic_cast<k3d::ianimation_render_engine*>(m_node))
+		if(dynamic_cast<k3d::ianimation_render_engine*>(m_node))
 		{
 			button::control* const control =
 				new button::control(m_parent, "render_animation", *Gtk::manage(new Gtk::Image(load_icon("render_animation", Gtk::ICON_SIZE_BUTTON))))
@@ -241,28 +264,8 @@ struct control::implementation :
 			toolbar_control->row(0).pack_start(*Gtk::manage(control), Gtk::PACK_SHRINK);
 		}
 
-		// Add controls for camera agnostic engines ...
-		if(dynamic_cast<k3d::ipreview_depict_engine*>(m_node))
-		{
-			button::control* const control =
-				new button::control(m_parent, "render_preview", *Gtk::manage(new Gtk::Image(load_icon("render_preview", Gtk::ICON_SIZE_BUTTON))))
-					<< connect_button(sigc::mem_fun(*this, &implementation::on_depict_preview))
-					<< set_tooltip(_("Render Preview"));
-
-			toolbar_control->row(0).pack_start(*Gtk::manage(control), Gtk::PACK_SHRINK);
-		}
-
-		if(dynamic_cast<k3d::istill_depict_engine*>(m_node))
-		{
-			button::control* const control =
-				new button::control(m_parent, "render_frame", *Gtk::manage(new Gtk::Image(load_icon("render_frame", Gtk::ICON_SIZE_BUTTON))))
-					<< connect_button(sigc::mem_fun(*this, &implementation::on_depict_frame))
-					<< set_tooltip(_("Render Frame"));
-
-			toolbar_control->row(0).pack_start(*Gtk::manage(control), Gtk::PACK_SHRINK);
-		}
-
 		// Get the node properties, grouped together ...
+		k3d::iproperty_collection* const property_collection = dynamic_cast<k3d::iproperty_collection*>(m_node);
 		k3d::iproperty_group_collection::groups_t property_groups;
 		if(property_collection)
 		{
@@ -361,10 +364,9 @@ struct control::implementation :
 				// String properties ...
 				else if(property_type == typeid(std::string))
 				{
-					if(k3d::ienumeration_property* const enumeration_property = dynamic_cast<k3d::ienumeration_property*>(&property))
+					if(dynamic_cast<k3d::ienumeration_property*>(&property))
 					{
-						const k3d::ienumeration_property::enumeration_values_t values(enumeration_property->property_values());
-						enumeration_chooser::control* const control = new enumeration_chooser::control(m_parent, property_name, enumeration_chooser::proxy(property, state_recorder, property_name), values);
+						enumeration_chooser::control* const control = new enumeration_chooser::control(m_parent, property_name, enumeration_chooser::proxy(property, state_recorder, property_name));
 						table->attach(*manage(control), prop_control_begin, prop_control_end, row, row + 1, Gtk::FILL | Gtk::SHRINK, Gtk::FILL | Gtk::SHRINK);
 
 						entry_list.push_back(control);
@@ -424,30 +426,6 @@ struct control::implementation :
 
 					entry_list.push_back(control);
 				}
-				// Position properties ...
-				else if(property_type == typeid(k3d::vector3) && property_name == "position")
-				{
-					position::control* const control = new position::control(m_parent, "position", position::proxy(property, state_recorder, property_name));
-					table->attach(*manage(control), prop_control_begin, prop_control_end, row, row + 1, Gtk::FILL | Gtk::SHRINK, Gtk::FILL | Gtk::SHRINK);
-
-					entry_list.push_back(control);
-				}
-				// Orientation properties ...
-				else if(property_type == typeid(k3d::angle_axis) && property_name == "orientation")
-				{
-					orientation::control* const control = new orientation::control(m_parent, "orientation", orientation::proxy(property, state_recorder, property_name));
-					table->attach(*manage(control), prop_control_begin, prop_control_end, row, row + 1, Gtk::FILL | Gtk::SHRINK, Gtk::FILL | Gtk::SHRINK);
-
-					entry_list.push_back(control);
-				}
-				// Scale properties ...
-				else if(property_type == typeid(k3d::vector3) && property_name == "scale")
-				{
-					scale::control* const control = new scale::control(m_parent, "scale", scale::proxy(property, state_recorder, property_name));
-					table->attach(*manage(control), prop_control_begin, prop_control_end, row, row + 1, Gtk::FILL | Gtk::SHRINK, Gtk::FILL | Gtk::SHRINK);
-
-					entry_list.push_back(control);
-				}
 				// k3d::bounding_box3 properties ...
 				else if(property_type == typeid(k3d::bounding_box3))
 				{
@@ -456,10 +434,10 @@ struct control::implementation :
 
 					entry_list.push_back(control);
 				}
-				// k3d::vector3 properties ...
-				else if(property_type == typeid(k3d::vector3))
+				// k3d::point3 properties ...
+				else if(property_type == typeid(k3d::point3) || property_type == typeid(k3d::vector3) || property_type == typeid(k3d::normal3))
 				{
-					position::control* const control = new position::control(m_parent, property_name, position::proxy(property));
+					point::control* const control = new point::control(m_parent, property_name, point::proxy(property));
 					table->attach(*manage(control), prop_control_begin, prop_control_end, row, row + 1, Gtk::FILL | Gtk::SHRINK, Gtk::FILL | Gtk::SHRINK);
 
 					entry_list.push_back(control);
@@ -467,7 +445,7 @@ struct control::implementation :
 				// k3d::angle_axis properties ...
 				else if(property_type == typeid(k3d::angle_axis))
 				{
-					orientation::control* const control = new orientation::control(m_parent, "orientation", orientation::proxy(property, state_recorder, property_name));
+					angle_axis::control* const control = new angle_axis::control(m_parent, property_name, angle_axis::proxy(property, state_recorder, property_name));
 					table->attach(*manage(control), prop_control_begin, prop_control_end, row, row + 1, Gtk::FILL | Gtk::SHRINK, Gtk::FILL | Gtk::SHRINK);
 
 					entry_list.push_back(control);
@@ -488,7 +466,7 @@ struct control::implementation :
 				}
 				else
 				{
-					k3d::log() << warning << "unknown property type: " << property_type.name() << " name: " << property_name << std::endl;
+					k3d::log() << warning << k3d_file_reference << "unknown property type: " << property_type.name() << " name: " << property_name << std::endl;
 				}
 
 				// Provide a "delete" button for user properties ...
@@ -503,6 +481,17 @@ struct control::implementation :
 				}
 			}
 
+			// Add controls for managing user properties ...
+			if(property_collection)
+			{
+				button::control* const control =
+					new button::control(m_parent, "add_user_property", *Gtk::manage(new Gtk::Image(Gtk::Stock::ADD, Gtk::ICON_SIZE_BUTTON)))
+						<< connect_button(sigc::mem_fun(*this, &implementation::on_add_user_property))
+						<< set_tooltip(_("Add a user property to this node"));
+
+				table->attach(*Gtk::manage(control), prop_delete_begin, prop_delete_end, row, row + 1, Gtk::SHRINK, Gtk::SHRINK);
+			}
+
 			// Set focus chain
 			table->set_focus_chain(entry_list);
 		}
@@ -510,67 +499,79 @@ struct control::implementation :
 		m_vbox.show_all();
 	}
 
-	void on_render_preview()
+	void on_render_camera_preview()
 	{
 		k3d::icamera* camera = dynamic_cast<k3d::icamera*>(m_node);
+		if(!camera)
+			camera = pick_camera(m_document_state);
+		if(!camera)
+			return;
+
+		k3d::icamera_preview_render_engine* render_engine = dynamic_cast<k3d::icamera_preview_render_engine*>(m_node);
+		if(!render_engine)
+			render_engine = pick_camera_preview_render_engine(m_document_state);
+		if(!render_engine)
+			return;
+
+		render_camera_preview(*camera, *render_engine);
+	}
+
+	void on_render_camera_frame()
+	{
+		k3d::icamera* camera = dynamic_cast<k3d::icamera*>(m_node);
+		if(!camera)
+			camera = pick_camera(m_document_state);
+		if(!camera)
+			return;
+
+		k3d::icamera_still_render_engine* render_engine = dynamic_cast<k3d::icamera_still_render_engine*>(m_node);
+		if(!render_engine)
+			render_engine = pick_camera_still_render_engine(m_document_state);
+		if(!render_engine)
+			return;
+
+		render_camera_frame(*camera, *render_engine);
+	}
+
+	void on_render_camera_animation()
+	{
+		k3d::icamera* camera = dynamic_cast<k3d::icamera*>(m_node);
+		if(!camera)
+			camera = pick_camera(m_document_state);
+		if(!camera)
+			return;
+
+		k3d::icamera_animation_render_engine* render_engine = dynamic_cast<k3d::icamera_animation_render_engine*>(m_node);
+		if(!render_engine)
+			render_engine = pick_camera_animation_render_engine(m_document_state);
+		if(!render_engine)
+			return;
+
+		render_camera_animation(m_document_state, *camera, *render_engine);
+	}
+
+	void on_render_preview()
+	{
 		k3d::ipreview_render_engine* render_engine = dynamic_cast<k3d::ipreview_render_engine*>(m_node);
-		render_preview(m_document_state, camera, render_engine);
+		return_if_fail(render_engine);
+
+		render_preview(*render_engine);
 	}
 
 	void on_render_frame()
 	{
-		k3d::icamera* camera = dynamic_cast<k3d::icamera*>(m_node);
-		if(!camera)
-			camera = pick_camera(m_document_state);
-		if(!camera)
-			return;
-
 		k3d::istill_render_engine* render_engine = dynamic_cast<k3d::istill_render_engine*>(m_node);
-		if(!render_engine)
-			render_engine = pick_still_render_engine(m_document_state);
-		if(!render_engine)
-			return;
+		return_if_fail(render_engine);
 
-		render_frame(*camera, *render_engine);
+		render_frame(*render_engine);
 	}
 
 	void on_render_animation()
 	{
-		k3d::icamera* camera = dynamic_cast<k3d::icamera*>(m_node);
-		if(!camera)
-			camera = pick_camera(m_document_state);
-		if(!camera)
-			return;
-
 		k3d::ianimation_render_engine* render_engine = dynamic_cast<k3d::ianimation_render_engine*>(m_node);
-		if(!render_engine)
-			render_engine = pick_animation_render_engine(m_document_state);
-		if(!render_engine)
-			return;
+		return_if_fail(render_engine);
 
-		render_animation(m_document_state, *camera, *render_engine);
-	}
-
-	void on_depict_preview()
-	{
-		k3d::ipreview_depict_engine* render_engine = dynamic_cast<k3d::ipreview_depict_engine*>(m_node);
-		if(!render_engine)
-			render_engine = pick_preview_depict_engine(m_document_state);
-		if(!render_engine)
-			return;
-
-		depict_preview(*render_engine);
-	}
-
-	void on_depict_frame()
-	{
-		k3d::istill_depict_engine* render_engine = dynamic_cast<k3d::istill_depict_engine*>(m_node);
-		if(!render_engine)
-			render_engine = pick_still_depict_engine(m_document_state);
-		if(!render_engine)
-			return;
-
-		depict_frame(*render_engine);
+		render_animation(m_document_state, *render_engine);
 	}
 
 	void on_delete_user_property(k3d::iproperty_collection* Collection, k3d::iproperty* Property)
@@ -598,13 +599,6 @@ struct control::implementation :
 		window->show();
 	}
 
-	void on_add_renderman_property()
-	{
-		return_if_fail(m_node);
-		add_renderman_property* const window = new add_renderman_property(*m_node, m_parent);
-		window->show();
-	}
-
 	/// Stores a reference to the owning document
 	document_state& m_document_state;
 	/// Stores a reference to the currently-selected node (if any)
@@ -626,6 +620,9 @@ struct control::implementation :
 	collapsible_frame::group m_collapsible_frame_group;
 
 	sigc::signal2<void, const std::string&, const std::string&> m_command_signal;
+
+	/// Signal that will be emitted whenever this control should grab the panel focus
+	sigc::signal<void> m_panel_grab_signal;
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -636,6 +633,8 @@ control::control(document_state& DocumentState, k3d::icommand_node& Parent) :
 	m_implementation(new implementation(DocumentState, *this))
 {
 	m_implementation->m_command_signal.connect(sigc::mem_fun(*this, &control::record_command));
+
+	m_implementation->m_scrolled_window.signal_button_press_event().connect(sigc::bind_return(sigc::hide(m_implementation->m_panel_grab_signal.make_slot()), false), false);
 	
 	pack_start(m_implementation->m_label, Gtk::PACK_SHRINK);
 	pack_start(m_implementation->m_scrolled_window, Gtk::PACK_EXPAND_WIDGET);
@@ -647,18 +646,12 @@ control::~control()
 	delete m_implementation;
 }
 
-bool control::execute_command(const std::string& Command, const std::string& Arguments)
+sigc::connection control::connect_focus_signal(const sigc::slot<void>& Slot)
 {
-/*
-	if(m_implementation->execute_command(Command, Arguments))
-		return true;
-*/
-
-	return ui_component::execute_command(Command, Arguments);
+	return m_implementation->m_panel_grab_signal.connect(Slot);
 }
 
 } // namespace node_properties
 
 } // namespace libk3dngui
 
-
