@@ -26,7 +26,6 @@
 #include <gtkmm/widget.h>
 
 #include "command_arguments.h"
-#include "cursors.h"
 #include "document_state.h"
 #include "icons.h"
 #include "interactive.h"
@@ -63,6 +62,7 @@ struct selection_input_model::implementation :
 		m_start_selection(k3d::selection::record::empty_record()),
 		m_motion_type(MOTION_NONE),
 		m_extended_mode(true),
+		m_extended_component_mode(true),
 		m_paint_mode(true),
 		m_double_click_mode(true)
 	{
@@ -75,7 +75,7 @@ struct selection_input_model::implementation :
 
 	void on_button_down(viewport::control& Viewport, const GdkEventButton& Event)
 	{
-		m_start_selection = Viewport.pick_object(k3d::vector2(Event.x, Event.y));
+		m_start_selection = Viewport.pick_object(k3d::point2(Event.x, Event.y));
 		m_motion_type = MOTION_NONE;
 	}
 
@@ -103,10 +103,24 @@ struct selection_input_model::implementation :
 		{
 			operation = REPLACE;
 
-			// Extended selection (except when clicking on nothing)
-			if(m_extended_mode && k3d::selection::get_node(m_start_selection))
+			// If something's under the mouse cursor
+			if(k3d::selection::get_node(m_start_selection))
 			{
-				operation = m_document_state.is_selected(m_start_selection) ? DESELECT : SELECT;
+				bool extended_mode;
+				switch(m_document_state.selection_mode().value())
+				{
+					case SELECT_NODES:
+						// Extended selection
+						extended_mode = m_extended_mode;
+					break;
+
+					default:
+						// Extended component selection
+						extended_mode = m_extended_component_mode;
+				}
+
+				if(extended_mode)
+					operation = m_document_state.is_selected(m_start_selection) ? DESELECT : SELECT;
 			}
 		}
 
@@ -117,7 +131,7 @@ struct selection_input_model::implementation :
 
 			case SELECT:
 			{
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				arguments.append("selection", m_start_selection);
 				m_command_signal.emit("pick_select", arguments);
@@ -129,7 +143,7 @@ struct selection_input_model::implementation :
 
 			case DESELECT:
 			{
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				arguments.append("selection", m_start_selection);
 				m_command_signal.emit("pick_deselect", arguments);
@@ -141,7 +155,7 @@ struct selection_input_model::implementation :
 
 			case REPLACE:
 			{
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				arguments.append("selection", m_start_selection);
 				m_command_signal.emit("pick_replace", arguments);
@@ -163,7 +177,7 @@ struct selection_input_model::implementation :
 		{
 			case SELECT_NODES:
 			{
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				m_command_signal.emit("deselect_all", arguments);
 
@@ -174,7 +188,7 @@ struct selection_input_model::implementation :
 
 			default:
 			{
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				m_command_signal.emit("node_selection", arguments);
 
@@ -193,11 +207,13 @@ struct selection_input_model::implementation :
 
 		if(modifiers.shift())
 		{
-			m_motion_type = (m_paint_mode && !m_start_selection.empty()) ? MOTION_PAINT_SELECT : MOTION_RUBBER_BAND_SELECT;
+			//m_motion_type = (m_paint_mode && !m_start_selection.empty()) ? MOTION_PAINT_SELECT : MOTION_RUBBER_BAND_SELECT;
+			m_motion_type = MOTION_RUBBER_BAND_SELECT;
 		}
 		else if(modifiers.control())
 		{
-			m_motion_type = (m_paint_mode && !m_start_selection.empty()) ? MOTION_PAINT_DESELECT : MOTION_RUBBER_BAND_DESELECT;
+			//m_motion_type = (m_paint_mode && !m_start_selection.empty()) ? MOTION_PAINT_DESELECT : MOTION_RUBBER_BAND_DESELECT;
+			m_motion_type = MOTION_RUBBER_BAND_DESELECT;
 		}
 		else
 		{
@@ -225,7 +241,7 @@ struct selection_input_model::implementation :
 
 			case MOTION_PAINT_SELECT:
 			{
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				arguments.append("selection", m_start_selection);
 				m_command_signal.emit("start_paint_select", arguments);
@@ -238,7 +254,7 @@ struct selection_input_model::implementation :
 
 			case MOTION_PAINT_DESELECT:
 			{
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				arguments.append("selection", m_start_selection);
 				m_command_signal.emit("start_paint_deselect", arguments);
@@ -254,7 +270,7 @@ struct selection_input_model::implementation :
 				m_rubber_band.box = k3d::rectangle(Event.x, Event.x, Event.y, Event.y);
 				m_rubber_band.draw(Viewport);
 
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				arguments.append("box", m_rubber_band.box);
 				m_command_signal.emit("start_rubber_band_replace", arguments);
@@ -267,7 +283,7 @@ struct selection_input_model::implementation :
 				m_rubber_band.box = k3d::rectangle(Event.x, Event.x, Event.y, Event.y);
 				m_rubber_band.draw(Viewport);
 
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				arguments.append_viewport_coordinates("box", Viewport, m_rubber_band.box);
 				m_command_signal.emit("start_rubber_band_select", arguments);
@@ -280,7 +296,7 @@ struct selection_input_model::implementation :
 				m_rubber_band.box = k3d::rectangle(Event.x, Event.x, Event.y, Event.y);
 				m_rubber_band.draw(Viewport);
 
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				arguments.append("box", m_rubber_band.box);
 				m_command_signal.emit("start_rubber_band_deselect", arguments);
@@ -293,7 +309,7 @@ struct selection_input_model::implementation :
 
 	void on_button_drag(viewport::control& Viewport, const GdkEventMotion& Event)
 	{
-		const k3d::vector2 coordinates(Event.x, Event.y);
+		const k3d::point2 coordinates(Event.x, Event.y);
 
 		switch(m_motion_type)
 		{
@@ -305,7 +321,7 @@ struct selection_input_model::implementation :
 				const k3d::selection::record selection = Viewport.pick_object(coordinates);
 				if(!m_document_state.is_selected(selection))
 				{
-					command_arguments arguments(m_document_state.document());
+					command_arguments arguments;
 					arguments.append_viewport_coordinates("mouse", Viewport, Event);
 					arguments.append("timestamp", m_timer.elapsed());
 					arguments.append("selection", selection);
@@ -315,7 +331,7 @@ struct selection_input_model::implementation :
 				}
 				else
 				{
-					command_arguments arguments(m_document_state.document());
+					command_arguments arguments;
 					arguments.append_viewport_coordinates("mouse", Viewport, Event);
 					arguments.append("timestamp", m_timer.elapsed());
 					m_command_signal.emit("paint_motion", arguments);
@@ -328,7 +344,7 @@ struct selection_input_model::implementation :
 				const k3d::selection::record selection = Viewport.pick_object(coordinates);
 				if(m_document_state.is_selected(selection))
 				{
-					command_arguments arguments(m_document_state.document());
+					command_arguments arguments;
 					arguments.append_viewport_coordinates("mouse", Viewport, Event);
 					arguments.append("timestamp", m_timer.elapsed());
 					arguments.append("selection", selection);
@@ -338,7 +354,7 @@ struct selection_input_model::implementation :
 				}
 				else
 				{
-					command_arguments arguments(m_document_state.document());
+					command_arguments arguments;
 					arguments.append_viewport_coordinates("mouse", Viewport, Event);
 					arguments.append("timestamp", m_timer.elapsed());
 					m_command_signal.emit("paint_motion", arguments);
@@ -355,7 +371,7 @@ struct selection_input_model::implementation :
 				m_rubber_band.box.bottom = Event.y;
 				m_rubber_band.draw(Viewport);
 
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				arguments.append("timestamp", m_timer.elapsed());
 				arguments.append_viewport_coordinates("box", Viewport, m_rubber_band.box);
@@ -374,7 +390,7 @@ struct selection_input_model::implementation :
 
 			case MOTION_PAINT_SELECT:
 			{
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				m_command_signal.emit("end_paint_select", arguments);
 				k3d::finish_state_change_set(m_document_state.document(), _("Paint Select"));
@@ -383,7 +399,7 @@ struct selection_input_model::implementation :
 
 			case MOTION_PAINT_DESELECT:
 			{
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				m_command_signal.emit("end_paint_deselect", arguments);
 				k3d::finish_state_change_set(m_document_state.document(), _("Paint Deselect"));
@@ -393,8 +409,8 @@ struct selection_input_model::implementation :
 			case MOTION_RUBBER_BAND_REPLACE:
 			{
 				const k3d::selection::records selection = Viewport.get_selectable_objects(m_rubber_band.box);
-				
-				command_arguments arguments(m_document_state.document());
+
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				arguments.append("selection", selection);
 				m_command_signal.emit("end_rubber_band_replace", arguments);
@@ -411,7 +427,7 @@ struct selection_input_model::implementation :
 			{
 				const k3d::selection::records selection = Viewport.get_selectable_objects(m_rubber_band.box);
 
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				arguments.append("selection", selection);
 				m_command_signal.emit("end_rubber_band_select", arguments);
@@ -427,7 +443,7 @@ struct selection_input_model::implementation :
 			{
 				const k3d::selection::records selection = Viewport.get_selectable_objects(m_rubber_band.box);
 
-				command_arguments arguments(m_document_state.document());
+				command_arguments arguments;
 				arguments.append_viewport_coordinates("mouse", Viewport, Event);
 				arguments.append("selection", selection);
 				m_command_signal.emit("end_rubber_band_deselect", arguments);
@@ -442,46 +458,46 @@ struct selection_input_model::implementation :
 		m_motion_type = MOTION_NONE;
 	}
 
-	bool execute_command(const std::string& Command, const std::string& Arguments)
+	const k3d::icommand_node::result execute_command(const std::string& Command, const std::string& Arguments)
 	{
 		try
 		{
 			if(Command == "pick_select")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
-				const k3d::selection::record selection = arguments.get_selection_record("selection");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
+				const k3d::selection::record selection = arguments.get_selection_record(m_document_state.document(), "selection");
 
 				interactive::move_pointer(viewport, mouse);
 
 				k3d::record_state_change_set change_set(m_document_state.document(), _("Select"));
 				m_document_state.select(selection);
 
-				return true;
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "pick_deselect")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
-				const k3d::selection::record selection = arguments.get_selection_record("selection");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
+				const k3d::selection::record selection = arguments.get_selection_record(m_document_state.document(), "selection");
 
 				interactive::move_pointer(viewport, mouse);
 
 				k3d::record_state_change_set change_set(m_document_state.document(), _("Deselect"));
 				m_document_state.deselect(selection);
 
-				return true;
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "pick_replace")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
-				const k3d::selection::record selection = arguments.get_selection_record("selection");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
+				const k3d::selection::record selection = arguments.get_selection_record(m_document_state.document(), "selection");
 
 				interactive::move_pointer(viewport, mouse);
 
@@ -489,43 +505,43 @@ struct selection_input_model::implementation :
 				m_document_state.deselect_all();
 				m_document_state.select(selection);
 
-				return true;
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "deselect_all")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
 
 				interactive::move_pointer(viewport, mouse);
 
 				k3d::record_state_change_set change_set(m_document_state.document(), _("Deselect All"));
 				m_document_state.deselect_all();
 
-				return true;
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "node_selection")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
 
 				interactive::move_pointer(viewport, mouse);
 
 				k3d::record_state_change_set change_set(m_document_state.document(), _("Node Selection"));
 				m_document_state.set_selection_mode(SELECT_NODES);
 
-				return true;
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "start_paint_select")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
-				const k3d::selection::record selection = arguments.get_selection_record("selection");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
+				const k3d::selection::record selection = arguments.get_selection_record(m_document_state.document(), "selection");
 
 				interactive::move_pointer(viewport, mouse);
 
@@ -533,16 +549,16 @@ struct selection_input_model::implementation :
 				m_document_state.select(selection);
 
 				m_timer.restart();
-				
-				return true;
+
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "start_paint_deselect")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
-				const k3d::selection::record selection = arguments.get_selection_record("selection");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
+				const k3d::selection::record selection = arguments.get_selection_record(m_document_state.document(), "selection");
 
 				interactive::move_pointer(viewport, mouse);
 
@@ -550,83 +566,83 @@ struct selection_input_model::implementation :
 				m_document_state.deselect(selection);
 
 				m_timer.restart();
-				
-				return true;
+
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "paint_motion")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
 				const double timestamp = arguments.get_double("timestamp");
 
 				interactive::warp_pointer(viewport, mouse, timestamp, m_timer);
 
-				return true;
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "paint_select")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
 				const double timestamp = arguments.get_double("timestamp");
-				const k3d::selection::record selection = arguments.get_selection_record("selection");
+				const k3d::selection::record selection = arguments.get_selection_record(m_document_state.document(), "selection");
 
 				interactive::warp_pointer(viewport, mouse, timestamp, m_timer);
 
 				m_document_state.select(selection);
 
-				return true;
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "paint_deselect")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
 				const double timestamp = arguments.get_double("timestamp");
-				const k3d::selection::record selection = arguments.get_selection_record("selection");
+				const k3d::selection::record selection = arguments.get_selection_record(m_document_state.document(), "selection");
 
 				interactive::warp_pointer(viewport, mouse, timestamp, m_timer);
 
 				m_document_state.deselect(selection);
 
-				return true;
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "end_paint_select")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
 
 				interactive::warp_pointer(viewport, mouse);
 
 				k3d::finish_state_change_set(m_document_state.document(), _("Paint Select"));
 
-				return true;
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "end_paint_deselect")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
 
 				interactive::warp_pointer(viewport, mouse);
 
 				k3d::finish_state_change_set(m_document_state.document(), _("Paint Deselect"));
 
-				return true;
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "start_rubber_band_replace")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
 				const k3d::rectangle box = arguments.get_viewport_rectangle("box");
 
 				interactive::move_pointer(viewport, mouse);
@@ -634,15 +650,15 @@ struct selection_input_model::implementation :
 				m_rubber_band.draw(viewport);
 
 				m_timer.restart();
-				
-				return true;
+
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "start_rubber_band_select")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
 				const k3d::rectangle box = arguments.get_viewport_rectangle("box");
 
 				interactive::move_pointer(viewport, mouse);
@@ -650,15 +666,15 @@ struct selection_input_model::implementation :
 				m_rubber_band.draw(viewport);
 
 				m_timer.restart();
-				
-				return true;
+
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "start_rubber_band_deselect")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
 				const k3d::rectangle box = arguments.get_viewport_rectangle("box");
 
 				interactive::move_pointer(viewport, mouse);
@@ -666,15 +682,15 @@ struct selection_input_model::implementation :
 				m_rubber_band.draw(viewport);
 
 				m_timer.restart();
-				
-				return true;
+
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "rubber_band_motion")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
 				const double timestamp = arguments.get_double("timestamp");
 				const k3d::rectangle box = arguments.get_viewport_rectangle("box");
 
@@ -683,15 +699,15 @@ struct selection_input_model::implementation :
 				m_rubber_band.box = box;
 				m_rubber_band.draw(viewport);
 
-				return true;
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "end_rubber_band_replace")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
-				const k3d::selection::records selection = arguments.get_selection_records("selection");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
+				const k3d::selection::records selection = arguments.get_selection_records(m_document_state.document(), "selection");
 
 				interactive::warp_pointer(viewport, mouse);
 				m_rubber_band.draw(viewport);
@@ -700,15 +716,15 @@ struct selection_input_model::implementation :
 				m_document_state.deselect_all();
 				m_document_state.select(selection);
 
-				return true;
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "end_rubber_band_select")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
-				const k3d::selection::records selection = arguments.get_selection_records("selection");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
+				const k3d::selection::records selection = arguments.get_selection_records(m_document_state.document(), "selection");
 
 				interactive::warp_pointer(viewport, mouse);
 				m_rubber_band.draw(viewport);
@@ -716,15 +732,15 @@ struct selection_input_model::implementation :
 				k3d::record_state_change_set change_set(m_document_state.document(), _("Rubber Band Select"));
 				m_document_state.select(selection);
 
-				return true;
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 
 			if(Command == "end_rubber_band_deselect")
 			{
-				command_arguments arguments(m_document_state.document(), Arguments);
+				command_arguments arguments(Arguments);
 				viewport::control& viewport = arguments.get_viewport();
-				const k3d::vector2 mouse = arguments.get_viewport_vector2("mouse");
-				const k3d::selection::records selection = arguments.get_selection_records("selection");
+				const k3d::point2 mouse = arguments.get_viewport_point2("mouse");
+				const k3d::selection::records selection = arguments.get_selection_records(m_document_state.document(), "selection");
 
 				interactive::warp_pointer(viewport, mouse);
 				m_rubber_band.draw(viewport);
@@ -732,15 +748,16 @@ struct selection_input_model::implementation :
 				k3d::record_state_change_set change_set(m_document_state.document(), _("Rubber Band Deselect"));
 				m_document_state.deselect(selection);
 
-				return true;
+				return k3d::icommand_node::RESULT_CONTINUE;
 			}
 		}
 		catch(std::exception& e)
 		{
 			k3d::log() << k3d_file_reference << ": caught exception: " << e.what() << std::endl;
+			return k3d::icommand_node::RESULT_ERROR;
 		}
 
-		return false;
+		return k3d::icommand_node::RESULT_UNKNOWN_COMMAND;
 	}
 
 	/// Stores a reference to the owning document
@@ -762,6 +779,7 @@ struct selection_input_model::implementation :
 	sigc::signal2<void, const std::string&, const std::string&> m_command_signal;
 
 	bool m_extended_mode;
+	bool m_extended_component_mode;
 	bool m_paint_mode;
 	bool m_double_click_mode;
 
@@ -786,6 +804,11 @@ void selection_input_model::set_extended_mode(const bool Enabled)
 	m_implementation->m_extended_mode = Enabled;
 }
 
+void selection_input_model::set_extended_component_mode(const bool Enabled)
+{
+	m_implementation->m_extended_component_mode = Enabled;
+}
+
 void selection_input_model::set_paint_mode(const bool Enabled)
 {
 	m_implementation->m_paint_mode = Enabled;
@@ -831,7 +854,7 @@ sigc::connection selection_input_model::connect_command_signal(const sigc::slot2
 	return m_implementation->m_command_signal.connect(Slot);
 }
 
-bool selection_input_model::execute_command(const std::string& Command, const std::string& Arguments)
+const k3d::icommand_node::result selection_input_model::execute_command(const std::string& Command, const std::string& Arguments)
 {
 	return m_implementation->execute_command(Command, Arguments);
 }
