@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2005, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -23,21 +23,22 @@
 */
 
 #include "application_state.h"
+#include "application_window.h"
+#include "black_box_recorder.h"
 #include "button.h"
 #include "document_state.h"
-#include "file_selection.h"
+#include "file_chooser_dialog.h"
 #include "main_document_window.h"
 #include "messages.h"
 #include "module.h"
 #include "open_uri.h"
+#include "options.h"
 #include "splash_box.h"
 #include "tutorial_menu.h"
 #include "tutorial_message.h"
 #include "tutorial_recorder.h"
-#include "user_interface.h"
 #include "utility.h"
 #include "widget_manip.h"
-#include "window.h"
 
 #include <k3dsdk/application.h>
 #include <k3dsdk/command_node.h>
@@ -46,6 +47,7 @@
 #include <k3dsdk/create_plugins.h>
 #include <k3dsdk/data.h>
 #include <k3dsdk/i18n.h>
+#include <k3dsdk/iapplication.h>
 #include <k3dsdk/icommand_tree.h>
 #include <k3dsdk/ideletable.h>
 #include <k3dsdk/iscript_engine.h>
@@ -54,6 +56,7 @@
 #include <k3dsdk/log.h>
 #include <k3dsdk/module.h>
 #include <k3dsdk/result.h>
+#include <k3dsdk/share.h>
 #include <k3dsdk/string_cast.h>
 #include <k3dsdk/system.h>
 #include <k3dsdk/version.h>
@@ -117,6 +120,7 @@ void setup_default_hotkeys()
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/edit/tools/rotate_tool", Gtk::AccelKey("e").get_key(), Gdk::ModifierType(0));
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/edit/tools/scale_tool", Gtk::AccelKey("r").get_key(), Gdk::ModifierType(0));
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/edit/tools/select_tool", Gtk::AccelKey("q").get_key(), Gdk::ModifierType(0));
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/edit/tools/snap_tool", Gtk::AccelKey("s").get_key(), Gdk::ModifierType(0));
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/edit/tools/unparent", Gtk::AccelKey("p").get_key(), Gdk::SHIFT_MASK);
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/edit/undo", Gtk::AccelKey("z").get_key(), Gdk::CONTROL_MASK);
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/file/close", Gtk::AccelKey("w").get_key(), Gdk::CONTROL_MASK);
@@ -127,15 +131,19 @@ void setup_default_hotkeys()
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/file/save_as", Gtk::AccelKey("s").get_key(), Gdk::CONTROL_MASK | Gdk::SHIFT_MASK);
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/help/manual", Gtk::AccelKey("F1").get_key(), Gdk::ModifierType(0));
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/layout/fullscreen", Gtk::AccelKey("F11").get_key(), Gdk::ModifierType(0));
-	Gtk::AccelMap::add_entry("<k3d-document>/actions/modify/Delete", Gtk::AccelKey("BackSpace").get_key(), Gdk::CONTROL_MASK);
-	Gtk::AccelMap::add_entry("<k3d-document>/actions/modify/Dissolve", Gtk::AccelKey("BackSpace").get_key(), Gdk::ModifierType(0));
-	Gtk::AccelMap::add_entry("<k3d-document>/actions/modify/ExtrudeFaces", Gtk::AccelKey("m").get_key(), Gdk::ModifierType(0));
-	Gtk::AccelMap::add_entry("<k3d-document>/actions/modify/FlipOrientation", Gtk::AccelKey("i").get_key(), Gdk::SHIFT_MASK);
-	Gtk::AccelMap::add_entry("<k3d-document>/actions/modify/GrowSelection", Gtk::AccelKey("equal").get_key(), Gdk::ModifierType(0));
-	Gtk::AccelMap::add_entry("<k3d-document>/actions/modify/SelectEdgeLoops", Gtk::AccelKey("semicolon").get_key(), Gdk::ModifierType(0));
-	Gtk::AccelMap::add_entry("<k3d-document>/actions/modify/SelectEdgeRings", Gtk::AccelKey("l").get_key(), Gdk::ModifierType(0));
-	Gtk::AccelMap::add_entry("<k3d-document>/actions/modify/SubdivideEdges", Gtk::AccelKey("k").get_key(), Gdk::ModifierType(0));
-	Gtk::AccelMap::add_entry("<k3d-document>/actions/modify/SubdivideFaces", Gtk::AccelKey("k").get_key(), Gdk::SHIFT_MASK);
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/layout/hide_unpinned", Gtk::AccelKey("grave").get_key(), Gdk::ModifierType(0));
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/modifier/CollapseEdges", Gtk::AccelKey("b").get_key(), Gdk::ModifierType(0));
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/modifier/ConnectVertices", Gtk::AccelKey("j").get_key(), Gdk::ModifierType(0));
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/modifier/Delete", Gtk::AccelKey("BackSpace").get_key(), Gdk::CONTROL_MASK);
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/modifier/Dissolve", Gtk::AccelKey("BackSpace").get_key(), Gdk::ModifierType(0));
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/modifier/ExtrudeFaces", Gtk::AccelKey("m").get_key(), Gdk::ModifierType(0));
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/modifier/FilletEdges", Gtk::AccelKey("n").get_key(), Gdk::ModifierType(0));
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/modifier/FlipOrientation", Gtk::AccelKey("i").get_key(), Gdk::SHIFT_MASK);
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/modifier/GrowSelection", Gtk::AccelKey("equal").get_key(), Gdk::ModifierType(0));
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/modifier/SelectEdgeLoops", Gtk::AccelKey("semicolon").get_key(), Gdk::ModifierType(0));
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/modifier/SelectEdgeRings", Gtk::AccelKey("l").get_key(), Gdk::ModifierType(0));
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/modifier/SubdivideEdges", Gtk::AccelKey("k").get_key(), Gdk::ModifierType(0));
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/modifier/SubdivideFaces", Gtk::AccelKey("k").get_key(), Gdk::SHIFT_MASK);
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/render/render_frame", Gtk::AccelKey("F12").get_key(), Gdk::SHIFT_MASK);
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/render/render_preview", Gtk::AccelKey("F12").get_key(), Gdk::ModifierType(0));
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/render/render_region", Gtk::AccelKey("F12").get_key(), Gdk::CONTROL_MASK);
@@ -157,18 +165,16 @@ void setup_default_hotkeys()
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/view/set_camera", Gtk::AccelKey("c").get_key(), Gdk::ModifierType(0));
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/view/show_all", Gtk::AccelKey("h").get_key(), Gdk::CONTROL_MASK);
 	Gtk::AccelMap::add_entry("<k3d-document>/actions/view/show_selection", Gtk::AccelKey("h").get_key(), Gdk::CONTROL_MASK | Gdk::SHIFT_MASK);
+	Gtk::AccelMap::add_entry("<k3d-document>/actions/view/toggle_projection", Gtk::AccelKey("o").get_key(), Gdk::ModifierType(0));
 }
 
-const std::string black_box_recorder_started = "Black Box Recorder Started:";
-const std::string black_box_recorder_stopped = "Black Box Recorder Stopped:";
-
 /////////////////////////////////////////////////////////////////////////////
 // assign_hotkeys_popup
 
 class assign_hotkeys_popup :
-	public window
+	public application_window
 {
-	typedef window base;
+	typedef application_window base;
 public:
 	assign_hotkeys_popup() :
 		base("assign_hotkeys", 0)
@@ -180,7 +186,7 @@ public:
 		Gtk::HButtonBox* const box2 = new Gtk::HButtonBox(Gtk::BUTTONBOX_END);
 		box2->pack_start(*Gtk::manage(
 			new button::control(*this, "close", Gtk::Stock::CLOSE)
-			<< connect_button(sigc::mem_fun(*this, &assign_hotkeys_popup::safe_close))
+			<< connect_button(sigc::mem_fun(*this, &assign_hotkeys_popup::close))
 			), Gtk::PACK_SHRINK);
 
 		Gtk::VBox* const box1 = new Gtk::VBox(false, 10);
@@ -193,19 +199,18 @@ public:
 
 		show_all();
 
-		application_state().assign_hotkeys().changed_signal().connect(sigc::mem_fun(*this, &assign_hotkeys_popup::on_assign_hotkeys));
+		application_state::instance().assign_hotkeys().changed_signal().connect(sigc::mem_fun(*this, &assign_hotkeys_popup::on_assign_hotkeys));
 	}
 
-	void on_assign_hotkeys()
+	void on_assign_hotkeys(k3d::iunknown*)
 	{
-		if(!application_state().assign_hotkeys().value())
+		if(!application_state::instance().assign_hotkeys().value())
 			close();
 	}
 
-	bool on_safe_to_close()
+	void on_close()
 	{
-		application_state().assign_hotkeys().set_value(false);
-		return false;
+		application_state::instance().assign_hotkeys().set_value(false);
 	}
 };
 
@@ -219,7 +224,6 @@ class user_interface_implementation :
 	public k3d::iuser_interface,
 	public k3d::iuser_interface_plugin,
 	public k3d::ideletable,
-	public user_interface,
 	public sigc::trackable
 {
 	typedef k3d::command_node::implementation base;
@@ -227,10 +231,8 @@ class user_interface_implementation :
 public:
 	user_interface_implementation() :
 		base("ui", 0),
-		m_show_tutorials(false),
-		m_record_tutorials(false),
-		m_bbr(init_value(true)),
-		m_bbr_script_engine(0)
+		m_show_tutorials(options::nag("show_tutorials")),
+		m_record_tutorials(false)
 	{
 		/// Redirect glib-based logging to our own standard logging mechanism
 		g_log_set_handler("", static_cast<GLogLevelFlags>(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION), log_handler, 0);
@@ -238,15 +240,16 @@ public:
 		g_log_set_handler("GdkPixbuf", static_cast<GLogLevelFlags>(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION), log_handler, 0);
 		g_log_set_handler("Glib", static_cast<GLogLevelFlags>(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION), log_handler, 0);
 		g_log_set_handler("Glib-GObject", static_cast<GLogLevelFlags>(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION), log_handler, 0);
+		g_log_set_handler("GLib-GObject", static_cast<GLogLevelFlags>(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION), log_handler, 0);
 		g_log_set_handler("Gtk", static_cast<GLogLevelFlags>(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION), log_handler, 0);
+		g_log_set_handler("Pango", static_cast<GLogLevelFlags>(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION), log_handler, 0);
 		g_log_set_handler("atkmm", static_cast<GLogLevelFlags>(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION), log_handler, 0);
 		g_log_set_handler("gdkmm", static_cast<GLogLevelFlags>(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION), log_handler, 0);
 		g_log_set_handler("glibmm", static_cast<GLogLevelFlags>(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION), log_handler, 0);
 		g_log_set_handler("gtkmm", static_cast<GLogLevelFlags>(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION), log_handler, 0);
 		g_log_set_handler("pangomm", static_cast<GLogLevelFlags>(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION), log_handler, 0);
 
-		application_state().assign_hotkeys().changed_signal().connect(sigc::mem_fun(*this, &user_interface_implementation::on_assign_hotkeys));
-		m_bbr.changed_signal().connect(sigc::mem_fun(*this, &user_interface_implementation::on_black_box_recorder));
+		application_state::instance().assign_hotkeys().changed_signal().connect(sigc::mem_fun(*this, &user_interface_implementation::on_assign_hotkeys));
 	}
 
 	~user_interface_implementation()
@@ -256,7 +259,8 @@ public:
 		k3d::log() << info << "Saving hotkeys to " << hotkey_path.native_file_string() << std::endl;
 		Gtk::AccelMap::save(hotkey_path.native_file_string());
 
-		stop_black_box_recorder();
+		/** \todo Figure out whether to keep the black box recorder or dump it */
+//		black_box_recorder::instance().stop();
 	}
 
 	static void log_handler(const gchar* LogDomain, GLogLevelFlags LogLevel, const gchar* Message, gpointer UserData)
@@ -289,11 +293,8 @@ public:
 		}
 	}
 
-	const arguments_t parse_command_line(const arguments_t& Arguments, const boost::filesystem::path& SharePath, bool& Quit, bool& Error)
+	const arguments_t parse_command_line(const arguments_t& Arguments, bool& Quit, bool& Error)
 	{
-		// Store the share path where we keep our resources ...
-		m_share_path = SharePath;
-
 		// Keep track of whether to display a splash screen or not ...
 		bool show_splash = true;
 
@@ -325,9 +326,18 @@ public:
 			k3d::log() << warning << "Could not locate or create " << rc_file.native_file_string() << " resource file" << std::endl;
 		}
 
-		const Glib::StringArrayHandle default_files = Gtk::RC::get_default_files();
-		for(Glib::StringArrayHandle::const_iterator file = default_files.begin(); file != default_files.end(); ++file)
-			k3d::log() << info << "Loading GTK resources from " << *file << std::endl;
+		try
+		{
+			const Glib::StringArrayHandle default_files = Gtk::RC::get_default_files();
+			for(Glib::StringArrayHandle::const_iterator file = default_files.begin(); file != default_files.end(); ++file)
+				k3d::log() << info << "Loading GTK resources from " << *file << std::endl;
+		}
+		catch(...)
+		{
+			/** \todo Figure-out why printing a path with a non-ASCII character throws an exception,
+			see http://sourceforge.net/tracker/index.php?func=detail&aid=1364098&group_id=11113&atid=111113 */
+			k3d::log() << error << "Caught exception logging GTK resource paths" << std::endl;
+		}
 
 		// gtkmm expects to parse and modify argc / argv, so create some temporary storage for it to fiddle with ...
 		std::vector<char*> argv_buffer;
@@ -354,7 +364,7 @@ public:
 		{
 			if((*argument) == "--batch")
 			{
-				application_state().batch_mode() = true;
+				application_state::instance().batch_mode() = true;
 			}
 			else if((*argument) == "--no-splash")
 			{
@@ -385,10 +395,10 @@ public:
 			}
 		}
 
-		Gtk::Window::set_default_icon(load_pixbuf(m_share_path / "ngui/pixmap", "default_icon.png"));
+		Gtk::Window::set_default_icon(load_pixbuf(k3d::share_path() / "ngui/pixmap", "default_icon.png"));
 
 		if(show_splash)
-			m_splash_box.reset(new splash_box(m_share_path / "ngui/pixmap"));
+			m_splash_box.reset(new splash_box(k3d::share_path() / "ngui/pixmap"));
 
 		return unused;
 	}
@@ -407,17 +417,13 @@ public:
 		k3d::log() << info << "Loading hotkeys from " << hotkey_path.native_file_string() << std::endl;
 		Gtk::AccelMap::load(hotkey_path.native_file_string());
 
-		k3d::application().close_signal().connect(sigc::mem_fun(*this, &user_interface_implementation::on_application_close));
-
-		m_splash_box.reset();
-
 		k3d::idocument* const document = k3d::application().create_document();
 		return_if_fail(document);
-		
+
 		populate_new_document(*document);
 
 		document_state* const state = new document_state(*document);
-		new main_document_window(*state, *this);
+		new main_document_window(*state);
 
 		if(m_show_tutorials)
 			create_tutorial_menu();
@@ -425,9 +431,10 @@ public:
 		if(m_record_tutorials)
 			create_tutorial_recorder();
 
-		k3d::command_tree().command_signal().connect(sigc::mem_fun(*this, &user_interface_implementation::on_command));
-		if(m_bbr.value())
-			start_black_box_recorder();
+		/** \todo Figure out whether to keep the black box recorder or dump it */
+//		black_box_recorder::instance().start();
+
+		m_splash_box.reset();
 	}
 
 	void start_event_loop()
@@ -440,13 +447,9 @@ public:
 		m_main->quit();
 	}
 
-	void on_application_close()
-	{
-	}
-
 	bool batch_mode()
 	{
-		return application_state().batch_mode();
+		return application_state::instance().batch_mode();
 	}
 
 	void browser_navigate(const std::string& URL)
@@ -471,18 +474,14 @@ public:
 
 	bool tutorial_message(const std::string& Message)
 	{
-		k3d::command_tree().command_signal().emit(*this, k3d::icommand_node::INTERACTIVE, "tutorial_message", Message);
-		return libk3dngui::tutorial_message(Message);
-	}
-
-	void tutorial_mouse_message(const std::string& Message, const mouse_action_t Action, const k3d::key_modifiers Modifiers)
-	{
-		libk3dngui::tutorial_mouse_message(Message, Action, Modifiers);
+		k3d::command_tree().command_signal().emit(*this, k3d::icommand_node::COMMAND_INTERACTIVE, "tutorial_message", Message);
+		return tutorial_message::instance().show_message(Message);
 	}
 
 	bool get_file_path(const k3d::ipath_property::mode_t Mode, const std::string& Type, const std::string& Prompt, const boost::filesystem::path& OldPath, boost::filesystem::path& Result)
 	{
-		return libk3dngui::get_file_path(Mode, Type, Prompt, OldPath, Result);
+		file_chooser_dialog dialog(Prompt, Type, Mode);
+		return dialog.get_file_path(Result);
 	}
 
 	bool show(iunknown& Object)
@@ -499,10 +498,10 @@ public:
 		return Glib::signal_timeout().connect(sigc::bind_return(Slot, true), interval);
 	}
 
-	bool execute_command(const std::string& Command, const std::string& Arguments)
+	const k3d::icommand_node::result execute_command(const std::string& Command, const std::string& Arguments)
 	{
 		if(Command == "tutorial_message")
-			return tutorial_message(Arguments);
+			return tutorial_message(Arguments) ? RESULT_CONTINUE : RESULT_STOP;
 
 		return base::execute_command(Command, Arguments);
 	}
@@ -517,144 +516,18 @@ public:
 		static k3d::plugin_factory<
 			k3d::application_plugin<user_interface_implementation>,
 			k3d::interface_list<k3d::iuser_interface_plugin> > factory(
-			get_class_id(),
-			"NextGenerationUI",
-			"Next Generation User Interface (NGUI)",
-			"");
+				get_class_id(),
+				"NextGenerationUI",
+				"Next Generation User Interface (NGUI)",
+				"");
 
 		return factory;
 	}
 
-	k3d_data(bool, no_name, change_signal, no_undo, local_storage, no_constraint, no_property, no_serialization)& black_box_recorder()
-	{
-		return m_bbr;
-	}
-
 private:
-	void on_black_box_recorder()
+	void on_assign_hotkeys(k3d::iunknown*)
 	{
-		if(m_bbr.value())
-		{
-			start_black_box_recorder();
-			return;
-		}
-
-		if(!application_state().batch_mode())
-		{
-			std::vector<std::string> buttons;
-			buttons.push_back(_("OK"));
-			buttons.push_back(_("Cancel"));
-
-			if(1 != query_message(_("Disable Black Box Recorder?  The Black Box Recorder is a powerful tool for reporting bugs to the K-3D development team."), _("Disable Black Box Recorder?"), 1, buttons))
-			{
-				m_bbr.set_value(true);
-				return;
-			}
-		}
-
-		stop_black_box_recorder();
-	}
-
-	void start_black_box_recorder()
-	{
-		if(m_bbr_log_stream.is_open())
-			return;
-
-		if(!m_bbr_script_engine)
-			m_bbr_script_engine = dynamic_cast<k3d::iscript_engine*>(k3d::create_plugin(k3d::classes::K3DScriptEngine()));
-		return_if_fail(m_bbr_script_engine);
-
-		const boost::filesystem::path bbr_log_path = k3d::system::get_home_directory() / boost::filesystem::path(".k3d/blackbox.log", boost::filesystem::native);
-		if(boost::filesystem::exists(bbr_log_path))
-		{
-			std::stringstream buffer;
-			k3d::filesystem::ifstream stream(bbr_log_path);
-			buffer << stream.rdbuf();
-			stream.close();
-
-			if(buffer.str().find(detail::black_box_recorder_started) != std::string::npos &&
-				buffer.str().rfind(detail::black_box_recorder_stopped) == std::string::npos)
-			{
-				unsigned long i = 1;
-				boost::filesystem::path storage = bbr_log_path;
-				while(boost::filesystem::exists(storage))
-					storage = k3d::system::get_home_directory() / boost::filesystem::path(".k3d/blackbox.log." + k3d::string_cast(i++), boost::filesystem::native);
-
-				try
-				{
-					boost::filesystem::rename(bbr_log_path, storage);
-				}
-				catch(std::exception& e)
-				{
-					k3d::log() << error << "Error archiving blackbox log: " << bbr_log_path.native_file_string() << ": " << e.what() << std::endl;
-				}
-
-				if(!application_state().batch_mode())
-				{
-					std::string command_line;
-					command_line += "k3d-bug-buddy";
-					command_line += " \"" + storage.native_file_string() + "\"";
-					if(!k3d::system::run_process(command_line))
-					{
-						// We fall-back on our old nag message if bug-buddy can't be run for any reason ...
-						boost::format warning_message(_("It appears that a previous K-3D session may have terminated unexpectedly.  A logfile has been saved as\n\n%1%\n\nPlease send this file to the K-3D development team for analysis."));
-						warning_message % storage.native_file_string();
-						message(warning_message.str(), _("Black Box Recorder:"));
-					}
-				}
-			}
-		}
-
-		m_bbr_log_stream.open(bbr_log_path);
-		return_if_fail(m_bbr_log_stream.good());
-
-		std::ostringstream buffer;
-		buffer << detail::black_box_recorder_started << " " << boost::posix_time::second_clock::universal_time() << " UTC\n";
-		buffer << "Package: " << K3D_PACKAGE << "\n";
-		buffer << "Version: " << K3D_VERSION << "\n";
-		buffer << "Platform: " << K3D_HOST << "\n";
-		buffer << "Compiler: " << __VERSION__ << "\n";
-		buffer << "Build Time: " << __DATE__ << " " << __TIME__ << " local\n";
-
-		m_bbr_script_engine->bless_script(m_bbr_log_stream);
-		m_bbr_script_engine->append_comment(m_bbr_log_stream, buffer.str());
-		m_bbr_log_stream.flush();
-	}
-
-	void stop_black_box_recorder()
-	{
-		if(!m_bbr_log_stream.is_open())
-			return;
-
-		std::ostringstream buffer;
-		buffer << detail::black_box_recorder_stopped << " " << boost::posix_time::second_clock::universal_time() << " UTC\n";
-
-		m_bbr_script_engine->append_comment(m_bbr_log_stream, buffer.str());
-		m_bbr_log_stream.flush();
-		m_bbr_log_stream.close();
-	}
-
-	void on_command(k3d::icommand_node& CommandNode, const k3d::icommand_node::type Type, const std::string& Name, const std::string& Arguments)
-	{
-		// We only want user interface commands ... !
-		if(Type != k3d::icommand_node::INTERACTIVE)
-			return;
-
-		// And only if recording is enabled ...
-		if(!m_bbr.value())
-			return;
-
-		// Sanity checks ...
-		return_if_fail(m_bbr_script_engine);
-		return_if_fail(m_bbr_log_stream.is_open());
-
-		m_bbr_script_engine->append_command(m_bbr_log_stream, CommandNode, Name, Arguments);
-		m_bbr_log_stream.flush();
-	}
-
-	void on_assign_hotkeys()
-	{
-		if(application_state().assign_hotkeys().value())
+		if(application_state::instance().assign_hotkeys().value())
 		{
 			new detail::assign_hotkeys_popup();
 			Gtk::Settings::get_default()->property_gtk_can_change_accels() = true;
@@ -669,20 +542,12 @@ private:
 	bool m_show_tutorials;
 	/// Set to true iff we should begin recording a tutorial immediately at startup
 	bool m_record_tutorials;
-	/// Stores the share path where resources are stored
-	boost::filesystem::path m_share_path;
 	/// Stores the path where tutorials should be displayed
 	boost::filesystem::path m_tutorials_path;
 	/// Stores the main loop
 	std::auto_ptr<Gtk::Main> m_main;
 	/// Stores the (optional) splash screen
 	std::auto_ptr<splash_box> m_splash_box;
-	/// Set to true while the black box recorder is enabled
-	k3d_data(bool, no_name, change_signal, no_undo, local_storage, no_constraint, no_property, no_serialization) m_bbr;
-	/// Stream for recording black box data
-	k3d::filesystem::ofstream m_bbr_log_stream;
-	/// Script engine for recording black box data
-	k3d::iscript_engine* m_bbr_script_engine;
 };
 
 /////////////////////////////////////////////////////////////////////////////
