@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2004, Timothy M. Shead
+// Copyright (c) 1995-2006, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -54,22 +54,22 @@ bool render_job(const boost::filesystem::path& GlobalOptionsPath, const boost::f
 {
 	// Sanity checks ...	
 	if(!boost::filesystem::exists(GlobalOptionsPath))
-		{
-			k3d::log() << error << "Options file " << GlobalOptionsPath.native_file_string() << " does not exist" << std::endl;
-			return false;
-		}
+	{
+		k3d::log() << error << "Options file " << GlobalOptionsPath.native_file_string() << " does not exist" << std::endl;
+		return false;
+	}
 
 	if(!boost::filesystem::exists(JobDirectory))
-		{
-			k3d::log() << error << "Job directory " << JobDirectory.native_file_string() << " does not exist" << std::endl;
-			return false;
-		}
+	{
+		k3d::log() << error << "Job directory " << JobDirectory.native_file_string() << " does not exist" << std::endl;
+		return false;
+	}
 
 	if(!boost::filesystem::is_directory(JobDirectory))
-		{
-			k3d::log() << error << "Job directory " << JobDirectory.native_file_string() << " is not a directory" << std::endl;
-			return false;
-		}
+	{
+		k3d::log() << error << "Job directory " << JobDirectory.native_file_string() << " is not a directory" << std::endl;
+		return false;
+	}
 		
 	// Skip the job if it's complete ...
 	if(boost::filesystem::exists(JobDirectory / "complete"))
@@ -85,10 +85,10 @@ bool render_job(const boost::filesystem::path& GlobalOptionsPath, const boost::f
 
 	// Make sure the job is ready ...
 	if(!boost::filesystem::exists(JobDirectory / "ready"))
-		{
-			k3d::log() << error << "Job " << JobDirectory.native_file_string() << " is not ready" << std::endl;
-			return false;
-		}
+	{
+		k3d::log() << error << "Job " << JobDirectory.native_file_string() << " is not ready" << std::endl;
+		return false;
+	}
 
 	// Standard logging ...
 	k3d::log() << info << "Starting Job " << JobDirectory.native_file_string() << std::endl;
@@ -98,13 +98,13 @@ bool render_job(const boost::filesystem::path& GlobalOptionsPath, const boost::f
 
 	// For each directory in the job directory (non-recursive) ...
 	for(boost::filesystem::directory_iterator frame(JobDirectory); frame != boost::filesystem::directory_iterator(); ++frame)
-		{
-			if(!is_directory(*frame))
-				continue;
+	{
+		if(!is_directory(*frame))
+			continue;
 
-			const std::string commandline("k3d-renderframe \"" + GlobalOptionsPath.native_file_string() + "\" \"" + frame->native_file_string() + "\"");
-			k3d::system::run_process(commandline);
-		}
+		const std::string commandline("k3d-renderframe \"" + GlobalOptionsPath.native_file_string() + "\" \"" + frame->native_file_string() + "\"");
+		k3d::system::run_process(commandline);
+	}
 
 	// Switch the job status to complete ...
 	boost::filesystem::rename(JobDirectory / "running", JobDirectory / "complete");
@@ -136,7 +136,7 @@ void usage(const std::string& Name, std::ostream& Stream)
 void print_version(std::ostream& Stream)
 {
 	Stream << "K-3D Version " << K3D_VERSION << std::endl;
-	Stream << "Copyright (c) 1995-2005, Timothy M. Shead.  See the AUTHORS file for contributors." << std::endl;
+	Stream << "Copyright (c) 1995-2006, Timothy M. Shead.  See the AUTHORS file for contributors." << std::endl;
 	Stream << "Licensed by the GNU General Public License.  See the COPYING file for details." << std::endl;
 	Stream << "K-3D Home Page: http://www.k-3d.org" << std::endl;
 	Stream << std::endl;
@@ -157,12 +157,12 @@ void setup_logging(const std::string& ProcessName, logbufs_t& LogBufs)
 		LogBufs.push_back(new k3d::tag_buf("[" + ProcessName + "]", k3d::log()));
 	
 	if(g_color_level)
-		{
+	{
 #ifdef K3D_PLATFORM_WIN32
-                        LogBufs.push_back(new k3d::ansi_to_win32_buf(k3d::log()));
+		LogBufs.push_back(new k3d::ansi_to_win32_buf(k3d::log()));
 #endif // K3D_PLATFORM_WIN32
-			LogBufs.push_back(new k3d::color_level_buf(k3d::log()));
-		}
+		LogBufs.push_back(new k3d::color_level_buf(k3d::log()));
+	}
 
 	LogBufs.push_back(new k3d::show_level_buf(k3d::log()));
 	
@@ -181,6 +181,15 @@ void setup_logging(const std::string& ProcessName, logbufs_t& LogBufs)
 /// Program main
 int main(int argc, char* argv[])
 {
+#ifndef K3D_PLATFORM_WIN32
+	// Fork ourselves so we don't become a zombie of the K-3D executable
+	const int fork_result = fork();
+	if(fork_result < 0)
+		return 1;
+	else if(fork_result > 0)
+		return 0;
+#endif // !K3D_PLATFORM_WIN32
+	
 	const std::string program_name = boost::filesystem::path(argv[0], boost::filesystem::native).leaf();
 
 	// Put our command-line arguments in a more useable form ...
@@ -188,24 +197,24 @@ int main(int argc, char* argv[])
 
 	// Print a "help" message ...
 	if(std::count(options.begin(), options.end(), "-h") || std::count(options.begin(), options.end(), "--help"))
-		{
-			usage(program_name, std::cout);
-			return 0;
-		}
+	{
+		usage(program_name, std::cout);
+		return 0;
+	}
 
 	// Print version data ...
 	if(std::count(options.begin(), options.end(), "--version"))
-		{
-			print_version(std::cout);
-			return 0;
-		}
+	{
+		print_version(std::cout);
+		return 0;
+	}
 
 	// Otherwise we should have a minimum of two arguments ...
 	if(options.size() < 2)
-		{
-			usage(program_name, k3d::log());
-			return 1;
-		}
+	{
+		usage(program_name, k3d::log());
+		return 1;
+	}
 
 	// Setup logging right away ...
 	logbufs_t logbufs;
@@ -214,12 +223,12 @@ int main(int argc, char* argv[])
 	// Each remaining argument should be a job path to render ...
 	int result = 0;
 	for(unsigned long j = 1; j < options.size(); j++)
-		{
-			if(!render_job(
-				boost::filesystem::path(options[0], boost::filesystem::native),
-				boost::filesystem::path(options[j], boost::filesystem::native)))
-				result = 1;
-		}
+	{
+		if(!render_job(
+			boost::filesystem::path(options[0], boost::filesystem::native),
+			boost::filesystem::path(options[j], boost::filesystem::native)))
+			result = 1;
+	}
 
 #ifndef K3D_PLATFORM_WIN32
 	// For reasons unknown, trying to clean-up the log bufs causes a segfault on Win32
