@@ -1,5 +1,5 @@
 // K-3D SDS preview
-// Copyright (c) 2005, Bart Janssens
+// Copyright (c) 2005-2006, Bart Janssens
 //
 // Contact: bart.janssens@lid.kviv.be
 //
@@ -26,8 +26,8 @@
 namespace k3d
 {
 
-// Comparison operator to sort k3d::vector3 in a map
-inline bool operator < (const k3d::vector3& a, const k3d::vector3& b)
+// Comparison operator to sort k3d::point3 in a map
+inline bool operator < (const k3d::point3& a, const k3d::point3& b)
 {
 	if(a[0] < b[0])
 		return true;
@@ -52,7 +52,7 @@ namespace sds
 
 bool k3d_cache_input::set_input(const k3d::mesh* Input)
 {
-	return_val_if_fail(m_input, false);
+	return_val_if_fail(Input, false);
 	if (m_npoints != Input->points.size() || m_npolyhedra != Input->polyhedra.size())
 		return false;
 	for (unsigned long i = 0; i < m_npolyhedra; ++i)
@@ -60,20 +60,20 @@ bool k3d_cache_input::set_input(const k3d::mesh* Input)
 		if (Input->polyhedra[i]->faces.size() != m_nfaces[i])
 			return false;
 	}
-	m_input = Input;
+	m_input_mesh = Input;
 	init_counters();
 	return true;
 }
 
 k3d_cache_input::k3d_cache_input(const k3d::mesh* Input) :
-		m_input(Input)
+		m_input_mesh(Input)
 {
 	init_counters();
-	for(unsigned long p = 0; p < m_input->polyhedra.size(); ++p)
+	for(unsigned long p = 0; p < m_input_mesh->polyhedra.size(); ++p)
 	{
 		facevertices_t* faces = new facevertices_t();
 		m_faces_per_polyhedron.push_back(faces);
-		k3d::polyhedron& polyhedron = *(m_input->polyhedra[p]);
+		k3d::polyhedron& polyhedron = *(m_input_mesh->polyhedra[p]);
 		for(unsigned long i = 0; i < polyhedron.faces.size(); ++i)
 		{
 			k3d::face* face = polyhedron.faces[i];
@@ -81,37 +81,61 @@ k3d_cache_input::k3d_cache_input(const k3d::mesh* Input) :
 			m_face_vertices.push_back(fv);
 			faces->push_back(fv);
 			k3d::split_edge* first_edge = face->first_edge;
-			k3d::split_edge* edge = first_edge;
-			do {
+			for (k3d::split_edge* edge = first_edge; edge; edge = edge->face_clockwise)
+			{
 				fv->edge_vertices.push_back(get_ev(edge, fv));
-				edge = edge->face_clockwise;
-			} while (edge != first_edge);
+				
+				if(edge->face_clockwise == first_edge)
+					break;
+			}
+			for(k3d::face::holes_t::iterator hole = face->holes.begin(); hole != face->holes.end(); ++hole)
+			{
+				for(k3d::split_edge* edge = *hole; edge; edge = edge->face_clockwise)
+				{
+					fv->edge_vertices.push_back(get_ev(edge, fv));
+					
+					if(edge->face_clockwise == *hole)
+						break;
+				}
+			}
 		}
 	}
 	m_n_segments = get_segment(m_face_vertices.size()-1) + 1;
 	k3d::log() << debug << "generated " << m_face_vertices.size() << " face vertices, " << m_edge_vertices.size() << " edge vertices and " << m_sds_vertices.size() << " sds vertices." << std::endl;
 }
 
-void k3d_cache_input::update_input(bool all, facevertices_t& updated_maps)
+void k3d_cache_input::update(bool all, facevertices_t& updated_maps)
 {
 	m_modified_segments.clear();
 	unsigned long global_index = 0;
-	for(unsigned long p = 0; p < m_input->polyhedra.size(); ++p)
+	for(unsigned long p = 0; p < m_input_mesh->polyhedra.size(); ++p)
 	{
 		facevertices_t& faces = *(m_faces_per_polyhedron[p]);
-		k3d::polyhedron& polyhedron = *(m_input->polyhedra[p]);
+		k3d::polyhedron& polyhedron = *(m_input_mesh->polyhedra[p]);
 		for(unsigned long i = 0; i < polyhedron.faces.size(); ++i)
 		{
 			k3d::face* face = polyhedron.faces[i];
-			if(all || selected(face->first_edge))
+			if(all || face->selection_weight != 0.0 || selected(face->first_edge))
 			{
 				positions_t points;
-				k3d::split_edge* edge = face->first_edge;
-				do
+				for (k3d::split_edge* edge = face->first_edge; edge; edge = edge->face_clockwise)
 				{
 					points.push_back(&edge->vertex->position);
-					edge = edge->face_clockwise;
-				} while (edge != face->first_edge);
+					m_selected_edges.insert(edge);
+
+					if(edge->face_clockwise == face->first_edge)
+						break;
+				}
+				for(k3d::face::holes_t::iterator hole = face->holes.begin(); hole != face->holes.end(); ++hole)
+				{
+					for(k3d::split_edge* edge = *hole; edge; edge = edge->face_clockwise)
+					{
+						points.push_back(&edge->vertex->position);
+						
+						if(edge->face_clockwise == *hole)
+							break;
+					}
+				}
 				faces[i]->update(points);
 				updated_maps.push_back(faces[i]);
 				m_modified_segments.insert(get_segment(global_index));
@@ -192,9 +216,7 @@ bool k3d_cache_input::selected(k3d::split_edge* first_edge, bool recurse)
 {
 	k3d::split_edge* edge = first_edge;
 	do {
-		//k3d::log() << debug << "selection_weights: " << edge->selection_weight << ", " << edge->vertex->selection_weight << std::endl;
-		if(edge->selection_weight != 0.0 || edge->vertex->selection_weight != 0.0 || (recurse && edge->companion && selected(edge->companion, false)))
-		//if(edge->visible_selection || edge->vertex->visible_selection || (recurse && edge->companion && selected(edge->companion, false)))
+		if(edge->selection_weight != 0.0 || edge->vertex->selection_weight != 0.0 || m_selected_edges.find(edge) != m_selected_edges.end() || (recurse && edge->companion && selected(edge->companion, false)))
 		{
 			return true;
 		}
@@ -213,8 +235,8 @@ void draw_quad(position_t** points, position_t** normals)
 	glBegin(GL_POLYGON);
 	for(int k = 0; k < 4; ++k)
 	{
-		glNormal3dv(*normals[k]);
-		glVertex3dv(*points[k]);
+		glNormal3dv(normals[k]->data());
+		glVertex3dv(points[k]->data());
 	}
 	glEnd();
 }
@@ -222,14 +244,13 @@ void draw_quad(position_t** points, position_t** normals)
 void k3d_opengl_sds_cache::client_output( k3d::mesh * Output )
 {
 	init_lists();
-	segments_t& ModifiedSegments = dynamic_cast<k3d_cache_input*>(m_input)->get_modified_segments();
-	//k3d::log() << debug << "outputting " << ModifiedSegments.size() << " segments" << std::endl;
+	segments_t& ModifiedSegments = dynamic_cast<k3d_cache_input*>(m_first_level_cache)->get_modified_segments();
 	unsigned long Level = m_levels-1;
 	for(segments_t::const_iterator segment_it = ModifiedSegments.begin(); segment_it != ModifiedSegments.end(); ++segment_it)
 	{
 		const unsigned long segment = *segment_it;
 		const unsigned long start = segment * segment_size;
-		const unsigned long end = (start + segment_size) > m_input->face_vertices().size() ? m_input->face_vertices().size() : (start + segment_size);
+		const unsigned long end = (start + segment_size) > m_first_level_cache->face_vertices().size() ? m_first_level_cache->face_vertices().size() : (start + segment_size);
 		glDeleteLists(m_display_lists[segment], 1);
 		m_display_lists[segment] = glGenLists(1);
 		glNewList(m_display_lists[segment],GL_COMPILE);
@@ -254,7 +275,7 @@ void k3d_opengl_sds_cache::client_output( k3d::mesh * Output )
 		}
 		glEndList();
 	}
-	for(unsigned long i = 0; i < dynamic_cast<k3d_cache_input*>(m_input)->get_n_segments(); ++i)
+	for(unsigned long i = 0; i < dynamic_cast<k3d_cache_input*>(m_first_level_cache)->get_n_segments(); ++i)
 		glCallList(m_display_lists[i]);
 	ModifiedSegments.clear();
 }
@@ -262,7 +283,7 @@ void k3d_opengl_sds_cache::client_output( k3d::mesh * Output )
 void k3d_opengl_sds_cache::client_output_nurbs( k3d::mesh * Output )
 {
 	init_lists();
-	segments_t& ModifiedSegments = dynamic_cast<k3d_cache_input*>(m_input)->get_modified_segments();
+	segments_t& ModifiedSegments = dynamic_cast<k3d_cache_input*>(m_first_level_cache)->get_modified_segments();
 	unsigned long Level = m_levels-1;
 	if(Level == 0)
 		return;
@@ -270,7 +291,7 @@ void k3d_opengl_sds_cache::client_output_nurbs( k3d::mesh * Output )
 	{
 		const unsigned long segment = *segment_it;
 		const unsigned long start = segment * segment_size;
-		const unsigned long end = (start + segment_size) > m_input->face_vertices().size() ? m_input->face_vertices().size() : (start + segment_size);
+		const unsigned long end = (start + segment_size) > m_first_level_cache->face_vertices().size() ? m_first_level_cache->face_vertices().size() : (start + segment_size);
 
 		if(!m_nurbs_renderer)
 		{
@@ -278,13 +299,13 @@ void k3d_opengl_sds_cache::client_output_nurbs( k3d::mesh * Output )
 			gluNurbsProperty(m_nurbs_renderer, GLU_AUTO_LOAD_MATRIX, GL_FALSE);
 			gluNurbsProperty(m_nurbs_renderer, GLU_CULLING, GL_TRUE);
 		}
-		
+
 		glDeleteLists(m_display_lists[segment], 1);
 		m_display_lists[segment] = glGenLists(1);
 		glNewList(m_display_lists[segment],GL_COMPILE);
 
 		gl_setup();
-		
+
 		GLfloat gl_modelview_matrix[16];
 		glGetFloatv(GL_MODELVIEW_MATRIX, gl_modelview_matrix);
 		return_if_fail(m_render_state);
@@ -293,14 +314,14 @@ void k3d_opengl_sds_cache::client_output_nurbs( k3d::mesh * Output )
 		gluNurbsProperty(m_nurbs_renderer, GLU_DISPLAY_MODE, GLU_FILL);
 
 		draw_nurbs_patches(start, end, Level);
-		
+
 		if(m_draw_borders)
 		{
 			draw_nurbs_borders(start, end, Level);
 		}
 		glEndList();
 	} // for all segments
-	for(unsigned long i = 0; i < dynamic_cast<k3d_cache_input*>(m_input)->get_n_segments(); ++i)
+	for(unsigned long i = 0; i < dynamic_cast<k3d_cache_input*>(m_first_level_cache)->get_n_segments(); ++i)
 		glCallList(m_display_lists[i]);
 	ModifiedSegments.clear();
 }
@@ -329,7 +350,7 @@ void k3d_opengl_sds_cache::draw_faces_first_level( unsigned long start, unsigned
 {
 	for(unsigned long face = start; face != end; ++face)
 	{
-		face_vertex& f = *m_input->face_vertices()[face];
+		face_vertex& f = *m_first_level_cache->face_vertices()[face];
 		unsigned long size = f.edge_vertices.size();
 		for(unsigned long i = 0; i < size; ++i)
 		{
@@ -347,15 +368,15 @@ void k3d_opengl_sds_cache::draw_borders_first_level( unsigned long start, unsign
 	glColor3d(0.0, 0.0, 1.0);
 	for(unsigned long face = start; face != end; ++face)
 	{
-		face_vertex& f = *m_input->face_vertices()[face];
+		face_vertex& f = *m_first_level_cache->face_vertices()[face];
 		unsigned long size = f.edge_vertices.size();
 		for(unsigned long i = 0; i < size; ++i)
 		{
 			glBegin(GL_LINES);
-			glVertex3dv(f.edge_vertices[i]->vertex);
-			glVertex3dv(f.corners[(i+1)%size]->vertex);
-			glVertex3dv(f.corners[(i+1)%size]->vertex);
-			glVertex3dv(f.edge_vertices[(i+1)%size]->vertex);
+			glVertex3dv(f.edge_vertices[i]->vertex.data());
+			glVertex3dv(f.corners[(i+1)%size]->vertex.data());
+			glVertex3dv(f.corners[(i+1)%size]->vertex.data());
+			glVertex3dv(f.edge_vertices[(i+1)%size]->vertex.data());
 			glEnd();
 		}
 	}
@@ -366,7 +387,7 @@ void k3d_opengl_sds_cache::draw_faces_higher_level( unsigned long start, unsigne
 	int size = static_cast<int>(pow(2.0, static_cast<double>(Level)))+1;
 	for(unsigned long face = start; face != end; ++face)
 	{
-		face_vertex& f = *m_input->face_vertices()[face];
+		face_vertex& f = *m_first_level_cache->face_vertices()[face];
 		for(face_vertex::mipmaps_t::iterator mipmap = f.mipmaps.begin(); mipmap != f.mipmaps.end(); ++mipmap)
 		{
 			const point_array& points = (*mipmap)->points(Level);
@@ -391,20 +412,20 @@ void k3d_opengl_sds_cache::draw_borders_higher_level( unsigned long start, unsig
 	glColor3d(0.0, 0.0, 1.0);
 	for(unsigned long face = start; face != end; ++face)
 	{
-		face_vertex& f = *m_input->face_vertices()[face];
+		face_vertex& f = *m_first_level_cache->face_vertices()[face];
 		for(face_vertex::mipmaps_t::iterator mipmap = f.mipmaps.begin(); mipmap != f.mipmaps.end(); ++mipmap)
 		{
 			const point_array& points = (*mipmap)->points(Level);
 			glBegin(GL_LINES);
 			for(int i = 0; i < size-1; ++i)
 			{
-				glVertex3dv(*points[0][i]);
-				glVertex3dv(*points[0][i+1]);
+				glVertex3dv((*points[0][i]).data());
+				glVertex3dv((*points[0][i+1]).data());
 			}
 			for(int i = 0; i < size-1; ++i)
 			{
-				glVertex3dv(*points[i][size-1]);
-				glVertex3dv(*points[i+1][size-1]);
+				glVertex3dv((*points[i][size-1]).data());
+				glVertex3dv((*points[i+1][size-1]).data());
 			}
 			glEnd();
 		}
@@ -418,7 +439,7 @@ void k3d_opengl_sds_cache::draw_nurbs_patches( unsigned long start, unsigned lon
 	int k_size = n_size + 4;
 	for(unsigned long face = start; face != end; ++face)
 	{
-		face_vertex& f = *m_input->face_vertices()[face];
+		face_vertex& f = *m_first_level_cache->face_vertices()[face];
 		for(face_vertex::mipmaps_t::iterator mipmap = f.mipmaps.begin(); mipmap != f.mipmaps.end(); ++mipmap)
 		{
 			const point_array& nurbs = (*mipmap)->nurbs(Level);
@@ -449,7 +470,7 @@ void k3d_opengl_sds_cache::draw_nurbs_borders( unsigned long start, unsigned lon
 	glColor3d(0.0, 0.0, 1.0);
 	for(unsigned long face = start; face != end; ++face)
 	{
-		face_vertex& f = *m_input->face_vertices()[face];
+		face_vertex& f = *m_first_level_cache->face_vertices()[face];
 		for(face_vertex::mipmaps_t::iterator mipmap = f.mipmaps.begin(); mipmap != f.mipmaps.end(); ++mipmap)
 		{
 			const point_array& nurbs = (*mipmap)->nurbs(Level);
@@ -475,11 +496,11 @@ void k3d_opengl_sds_cache::draw_nurbs_borders( unsigned long start, unsigned lon
 	}
 }
 
-typedef std::map<k3d::vector3, k3d::point*> vertex_map_t;
+typedef std::map<const k3d::point3* const, k3d::point*> vertex_map_t;
 
-k3d::point* create_unique_point(vertex_map_t& VertexMap, const k3d::vector3& Position, k3d::mesh& Mesh)
+k3d::point* create_unique_point(vertex_map_t& VertexMap, const k3d::point3& Position, k3d::mesh& Mesh)
 {
-	vertex_map_t::iterator i = VertexMap.find(Position);
+	vertex_map_t::iterator i = VertexMap.find(&Position);
 	if(i != VertexMap.end())
 		return i->second;
 
@@ -488,53 +509,79 @@ k3d::point* create_unique_point(vertex_map_t& VertexMap, const k3d::vector3& Pos
 	return_val_if_fail(new_point, 0);
 	Mesh.points.push_back(new_point);
 
-	VertexMap.insert(std::make_pair(Position, new_point));
+	VertexMap.insert(std::make_pair(&Position, new_point));
 
 	return new_point;
 }
 
+typedef std::vector<k3d::split_edge*> edges_t;
+typedef std::vector<edges_t> subdivided_edges_t; // each element of this type is a vector of edges that resulted from one edge in the original mesh
+
+/// Copy attributes (selection, material, ...) of the original face to the destination faces
+void copy_face_attributes(const k3d::face& SourceFace, k3d::polyhedron::faces_t& Faces, subdivided_edges_t& Edges, k3d::mesh::points_t& Points) {
+	for (unsigned long face = 0; face < Faces.size(); ++face)
+	{
+		// face attributes to be copied go here:
+		Faces[face]->material = SourceFace.material;
+		Faces[face]->selection_weight = SourceFace.selection_weight;
+	}
+	
+	int edge_number = 0;
+	for (k3d::split_edge* edge = SourceFace.first_edge; edge; edge = edge->face_clockwise)
+	{
+		// vertex attributes to be copied go here:
+		Points[(edge_number-1+Points.size())%Points.size()]->selection_weight = edge->vertex->selection_weight;
+		edges_t& edges = Edges[edge_number];
+		for (edges_t::const_iterator new_edge = edges.begin(); new_edge != edges.end(); ++new_edge)
+			(*new_edge)->selection_weight = edge->selection_weight; // edge attributes to be copied go in this loop
+
+		++edge_number;
+		
+		if(edge->face_clockwise == SourceFace.first_edge)
+			break;
+	}
+}
+
 void k3d_mesh_sds_cache::client_output(k3d::mesh* Output)
 {
 	k3d::mesh& Mesh = *Output;
 	const unsigned long Level = m_levels;
-	/** \todo link materials to original face materials. */
-	k3d_cache_input* input = dynamic_cast<k3d_cache_input*>(m_input);
+	k3d_cache_input* input = dynamic_cast<k3d_cache_input*>(m_first_level_cache);
 	return_if_fail(input);
 	return_if_fail(!input->mesh().polyhedra.empty());
 	return_if_fail(!input->mesh().polyhedra.front()->faces.empty());
-	k3d::imaterial* Material = input->mesh().polyhedra.front()->faces.front()->material;
+	
+	const k3d_cache_input::faces_per_polyhedron_t& faces_per_poly = input->faces_per_polyhedron();
 
-	const segments_t& ModifiedSegments = input->get_modified_segments();
-	const facevertices_t& faces = input->face_vertices();
+	vertex_map_t vertex_map;
 
-	k3d::polyhedron* const polyhedron = new k3d::polyhedron();
-	Mesh.polyhedra.push_back(polyhedron);
-
-	for(segments_t::const_iterator segment_it = ModifiedSegments.begin(); segment_it != ModifiedSegments.end(); ++segment_it)
+	for(unsigned long polynumber = 0; polynumber < faces_per_poly.size(); ++polynumber)
 	{
-		const unsigned long segment = *segment_it;
-		const unsigned long start = segment * segment_size;
-		const unsigned long end = (start + segment_size) > faces.size() ? faces.size() : (start + segment_size);
-		int size = static_cast<int>(pow(2.0, static_cast<double>(Level-1)))+1;
-
-		vertex_map_t vertex_map;
+		k3d::polyhedron* const polyhedron = new k3d::polyhedron();
+		Mesh.polyhedra.push_back(polyhedron);
+		const facevertices_t& faces = *faces_per_poly[polynumber];
 		if(Level-1 == 0)
 		{
-			for(unsigned long face = start; face != end; ++face)
+			for(unsigned long face = 0; face != faces.size(); ++face)
 			{
+				k3d::face& original_face = *(input->mesh().polyhedra[polynumber]->faces[face]);
 				face_vertex& f = *faces[face];
 				unsigned long size = f.edge_vertices.size();
+				k3d::polyhedron::faces_t new_faces;
+				subdivided_edges_t new_edges(size);
+				k3d::mesh::points_t new_corners;
+				
 				for(unsigned long i = 0; i < size; ++i)
 				{
-					const k3d::vector3 vertex1 = f.vertex;
-					const k3d::vector3 vertex2 = f.edge_vertices[i]->vertex;
-					const k3d::vector3 vertex3 = f.corners[(i+1)%size]->vertex;
-					const k3d::vector3 vertex4 = f.edge_vertices[(i+1)%size]->vertex;
+					const k3d::point3& vertex1 = f.vertex;
+					const k3d::point3& vertex2 = f.edge_vertices[i]->vertex;
+					const k3d::point3& vertex3 = f.corners[(i+1)%size]->vertex;
+					const k3d::point3& vertex4 = f.edge_vertices[(i+1)%size]->vertex;
 					k3d::point* point1 = create_unique_point(vertex_map, vertex1, Mesh);
 					k3d::point* point2 = create_unique_point(vertex_map, vertex2, Mesh);
 					k3d::point* point3 = create_unique_point(vertex_map, vertex3, Mesh);
 					k3d::point* point4 = create_unique_point(vertex_map, vertex4, Mesh);
-						//const k3d::vector3 normal = normal(f.vertex, f.edge_vertices[i]->vertex, f.corners[(i+1)%size]->vertex, f.edge_vertices[(i+1)%size]->vertex);
+						//const k3d::point3 normal = normal(f.vertex, f.edge_vertices[i]->vertex, f.corners[(i+1)%size]->vertex, f.edge_vertices[(i+1)%size]->vertex);
 
 					k3d::split_edge* edge1 = new k3d::split_edge(point1);
 					k3d::split_edge* edge2 = new k3d::split_edge(point2);
@@ -546,58 +593,80 @@ void k3d_mesh_sds_cache::client_output(k3d::mesh* Output)
 					edge3->face_clockwise = edge4;
 					edge4->face_clockwise = edge1;
 
-					k3d::face* new_face = new k3d::face(edge1, Material);
+					k3d::face* new_face = new k3d::face(edge1, 0);
 					assert_warning(new_face);
-
+					
 					polyhedron->faces.push_back(new_face);
+					new_faces.push_back(new_face);
+					new_edges[i].push_back(edge2);
+					new_edges[(i+1)%size].push_back(edge3);
+					new_corners.push_back(point3);
 				}
+				copy_face_attributes(original_face, new_faces, new_edges, new_corners);
 			}
 		}
-
-		for(unsigned long face = start; face != end; ++face)
+		else
 		{
-			face_vertex& f = *faces[face];
-			for(face_vertex::mipmaps_t::iterator mipmap = f.mipmaps.begin(); mipmap != f.mipmaps.end(); ++mipmap)
+			int size = static_cast<int>(pow(2.0, static_cast<double>(Level-1)))+1;
+			for(unsigned long face = 0; face != faces.size(); ++face)
 			{
-				const point_array& points = (*mipmap)->points(Level-1);
-						//const point_array& normals = (*mipmap)->normals(Level-1);
-				for(int i = 0; i < size-1; ++i)
+				k3d::face& original_face = *(input->mesh().polyhedra[polynumber]->faces[face]);
+				face_vertex& f = *faces[face];
+				k3d::polyhedron::faces_t new_faces;
+				subdivided_edges_t new_edges(f.edge_vertices.size());
+				k3d::mesh::points_t new_corners(f.edge_vertices.size());
+				int corner_number = 0;
+				for(face_vertex::mipmaps_t::iterator mipmap = f.mipmaps.begin(); mipmap != f.mipmaps.end(); ++mipmap)
 				{
-					for(int j = 0; j < size-1; ++j)
+					const point_array& points = (*mipmap)->points(Level-1);
+							//const point_array& normals = (*mipmap)->normals(Level-1);
+					for(int i = 0; i < size-1; ++i)
 					{
-						const k3d::vector3 vertex1 = *points[i][j];
-						const k3d::vector3 vertex2 = *points[i][j+1];
-						const k3d::vector3 vertex3 = *points[i+1][j+1];
-						const k3d::vector3 vertex4 = *points[i+1][j];
-						k3d::point* point1 = create_unique_point(vertex_map, vertex1, Mesh);
-						k3d::point* point2 = create_unique_point(vertex_map, vertex2, Mesh);
-						k3d::point* point3 = create_unique_point(vertex_map, vertex3, Mesh);
-						k3d::point* point4 = create_unique_point(vertex_map, vertex4, Mesh);
-								//const k3d::vector3 normal1 = *normals[i][j];
-								//const k3d::vector3 normal2 = *normals[i][j+1];
-								//const k3d::vector3 normal3 = *normals[i+1][j+1];
-								//const k3d::vector3 normal4 = *normals[i+1][j];
-
-						k3d::split_edge* edge1 = new k3d::split_edge(point1);
-						k3d::split_edge* edge2 = new k3d::split_edge(point2);
-						k3d::split_edge* edge3 = new k3d::split_edge(point3);
-						k3d::split_edge* edge4 = new k3d::split_edge(point4);
-
-						edge1->face_clockwise = edge2;
-						edge2->face_clockwise = edge3;
-						edge3->face_clockwise = edge4;
-						edge4->face_clockwise = edge1;
-
-						k3d::face* new_face = new k3d::face(edge1, Material);
-						assert_warning(new_face);
-
-						polyhedron->faces.push_back(new_face);
+						for(int j = 0; j < size-1; ++j)
+						{
+							const k3d::point3& vertex1 = *points[i][j];
+							const k3d::point3& vertex2 = *points[i][j+1];
+							const k3d::point3& vertex3 = *points[i+1][j+1];
+							const k3d::point3& vertex4 = *points[i+1][j];
+							k3d::point* point1 = create_unique_point(vertex_map, vertex1, Mesh);
+							k3d::point* point2 = create_unique_point(vertex_map, vertex2, Mesh);
+							k3d::point* point3 = create_unique_point(vertex_map, vertex3, Mesh);
+							k3d::point* point4 = create_unique_point(vertex_map, vertex4, Mesh);
+									//const k3d::point3 normal1 = *normals[i][j];
+									//const k3d::point3 vector2 = *normals[i][j+1];
+									//const k3d::point3 vector3 = *normals[i+1][j+1];
+									//const k3d::point3 normal4 = *normals[i+1][j];
+	
+							k3d::split_edge* edge1 = new k3d::split_edge(point1);
+							k3d::split_edge* edge2 = new k3d::split_edge(point2);
+							k3d::split_edge* edge3 = new k3d::split_edge(point3);
+							k3d::split_edge* edge4 = new k3d::split_edge(point4);
+	
+							edge1->face_clockwise = edge2;
+							edge2->face_clockwise = edge3;
+							edge3->face_clockwise = edge4;
+							edge4->face_clockwise = edge1;
+	
+							k3d::face* new_face = new k3d::face(edge1, 0);
+							assert_warning(new_face);
+	
+							polyhedron->faces.push_back(new_face);
+							new_faces.push_back(new_face);
+							if (i == 0)
+								new_edges[(corner_number-1+new_edges.size())%new_edges.size()].push_back(edge1);
+							if (j == size-2)
+								new_edges[(corner_number)%new_edges.size()].push_back(edge2);
+							if (i == 0 && j == size-2)
+								new_corners[(corner_number-1 + new_corners.size())%new_corners.size()] = point2;
+						}
 					}
+					++corner_number;
 				}
+				copy_face_attributes(original_face, new_faces, new_edges, new_corners);
 			}
 		}
+		k3d::set_companions(*polyhedron);
 	}
-	k3d::set_companions(*polyhedron);
 }
 
 } // namespace sds
