@@ -67,6 +67,7 @@ void edge_vertex::update()
 {
 	if(m_valid && face_vertices[1]) // mesh border needs updating beyond the selection, so we just recalculate all of them
 		return;
+	return_if_fail(corners[1]);
 	vertex = face_vertices[1] ? (corners[0]->original + corners[1]->original + face_vertices[0]->vertex + face_vertices[1]->vertex)/4.0 : (corners[0]->original + corners[1]->original)/2.0;
 	// edge of object
 	if(!face_vertices[1])
@@ -494,6 +495,8 @@ void sds_mipmap::update_normals(int Level)
 
 void face_vertex::update(positions_t& Points)
 {
+	if (Points.size() != corners.size())
+		throw cache_invalid_exception();
 	unsigned long n = Points.size();
 	position_t position(0,0,0);
 	position_t zero(0,0,0);
@@ -607,7 +610,7 @@ void patch_corner::smooth()
 	if(n > 4 && (m_edge_vertices.size() % 2) == 0)
 	{
 		position_t r = s.r(m_Q40);
-		if(r != 0.0)
+		if(r != k3d::point3(0, 0, 0))
 			s.add_h(r, m_Q40, m_Q41, m_Q14);
 	}
 	if(n != 4.0)
@@ -904,7 +907,7 @@ void corner_smoothing::q10(positions_t & Q10, const position_t & Q00)
 	A(n, Q10);
 	for(int i = 0; i < n; ++i)
 	{
-		k3d::vector3& p1 = *Q10[i];
+		k3d::point3& p1 = *Q10[i];
 		k3d::log() << debug << "modified " << p1[0] << ", " << p1[1] << ", " << p1[2];
 		*Q10[i] += Q00;
 		k3d::log() << debug << " to " << p1[0] << ", " << p1[1] << ", " << p1[2] << std::endl;
@@ -1016,12 +1019,12 @@ void corner_smoothing::mat_mult(matrix & A, positions_t & p)
 	}
 }
 
-void corner_smoothing::q12(positions_t & Q21, positions_t & Q12, const positions_t & Q20, const positions_t & Q20_orig)
+void corner_smoothing::q12(positions_t& Q21, positions_t& Q12, const positions_t& Q20, const positions_t& Q20_orig)
 {
 	int n = Q21.size();
 	for(int i = 0; i < n; ++i)
 	{
-		position_t diff = Q20[i] - Q20_orig[i];
+		position_t diff = *Q20[i] - *Q20_orig[i];
 		*Q21[i] += diff;
 		*Q12[(n+i-1)%n] += diff;
 	}
@@ -1038,7 +1041,7 @@ higher_level_cache::higher_level_cache(const unsigned long Levels, const bool Us
 	generate(m_levels-1, Faces);
 }
 
-void higher_level_cache::update_points(facevertices_t& UpdatedMaps)
+void higher_level_cache::update(facevertices_t& UpdatedMaps)
 {
 	//k3d::timer dftimer;
 	for(unsigned long i = 1; i < m_levels; ++i)
