@@ -39,9 +39,12 @@ namespace sds
 {
 
 /// Encapsulates a point position
-typedef k3d::vector3 position_t;
+typedef k3d::point3 position_t;
 typedef std::vector<position_t*> positions_t;
 
+/// Exception thrown when the cache turns out to be invalid (despite point and face-count check)
+class cache_invalid_exception {};
+
 /// Super class for points that can be invalidated
 class valid_point
 {
@@ -750,7 +753,7 @@ public:
 	virtual ~cache_input() {}
 	
 	/// update the first level using the latest input and place all updated face_vertices in updated_maps. If all is true, everything gets updated. First update selected face_vertices using corner point info, then update all edge_vertices and finally all sds_vertices using the update() methods in those classes.
-	virtual void update_input(bool all, facevertices_t& updated_maps) = 0;
+	virtual void update(bool all, facevertices_t& updated_maps) = 0;
 
 	/// Set the input. This depends on the client of course. Returns false if the new input is incompatible with the cache (i.e. different point or faces count).
 	virtual bool set_input(const mesh_type* Input) = 0;
@@ -775,7 +778,7 @@ public:
 	~higher_level_cache();
 
 	/// Update point positions for all faces given in UpdatedMaps
-	void update_points(facevertices_t& UpdatedMaps);
+	void update(facevertices_t& UpdatedMaps);
 
 	/// Get level number
 	const unsigned long levels() const
@@ -818,12 +821,12 @@ template <typename mesh_type> class catmull_clark_cache
 {
 public:
 
-	catmull_clark_cache() : m_input(0), m_cache(0), m_levels(2), m_use_nurbs(false), m_valid(false) {}
+	catmull_clark_cache() : m_first_level_cache(0), m_higher_level_cache(0), m_levels(2), m_use_nurbs(false), m_valid(false) {}
 
 	virtual ~catmull_clark_cache()
 	{
-		delete m_input;
-		delete m_cache;
+		delete m_first_level_cache;
+		delete m_higher_level_cache;
 	}
 
 	/// The number of SDS levels required from 1 to ...
@@ -846,7 +849,7 @@ public:
 	/// Output of the subdivided mesh, which may be placed in the Output argument, or drawn directly to an output device if Output == 0.
 	void output(mesh_type* Output = 0)
 	{
-		if (m_input->face_vertices().size() == 0)
+		if (m_first_level_cache->face_vertices().size() == 0)
 			return;
 		if (m_use_nurbs)
 		{
@@ -861,16 +864,26 @@ public:
 	/// Update the cache.
 	void update()
 	{
-		return_if_fail(m_input);
+		return_if_fail(m_first_level_cache);
 		if (m_valid)
 		{
-			update_detail(false);
+			try
+			{
+				update_detail(false);
+			}
+			catch (cache_invalid_exception& E)
+			{
+				k3d::log() << debug << "Invalidating cache during update" << std::endl;
+				m_valid = false;
+				set_input(m_first_level_cache_mesh);
+				update();
+			}
 		}
 		else // invalid cache
 		{
-			delete m_cache;
+			delete m_higher_level_cache;
 			k3d::log() << debug << "cache reset" << std::endl;
-			m_cache = new higher_level_cache(m_levels, m_use_nurbs, m_input->face_vertices());
+			m_higher_level_cache = new higher_level_cache(m_levels, m_use_nurbs, m_first_level_cache->face_vertices());
 			update_detail(true);
 			m_valid = true;
 		}
@@ -879,11 +892,12 @@ public:
 	/// Set the input,using the structure of the client program
 	void set_input(const mesh_type* const Input)
 	{
-		if (m_valid && m_input && m_input->set_input(Input))
+		m_first_level_cache_mesh = Input;
+		if (m_valid && m_first_level_cache && m_first_level_cache->set_input(Input))
 			return;
-		delete m_input;
+		delete m_first_level_cache;
 		k3d::log() << debug << "input reset" << std::endl;
-		m_input = create_cache_input(Input);
+		m_first_level_cache = create_cache_input(Input);
 		m_valid = false;
 	}
 
@@ -894,24 +908,31 @@ protected:
 	virtual void client_output(mesh_type* Output = 0) = 0;
 	virtual void client_output_nurbs(mesh_type* Output = 0) = 0;
 
-	cache_input<mesh_type>* m_input; // first level cache
-	higher_level_cache* m_cache; // higher levels
+	// cache for the first level (allowing non-quadrilateral faces)
+	cache_input<mesh_type>* m_first_level_cache; // first level cache
+	// input mesh
+	const mesh_type* m_first_level_cache_mesh;
+	// cache for levels > 1 (taking advantage of all-quadrilateral structure)
+	higher_level_cache* m_higher_level_cache;
+	// number of levels to use
 	unsigned long m_levels;
+	// calculate NURBS control points?
 	bool m_use_nurbs;
+	// false if the cache became invalid
 	bool m_valid;
 private:
 	// update helper
 	void update_detail(bool All)
 	{
 		// try to retain our sanity:
-		return_if_fail(m_input);
-		return_if_fail(m_cache);
+		return_if_fail(m_first_level_cache);
+		return_if_fail(m_higher_level_cache);
 
 		// update first level:
 		facevertices_t updated_faces;
-		m_input->update_input(All, updated_faces);
+		m_first_level_cache->update(All, updated_faces);
 		// and the higher levels:
-		m_cache->update_points(updated_faces);
+		m_higher_level_cache->update(updated_faces);
 	}	
 };
 
